@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//  $Id: app.h,v 1.34.2.6 2006/03/25 21:13:15 spamatica Exp $
+//  $Id: app.h,v 1.34.2.14 2009/11/16 11:29:33 lunar_shuttle Exp $
 //
 //  (C) Copyright 1999-2004 Werner Schweer (ws@seh.de)
 //=========================================================
@@ -54,8 +54,10 @@ class MidiPort;
 class ShortcutConfig;
 class Appearance;
 class WaveTrack;
+class AudioOutput;
+class EditInstrument;
 
-extern void configMidiController();
+#define MENU_ADD_SYNTH_ID_BASE 0x1000
 
 //---------------------------------------------------------
 //   MusE
@@ -64,7 +66,8 @@ extern void configMidiController();
 class MusE : public QMainWindow
       {
       Q_OBJECT
-      enum {CMD_CUT, CMD_COPY, CMD_PASTE, CMD_DELETE,
+      enum {CMD_CUT, CMD_COPY, CMD_PASTE, CMD_PASTE_CLONE, 
+            CMD_PASTE_TO_TRACK, CMD_PASTE_CLONE_TO_TRACK, CMD_DELETE,
             CMD_SELECT_ALL, CMD_SELECT_NONE, CMD_SELECT_INVERT,
             CMD_SELECT_ILOOP, CMD_SELECT_OLOOP, CMD_SELECT_PARTS,
             CMD_FOLLOW_NO, CMD_FOLLOW_JUMP, CMD_FOLLOW_CONTINUOUS ,
@@ -73,7 +76,7 @@ class MusE : public QMainWindow
 
       //File menu items:
       enum { CMD_OPEN_RECENT=0, CMD_LOAD_TEMPLATE, CMD_SAVE_AS, CMD_IMPORT_MIDI,
-            CMD_EXPORT_MIDI, CMD_IMPORT_PART, CMD_IMPORT_AUDIO, CMD_QUIT, CMD_OPEN_DRUMS,
+            CMD_EXPORT_MIDI, CMD_IMPORT_PART, CMD_IMPORT_AUDIO, CMD_QUIT, CMD_OPEN_DRUMS, CMD_OPEN_WAVE,
             CMD_OPEN_LIST, CMD_OPEN_LIST_MASTER, CMD_GLOBAL_CONFIG,
             CMD_OPEN_GRAPHIC_MASTER, CMD_OPEN_MIDI_TRANSFORM, CMD_TRANSPOSE,
             CMD_GLOBAL_CUT, CMD_GLOBAL_INSERT, CMD_GLOBAL_SPLIT, CMD_COPY_RANGE,
@@ -95,14 +98,17 @@ class MusE : public QMainWindow
 
       Transport* transport;
       BigTime* bigtime;
+      EditInstrument* editInstrument;
+      
       QPopupMenu *menu_file, *menuView, *menuSettings, *menu_help;
       QPopupMenu *menuEdit, *menuStructure;
       QPopupMenu* menu_audio, *menuAutomation;
-      QPopupMenu* menu_functions;
+      QPopupMenu* menu_functions, *menuScriptPlugins;
       QPopupMenu* select, *master, *midiEdit, *addTrack;
 
       int aid1, aid2, aid3, autoId;
       int tr_id, bt_id, mr_id;
+      int cc_id;
       QPopupMenu* follow;
       int fid0, fid1, fid2;
       QPopupMenu* midiInputPlugins;
@@ -171,7 +177,6 @@ class MusE : public QMainWindow
       void launchBrowser(QString &whereTo);
       void importMidi();
       void importWave();
-      bool importWave(const QString&);
       void importPart();
       void exportMidi();
 
@@ -198,12 +203,13 @@ class MusE : public QMainWindow
       void startPianoroll(PartList* pl);
       void startWaveEditor();
       void startWaveEditor(PartList*);
+      void startSongInfo(bool editable=true);
+
       void startMidiTransformer();
       void writeGlobalConfiguration() const;
       void startEditInstrument();
       void startClipList();
-      //void startMarkerView();
-
+      
       void openRecentMenu();
       void selectProject(int id);
       void cmd(int);
@@ -242,10 +248,14 @@ class MusE : public QMainWindow
       void clearAutomation();
       void bigtimeClosed();
       void mixerClosed();
+      void markerClosed();
+
+      void execDeliveredScript(int);
+      void execUserScript(int);
 
    public slots:
       bool saveAs();
-      void bounceToFile();
+      void bounceToFile(AudioOutput* ao = 0);
       void closeEvent(QCloseEvent*e);
       void loadProjectFile(const QString&);
       void loadProjectFile(const QString&, bool songTemplate, bool loadAll);
@@ -258,6 +268,7 @@ class MusE : public QMainWindow
       void showMarker(bool);
       void importMidi(const QString &file);
       void setUsedTool(int);
+      void showDidYouKnowDialog();
 
    public:
       MusE(int argc, char** argv);
@@ -274,8 +285,7 @@ class MusE : public QMainWindow
       QWidget* mixerWindow();
       QWidget* transportWindow();
       QWidget* bigtimeWindow();
-      //bool importWaveToTrack(QString& name, int track);
-      bool importWaveToTrack(QString& name, unsigned tick, Track* track);
+      bool importWaveToTrack(QString& name, unsigned tick=0, Track* track=NULL);
       void importPartToTrack(QString& filename, unsigned tick, Track* track);
 
       void showTransport(bool flag);
