@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//    $Id: arranger.cpp,v 1.33.2.3 2006/01/05 18:14:38 spamatica Exp $
+//    $Id: arranger.cpp,v 1.33.2.21 2009/11/17 22:08:22 terminator356 Exp $
 //  (C) Copyright 1999-2004 Werner Schweer (ws@seh.de)
 //=========================================================
 
@@ -9,6 +9,7 @@
 
 #include <stdio.h>
 
+#include <values.h>
 #include <qlayout.h>
 #include <qcombobox.h>
 #include <qtoolbutton.h>
@@ -24,7 +25,6 @@
 #include <qstringlist.h>
 #include <qfiledialog.h>
 #include <qcheckbox.h>
-#include <qspinbox.h>
 #include <qpushbutton.h>
 #include <qmainwindow.h>
 #include <qwidgetstack.h>
@@ -36,7 +36,6 @@
 #include "scrollscale.h"
 #include "pcanvas.h"
 #include "poslabel.h"
-#include "intlabel.h"
 #include "xml.h"
 #include "splitter.h"
 #include "lcombo.h"
@@ -57,7 +56,7 @@
 #include "mpevent.h"
 #include "gconfig.h"
 #include "mixer/astrip.h"
-
+#include "spinbox.h"
 //---------------------------------------------------------
 //   TWhatsThis::text
 //---------------------------------------------------------
@@ -91,12 +90,15 @@ Arranger::Arranger(QMainWindow* parent, const char* name)
       {
       _raster  = 0;      // measure
       selected = 0;
-      program  = -1;
-      pan      = -66;
+      // Since program covers 3 controls at once, it is in 'midi controller' units rather than 'gui control' units.
+      //program  = -1;
+      program  = CTRL_VAL_UNKNOWN;
+      pan      = -65;
       volume   = -1;
       setMinimumSize(600, 50);
       showTrackinfoFlag = true;
-
+      
+      cursVal = MAXINT;
       //---------------------------------------------------
       //  ToolBar
       //    create toolbar in toplevel widget
@@ -130,9 +132,11 @@ Arranger::Arranger(QMainWindow* parent, const char* name)
       // song length is limited to 10000 bars; the real song len is limited
       // by overflows in tick computations
       //
-      lenEntry = new QSpinBox(1, 10000, 1, toolbar);
+      lenEntry = new SpinBox(1, 10000, 1, toolbar);
       lenEntry->setValue(song->len());
       connect(lenEntry, SIGNAL(valueChanged(int)), SLOT(songlenChanged(int)));
+      QToolTip::add(lenEntry, tr("song length - bars"));
+      QWhatsThis::add(lenEntry, tr("song length - bars"));
 
       typeBox = new LabelCombo(tr("Type"), toolbar);
       typeBox->insertItem(tr("NO"), 0);
@@ -147,22 +151,20 @@ Arranger::Arranger(QMainWindow* parent, const char* name)
       label = new QLabel(tr("Pitch"), toolbar, "Pitch");
       label->setAlignment(AlignRight|AlignVCenter);
       label->setIndent(3);
-      QSpinBox* globalPitchSpinBox = new QSpinBox(-127, 127, 1, toolbar);
+      globalPitchSpinBox = new SpinBox(-127, 127, 1, toolbar);
       globalPitchSpinBox->setValue(song->globalPitchShift());
       QToolTip::add(globalPitchSpinBox, tr("midi pitch"));
       QWhatsThis::add(globalPitchSpinBox, tr("global midi pitch shift"));
       connect(globalPitchSpinBox, SIGNAL(valueChanged(int)), SLOT(globalPitchChanged(int)));
-
       label = new QLabel(tr("Tempo"), toolbar, "Tempo");
       label->setAlignment(AlignRight|AlignVCenter);
       label->setIndent(3);
-      globalTempoSpinBox = new QSpinBox(50, 200, 1, toolbar);
+      globalTempoSpinBox = new SpinBox(50, 200, 1, toolbar);
       globalTempoSpinBox->setSuffix(QString("%"));
       globalTempoSpinBox->setValue(tempomap.globalTempo());
       QToolTip::add(globalTempoSpinBox, tr("midi tempo"));
       QWhatsThis::add(globalTempoSpinBox, tr("midi tempo"));
       connect(globalTempoSpinBox, SIGNAL(valueChanged(int)), SLOT(globalTempoChanged(int)));
-
       QToolButton* tempo50  = new QToolButton(toolbar, "tempo50");
       tempo50->setText(QString("50%"));
       connect(tempo50, SIGNAL(clicked()), SLOT(setTempo50()));
@@ -193,7 +195,11 @@ Arranger::Arranger(QMainWindow* parent, const char* name)
 
       QWidget* editor = new QWidget(split);
       split->setResizeMode(editor, QSplitter::Stretch);
-      editor->setSizePolicy(QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding, 1000, 100));
+      editor->setSizePolicy(QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding,
+        // Changed by T356. Was causing "large int implicitly truncated" warning. These are UCHAR values...
+        //1000, 100));
+        //232, 100)); // 232 is what it was being truncated to, but what is the right value?...
+        255, 100));
 
       //---------------------------------------------------
       //    Track Info
@@ -210,7 +216,11 @@ Arranger::Arranger(QMainWindow* parent, const char* name)
 
       connect(ib, SIGNAL(toggled(bool)), SLOT(showTrackInfo(bool)));
 
+      // Changed by T356 Mar 8 2008. Did someone accidentally change to header1 recently? 
+      // Muse was saying Tlist: unknown tag <header> at line 32
+      //header = new Header(tracklist, "header1");
       header = new Header(tracklist, "header");
+      
       header->setFixedHeight(30);
 
       QFontMetrics fm1(header->font());
@@ -221,7 +231,8 @@ Arranger::Arranger(QMainWindow* parent, const char* name)
       header->addLabel(tr("S"), fm1.width('S')+fw);
       header->addLabel(tr("C"), fm1.width('C')+fw);
       header->addLabel(tr("Track"), 100);
-      header->addLabel(tr("O-Port"), 60);
+      //header->addLabel(tr("O-Port"), 60);
+      header->addLabel(tr("Port"), 60);
       header->addLabel(tr("Ch"), 30);
       header->addLabel(tr("T"), fm1.width('T')+fw);
       header->setResizeEnabled(false, COL_RECORD);
@@ -295,8 +306,8 @@ Arranger::Arranger(QMainWindow* parent, const char* name)
       connect(canvas, SIGNAL(trackChanged(Track*)), list, SLOT(selectTrack(Track*)));
       connect(list, SIGNAL(keyPressExt(QKeyEvent*)), canvas, SLOT(redirKeypress(QKeyEvent*)));
 
-      // no longer available?
-      //connect(canvas, SIGNAL(horizontalScroll(int)), hscroll, SIGNAL(scrollChanged(int)));
+      connect(this, SIGNAL(redirectWheelEvent(QWheelEvent*)), canvas, SLOT(redirectedWheelEvent(QWheelEvent*)));
+      connect(list, SIGNAL(redirectWheelEvent(QWheelEvent*)), canvas, SLOT(redirectedWheelEvent(QWheelEvent*)));
       
       egrid->addMultiCellWidget(time,           0, 0, 0, 1);
       egrid->addMultiCellWidget(hLine(editor),  1, 1, 0, 1);
@@ -310,9 +321,10 @@ Arranger::Arranger(QMainWindow* parent, const char* name)
       connect(vscroll, SIGNAL(valueChanged(int)), list,   SLOT(setYPos(int)));
       connect(hscroll, SIGNAL(scrollChanged(int)), time,   SLOT(setXPos(int)));
       connect(hscroll, SIGNAL(scaleChanged(int)),  time,   SLOT(setXMag(int)));
-
       connect(canvas,  SIGNAL(timeChanged(unsigned)),   SLOT(setTime(unsigned)));
       connect(canvas,  SIGNAL(verticalScroll(unsigned)),SLOT(verticalScrollSetYpos(unsigned)));
+      connect(canvas,  SIGNAL(horizontalScroll(unsigned)),hscroll, SLOT(setPos(unsigned)));
+      connect(canvas,  SIGNAL(horizontalScrollNoLimit(unsigned)),hscroll, SLOT(setPosNoLimit(unsigned))); 
       connect(time,    SIGNAL(timeChanged(unsigned)),   SLOT(setTime(unsigned)));
 
       connect(canvas, SIGNAL(tracklistChanged()), list, SLOT(tracklistChanged()));
@@ -326,13 +338,33 @@ Arranger::Arranger(QMainWindow* parent, const char* name)
       connect(canvas, SIGNAL(dropMidiFile(const QString&)), SIGNAL(dropMidiFile(const QString&)));
 
       connect(canvas, SIGNAL(toolChanged(int)), SIGNAL(toolChanged(int)));
-      connect(song, SIGNAL(posChanged(int, unsigned, bool)), SLOT(seek()));
+//      connect(song, SIGNAL(posChanged(int, unsigned, bool)), SLOT(seek()));
 
       connect(time, SIGNAL(addMarker(int)), SIGNAL(addMarker(int)));
+      
       configChanged();  // set configuration values
       showTrackInfo(showTrackinfoFlag);
       }
 
+//---------------------------------------------------------
+//   updateHScrollRange
+//---------------------------------------------------------
+
+//void Arranger::updateHScrollRange()
+//{
+//      int s = 0, e = song->len();
+      // Show one more measure.
+//      e += sigmap.ticksMeasure(e);  
+      // Show another quarter measure due to imprecise drawing at canvas end point.
+//      e += sigmap.ticksMeasure(e) / 4;
+      // Compensate for the fixed vscroll width. 
+//      e += canvas->rmapxDev(-vscroll->width()); 
+//      int s1, e1;
+//      hscroll->range(&s1, &e1);
+//      if(s != s1 || e != e1) 
+//        hscroll->setRange(s, e);
+//}
+
 //---------------------------------------------------------
 //   headerMoved
 //---------------------------------------------------------
@@ -351,6 +383,7 @@ void Arranger::setTime(unsigned tick)
       if (tick == MAXINT)
             cursorPos->setEnabled(false);
       else {
+            cursVal = tick;
             cursorPos->setEnabled(true);
             cursorPos->setValue(tick);
             time->setPos(3, tick, false);
@@ -381,8 +414,12 @@ void Arranger::dclickPart(Track* t)
 
 void Arranger::configChanged()
       {
+      // Added by Tim. p3.3.6
+      //printf("Arranger::configChanged\n");
+      
       if (config.canvasBgPixmap.isEmpty()) {
             canvas->setBg(config.partCanvasBg);
+            canvas->setBg(QPixmap());
             //printf("Arranger::configChanged - no bitmap!\n");
       }
       else {
@@ -391,6 +428,7 @@ void Arranger::configChanged()
             canvas->setBg(QPixmap(config.canvasBgPixmap));
       }
       midiTrackInfo->setFont(config.fonts[2]);
+      //updateTrackInfo(type);
       }
 
 //---------------------------------------------------------
@@ -401,39 +439,62 @@ void Arranger::songlenChanged(int n)
       {
       int newLen = sigmap.bar2tick(n, 0, 0);
       song->setLen(newLen);
-      lenEntry->clearFocus();  // ??
       }
-
 //---------------------------------------------------------
 //   songChanged
 //---------------------------------------------------------
 
 void Arranger::songChanged(int type)
       {
-      unsigned endTick = song->len();
-      int offset  = sigmap.ticksMeasure(0);
-      hscroll->setRange(-offset, endTick + offset);  //DEBUG
-      canvas->setOrigin(-offset, 0);
-      time->setOrigin(-offset, 0);
-
-      int bar, beat;
-      unsigned tick;
-      sigmap.tickValues(endTick, &bar, &beat, &tick);
-      if (tick || beat)
-            ++bar;
-      lenEntry->blockSignals(true);
-      lenEntry->setValue(bar);
-      lenEntry->blockSignals(false);
-
-      trackSelectionChanged();
-      canvas->partsChanged();
-      typeBox->setCurrentItem(int(song->mtype()));
-      if (type & SC_SIG)
-            time->redraw();
-      if (type & SC_TEMPO)
-            setGlobalTempo(tempomap.globalTempo());
-      updateTrackInfo();
+      // Is it simply a midi controller value adjustment? Forget it.
+      if(type != SC_MIDI_CONTROLLER)
+      {
+        unsigned endTick = song->len();
+        int offset  = sigmap.ticksMeasure(endTick);
+        hscroll->setRange(-offset, endTick + offset);  //DEBUG
+        canvas->setOrigin(-offset, 0);
+        time->setOrigin(-offset, 0);
+  
+        int bar, beat;
+        unsigned tick;
+        sigmap.tickValues(endTick, &bar, &beat, &tick);
+        if (tick || beat)
+              ++bar;
+        lenEntry->blockSignals(true);
+        lenEntry->setValue(bar);
+        lenEntry->blockSignals(false);
+  
+        trackSelectionChanged();
+        canvas->partsChanged();
+        typeBox->setCurrentItem(int(song->mtype()));
+        if (type & SC_SIG)
+              time->redraw();
+        if (type & SC_TEMPO)
+              setGlobalTempo(tempomap.globalTempo());
+              
+        if(type & SC_TRACK_REMOVED)
+        {
+          AudioStrip* w = (AudioStrip*)(trackInfo->getWidget(2));
+          if(w)
+          {
+            Track* t = w->getTrack();
+            if(t)
+            {
+              TrackList* tl = song->tracks();
+              iTrack it = tl->find(t);
+              if(it == tl->end())
+              {
+                delete w;
+                trackInfo->addWidget(0, 2);
+                selected = 0;
+              } 
+            }   
+          } 
+        }
       }
+            
+      updateTrackInfo(type);
+    }
 
 //---------------------------------------------------------
 //   trackSelectionChanged
@@ -452,7 +513,7 @@ void Arranger::trackSelectionChanged()
       if (track == selected)
             return;
       selected = track;
-      updateTrackInfo();
+      updateTrackInfo(-1);
       }
 
 //---------------------------------------------------------
@@ -462,7 +523,7 @@ void Arranger::trackSelectionChanged()
 void Arranger::modeChange(int mode)
       {
       song->setMType(MType(mode));
-      updateTrackInfo();
+      updateTrackInfo(-1);
       }
 
 //---------------------------------------------------------
@@ -543,6 +604,8 @@ void Arranger::_setRaster(int index)
             1, 0, 768, 384, 192, 96
             };
       _raster = rasterTable[index];
+      // Set the audio record part snapping.
+      song->setRecRaster(_raster);
       canvas->redraw();
       }
 
@@ -577,6 +640,15 @@ void Arranger::cmd(int cmd)
             case CMD_PASTE_PART:
                   ncmd = PartCanvas::CMD_PASTE_PART;
                   break;
+            case CMD_PASTE_CLONE_PART:
+                  ncmd = PartCanvas::CMD_PASTE_CLONE_PART;
+                  break;
+            case CMD_PASTE_PART_TO_TRACK:
+                  ncmd = PartCanvas::CMD_PASTE_PART_TO_TRACK;
+                  break;
+            case CMD_PASTE_CLONE_PART_TO_TRACK:
+                  ncmd = PartCanvas::CMD_PASTE_CLONE_PART_TO_TRACK;
+                  break;
             default:
                   return;
             }
@@ -599,6 +671,7 @@ void Arranger::globalPitchChanged(int val)
 void Arranger::globalTempoChanged(int val)
       {
       audio->msgSetGlobalTempo(val);
+      song->tempoChanged();
       }
 
 //---------------------------------------------------------
@@ -634,7 +707,8 @@ void Arranger::setTempo200()
 
 void Arranger::setGlobalTempo(int val)
       {
-      globalTempoSpinBox->setValue(val);
+      if(val != globalTempoSpinBox->value())
+        globalTempoSpinBox->setValue(val);
       }
 
 //---------------------------------------------------------
@@ -645,6 +719,75 @@ void Arranger::verticalScrollSetYpos(unsigned ypos)
       vscroll->setValue(ypos);
       }
 
+//---------------------------------------------------------
+//   progRecClicked
+//---------------------------------------------------------
+
+void Arranger::progRecClicked()
+      {
+      MidiTrack* track = (MidiTrack*)selected;
+      int portno       = track->outPort();
+      int channel      = track->outChannel();
+      MidiPort* port   = &midiPorts[portno];
+      int program      = port->hwCtrlState(channel, CTRL_PROGRAM);
+      if(program == CTRL_VAL_UNKNOWN || program == 0xffffff) 
+        return;
+
+      unsigned tick = song->cpos();
+      Event a(Controller);
+      a.setTick(tick);
+      a.setA(CTRL_PROGRAM);
+      a.setB(program);
+
+      song->recordEvent(track, a);
+      }
+
+//---------------------------------------------------------
+//   volRecClicked
+//---------------------------------------------------------
+
+void Arranger::volRecClicked()
+      {
+      MidiTrack* track = (MidiTrack*)selected;
+      int portno       = track->outPort();
+      int channel      = track->outChannel();
+      MidiPort* port   = &midiPorts[portno];
+      int volume       = port->hwCtrlState(channel, CTRL_VOLUME);
+      if(volume == CTRL_VAL_UNKNOWN) 
+        return;
+
+      unsigned tick = song->cpos();
+      Event a(Controller);
+      a.setTick(tick);
+      a.setA(CTRL_VOLUME);
+      a.setB(volume);
+
+      song->recordEvent(track, a);
+      }
+
+//---------------------------------------------------------
+//   panRecClicked
+//---------------------------------------------------------
+
+void Arranger::panRecClicked()
+      {
+      MidiTrack* track = (MidiTrack*)selected;
+      int portno       = track->outPort();
+      int channel      = track->outChannel();
+      MidiPort* port   = &midiPorts[portno];
+      int pan          = port->hwCtrlState(channel, CTRL_PANPOT);
+      if(pan == CTRL_VAL_UNKNOWN) 
+        return;
+
+      unsigned tick = song->cpos();
+      Event a(Controller);
+      a.setTick(tick);
+      a.setA(CTRL_PANPOT);
+      a.setB(pan);
+
+      song->recordEvent(track, a);
+      }
+
 //---------------------------------------------------------
 //   recordClicked
 //---------------------------------------------------------
@@ -655,26 +798,36 @@ void Arranger::recordClicked()
       int portno       = track->outPort();
       int channel      = track->outChannel();
       MidiPort* port   = &midiPorts[portno];
-      int program      = port->hwCtrlState(channel, CTRL_PROGRAM);
-      int volume       = port->hwCtrlState(channel, CTRL_VOLUME);
-      int pan          = port->hwCtrlState(channel, CTRL_PANPOT);
-
       unsigned tick = song->cpos();
-      Event a(Controller), b(Controller), c(Controller);
-      a.setTick(tick);
-      b.setTick(tick);
-      c.setTick(tick);
-      a.setA(CTRL_PROGRAM);
-      a.setB(program);
-      b.setA(CTRL_VOLUME);
-      b.setB(volume);
-      c.setA(CTRL_PANPOT);
-      c.setB(pan);
-
-      song->recordEvent(track, a);
-      song->recordEvent(track, b);
-      song->recordEvent(track, c);
+      
+      int program = port->hwCtrlState(channel, CTRL_PROGRAM);
+      if(program != CTRL_VAL_UNKNOWN && program != 0xffffff) 
+      {
+        Event a(Controller);
+        a.setTick(tick);
+        a.setA(CTRL_PROGRAM);
+        a.setB(program);
+        song->recordEvent(track, a);
       }
+      int volume = port->hwCtrlState(channel, CTRL_VOLUME);
+      if(volume != CTRL_VAL_UNKNOWN) 
+      {
+        Event a(Controller);
+        a.setTick(tick);
+        a.setA(CTRL_VOLUME);
+        a.setB(volume);
+        song->recordEvent(track, a);
+      }
+      int pan = port->hwCtrlState(channel, CTRL_PANPOT);
+      if(pan != CTRL_VAL_UNKNOWN) 
+      {
+        Event a(Controller);
+        a.setTick(tick);
+        a.setA(CTRL_PANPOT);
+        a.setB(pan);
+        song->recordEvent(track, a);
+      }
+    }
 
 //---------------------------------------------------------
 //   trackInfoScroll
@@ -720,19 +873,19 @@ void WidgetStack::raiseWidget(int idx)
 //   addWidget
 //---------------------------------------------------------
 
-void WidgetStack::addWidget(QWidget* w, int n)
+void WidgetStack::addWidget(QWidget* w, unsigned int n)
       {
       if (w)
             w->hide();
-      if (stack.size() <= n)
+      if (stack.size() <= n )
             stack.push_back(w);
       else
             stack[n] = w;
       }
 
-QWidget* WidgetStack::getWidget(int n)
+QWidget* WidgetStack::getWidget(unsigned int n)
       {
-      if (stack.size() <= n)
+      if (stack.size() <= n )
             return 0;
       return stack[n];
       }
@@ -781,3 +934,7 @@ void Arranger::clear()
       selected = 0;
       }
 
+void Arranger::wheelEvent(QWheelEvent* ev)
+      {
+      emit redirectWheelEvent(ev);
+      }
