@@ -1,10 +1,12 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//    $Id: tlist.cpp,v 1.31.2.11 2005/08/14 21:50:11 spamatica Exp $
+//    $Id: tlist.cpp,v 1.31.2.31 2009/12/15 03:39:58 terminator356 Exp $
 //  (C) Copyright 1999 Werner Schweer (ws@seh.de)
 //=========================================================
 
+//#include "config.h"
+
 #include <cmath>
 
 #include <qpainter.h>
@@ -22,6 +24,7 @@
 #include "xml.h"
 #include "mididev.h"
 #include "midiport.h"
+#include "midiseq.h"
 #include "comment.h"
 #include "track.h"
 #include "song.h"
@@ -33,6 +36,9 @@
 #include "gconfig.h"
 #include "event.h"
 #include "midiedit/drummap.h"
+#include "synth.h"
+
+extern QPopupMenu* populateAddSynth(QWidget* parent, QObject* obj = 0, const char* slot = 0);
 
 static const int MIN_TRACKHEIGHT = 20;
 static const int WHEEL_DELTA = 120;
@@ -57,7 +63,8 @@ void THeaderTip::maybeTip(const QPoint &pos)
             case COL_CLASS:    p = QHeader::tr("Track Type"); break;
             case COL_NAME:     p = QHeader::tr("Track Name"); break;
             case COL_OCHANNEL: p = QHeader::tr("Output Channel Number"); break;
-            case COL_OPORT:    p = QHeader::tr("Output Port"); break;
+            //case COL_OPORT:    p = QHeader::tr("Output Port"); break;
+            case COL_OPORT:    p = QHeader::tr("Output midi port, or synth midi port"); break;
             case COL_TIMELOCK: p = QHeader::tr("Time Lock"); break;
             default: return;
             }
@@ -96,7 +103,7 @@ TList::TList(Header* hdr, QWidget* parent, const char* name)
 void TList::songChanged(int flags)
       {
       if (flags & (SC_MUTE | SC_SOLO | SC_RECFLAG | SC_TRACK_INSERTED
-         | SC_TRACK_REMOVED | SC_TRACK_MODIFIED | SC_ROUTE | SC_CHANNELS))
+         | SC_TRACK_REMOVED | SC_TRACK_MODIFIED | SC_ROUTE | SC_CHANNELS | SC_MIDI_CHANNEL))
             redraw();
       if (flags & (SC_TRACK_INSERTED | SC_TRACK_REMOVED | SC_TRACK_MODIFIED))
             adjustScrollbar();
@@ -263,7 +270,8 @@ void TList::paint(const QRect& r)
                                           pm = addtrack_auxsendIcon;
                                           break;
                                     case Track::AUDIO_SOFTSYNTH:
-                                          pm = waveIcon;
+                                          //pm = waveIcon;
+                                          pm = synthIcon;
                                           break;
                                     }
                               drawCenteredPixmap(p, pm, r);
@@ -276,6 +284,12 @@ void TList::paint(const QRect& r)
                                     drawCenteredPixmap(p, editmuteSIcon, r);
                               break;
                         case COL_SOLO:
+                              if(track->solo() && track->internalSolo())
+                                    drawCenteredPixmap(p, blacksqcheckIcon, r);
+                              else      
+                              if(track->internalSolo())
+                                    drawCenteredPixmap(p, blacksquareIcon, r);
+                              else
                               if (track->solo())
                                     drawCenteredPixmap(p, bluedotIcon, r);
                               break;
@@ -308,13 +322,24 @@ void TList::paint(const QRect& r)
                               QString s;
                               if (track->isMidiTrack()) {
                                     int outport = ((MidiTrack*)track)->outPort();
-                                    MidiDevice* dev = midiPorts[outport].device();
-                                    if (dev)
-                                          s.sprintf("%d(%s)", outport + 1,
-                                             dev->name().latin1());
-                                    else
-                                          s.sprintf("%d(none)", outport + 1);
+                                    s.sprintf("%d:%s", outport+1, midiPorts[outport].portname().latin1());
                                     }
+                              // Added by Tim. p3.3.9
+                              
+                              else
+                              if(track->type() == Track::AUDIO_SOFTSYNTH)
+                              {
+                                MidiDevice* md = dynamic_cast<MidiDevice*>(track);  
+                                if(md)
+                                {
+                                  int outport = md->midiPort();
+                                  if((outport >= 0) && (outport < MIDI_PORTS))
+                                    s.sprintf("%d:%s", outport+1, midiPorts[outport].portname().latin1());
+                                  else
+                                    s = tr("<none>");
+                                }  
+                              }  
+                              
                               p.drawText(r, Qt::AlignVCenter|Qt::AlignLeft, s);
                               }
                               break;
@@ -371,7 +396,8 @@ void TList::returnPressed()
                         return;
                         }
                   }
-            Track* track = editTrack->clone();
+            //Track* track = editTrack->clone();
+            Track* track = editTrack->clone(false);
             editTrack->setName(editor->text());
             audio->msgChangeTrack(track, editTrack);
             }
@@ -458,32 +484,91 @@ void TList::portsPopupMenu(Track* t, int x, int y)
       switch(t->type()) {
             case Track::MIDI:
             case Track::DRUM:
+            case Track::AUDIO_SOFTSYNTH: 
                   {
                   MidiTrack* track = (MidiTrack*)t;
-                  QPopupMenu* p = midiPortsPopup(0);
+                  
+                  //QPopupMenu* p = midiPortsPopup(0);
+                  MidiDevice* md = 0;
+                  int port = -1; 
+                  if(t->type() == Track::AUDIO_SOFTSYNTH) 
+                  {
+                    //MidiDevice* md = dynamic_cast<MidiDevice*>((SynthI*)t);
+                    md = dynamic_cast<MidiDevice*>(t);
+                    if(md)
+                      port = md->midiPort(); 
+                  }
+                  else   
+                    port = track->outPort();
+                    
+                  QPopupMenu* p = midiPortsPopup(0, port);
                   int n = p->exec(mapToGlobal(QPoint(x, y)), 0);
                   if (n != -1) {
-                        track->setOutPort(n);
-                        song->update();
+                        // Changed by T356.
+                        //track->setOutPort(n);
+                        //audio->msgSetTrackOutPort(track, n);
+                        
+                        //song->update();
                         if (t->type() == Track::DRUM) {
                               bool change = QMessageBox::question(this, tr("Update drummap?"),
                                              tr("Do you want to use same port for all instruments in the drummap?"),
                                              tr("&Yes"), tr("&No"), QString::null, 0, 1);
-                              if (!change) {
+                              audio->msgIdle(true);
+                              if (!change) 
+                              {
+                                    // Delete all port controller events.
+                                    //audio->msgChangeAllPortDrumCtrlEvents(false);
+                                    song->changeAllPortDrumCtrlEvents(false);
+                                    track->setOutPort(n);
+                        
                                     for (int i=0; i<DRUM_MAPSIZE; i++) //Remap all drum instruments to this port
                                           drumMap[i].port = track->outPort();
-                                    }
+                                    // Add all port controller events.
+                                    //audio->msgChangeAllPortDrumCtrlEvents(true);
+                                    song->changeAllPortDrumCtrlEvents(true);
                               }
+                              else
+                              {
+                                //audio->msgSetTrackOutPort(track, n);
+                                track->setOutPortAndUpdate(n);
+                              }
+                              audio->msgIdle(false);
+                              song->update();
                         }
+                        else
+                        if (t->type() == Track::AUDIO_SOFTSYNTH) 
+                        {
+                          if(md != 0)
+                          {
+                            // Idling is already handled in msgSetMidiDevice.
+                            //audio->msgIdle(true);
+                            
+                            // Compiler complains if simple cast from Track to SynthI...
+                            midiSeq->msgSetMidiDevice(&midiPorts[n], (midiPorts[n].device() == md) ? 0 : md);
+                            muse->changeConfig(true);     // save configuration file
+                          
+                            //audio->msgIdle(false);
+                            song->update();
+                          }
+                        }
+                        else
+                        {
+                          audio->msgIdle(true);
+                          //audio->msgSetTrackOutPort(track, n);
+                          track->setOutPortAndUpdate(n);
+                          audio->msgIdle(false);
+                          song->update();
+                        }
+                      }
                   delete p;
                   }
                   break;
+                  
             case Track::WAVE:
             case Track::AUDIO_OUTPUT:
             case Track::AUDIO_INPUT:
             case Track::AUDIO_GROUP:
-            case Track::AUDIO_AUX:
-            case Track::AUDIO_SOFTSYNTH: //TODO
+            case Track::AUDIO_AUX:    //TODO
                   break;
             }
       }
@@ -494,6 +579,28 @@ void TList::portsPopupMenu(Track* t, int x, int y)
 
 void TList::oportPropertyPopupMenu(Track* t, int x, int y)
       {
+      // Added by Tim. p3.3.9
+      if(t->type() == Track::AUDIO_SOFTSYNTH)
+      {
+        SynthI* synth = (SynthI*)t;
+  
+        QPopupMenu* p = new QPopupMenu(this);
+        p->setCheckable(true);
+        p->insertItem(tr("Show Gui"), 0);
+  
+        p->setItemEnabled(0, synth->hasGui());
+        p->setItemChecked(0, synth->guiVisible());
+  
+        int n = p->exec(mapToGlobal(QPoint(x, y)), 0);
+        if (n == 0) {
+              bool show = !synth->guiVisible();
+              audio->msgShowInstrumentGui(synth, show);
+              }
+        delete p;
+        return;
+      }
+      
+      
       if (t->type() != Track::MIDI && t->type() != Track::DRUM)
             return;
       int oPort      = ((MidiTrack*)t)->outPort();
@@ -512,6 +619,7 @@ void TList::oportPropertyPopupMenu(Track* t, int x, int y)
             audio->msgShowInstrumentGui(port->instrument(), show);
             }
       delete p;
+      
       }
 
 //---------------------------------------------------------
@@ -616,29 +724,78 @@ void TList::mousePressEvent(QMouseEvent* ev)
                   QPopupMenu* p = new QPopupMenu(this);
                   p->clear();
                   p->insertItem(*addtrack_addmiditrackIcon,
-                     tr("Add Midi Track"), Track::MIDI);
+                     tr("Add Midi Track"), Track::MIDI, 0);
                   p->insertItem(*addtrack_drumtrackIcon,
-                     tr("Add Drum Track"),Track::DRUM);
+                     tr("Add Drum Track"),Track::DRUM, 1);
                   p->insertItem(*addtrack_wavetrackIcon,
-                     tr("Add Wave Track"), Track::WAVE);
+                     tr("Add Wave Track"), Track::WAVE, 2);
                   p->insertItem(*addtrack_audiooutputIcon,
-                     tr("Add Output"), Track::AUDIO_OUTPUT);
+                     tr("Add Output"), Track::AUDIO_OUTPUT, 3);
                   p->insertItem(*addtrack_audiogroupIcon,
-                     tr("Add Group"), Track::AUDIO_GROUP);
+                     tr("Add Group"), Track::AUDIO_GROUP, 4);
                   p->insertItem(*addtrack_audioinputIcon,
-                     tr("Add Input"), Track::AUDIO_INPUT);
+                     tr("Add Input"), Track::AUDIO_INPUT, 5);
                   p->insertItem(*addtrack_auxsendIcon,
-                     tr("Add Aux Send"), Track::AUDIO_AUX);
+                     tr("Add Aux Send"), Track::AUDIO_AUX, 6);
+                  
+                  // Create a sub-menu and fill it with found synth types. Make p the owner.
+                  QPopupMenu* synp = populateAddSynth(p);
+                  // Add the 'Add Synth' sub-menu to the menu.
+                  p->insertItem(*synthIcon, tr("Add Synth"), synp, Track::AUDIO_SOFTSYNTH);
+                  
+                  // Show the menu
                   int n = p->exec(ev->globalPos(), 0);
 
-                  if (n != -1) {
-                        t = song->addTrack((Track::TrackType)n);
-                        song->deselectTracks();
-                        t->setSelected(true);
-                        emit selectionChanged();
-                        adjustScrollbar();
+                  // Valid click?
+                  if((n >= 0) && ((Track::TrackType)n != Track::AUDIO_SOFTSYNTH))
+                  {
+                    // Synth sub-menu id?
+                    if(n >= MENU_ADD_SYNTH_ID_BASE)
+                    {
+                      n -= MENU_ADD_SYNTH_ID_BASE;
+                      //if(n < synthis.size())
+                      //  t = song->createSynthI(synthis[n]->baseName());
+                      //if((n - MENU_ADD_SYNTH_ID_BASE) < (int)synthis.size())
+                      if(n < (int)synthis.size())
+                      {
+                        //t = song->createSynthI(synp->text(n));
+                        //t = song->createSynthI(synthis[n]->name());                        
+                        t = song->createSynthI(synthis[n]->baseName(), synthis[n]->name());
+                        
+                        if(t)
+                        {
+                          // Add instance last in midi device list.
+                          for (int i = 0; i < MIDI_PORTS; ++i) 
+                          {
+                            MidiPort* port  = &midiPorts[i];
+                            MidiDevice* dev = port->device();
+                            if (dev==0) 
+                            {
+                              midiSeq->msgSetMidiDevice(port, (SynthI*)t);
+                              muse->changeConfig(true);     // save configuration file
+                              song->update();
+                              break;
+                            }
+                          }  
                         }
-                 delete p;
+                      }  
+                    }  
+                    // Normal track.
+                    else
+                      t = song->addTrack((Track::TrackType)n);
+                    
+                    if(t)
+                    {
+                      song->deselectTracks();
+                      t->setSelected(true);
+                      emit selectionChanged();
+                      adjustScrollbar();
+                    }  
+                  }
+                  
+                  // Just delete p, and all its children will go too, right?
+                  //delete synp;
+                  delete p;
                  }
             return;
             }
@@ -686,8 +843,7 @@ void TList::mousePressEvent(QMouseEvent* ev)
                   if (!t->isMidiTrack()) {
                         if (t->type() == Track::AUDIO_OUTPUT) {
                               if (val && t->recordFlag() == false) {
-                                    muse->bounceToFile();
-                                    audio->msgSetRecord((AudioOutput*)t, val);
+                                    muse->bounceToFile((AudioOutput*)t);
                                     }
                               audio->msgSetRecord((AudioOutput*)t, val);
                               if (!((AudioOutput*)t)->recFile())
@@ -708,10 +864,18 @@ void TList::mousePressEvent(QMouseEvent* ev)
                         classesPopupMenu(t, x, t->y() - ypos);
                   break;
             case COL_OPORT:
+                  // Changed by Tim. p3.3.9
+                  // Reverted.
                   if (button == QMouseEvent::LeftButton)
                         portsPopupMenu(t, x, t->y() - ypos);
                   else if (button == QMouseEvent::RightButton)
                         oportPropertyPopupMenu(t, x, t->y() - ypos);
+                  //if(((button == QMouseEvent::LeftButton) && (t->type() == Track::AUDIO_SOFTSYNTH)) || (button == QMouseEvent::RightButton))
+                  //  oportPropertyPopupMenu(t, x, t->y() - ypos);      
+                  //else      
+                  //if(button == QMouseEvent::LeftButton)
+                  //  portsPopupMenu(t, x, t->y() - ypos);
+                    
                   break;
             case COL_MUTE:
                   if (t->off())
@@ -720,9 +884,8 @@ void TList::mousePressEvent(QMouseEvent* ev)
                         t->setMute(!t->mute());
                   song->update(SC_MUTE);
                   break;
-
             case COL_SOLO:
-                  t->setSolo(!t->solo());
+                  audio->msgSetSolo(t, !t->solo());
                   song->update(SC_SOLO);
                   break;
 
@@ -749,13 +912,14 @@ void TList::mousePressEvent(QMouseEvent* ev)
                               switch (n) {
                                     case 0:     // delete track
                                           song->removeTrack0(t);
+                                          audio->msgUpdateSoloStates();
                                           break;
 
                                     case 1:     // show track comment
                                           {
                                           TrackComment* tc = new TrackComment(t, 0);
                                           tc->show();
-                                          QToolTip::add( this, "FOOOOOOOOOOOOO" );
+                                          //QToolTip::add( this, "FOOOOOOOOOOOOO" );
                                           }
                                           break;
 
@@ -775,29 +939,67 @@ void TList::mousePressEvent(QMouseEvent* ev)
 
             case COL_OCHANNEL:
                   {
-                  MidiTrack* mt = dynamic_cast<MidiTrack*>(t);
-                  if (mt == 0)
-                        break;
-                  int channel = mt->outChannel();
-                  if (button == QMouseEvent::RightButton) {
-                        if (channel < 15)
-                              ++channel;
-                        }
-                  else if (button == QMouseEvent::MidButton) {
-                        if (channel > 0)
-                              --channel;
-                        }
-                  if (channel != ((MidiTrack*)t)->outChannel()) {
-                        mt->setOutChannel(channel);
-                        /* --- I really don't like this, you can mess up the whole map "as easy as dell"
-                        if (mt->type() == MidiTrack::DRUM) {//Change channel on all drum instruments
-                              for (int i=0; i<DRUM_MAPSIZE; i++)
-                                    drumMap[i].channel = channel;
-                              }*/
-                        song->update(-1);
-                        }
+                    int delta = 0;
+                    if (button == QMouseEvent::RightButton) 
+                      delta = 1;
+                    else if (button == QMouseEvent::MidButton) 
+                      delta = -1;
+                    if (t->isMidiTrack()) 
+                    {
+                      MidiTrack* mt = dynamic_cast<MidiTrack*>(t);
+                      if (mt == 0)
+                      break;
+                    
+                      int channel = mt->outChannel();
+                      channel += delta;
+                      if(channel >= MIDI_CHANNELS)
+                        channel = MIDI_CHANNELS - 1;
+                      if(channel < 0)
+                        channel = 0;
+                      //if (channel != ((MidiTrack*)t)->outChannel()) 
+                      if (channel != mt->outChannel()) 
+                      {
+                            // Changed by T356.
+                            //mt->setOutChannel(channel);
+                            audio->msgIdle(true);
+                            //audio->msgSetTrackOutChannel(mt, channel);
+                            mt->setOutChanAndUpdate(channel);
+                            audio->msgIdle(false);
+                            
+                            /* --- I really don't like this, you can mess up the whole map "as easy as dell"
+                            if (mt->type() == MidiTrack::DRUM) {//Change channel on all drum instruments
+                                  for (int i=0; i<DRUM_MAPSIZE; i++)
+                                        drumMap[i].channel = channel;
+                                  }*/
+                            
+                            // may result in adding/removing mixer strip:
+                            //song->update(-1);
+                            //song->update(SC_CHANNELS);
+                            song->update(SC_MIDI_CHANNEL);
+                      }
+                    }
+                    else
+                    {
+                        if(t->type() != Track::AUDIO_SOFTSYNTH)
+                        {
+                          AudioTrack* at = dynamic_cast<AudioTrack*>(t);
+                          if (at == 0)
+                            break;
+                    
+                          int n = t->channels() + delta;
+                          if (n > MAX_CHANNELS)
+                                n = MAX_CHANNELS;
+                          else if (n < 1)
+                                n = 1;
+                          if (n != t->channels()) {
+                                audio->msgSetChannels(at, n);
+                                song->update(SC_CHANNELS);
+                                }
+                        }         
+                    }      
                   }
-            }
+                  break;
+          }        
       redraw();
       }
 
@@ -808,6 +1010,10 @@ void TList::selectTrack(Track* tr)
       {
       song->deselectTracks();
       tr->setSelected(true);
+      
+      // By T356. Force a redraw for wave tracks, since it does not seem to happen.
+      if(!tr->isMidiTrack())
+        redraw();
       emit selectionChanged();
       }
 //---------------------------------------------------------
@@ -824,17 +1030,14 @@ void TList::mouseMoveEvent(QMouseEvent* ev)
             for (it = tracks->begin(); it != tracks->end(); ++it) {
                   int h = (*it)->height();
                   ty += h;
-                  if ( y >= (ty-2) ) {
-                  
-                        if ( (*it) == tracks->back() && y > ty ) {
-                              //printf("tracks->back() && y > ty\n");
+                  if (y >= (ty-2)) { 
+                        if ( (*it) == tracks->back() && y >= ty ) {
+                              // outside last track don't change to splitVCursor
                         }
                         else if ( y > (ty+2) ) {
                               //printf(" y > (ty+2) \n");
                         }
                         else {
-                              //printf("ogga ogga\n");
-                        
                               if (!resizeFlag) {
                                     resizeFlag = true;
                                     setCursor(QCursor(splitVCursor));
@@ -877,13 +1080,19 @@ void TList::mouseMoveEvent(QMouseEvent* ev)
                   break;
             case RESIZE:
                   {
-                  Track* t = song->tracks()->index(sTrack);
-                  int h  = t->height() + delta;
-                  startY = curY;
-                  if (h < MIN_TRACKHEIGHT)
-                        h = MIN_TRACKHEIGHT;
-                  t->setHeight(h);
-                  song->update(SC_TRACK_MODIFIED);
+                    if(sTrack >= 0 && sTrack < song->tracks()->size())
+                    {
+                      Track* t = song->tracks()->index(sTrack);
+                      if(t)
+                      {
+                        int h  = t->height() + delta;
+                        startY = curY;
+                        if (h < MIN_TRACKHEIGHT)
+                              h = MIN_TRACKHEIGHT;
+                        t->setHeight(h);
+                        song->update(SC_TRACK_MODIFIED);
+                      }  
+                    }  
                   }
                   break;
             }
@@ -921,8 +1130,10 @@ void TList::wheelEvent(QWheelEvent* ev)
       int x           = ev->x();
       int y           = ev->y();
       Track* t        = y2Track(y + ypos);
-      if (t == 0)
+      if (t == 0) {
+            emit redirectWheelEvent(ev);
             return;
+            }
       TrackColumn col = TrackColumn(header->sectionAt(x));
       int delta       = ev->delta() / WHEEL_DELTA;
       ev->accept();
@@ -942,7 +1153,7 @@ void TList::wheelEvent(QWheelEvent* ev)
                   break;
 
             case COL_SOLO:
-                  t->setSolo(!t->solo());
+                  audio->msgSetSolo(t, !t->solo());
                   song->update(SC_SOLO);
                   break;
 
@@ -960,7 +1171,13 @@ void TList::wheelEvent(QWheelEvent* ev)
                         else if (port < 0)
                               port = 0;
                         if (port != ((MidiTrack*)t)->outPort()) {
-                              mt->setOutPort(port);
+                              // Changed by T356.
+                              //mt->setOutPort(port);
+                              audio->msgIdle(true);
+                              //audio->msgSetTrackOutPort(mt, port);
+                              mt->setOutPortAndUpdate(port);
+                              audio->msgIdle(false);
+                              
                               song->update(SC_ROUTE);
                               }
                         }
@@ -976,9 +1193,16 @@ void TList::wheelEvent(QWheelEvent* ev)
                         else if (channel < 0)
                               channel = 0;
                         if (channel != ((MidiTrack*)t)->outChannel()) {
-                              mt->setOutChannel(channel);
+                              // Changed by T356.
+                              //mt->setOutChannel(channel);
+                              audio->msgIdle(true);
+                              //audio->msgSetTrackOutChannel(mt, channel);
+                              mt->setOutChanAndUpdate(channel);
+                              audio->msgIdle(false);
+                              
                               // may result in adding/removing mixer strip:
-                              song->update(-1);
+                              //song->update(-1);
+                              song->update(SC_MIDI_CHANNEL);
                               }
                         }
                   else {
@@ -993,6 +1217,8 @@ void TList::wheelEvent(QWheelEvent* ev)
                               }
                         }
                   break;
+            default:
+                  break;
             }
       }
 
@@ -1097,16 +1323,33 @@ void TList::classesPopupMenu(Track* t, int x, int y)
             //
             audio->msgIdle(true);
             PartList* pl = t->parts();
+            MidiTrack* m = (MidiTrack*) t;
             for (iPart ip = pl->begin(); ip != pl->end(); ++ip) {
                   EventList* el = ip->second->events();
                   for (iEvent ie = el->begin(); ie != el->end(); ++ie) {
                         Event ev = ie->second;
-                        if (ev.type() != Note)
-                              continue;
-                        int pitch = ev.pitch();
-                        pitch = drumMap[pitch].anote;
-                        ev.setPitch(pitch);
+                        if(ev.type() == Note)
+                        {
+                          int pitch = ev.pitch();
+                          // Changed by T356.
+                          // Tested: Notes were being mixed up switching back and forth between midi and drum.
+                          //pitch = drumMap[pitch].anote;
+                          pitch = drumMap[pitch].enote;
+                          
+                          ev.setPitch(pitch);
                         }
+                        else
+                        if(ev.type() == Controller)
+                        {
+                          int ctl = ev.dataA();
+                          // Is it a drum controller event, according to the track port's instrument?
+                          MidiController *mc = midiPorts[m->outPort()].drumController(ctl);
+                          if(mc)
+                            // Change the controller event's index into the drum map to an instrument note.
+                            ev.setA((ctl & ~0xff) | drumMap[ctl & 0x7f].enote);
+                        }
+                          
+                      }
                   }
             t->setType(Track::MIDI);
             audio->msgIdle(false);
@@ -1118,6 +1361,12 @@ void TList::classesPopupMenu(Track* t, int x, int y)
             bool change = QMessageBox::question(this, tr("Update drummap?"),
                            tr("Do you want to use same port and channel for all instruments in the drummap?"),
                            tr("&Yes"), tr("&No"), QString::null, 0, 1);
+            
+            audio->msgIdle(true);
+            // Delete all port controller events.
+            //audio->msgChangeAllPortDrumCtrlEvents(false);
+            song->changeAllPortDrumCtrlEvents(false);
+            
             if (!change) {
                   MidiTrack* m = (MidiTrack*) t;
                   for (int i=0; i<DRUM_MAPSIZE; i++) {
@@ -1126,20 +1375,42 @@ void TList::classesPopupMenu(Track* t, int x, int y)
                         }
                   }
 
-            audio->msgIdle(true);
+            //audio->msgIdle(true);
             PartList* pl = t->parts();
+            MidiTrack* m = (MidiTrack*) t;
             for (iPart ip = pl->begin(); ip != pl->end(); ++ip) {
                   EventList* el = ip->second->events();
                   for (iEvent ie = el->begin(); ie != el->end(); ++ie) {
                         Event ev = ie->second;
-                        if (ev.type() != Note)
-                              continue;
-                        int pitch = ev.pitch();
-                        pitch = drumInmap[pitch];
-                        ev.setPitch(pitch);
+                        if (ev.type() == Note)
+                        {
+                          int pitch = ev.pitch();
+                          pitch = drumInmap[pitch];
+                          ev.setPitch(pitch);
+                        }  
+                        else
+                        {
+                          if(ev.type() == Controller)
+                          {
+                            int ctl = ev.dataA();
+                            // Is it a drum controller event, according to the track port's instrument?
+                            MidiController *mc = midiPorts[m->outPort()].drumController(ctl);
+                            if(mc)
+                              // Change the controller event's instrument note to an index into the drum map.
+                              ev.setA((ctl & ~0xff) | drumInmap[ctl & 0x7f]);
+                          }
+                          
                         }
+                        
+                      }
                   }
             t->setType(Track::DRUM);
+            
+            // Add all port controller events.
+            //audio->msgChangeAllPortDrumCtrlEvents(true);
+            song->changeAllPortDrumCtrlEvents(true);
+            
             audio->msgIdle(false);
             }
       }
+
