@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//  $Id: audio.h,v 1.25.2.2 2005/09/26 18:17:51 spamatica Exp $
+//  $Id: audio.h,v 1.25.2.13 2009/12/20 05:00:35 terminator356 Exp $
 //
 //  (C) Copyright 2001 Werner Schweer (ws@seh.de)
 //=========================================================
@@ -48,17 +48,31 @@ enum {
       SEQM_MIDI_LOCAL_OFF,
       SEQM_SET_MIDI_DEVICE,
       SEQM_PLAY_MIDI_EVENT,
+      SEQM_SET_HW_CTRL_STATE,
+      SEQM_SET_HW_CTRL_STATES,
+      SEQM_SET_TRACK_OUT_PORT,
+      SEQM_SET_TRACK_OUT_CHAN,
+      SEQM_REMAP_PORT_DRUM_CTL_EVS,
+      SEQM_CHANGE_ALL_PORT_DRUM_CTL_EVS,
       SEQM_SCAN_ALSA_MIDI_PORTS,
       SEQM_SET_AUX,
+      SEQM_UPDATE_SOLO_STATES,
       MIDI_SHOW_INSTR_GUI,
-      //
       AUDIO_RECORD,
       AUDIO_ROUTEADD, AUDIO_ROUTEREMOVE,
       AUDIO_VOL, AUDIO_PAN,
       AUDIO_ADDPLUGIN,
       AUDIO_SET_SEG_SIZE,
       AUDIO_SET_PREFADER, AUDIO_SET_CHANNELS,
-
+      AUDIO_SET_PLUGIN_CTRL_VAL,
+      AUDIO_SWAP_CONTROLLER_IDX,
+      AUDIO_CLEAR_CONTROLLER_EVENTS,
+      AUDIO_SEEK_PREV_AC_EVENT,
+      AUDIO_SEEK_NEXT_AC_EVENT,
+      AUDIO_ERASE_AC_EVENT,
+      AUDIO_ERASE_RANGE_AC_EVENTS,
+      AUDIO_ADD_AC_EVENT,
+      AUDIO_SET_SOLO, AUDIO_SET_SEND_METRONOME, 
       MS_PROCESS, MS_STOP, MS_SET_RTC, MS_UPDATE_POLL_FD,
       SEQM_IDLE, SEQM_SEEK,
       };
@@ -108,8 +122,9 @@ class Audio {
       bool idle;              // do nothing in idle mode
       bool _freewheel;
       bool _bounce;
-      bool loopPassed;
-      unsigned loopFrame;     // startframe of loop if in LOOP mode
+      //bool loopPassed;
+      unsigned _loopFrame;     // Startframe of loop if in LOOP mode. Not quite the same as left marker !
+      int _loopCount;         // Number of times we have looped so far
 
       Pos _pos;               // current play position
 
@@ -151,7 +166,7 @@ class Audio {
       void processMsg(AudioMsg* msg);
       void process1(unsigned samplePos, unsigned offset, unsigned samples);
 
-      void collectEvents(MidiTrack*, int startTick, int endTick);
+      void collectEvents(MidiTrack*, unsigned int startTick, unsigned int endTick);
 
    public:
       Audio();
@@ -185,13 +200,18 @@ class Audio {
       void msgMoveTrack(int idx1, int dx2, bool u = true);
       void msgAddPart(Part*, bool u = true);
       void msgRemovePart(Part*, bool u = true);
-      void msgChangePart(Part* oldPart, Part* newPart, bool u = true);
-      void msgAddEvent(Event&, Part*, bool u = true);
-      void msgDeleteEvent(Event&, Part*, bool u = true);
-      void msgChangeEvent(Event&, Event&, Part*, bool u = true);
+      //void msgChangePart(Part* oldPart, Part* newPart, bool u = true);
+      void msgChangePart(Part* oldPart, Part* newPart, bool u = true, bool doCtrls = true, bool doClones = false);
+      //void msgAddEvent(Event&, Part*, bool u = true);
+      void msgAddEvent(Event&, Part*, bool u = true, bool doCtrls = true, bool doClones = false);
+      //void msgDeleteEvent(Event&, Part*, bool u = true);
+      void msgDeleteEvent(Event&, Part*, bool u = true, bool doCtrls = true, bool doClones = false);
+      //void msgChangeEvent(Event&, Event&, Part*, bool u = true);
+      void msgChangeEvent(Event&, Event&, Part*, bool u = true, bool doCtrls = true, bool doClones = false);
       void msgScanAlsaMidiPorts();
       void msgAddTempo(int tick, int tempo, bool doUndoFlag = true);
       void msgSetTempo(int tick, int tempo, bool doUndoFlag = true);
+      void msgUpdateSoloStates();
       void msgSetAux(AudioTrack*, int, double);
       void msgSetGlobalTempo(int val);
       void msgDeleteTempo(int tick, int tempo, bool doUndoFlag = true);
@@ -223,6 +243,22 @@ class Audio {
       void msgResetMidiDevices();
       void msgIdle(bool);
       void msgBounce();
+      void msgSetPluginCtrlVal(PluginI* plugin, int param, double val);
+      void msgSwapControllerIDX(AudioTrack*, int, int);
+      void msgClearControllerEvents(AudioTrack*, int);
+      void msgSeekPrevACEvent(AudioTrack*, int);
+      void msgSeekNextACEvent(AudioTrack*, int);
+      void msgEraseACEvent(AudioTrack*, int, int);
+      void msgEraseRangeACEvents(AudioTrack*, int, int, int);
+      void msgAddACEvent(AudioTrack*, int, int, double);
+      void msgSetSolo(Track*, bool);
+      void msgSetHwCtrlState(MidiPort*, int, int, int);
+      void msgSetHwCtrlStates(MidiPort*, int, int, int, int);
+      void msgSetTrackOutChannel(MidiTrack*, int);
+      void msgSetTrackOutPort(MidiTrack*, int);
+      void msgRemapPortDrumCtlEvents(int, int, int, int);
+      void msgChangeAllPortDrumCtrlEvents(bool, bool);
+      void msgSetSendMetronome(AudioTrack*, bool);
 
       void msgPlayMidiEvent(const MidiPlayEvent* event);
       void rescanAlsaPorts();
@@ -232,6 +268,8 @@ class Audio {
       const Pos& pos() const { return _pos; }
       const Pos& getStartRecordPos() const { return startRecordPos; }
       const Pos& getEndRecordPos() const { return endRecordPos; }
+      int loopCount() { return _loopCount; }         // Number of times we have looped so far
+      unsigned loopFrame() { return _loopFrame; }          
 
       int tickPos() const         { return curTickPos; }
       int timestamp() const;
