@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//  $Id: audioprefetch.cpp,v 1.14.2.2 2005/12/28 22:17:55 spamatica Exp $
+//  $Id: audioprefetch.cpp,v 1.14.2.7 2009/12/20 05:00:35 terminator356 Exp $
 //
 //  (C) Copyright 2001 Werner Schweer (ws@seh.de)
 //=========================================================
@@ -18,6 +18,9 @@
 #include "audio.h"
 #include "sync.h"
 
+// Added by Tim. p3.3.20
+//#define AUDIOPREFETCH_DEBUG
+
 enum { PREFETCH_TICK, PREFETCH_SEEK
       };
 
@@ -35,12 +38,15 @@ AudioPrefetch* audioPrefetch;
 //   AudioPrefetch
 //---------------------------------------------------------
 
-AudioPrefetch::AudioPrefetch(int prio, const char* name)
-   : Thread(prio,name)
+//AudioPrefetch::AudioPrefetch(int prio, const char* name)
+//   : Thread(prio,name)
+AudioPrefetch::AudioPrefetch(const char* name)
+   : Thread(name)
       {
       seekPos  = ~0;
       writePos = ~0;
-      seekDone = true;
+      //seekDone = true;
+      seekCount = 0;
       }
 
 //---------------------------------------------------------
@@ -57,11 +63,13 @@ static void readMsgP(void* p, void*)
 //   start
 //---------------------------------------------------------
 
-void AudioPrefetch::start()
+//void AudioPrefetch::start()
+void AudioPrefetch::start(int priority)
       {
       clearPollFd();
       addPollFd(toThreadFdr, POLLIN, ::readMsgP, this, 0);
-      Thread::start();
+      //Thread::start();
+      Thread::start(priority);
       }
 
 //---------------------------------------------------------
@@ -81,12 +89,22 @@ void AudioPrefetch::processMsg1(const void* m)
       const PrefetchMsg* msg = (PrefetchMsg*)m;
       switch(msg->id) {
             case PREFETCH_TICK:
-                  if (audio->isRecording())
+                  if (audio->isRecording()) {
+                  //puts("writeTick");
                         audio->writeTick();
-                  prefetch();
+                        }
+                  // Indicate do not seek file before each read.
+                  // Changed by Tim. p3.3.17 
+                  //prefetch();
+                  prefetch(false);
+                  
                   seekPos = ~0;     // invalidate cached last seek position
                   break;
             case PREFETCH_SEEK:
+                  #ifdef AUDIOPREFETCH_DEBUG
+                  printf("AudioPrefetch::processMsg1 PREFETCH_SEEK msg->pos:%d\n", msg->pos); 
+                  #endif
+                  
                   // process seek in background
                   seek(msg->pos);
                   break;
@@ -116,10 +134,17 @@ void AudioPrefetch::msgTick()
 void AudioPrefetch::msgSeek(unsigned samplePos, bool force)
       {
       if (samplePos == seekPos && !force) {
-            seekDone = true;
+            //seekDone = true;
             return;
             }
-      seekDone = false;
+
+      ++seekCount;
+      //seekDone = false;
+      
+      #ifdef AUDIOPREFETCH_DEBUG
+      printf("AudioPrefetch::msgSeek samplePos:%u force:%d seekCount:%d\n", samplePos, force, seekCount); 
+      #endif
+      
       PrefetchMsg msg;
       msg.id  = PREFETCH_SEEK;
       msg.pos = samplePos;
@@ -133,10 +158,11 @@ void AudioPrefetch::msgSeek(unsigned samplePos, bool force)
 //   prefetch
 //---------------------------------------------------------
 
-void AudioPrefetch::prefetch()
+//void AudioPrefetch::prefetch()
+void AudioPrefetch::prefetch(bool doSeek)
       {
       if (writePos == ~0U) {
-            printf("prefetch(): invalid write position\n");
+            printf("AudioPrefetch::prefetch: invalid write position\n");
             return;
             }
       if (song->loop() && !audio->bounce() && !extSyncFlag.value()) {
@@ -158,10 +184,11 @@ void AudioPrefetch::prefetch()
             float* bp[ch];
 // printf("prefetch %d\n", writePos);
             if (track->prefetchFifo()->getWriteBuffer(ch, segmentSize, bp, writePos)) {
-                  printf("Prefetch: NO BUFFER\n");
+                  printf("AudioPrefetch::prefetch No write buffer!\n");
                   continue;
                   }
-            track->fetchData(writePos, segmentSize, bp);
+            //track->fetchData(writePos, segmentSize, bp);
+            track->fetchData(writePos, segmentSize, bp, doSeek);
             }
       writePos += segmentSize;
       }
@@ -173,15 +200,54 @@ void AudioPrefetch::prefetch()
 void AudioPrefetch::seek(unsigned seekTo)
       {
 // printf("seek %d\n", seekTo);
+      #ifdef AUDIOPREFETCH_DEBUG
+      printf("AudioPrefetch::seek to:%u seekCount:%d\n", seekTo, seekCount); 
+      #endif
+      
+      // Speedup: More than one seek message pending?
+      // Eat up seek messages until we get to the very LATEST one, 
+      //  because all the rest which came before it are irrelevant now,
+      //  and processing them all was taking extreme time, especially with
+      //  resampling enabled.
+      // In particular, when the user 'slides' the play cursor back and forth   
+      //  there are MANY seek messages in the pipe, and with resampling enabled
+      //  it was taking minutes to finish seeking. If the user hit play during that time,
+      //  things were messed up (FIFO underruns, choppy intermittent sound etc).
+      // Added by Tim. p3.3.20
+      if(seekCount > 1)
+      {
+        --seekCount;
+        return;
+      }
+      
       writePos = seekTo;
       WaveTrackList* tl = song->waves();
       for (iWaveTrack it = tl->begin(); it != tl->end(); ++it) {
             WaveTrack* track = *it;
             track->clearPrefetchFifo();
             }
-      for (int i = 0; i < (fifoLength-1); ++i)
-            prefetch();
+      
+      bool isFirstPrefetch = true;
+      for (unsigned int i = 0; i < (fifoLength)-1; ++i)//prevent compiler warning: comparison of signed/unsigned
+      {      
+            // Indicate do a seek command before read, but only on the first pass. 
+            // Changed by Tim. p3.3.17 
+            //prefetch();
+            prefetch(isFirstPrefetch);
+            
+            isFirstPrefetch = false;
+            
+            // To help speed things up even more, check the count again. Return if more seek messages are pending.
+            // Added by Tim. p3.3.20
+            if(seekCount > 1)
+            {
+              --seekCount;
+              return;
+            }  
+      }
+            
       seekPos  = seekTo;
-      seekDone = true;
+      //seekDone = true;
+      --seekCount;
       }
 
