@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//  $Id: audioprefetch.h,v 1.3.2.1 2005/06/04 19:25:15 spamatica Exp $
+//  $Id: audioprefetch.h,v 1.3.2.2 2009/12/20 05:00:35 terminator356 Exp $
 //
 //  (C) Copyright 2001 Werner Schweer (ws@seh.de)
 //=========================================================
@@ -20,17 +20,25 @@ class AudioPrefetch : public Thread {
       unsigned seekPos; // remember last seek to optimize seeks
 
       virtual void processMsg1(const void*);
-      void prefetch();
+      //void prefetch();
+      void prefetch(bool doSeek);
       void seek(unsigned pos);
 
+      volatile int seekCount;
+      
    public:
-      AudioPrefetch(int prio, const char* name);
+      //AudioPrefetch(int prio, const char* name);
+      AudioPrefetch(const char* name);
+      
       ~AudioPrefetch();
-      virtual void start();
+      //virtual void start();
+      virtual void start(int);
 
       void msgTick();
       void msgSeek(unsigned samplePos, bool force=false);
-      volatile bool seekDone;
+      
+      //volatile bool seekDone;
+      bool seekDone() const { return seekCount == 0; }
       };
 
 extern AudioPrefetch* audioPrefetch;
