@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//  $Id: conf.cpp,v 1.33.2.5 2006/01/01 20:51:55 spamatica Exp $
+//  $Id: conf.cpp,v 1.33.2.18 2009/12/01 03:52:40 terminator356 Exp $
 //
 //  (C) Copyright 1999-2003 Werner Schweer (ws@seh.de)
 //=========================================================
@@ -159,7 +159,9 @@ static void readController(Xml& xml, int midiPort, int channel)
                   case Xml::TagEnd:
                         if (tag == "controller") {
                               MidiPort* port = &midiPorts[midiPort];
-                              port->addManagedController(channel, id);
+                              //port->addManagedController(channel, id);
+                              val = port->limitValToInstrCtlRange(id, val);
+                              // The value here will actually be sent to the device LATER, in MidiPort::setMidiDevice()
                               port->setHwCtrlState(channel, id, val);
                               return;
                               }
@@ -213,6 +215,7 @@ static void readConfigMidiPort(Xml& xml)
       QString instrument;
       int openFlags = 1;
       bool thruFlag = false;
+      MidiSyncInfo tmpSi;
 
       for (;;) {
             Xml::Token token = xml.parse();
@@ -230,6 +233,8 @@ static void readConfigMidiPort(Xml& xml)
                               }
                         else if (tag == "openFlags")
                               openFlags = xml.parseInt();
+                        else if (tag == "midiSyncInfo")
+                              tmpSi.read(xml);
                         else if (tag == "instrument") {
                               instrument = xml.parse1();
                               midiPorts[idx].setInstrument(
@@ -251,13 +256,15 @@ static void readConfigMidiPort(Xml& xml)
                         break;
                   case Xml::TagEnd:
                         if (tag == "midiport") {
-                              if (idx > MIDI_PORTS) {
+                              //if (idx > MIDI_PORTS) {
+                              if (idx < 0 || idx >= MIDI_PORTS) {
                                     fprintf(stderr, "bad midi port %d (>%d)\n",
                                        idx, MIDI_PORTS);
                                     idx = 0;
                                     }
                               MidiDevice* dev = midiDevices.find(device);
                               MidiPort* mp = &midiPorts[idx];
+                              mp->syncInfo().copyParams(tmpSi);
                               if (dev) {
                                     dev->setOpenFlags(openFlags);
                                     midiSeq->msgSetMidiDevice(mp, dev);
@@ -270,6 +277,83 @@ static void readConfigMidiPort(Xml& xml)
             }
       }
 
+/*
+//---------------------------------------------------------
+//   readConfigMidiSyncInfo
+//---------------------------------------------------------
+
+static void readConfigMidiSyncInfo(Xml& xml)
+{
+      QString device;
+      int idOut       = 127;
+      int idIn        = 127;
+      bool sendMC     = false;
+      bool sendMMC    = false;
+      bool sendMTC    = false;
+      bool recMC      = false;
+      bool recMMC     = false;
+      bool recMTC     = false;
+      
+      for (;;) {
+            Xml::Token token = xml.parse();
+            if (token == Xml::Error || token == Xml::End)
+                  break;
+            QString tag = xml.s1();
+            switch (token) {
+                  case Xml::TagStart:
+                        if (tag == "device")
+                              device = xml.parse1();
+                        else if (tag == "idOut")
+                              idOut = (xml.parseInt());
+                        else if (tag == "idIn")
+                              idIn = xml.parseInt();
+                        else if (tag == "sendMC")
+                              sendMC = xml.parseInt();
+                        else if (tag == "sendMMC")
+                              sendMMC = xml.parseInt();
+                        else if (tag == "sendMTC")
+                              sendMTC = xml.parseInt();
+                        else if (tag == "recMC")
+                              recMC = xml.parseInt();
+                        else if (tag == "recMMC")
+                              recMMC = xml.parseInt();
+                        else if (tag == "recMTC")
+                              recMTC = xml.parseInt();
+                        else
+                              xml.unknown("midiSyncInfo");
+                        break;
+                  case Xml::Attribut:
+                        break;
+                  case Xml::TagEnd:
+                        if(tag == "midiSyncInfo") 
+                        {
+                          MidiDevice* dev = midiDevices.find(device);
+                          if(dev) 
+                          {
+                            MidiSyncInfo& si = dev->syncInfo();
+                            si.setIdIn(idIn);
+                            si.setIdOut(idOut);
+                            
+                            si.setMCIn(recMC);
+                            si.setMMCIn(recMMC);
+                            si.setMTCIn(recMTC);
+                            
+                            si.setMCOut(sendMC);
+                            si.setMMCOut(sendMMC);
+                            si.setMTCOut(sendMTC);
+                          }
+                          else
+                            fprintf(stderr, "Read configuration: Sync device: %s not found\n", device.latin1());
+                            
+                          return;
+                        }
+                  default:
+                        break;
+                  }
+            }
+}
+*/
+
 //---------------------------------------------------------
 //   loadConfigMetronom
 //---------------------------------------------------------
@@ -369,7 +453,7 @@ static void readSeqConfiguration(Xml& xml)
 //   readConfiguration
 //---------------------------------------------------------
 
-void readConfiguration(Xml& xml)
+void readConfiguration(Xml& xml, bool readOnlySequencer)
       {
       for (;;) {
             Xml::Token token = xml.parse();
@@ -378,8 +462,28 @@ void readConfiguration(Xml& xml)
             QString tag = xml.s1();
             switch (token) {
                   case Xml::TagStart:
+                        /* the reading of configuration is split in two; read
+                           "sequencer" and read ALL. The reason is that it is
+                           possible to load a song without configuration. In
+                           this case the <configuration> chapter in the song
+                           file should be skipped. However the sub part
+                           <sequencer> contains elements that are necessary
+                           to preserve composition consistency. Mainly
+                           midiport configuration and VOLUME.
+                        */
+                        if (tag == "sequencer") {
+                              readSeqConfiguration(xml);
+                              break;
+                              }
+                        else if (readOnlySequencer) {
+                              xml.skip(tag);
+                              break;
+                              }
+                        
                         if (tag == "theme")
                               config.style = xml.parse1();
+                        else if (tag == "useOldStyleStopShortCut")
+                              config.useOldStyleStopShortCut = xml.parseInt();
                         else if (tag == "externalWavEditor")
                               config.externalWavEditor = xml.parse1();
                         else if (tag == "font0")
@@ -394,6 +498,8 @@ void readConfiguration(Xml& xml)
                               config.fonts[4].fromString(xml.parse1());
                         else if (tag == "font5")
                               config.fonts[5].fromString(xml.parse1());
+                        else if (tag == "font6")
+                              config.fonts[6].fromString(xml.parse1());
                         else if (tag == "palette0")
                               config.palette[0] = readColor(xml);
                         else if (tag == "palette1")
@@ -430,8 +536,14 @@ void readConfiguration(Xml& xml)
                               config.palette[16] = readColor(xml);
                         else if (tag == "trackBg")
                               config.trackBg = readColor(xml);
+                        else if (tag == "selectTrackBg")
+                              config.selectTrackBg = readColor(xml);
+                        else if (tag == "selectTrackFg")
+                              config.selectTrackFg = readColor(xml);
                         else if (tag == "midiTrackBg")
                               config.midiTrackBg = readColor(xml);
+                        else if (tag == "ctrlGraphFg")
+                              config.ctrlGraphFg = readColor(xml);
                         else if (tag == "drumTrackBg")
                               config.drumTrackBg = readColor(xml);
                         else if (tag == "waveTrackBg")
@@ -454,6 +566,10 @@ void readConfiguration(Xml& xml)
                               config.copyright = xml.parse1();
                         else if (tag == "smfFormat")
                               config.smfFormat = xml.parseInt();
+                        else if (tag == "exp2ByteTimeSigs")
+                              config.exp2ByteTimeSigs = xml.parseInt();
+                        else if (tag == "expOptimNoteOffs")
+                              config.expOptimNoteOffs = xml.parseInt();
                         else if (tag == "midiInputDevice")
                               midiInputPorts = xml.parseInt();
                         else if (tag == "midiInputChannel")
@@ -507,35 +623,48 @@ void readConfiguration(Xml& xml)
                         else if (tag == "transportHandleColor")
                               config.transportHandleColor = readColor(xml);
                         else if (tag == "txDeviceId")
-                              txDeviceId = xml.parseInt();
+                                //txDeviceId = xml.parseInt();
+                                xml.parseInt();
                         else if (tag == "rxDeviceId")
-                              rxDeviceId = xml.parseInt();
+                                //rxDeviceId = xml.parseInt();
+                                xml.parseInt();
                         else if (tag == "txSyncPort")
-                              txSyncPort= xml.parseInt();
+                                //txSyncPort= xml.parseInt();
+                                xml.parseInt();
                         else if (tag == "rxSyncPort")
-                              rxSyncPort= xml.parseInt();
+                                //rxSyncPort= xml.parseInt();
+                                xml.parseInt();
                         else if (tag == "mtctype")
                               mtcType= xml.parseInt();
                         else if (tag == "extSync")
                               extSyncFlag.setValue(xml.parseInt());
+                        else if (tag == "useJackTransport")
+                              useJackTransport = xml.parseInt();
                         else if (tag == "syncgentype") {
                               // for compatibility
-                              int syncGenType= xml.parseInt();
-                              genMTCSync = syncGenType == 1;
-                              genMCSync = syncGenType == 2;
+                              //int syncGenType= xml.parseInt();
+                              //genMTCSync = syncGenType == 1;
+                              //genMCSync = syncGenType == 2;
+                              xml.parseInt();
                               }
                         else if (tag == "genMTCSync")
-                              genMTCSync = xml.parseInt();
+                              //genMTCSync = xml.parseInt();
+                              xml.parseInt();
                         else if (tag == "genMCSync")
-                              genMCSync = xml.parseInt();
+                              //genMCSync = xml.parseInt();
+                              xml.parseInt();
                         else if (tag == "genMMC")
-                              genMMC = xml.parseInt();
+                              //genMMC = xml.parseInt();
+                              xml.parseInt();
                         else if (tag == "acceptMTC")
-                              acceptMTC = xml.parseInt();
+                              //acceptMTC = xml.parseInt();
+                              xml.parseInt();
                         else if (tag == "acceptMMC")
-                              acceptMMC = xml.parseInt();
+                              //acceptMMC = xml.parseInt();
+                              xml.parseInt();
                         else if (tag == "acceptMC")
-                              acceptMC = xml.parseInt();
+                              //acceptMC = xml.parseInt();
+                              xml.parseInt();
                         else if (tag == "mtcoffset") {
                               QString qs(xml.parse1());
                               const char* str = qs.latin1();
@@ -543,8 +672,8 @@ void readConfiguration(Xml& xml)
                               sscanf(str, "%d:%d:%d:%d:%d", &h, &m, &s, &f, &sf);
                               mtcOffset = MTC(h, m, s, f, sf);
                               }
-                        else if (tag == "sequencer")
-                              readSeqConfiguration(xml);
+                        //else if (tag == "midiSyncInfo")
+                        //      readConfigMidiSyncInfo(xml);
                         else if (tag == "arranger") {
                               if (muse && muse->arranger)
                                     muse->arranger->readStatus(xml);
@@ -577,6 +706,16 @@ void readConfiguration(Xml& xml)
                               config.minMeter = xml.parseInt();
                         else if (tag == "minSlider")
                               config.minSlider = xml.parseDouble();
+                        else if (tag == "freewheelMode")
+                              config.freewheelMode = xml.parseInt();
+                        else if (tag == "denormalProtection")
+                              config.useDenormalBias = xml.parseInt();
+                        else if (tag == "didYouKnow")
+                              config.showDidYouKnow = xml.parseInt();
+                        else if (tag == "outputLimiter")
+                              config.useOutputLimiter = xml.parseInt();
+                        else if (tag == "vstInPlace")
+                              config.vstInPlace = xml.parseInt();
                         else if (tag == "guiRefresh")
                               config.guiRefresh = xml.parseInt();
                         else if (tag == "helpBrowser")
@@ -600,6 +739,8 @@ void readConfiguration(Xml& xml)
                         printf("text <%s>\n", xml.s1().latin1());
                         break;
                   case Xml::Attribut:
+                        if (readOnlySequencer)
+                              break;
                         if (tag == "version") {
                               int major = xml.s2().section('.', 0, 0).toInt();
                               int minor = xml.s2().section('.', 1, 1).toInt();
@@ -684,7 +825,7 @@ bool readConfiguration()
                         else if (skipmode)
                               break;
                         else if (tag == "configuration")
-                              readConfiguration(xml);
+                              readConfiguration(xml,false);
                         else
                               xml.unknown("muse config");
                         break;
@@ -764,8 +905,12 @@ static void writeSeqConfiguration(int level, Xml& xml, bool writePortInfo)
                   xml.strTag(level, "instrument", mport->instrument()->iname());
                   if (dev) {
                         xml.strTag(level, "name",   dev->name());
-                        xml.intTag(level, "record", dev->rwFlags() & 0x2 ? 1 : 0);
+                        // Changed by T356. "record" is old and by mistake written as rwFlags here. 
+                        // openFlags was read before, but never written here.
+                        //xml.intTag(level, "record", dev->rwFlags() & 0x2 ? 1 : 0);
+                        xml.intTag(level, "openFlags", dev->openFlags());
                         }
+                  mport->syncInfo().write(level, xml);
                   // write out registered controller for all channels
                   MidiCtrlValListList* vll = mport->controller();
                   for (int k = 0; k < MIDI_CHANNELS; ++k) {
@@ -818,6 +963,12 @@ void MusE::writeGlobalConfiguration(int level, Xml& xml) const
       xml.intTag(level, "rtcTicks", config.rtcTicks);
       xml.intTag(level, "minMeter", config.minMeter);
       xml.doubleTag(level, "minSlider", config.minSlider);
+      xml.intTag(level, "freewheelMode", config.freewheelMode);
+      xml.intTag(level, "denormalProtection", config.useDenormalBias);
+      xml.intTag(level, "didYouKnow", config.showDidYouKnow);
+      xml.intTag(level, "outputLimiter", config.useOutputLimiter);
+      xml.intTag(level, "vstInPlace", config.vstInPlace);
+
       xml.intTag(level, "guiRefresh", config.guiRefresh);
       xml.strTag(level, "helpBrowser", config.helpBrowser);
       xml.intTag(level, "extendedMidi", config.extendedMidi);
@@ -825,6 +976,8 @@ void MusE::writeGlobalConfiguration(int level, Xml& xml) const
       xml.intTag(level, "guiDivision", config.guiDivision);
       xml.strTag(level, "copyright", config.copyright);
       xml.intTag(level, "smfFormat", config.smfFormat);
+      xml.intTag(level, "exp2ByteTimeSigs", config.exp2ByteTimeSigs);
+      xml.intTag(level, "expOptimNoteOffs", config.expOptimNoteOffs);
       xml.intTag(level, "startMode", config.startMode);
       xml.strTag(level, "startSong", config.startSong);
 
@@ -836,12 +989,16 @@ void MusE::writeGlobalConfiguration(int level, Xml& xml) const
       xml.intTag(level, "midiFilterCtrl2", midiFilterCtrl2);
       xml.intTag(level, "midiFilterCtrl3", midiFilterCtrl3);
       xml.intTag(level, "midiFilterCtrl4", midiFilterCtrl4);
-      xml.intTag(level, "txDeviceId", txDeviceId);
-      xml.intTag(level, "rxDeviceId", rxDeviceId);
+      // Removed by Tim. p3.3.6
+      
+      //xml.intTag(level, "txDeviceId", txDeviceId);
+      //xml.intTag(level, "rxDeviceId", rxDeviceId);
       xml.strTag(level, "theme", config.style);
       xml.strTag(level, "externalWavEditor", config.externalWavEditor);
+      xml.intTag(level, "useOldStyleStopShortCut", config.useOldStyleStopShortCut);
 
-      for (int i = 0; i < 6; ++i) {
+      //for (int i = 0; i < 6; ++i) {
+      for (int i = 0; i < NUM_FONTS; ++i) {
             char buffer[32];
             sprintf(buffer, "font%d", i);
             xml.strTag(level, buffer, config.fonts[i].toString());
@@ -853,7 +1010,10 @@ void MusE::writeGlobalConfiguration(int level, Xml& xml) const
             }
 
       xml.colorTag(level, "trackBg",       config.trackBg);
+      xml.colorTag(level, "selectTrackBg", config.selectTrackBg);
+      xml.colorTag(level, "selectTrackFg", config.selectTrackFg);
       xml.colorTag(level, "midiTrackBg",   config.midiTrackBg);
+      xml.colorTag(level, "ctrlGraphFg",   config.ctrlGraphFg);
       xml.colorTag(level, "drumTrackBg",   config.drumTrackBg);
       xml.colorTag(level, "waveTrackBg",   config.waveTrackBg);
       xml.colorTag(level, "outputTrackBg", config.outputTrackBg);
@@ -862,19 +1022,53 @@ void MusE::writeGlobalConfiguration(int level, Xml& xml) const
       xml.colorTag(level, "auxTrackBg",    config.auxTrackBg);
       xml.colorTag(level, "synthTrackBg",  config.synthTrackBg);
 
-      xml.intTag(level, "txSyncPort", txSyncPort);
-      xml.intTag(level, "rxSyncPort", rxSyncPort);
+      // Changed by Tim. p3.3.6
+      
+      //xml.intTag(level, "txSyncPort", txSyncPort);
+      /*
+      // To keep old muse versions happy...
+      bool mcsync = mmc = mtc = false;
+      for(int sp = 0; sp < MIDI_PORTS; ++sp)
+      {
+        MidiSyncTxPort* txPort = &midiSyncTxPorts[sp];
+        if(txPort->doMCSync() || txPort->doMMC() || txPort->doMTC())
+        {
+          if(txPort->doMCSync())
+            mcsync = true;
+          if(txPort->doMMC())
+            mmc = true;
+          if(txPort->doMTC())
+            mtc = true;
+          xml.intTag(level, "txSyncPort", sp);
+          break;
+        }  
+      }
+      */
+      
+      // Added by Tim. p3.3.6
+      
+      //xml.tag(level++, "midiSyncInfo");
+      //for(iMidiDevice id = midiDevices.begin(); id != midiDevices.end(); ++id) 
+      //{
+      //  MidiDevice* md = *id;
+      //  (*id)->syncInfo().write(level, xml, md);
+      //}      
+      //xml.etag(level, "midiSyncInfo");
+
+      //xml.intTag(level, "rxSyncPort", rxSyncPort);
       xml.intTag(level, "mtctype", mtcType);
       xml.nput(level, "<mtcoffset>%02d:%02d:%02d:%02d:%02d</mtcoffset>\n",
         mtcOffset.h(), mtcOffset.m(), mtcOffset.s(),
         mtcOffset.f(), mtcOffset.sf());
+      //xml.intTag(level, "useJackTransport", useJackTransport);
       extSyncFlag.save(level, xml);
-      xml.intTag(level, "genMTCSync", genMTCSync);
-      xml.intTag(level, "genMCSync", genMCSync);
-      xml.intTag(level, "genMMC", genMMC);
-      xml.intTag(level, "acceptMTC", acceptMTC);
-      xml.intTag(level, "acceptMMC", acceptMMC);
-      xml.intTag(level, "acceptMC", acceptMC);
+      
+//      xml.intTag(level, "genMTCSync", genMTCSync);
+//      xml.intTag(level, "genMCSync", genMCSync);
+//      xml.intTag(level, "genMMC", genMMC);
+//      xml.intTag(level, "acceptMTC", acceptMTC);
+//      xml.intTag(level, "acceptMMC", acceptMMC);
+//      xml.intTag(level, "acceptMC", acceptMC);
 
       xml.qrectTag(level, "geometryMain",      config.geometryMain);
       xml.qrectTag(level, "geometryTransport", config.geometryTransport);
@@ -925,22 +1119,58 @@ void MusE::writeConfiguration(int level, Xml& xml) const
       xml.intTag(level, "midiFilterCtrl2",  midiFilterCtrl2);
       xml.intTag(level, "midiFilterCtrl3",  midiFilterCtrl3);
       xml.intTag(level, "midiFilterCtrl4",  midiFilterCtrl4);
-      xml.intTag(level, "txDeviceId", txDeviceId);
-      xml.intTag(level, "rxDeviceId", rxDeviceId);
+      // Removed by Tim. p3.3.6
+      
+      //xml.intTag(level, "txDeviceId", txDeviceId);
+      //xml.intTag(level, "rxDeviceId", rxDeviceId);
 
-      xml.intTag(level, "txSyncPort", txSyncPort);
-      xml.intTag(level, "rxSyncPort", rxSyncPort);
+      // Changed by Tim. p3.3.6
+      
+      //xml.intTag(level, "txSyncPort", txSyncPort);
+      /*
+      // To keep old muse versions happy...
+      bool mcsync = mmc = mtc = false;
+      for(int sp = 0; sp < MIDI_PORTS; ++sp)
+      {
+        MidiSyncTxPort* txPort = &midiSyncTxPorts[sp];
+        if(txPort->doMCSync() || txPort->doMMC() || txPort->doMTC())
+        {
+          if(txPort->doMCSync())
+            mcsync = true;
+          if(txPort->doMMC())
+            mmc = true;
+          if(txPort->doMTC())
+            mtc = true;
+          xml.intTag(level, "txSyncPort", sp);
+          break;
+        }  
+      }
+      */
+      
+      // Added by Tim. p3.3.6
+      
+      //xml.tag(level++, "midiSyncInfo");
+      //for(iMidiDevice id = midiDevices.begin(); id != midiDevices.end(); ++id) 
+      //{
+      //  MidiDevice* md = *id;
+      //  md->syncInfo().write(level, xml, md);
+      //}      
+      //xml.etag(level, "midiSyncInfo");
+
+      //xml.intTag(level, "rxSyncPort", rxSyncPort);
       xml.intTag(level, "mtctype", mtcType);
       xml.nput(level, "<mtcoffset>%02d:%02d:%02d:%02d:%02d</mtcoffset>\n",
         mtcOffset.h(), mtcOffset.m(), mtcOffset.s(),
         mtcOffset.f(), mtcOffset.sf());
+      xml.intTag(level, "useJackTransport", useJackTransport);
       extSyncFlag.save(level, xml);
-      xml.intTag(level, "genMTCSync", genMTCSync);
-      xml.intTag(level, "genMCSync", genMCSync);
-      xml.intTag(level, "genMMC", genMMC);
-      xml.intTag(level, "acceptMTC", acceptMTC);
-      xml.intTag(level, "acceptMMC", acceptMMC);
-      xml.intTag(level, "acceptMC", acceptMC);
+      
+//      xml.intTag(level, "genMTCSync", genMTCSync);
+//      xml.intTag(level, "genMCSync", genMCSync);
+//      xml.intTag(level, "genMMC", genMMC);
+//      xml.intTag(level, "acceptMTC", acceptMTC);
+//      xml.intTag(level, "acceptMMC", acceptMMC);
+//      xml.intTag(level, "acceptMC", acceptMC);
 
       xml.intTag(level, "bigtimeVisible",   menuView->isItemChecked(bt_id));
       xml.intTag(level, "transportVisible", menuView->isItemChecked(tr_id));
@@ -975,8 +1205,9 @@ void MusE::writeConfiguration(int level, Xml& xml) const
 void MusE::configMidiSync()
       {
       if (!midiSyncConfig)
-            midiSyncConfig = new MidiSyncConfig(this);
-
+        //midiSyncConfig = new MidiSyncConfig(this);
+        midiSyncConfig = new MidiSyncConfig(0, (char*) "midiSyncConfig");
+        
       if (midiSyncConfig->isVisible()) {
           midiSyncConfig->raise();
           midiSyncConfig->setActiveWindow();
@@ -1031,6 +1262,8 @@ void MidiFileConfig::updateValues()
       formatCombo->setCurrentItem(config.smfFormat);
       extendedFormat->setChecked(config.extendedMidi);
       copyrightEdit->setText(config.copyright);
+      optNoteOffs->setChecked(config.expOptimNoteOffs);
+      twoByteTimeSigs->setChecked(config.exp2ByteTimeSigs);
       }
 
 //---------------------------------------------------------
@@ -1047,6 +1280,8 @@ void MidiFileConfig::okClicked()
       config.extendedMidi = extendedFormat->isChecked();
       config.smfFormat    = formatCombo->currentItem();
       config.copyright    = copyrightEdit->text();
+      config.expOptimNoteOffs = optNoteOffs->isChecked();
+      config.exp2ByteTimeSigs = twoByteTimeSigs->isChecked();
 
       muse->changeConfig(true);  // write config file
       close();
