@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//  $Id: conf.h,v 1.4 2004/06/01 14:25:47 wschweer Exp $
+//  $Id: conf.h,v 1.4.2.1 2006/09/28 19:22:25 spamatica Exp $
 //
 //  (C) Copyright 1999-2003 Werner Schweer (ws@seh.de)
 //=========================================================
@@ -33,6 +33,6 @@ class MidiFileConfig : public ConfigMidiFileBase {
 
 class Xml;
 extern bool readConfiguration();
-extern void readConfiguration(Xml&);
+extern void readConfiguration(Xml&, bool readOnlySequencer);
 #endif
 
