@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//    $Id: ctrledit.h,v 1.4 2004/01/17 11:34:00 wschweer Exp $
+//    $Id: ctrledit.h,v 1.4.2.1 2008/05/21 00:28:53 terminator356 Exp $
 //  (C) Copyright 1999 Werner Schweer (ws@seh.de)
 //=========================================================
 
@@ -19,6 +19,7 @@ class CtrlView;
 class CtrlPanel;
 class Xml;
 
+#define CTRL_PANEL_FIXED_WIDTH 40
 //---------------------------------------------------------
 //   CtrlEdit
 //---------------------------------------------------------
@@ -35,8 +36,7 @@ class CtrlEdit : public QWidget {
       void setTool(int tool);
       void setXPos(int val)           { canvas->setXPos(val); }
       void setXMag(int val)           { canvas->setXMag(val); }
-      void setCanvasWidth(int w)      { canvas->setFixedWidth(w); }
-
+      void setCanvasWidth(int w);
    signals:
       void timeChanged(unsigned);
       void destroyedCtrl(CtrlEdit*);
