@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//  $Id: alsamidi.cpp,v 1.8.2.1 2004/11/27 23:38:56 spamatica Exp $
+//  $Id: alsamidi.cpp,v 1.8.2.7 2009/11/19 04:20:33 terminator356 Exp $
 //  (C) Copyright 2000-2001 Werner Schweer (ws@seh.de)
 //=========================================================
 
@@ -16,6 +16,7 @@
 #include "../midictrl.h"
 #include "../audio.h"
 #include "mpevent.h"
+//#include "sync.h"
 
 static int alsaSeqFdi = -1;
 static int alsaSeqFdo = -1;
@@ -48,53 +49,124 @@ int MidiAlsaDevice::selectWfd()
 //---------------------------------------------------------
 
 QString MidiAlsaDevice::open()
-      {
+{
       _openFlags &= _rwFlags; // restrict to available bits
       snd_seq_port_subscribe_t* subs;
+      // Allocated on stack, no need to call snd_seq_port_subscribe_free() later.
       snd_seq_port_subscribe_alloca(&subs);
 
+      QString estr;
+      int wer = 0;
+      int rer = 0;
+      
       // subscribe for writing
-      if (_openFlags & 1) {
+      if (_openFlags & 1) 
+      {
             snd_seq_port_subscribe_set_sender(subs, &musePort);
             snd_seq_port_subscribe_set_dest(subs, &adr);
-            int error = snd_seq_subscribe_port(alsaSeq, subs);
-            if (error < 0)
-                  return QString("Play: ")+QString(snd_strerror(error));
-            }
+            // Not already subscribed (or error)? Then try subscribing.
+            if(snd_seq_get_port_subscription(alsaSeq, subs) < 0)
+            {
+              //int error = snd_seq_subscribe_port(alsaSeq, subs);
+              wer = snd_seq_subscribe_port(alsaSeq, subs);
+              //if (error < 0)
+              if(wer < 0)
+                    //return QString("Play: ")+QString(snd_strerror(error));
+                    estr += (QString("Play: ") + QString(snd_strerror(wer)) + QString(" "));
+            }        
+            if(!wer)
+              _writeEnable = true;      
+      }
 
       // subscribe for reading
-      if (_openFlags & 2) {
+      if (_openFlags & 2) 
+      {
             snd_seq_port_subscribe_set_dest(subs, &musePort);
-	      snd_seq_port_subscribe_set_sender(subs, &adr);
-            int error = snd_seq_subscribe_port(alsaSeq, subs);
-            if (error < 0)
-                  return QString("Rec: ") + QString(snd_strerror(error));
-            }
-      return QString("OK");
+	          snd_seq_port_subscribe_set_sender(subs, &adr);
+            // Not already subscribed (or error)? Then try subscribing.
+            if(snd_seq_get_port_subscription(alsaSeq, subs) < 0)
+            {
+              //int error = snd_seq_subscribe_port(alsaSeq, subs);
+              rer = snd_seq_subscribe_port(alsaSeq, subs);
+              //if (error < 0)
+              if(rer < 0)
+                    //return QString("Rec: ") + QString(snd_strerror(error));
+                    estr += (QString("Rec: ") + QString(snd_strerror(rer)));
+            }        
+            if(!rer)
+              _readEnable = true;      
       }
+      
+
+      if(wer < 0 || rer < 0)
+        return estr;
+        
+      return QString("OK");
+}
 
 //---------------------------------------------------------
 //   close
 //---------------------------------------------------------
 
 void MidiAlsaDevice::close()
-      {
+{
       snd_seq_port_subscribe_t* subs;
-
+      // Allocated on stack, no need to call snd_seq_port_subscribe_free() later.
       snd_seq_port_subscribe_alloca(&subs);
-      if (_openFlags & 1) {
+      
+      // Changed by T356. This function appears to be called only by MidiPort::setMidiDevice(), 
+      //  which closes then opens the device.
+      // Because the open flags are set BEFORE setMidiDevice() is called, we must ignore the flags.
+      //
+      // NOTE: Tested: The read unsubscribe works ok but not the write.
+      //               As viewed in say, qjackctl, the connection is clearly lost, 
+      //                but strangely the events are still accepted, ie, playback notes 
+      //                are still heard etc. Tried an alsa midi device AND external fluidsynth inst.
+      //
+      //               Also, jack running and with jack midi disabled, we get messages like
+      //                MidiAlsaDevice::0x84512c0 putEvent(): midi write error: No such device
+      //                 dst 16:0
+      //                only sometimes (not when playing notes), but with jack midi turned on, 
+      //                we don't get the messages. With jack stopped we get the messages
+      //                no matter if jack midi is turned on or not.
+
+      //if (_openFlags & 1) {
+      //if (!(_openFlags & 1)) 
+      {
             snd_seq_port_subscribe_set_sender(subs, &musePort);
             snd_seq_port_subscribe_set_dest(subs, &adr);
-            snd_seq_unsubscribe_port(alsaSeq, subs);
-            }
-
-      if (_openFlags & 2) {
-	      snd_seq_port_subscribe_set_dest(subs, &musePort);
-	      snd_seq_port_subscribe_set_sender(subs, &adr);
-            snd_seq_unsubscribe_port(alsaSeq, subs);
-            }
+            
+            // Already subscribed? Then unsubscribe.
+            if(!snd_seq_get_port_subscription(alsaSeq, subs))
+            {
+              if(!snd_seq_unsubscribe_port(alsaSeq, subs))
+                _writeEnable = false;      
+              else
+                printf("MidiAlsaDevice::close Error unsubscribing alsa midi port for writing\n");
+            }   
+            else
+              _writeEnable = false;      
       }
 
+      //if (_openFlags & 2) {
+      //if (!(_openFlags & 2)) 
+      {
+            snd_seq_port_subscribe_set_dest(subs, &musePort);
+            snd_seq_port_subscribe_set_sender(subs, &adr);
+            
+            // Already subscribed? Then unsubscribe.
+            if(!snd_seq_get_port_subscription(alsaSeq, subs))
+            {
+              if(!snd_seq_unsubscribe_port(alsaSeq, subs))
+                _readEnable = false;      
+              else  
+                printf("MidiAlsaDevice::close Error unsubscribing alsa midi port for reading\n");
+            }  
+            else
+              _readEnable = false;      
+      }
+}
+
 //---------------------------------------------------------
 //   putEvent
 //---------------------------------------------------------
@@ -316,6 +388,26 @@ bool initMidiAlsa()
                            adr.client, adr.port,
                            flags, capability);
                   midiDevices.add(dev);
+                  
+                  /*
+                  // Experimental... Need to list 'sensible' devices first and ignore unwanted ones...
+                  // Add instance last in midi device list.
+                  for(int i = 0; i < MIDI_PORTS; ++i) 
+                  {
+                    MidiPort* mp  = &midiPorts[i];
+                    if(mp->device() == 0) 
+                    {
+                      // midiSeq might not be initialzed yet!
+                      //midiSeq->msgSetMidiDevice(mp, dev);
+                      mp->setMidiDevice(dev);
+                      
+                      //muse->changeConfig(true);     // save configuration file
+                      //update();
+                      break;
+                    }
+                  }
+                  */
+                  
                   }
             }
       snd_seq_set_client_name(alsaSeq, "MusE Sequencer");
@@ -496,10 +588,12 @@ int alsaSelectWfd()
 //---------------------------------------------------------
 
 void alsaProcessMidiInput()
-      {
+{
       MidiRecordEvent event;
       snd_seq_event_t* ev;
-      for (;;) {
+      
+      for (;;) 
+      {
             int rv = snd_seq_event_input(alsaSeq, &ev);
 // printf("AlsaInput %d\n", rv);
             if (rv < 0) {
@@ -537,6 +631,7 @@ void alsaProcessMidiInput()
                         mdev = d;
                         }
                   }
+            
             if (mdev == 0 || curPort == -1) {
                   if (debugMsg) {
                         fprintf(stderr, "no port %d:%d found for received alsa event\n",
@@ -545,11 +640,38 @@ void alsaProcessMidiInput()
                   snd_seq_free_event(ev);
                   return;
                   }
+            
+            /*
+            if(curPort == -1) 
+            {
+                if(mdev == 0)
+                {  
+                  if (debugMsg) 
+                  {
+                    fprintf(stderr, "no port %d:%d found for received alsa event\n",
+                      ev->source.client, ev->source.port);
+                  }
+                }
+                else
+                {
+                  // Allow the sync detect mechanisms to work, even if device is not assigned to a port.
+                  if(ev->type == SND_SEQ_EVENT_CLOCK)
+                    mdev->syncInfo().trigMCSyncDetect();
+                  else  
+                  if(ev->type == SND_SEQ_EVENT_TICK)
+                    mdev->syncInfo().trigTickDetect();
+                }
+                snd_seq_free_event(ev);
+                return;
+            }
+            */      
+                  
             event.setType(0);      // mark as unused
             event.setPort(curPort);
             event.setB(0);
 
-            switch(ev->type) {
+            switch(ev->type) 
+            {
                   case SND_SEQ_EVENT_NOTEON:
                   case SND_SEQ_EVENT_KEYPRESS:
                         event.setChannel(ev->data.note.channel);
@@ -592,6 +714,7 @@ void alsaProcessMidiInput()
 
                   case SND_SEQ_EVENT_CLOCK:
                         midiSeq->realtimeSystemInput(curPort, 0xf8);
+                        //mdev->syncInfo().trigMCSyncDetect();
                         break;
 
                   case SND_SEQ_EVENT_START:
@@ -608,6 +731,7 @@ void alsaProcessMidiInput()
 
                   case SND_SEQ_EVENT_TICK:
                         midiSeq->realtimeSystemInput(curPort, 0xf9);
+                        //mdev->syncInfo().trigTickDetect();
                         break;
 
                   case SND_SEQ_EVENT_SYSEX:
@@ -647,12 +771,18 @@ void alsaProcessMidiInput()
                   default:
                         printf("ALSA Midi input: type %d not handled\n", ev->type);
                         break;
-                  }
-            if (event.type())
-                  mdev->recordEvent(event);
+            }
+            if(event.type())
+            {
+              mdev->recordEvent(event);
+              if(ev->type != SND_SEQ_EVENT_SYSEX)
+                // Trigger general activity indicator detector. Sysex has no channel, don't trigger.
+                midiPorts[curPort].syncInfo().trigActDetect(event.channel());
+            }
+                  
             snd_seq_free_event(ev);
             if (rv == 0)
                   break;
-            }
       }
+}
 
