@@ -1,7 +1,7 @@
   //=========================================================
   //  MusE
   //  Linux Music Editor
-  //  $Id: alsatimer.cpp,v 1.1.2.6 2005/08/21 18:11:27 spamatica Exp $
+  //  $Id: alsatimer.cpp,v 1.1.2.9 2009/03/28 01:46:10 terminator356 Exp $
   //
   //  Plenty of code borrowed from timer.c example in 
   //  alsalib 1.0.7
@@ -10,6 +10,7 @@
   //=========================================================
         
   #include "alsatimer.h"
+  #include <climits>
 
 #define TIMER_DEBUG 0
   
@@ -21,6 +22,7 @@
      id = NULL;
      info = NULL;
      params = NULL;
+     findBest = true;
      }
      
   AlsaTimer::~AlsaTimer()
@@ -29,9 +31,9 @@
        fprintf(stderr,"AlsaTimer::~AlsaTimer(this=%p) called\n",this);
     if (handle)
       snd_timer_close(handle);
-    if (id) free(id);
-    if (info) free(info);
-    if (params) free(params);
+    if (id) snd_timer_id_free(id);
+    if (info) snd_timer_info_free(info);
+    if (params) snd_timer_params_free(params);
     }
   
   signed int AlsaTimer::initTimer()
@@ -45,6 +47,16 @@
     int card = 0;
     int device = SND_TIMER_GLOBAL_SYSTEM;
     int subdevice = 0;
+    int test_ids[] = { SND_TIMER_GLOBAL_SYSTEM
+                     , SND_TIMER_GLOBAL_RTC
+#ifdef SND_TIMER_GLOBAL_HPET
+                     , SND_TIMER_GLOBAL_HPET
+#endif
+                     };
+    int max_ids = sizeof(test_ids) / sizeof(int);
+    long best_res = LONG_MAX;
+    int best_dev = SND_TIMER_GLOBAL_SYSTEM;
+    int i;
 
     if (id || info || params) {
       fprintf(stderr,"AlsaTimer::initTimer(): called on initialised timer!\n");
@@ -54,6 +66,29 @@
     snd_timer_info_malloc(&info);
     snd_timer_params_malloc(&params);
 
+    if (findBest) {
+      for (i = 0; i < max_ids; ++i) {
+        device = test_ids[i];
+        sprintf(timername, "hw:CLASS=%i,SCLASS=%i,CARD=%i,DEV=%i,SUBDEV=%i", devclass, sclass, card, device, subdevice);
+        if ((err = snd_timer_open(&handle, timername, SND_TIMER_OPEN_NONBLOCK)) < 0) {
+          continue;
+          }
+        if ((err = snd_timer_info(handle, info)) < 0) {
+          snd_timer_close(handle);
+          continue;
+          }
+        // select a non slave timer with the lowest resolution value
+        int is_slave = snd_timer_info_is_slave(info);
+        long res = snd_timer_info_get_resolution(info);
+        if ((is_slave == 0) && (best_res > res)) {
+          best_res = res;
+          best_dev = device;
+          }
+        snd_timer_close(handle);
+        }
+      device = best_dev;
+      }
+
     sprintf(timername, "hw:CLASS=%i,SCLASS=%i,CARD=%i,DEV=%i,SUBDEV=%i", devclass, sclass, card, device, subdevice);
     if ((err = snd_timer_open(&handle, timername, SND_TIMER_OPEN_NONBLOCK))<0) {
       fprintf(stderr, "AlsaTimer::initTimer(): timer open %i (%s)\n", err, snd_strerror(err));
@@ -64,6 +99,9 @@
       return -1;
       }
 
+    //if(debugMsg)
+      fprintf(stderr, "AlsaTimer::initTimer(): best available ALSA timer: %s\n", snd_timer_info_get_name(info));
+
     snd_timer_params_set_auto_start(params, 1);
     snd_timer_params_set_ticks(params, 1);
       
@@ -106,8 +144,9 @@
     if (setTick == 0) {
       fprintf(stderr,"AlsaTimer::setTimerTicks(): requested freq %u Hz too high for timer (max is %g)\n",
         freq, 1000000000.0 / snd_timer_info_get_resolution(info));
-      fprintf(stderr,"  freq stays at %d Hz\n",
-        (1000000000 / snd_timer_info_get_resolution(info)) / snd_timer_params_get_ticks(params));
+      fprintf(stderr,"  freq stays at %ld Hz\n",
+        (long int)((1000000000.0 / snd_timer_info_get_resolution(info)) / snd_timer_params_get_ticks(params)) 
+      );
 
       return 0;
     }
