@@ -1,11 +1,12 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//    $Id: dummyaudio.cpp,v 1.3.2.6 2006/03/19 16:26:48 spamatica Exp $
+//    $Id: dummyaudio.cpp,v 1.3.2.16 2009/12/20 05:00:35 terminator356 Exp $
 //  (C) Copyright 2002-2003 Werner Schweer (ws@seh.de)
 //=========================================================
 
 #include <stdio.h>
+#include <stdlib.h>
 #include <errno.h>
 #include <stdarg.h>
 #include <pthread.h>
@@ -17,12 +18,15 @@
 #include "globals.h"
 #include "song.h"
 #include "driver/alsatimer.h"
+#include "pos.h"
 
 #define DEBUG_DUMMY 0
 //---------------------------------------------------------
 //   DummyAudioDevice
 //---------------------------------------------------------
 
+static const unsigned dummyFrames = 1024;
+
 enum Cmd {
 trSeek,
 trStart,
@@ -37,7 +41,10 @@ struct Msg {
 
 class DummyAudioDevice : public AudioDevice {
       pthread_t dummyThread;
-      float buffer[1024];
+      // Changed by Tim. p3.3.15
+      //float buffer[1024];
+      float* buffer;
+      int _realTimePriority;
 
    public:
       std::list<Msg> cmdQueue;
@@ -47,9 +54,15 @@ class DummyAudioDevice : public AudioDevice {
       bool realtimeFlag;
       
       DummyAudioDevice();
-      virtual ~DummyAudioDevice() {}
+      virtual ~DummyAudioDevice()
+      { 
+        // Added by Tim. p3.3.15
+        free(buffer); 
+      }
 
-      virtual void start();
+      //virtual void start();
+      virtual void start(int);
+      
       virtual void stop ();
       virtual int framePos() const { 
             if(DEBUG_DUMMY)
@@ -59,7 +72,7 @@ class DummyAudioDevice : public AudioDevice {
 
       virtual float* getBuffer(void* /*port*/, unsigned long nframes)
             {
-            if (nframes > 1024) {
+            if (nframes > dummyFrames) {
                   printf("error: segment size > 1024\n");
                   exit(-1);
                   }
@@ -96,6 +109,8 @@ class DummyAudioDevice : public AudioDevice {
       
       return _framePos; }
       virtual bool isRealtime() { return realtimeFlag; }
+      //virtual int realtimePriority() const { return 40; }
+      virtual int realtimePriority() const { return _realTimePriority; }
       virtual void startTransport() {
             if(DEBUG_DUMMY)
                 printf("DummyAudioDevice::startTransport playPos=%d\n", playPos);
@@ -112,8 +127,20 @@ class DummyAudioDevice : public AudioDevice {
                 printf("DummyAudioDevice::stopTransport, playPos=%d\n", playPos);
             state = Audio::STOP;
             }
+      virtual int setMaster(bool) { return 1; }
+
+      virtual void seekTransport(const Pos &p)
+      {
+            if(DEBUG_DUMMY)
+                printf("DummyAudioDevice::seekTransport frame=%d topos=%d\n",playPos, p.frame());
+            Msg trcmd;
+            trcmd.cmd = trSeek;
+            trcmd.arg = p.frame();
+            cmdQueue.push_front(trcmd);
+            playPos = p.frame();
+      }
       virtual void seekTransport(unsigned pos) {
-            //if(DEBUG_DUMMY)
+            if(DEBUG_DUMMY)
                 printf("DummyAudioDevice::seekTransport frame=%d topos=%d\n",playPos,pos);
             Msg trcmd;
             trcmd.cmd = trSeek;
@@ -130,10 +157,13 @@ class DummyAudioDevice : public AudioDevice {
       void setRealTime() { realtimeFlag = true; }
       };
 
-DummyAudioDevice* dummyAudio;
+DummyAudioDevice* dummyAudio = 0;
 
 DummyAudioDevice::DummyAudioDevice() 
       {
+      // Added by Tim. p3.3.15
+      posix_memalign((void**)&buffer, 16, sizeof(float) * dummyFrames);
+      
       realtimeFlag = false;
       state = Audio::STOP;
       _framePos = 0;
@@ -141,8 +171,17 @@ DummyAudioDevice::DummyAudioDevice()
       cmdQueue.clear();
       }
 
+//---------------------------------------------------------
+//   exitDummyAudio
+//---------------------------------------------------------
 
-
+void exitDummyAudio()
+{
+      if(dummyAudio)
+        delete dummyAudio;
+      dummyAudio = NULL;      
+      audioDevice = NULL;      
+}
 
 //---------------------------------------------------------
 //   initDummyAudio
@@ -185,10 +224,14 @@ std::list<QString> DummyAudioDevice::inputPorts()
 
 static void* dummyLoop(void* ptr)
       {
-      unsigned int tickRate = 25;
+      //unsigned int tickRate = 25;
       sampleRate = 25600;
-      segmentSize = 1024;
+      segmentSize = dummyFrames;
+      unsigned int tickRate = sampleRate / dummyFrames;
+      
       AlsaTimer timer;
+      fprintf(stderr, "Finding best alsa timer for dummy driver:\n");
+      timer.setFindBestTimer(false);
       int fd = timer.initTimer();
       
       /* Depending on nature of the timer, the requested tickRate might not
@@ -207,6 +250,7 @@ static void* dummyLoop(void* ptr)
       myPollFd.fd = fd;
       myPollFd.events = POLLIN;
 
+      /*
       doSetuid();
       struct sched_param rt_param;
       int rv;
@@ -219,7 +263,8 @@ static void* dummyLoop(void* ptr)
             fprintf(stderr, "Driver thread not running SCHED_FIFO, trying to set...\n");
 
             memset(&rt_param, 0, sizeof(sched_param));
-            rt_param.sched_priority = 1;
+            //rt_param.sched_priority = 1;
+            rt_param.sched_priority = realtimePriority();
             rv = pthread_setschedparam(pthread_self(), SCHED_FIFO, &rt_param);
             if (rv != 0)
                   perror("set realtime scheduler");
@@ -236,13 +281,45 @@ static void* dummyLoop(void* ptr)
                   }
             }
       undoSetuid();
-      unsigned long tick = 0;
+      */
+      
+#ifndef __APPLE__
+      doSetuid();
+      //if (realTimePriority) {
+      if (realTimeScheduling) {
+            //
+            // check if we really got realtime priviledges
+            //
+            int policy;
+            if ((policy = sched_getscheduler (0)) < 0) {
+                printf("cannot get current client scheduler for audio dummy thread: %s!\n", strerror(errno));
+                }
+            else
+                {
+                if (policy != SCHED_FIFO)
+                          printf("audio dummy thread _NOT_ running SCHED_FIFO\n");
+                else if (debugMsg) {
+                        struct sched_param rt_param;
+                    memset(&rt_param, 0, sizeof(sched_param));
+                        int type;
+                    int rv = pthread_getschedparam(pthread_self(), &type, &rt_param);
+                        if (rv == -1)
+                                perror("get scheduler parameter");
+                    printf("audio dummy thread running SCHED_FIFO priority %d\n",
+                             rt_param.sched_priority);
+                    }
+                }
+            }
+      undoSetuid();
+#endif
+      
+      /* unsigned long tick = 0;*/    // prevent compiler warning: unused variable
       for (;;) {
             int _pollWait = 10;   // ms
             unsigned long count = 0;
             while (count < 1 /*250/tickRate*/) // will loop until the next tick occurs
                 {
-                int n = poll(&myPollFd, 1 /* npfd */, _pollWait);
+                /*int n = */  poll(&myPollFd, 1 /* npfd */, _pollWait);
                 count += timer.getTimerTicks();
                 while (drvPtr->cmdQueue.size())
                     {
@@ -251,7 +328,7 @@ static void* dummyLoop(void* ptr)
                     switch(msg.cmd) {
                           case trSeek:
                             {
-                            printf("trSeek\n");
+                            //printf("trSeek\n");
                             drvPtr->playPos = msg.arg;
                             Audio::State tempState = drvPtr->state;
                             drvPtr->state = Audio::START_PLAY;
@@ -261,7 +338,7 @@ static void* dummyLoop(void* ptr)
                             break;
                           case trStart:
                             {
-                            printf("trStart\n");
+                            //printf("trStart\n");
                             drvPtr->state = Audio::START_PLAY;
                             audio->sync(drvPtr->state, msg.arg);
                             drvPtr->state = Audio::PLAY;
@@ -270,11 +347,10 @@ static void* dummyLoop(void* ptr)
                           case trStop:
                             break;
                           default:
-                            printf("Woaaa\n");
-                          }
+                            printf("dummyLoop: Unknown command!\n");
+                        }
                     }
                 }
-
             audio->process(segmentSize);
             int increment = segmentSize; // 1 //tickRate / sampleRate * segmentSize;
             drvPtr->_framePos+=increment;
@@ -287,13 +363,39 @@ static void* dummyLoop(void* ptr)
       pthread_exit(0);
       }
 
-void DummyAudioDevice::start()
+//void DummyAudioDevice::start()
+void DummyAudioDevice::start(int priority)
       {
-      pthread_attr_t* attributes = (pthread_attr_t*) malloc(sizeof(pthread_attr_t));
-      pthread_attr_init(attributes);
+      //realTimePriority = priority;
+      _realTimePriority = priority;
+      pthread_attr_t* attributes = 0;
+
+      //if (priority) {
+      if (realTimeScheduling && priority > 0) {
+            attributes = (pthread_attr_t*) malloc(sizeof(pthread_attr_t));
+            pthread_attr_init(attributes);
+
+            if (pthread_attr_setschedpolicy(attributes, SCHED_FIFO)) {
+                  printf("cannot set FIFO scheduling class for RT thread\n");
+                  }
+            if (pthread_attr_setscope (attributes, PTHREAD_SCOPE_SYSTEM)) {
+                  printf("Cannot set scheduling scope for RT thread\n");
+                  }
+            struct sched_param rt_param;
+            memset(&rt_param, 0, sizeof(rt_param));
+            rt_param.sched_priority = priority;
+            if (pthread_attr_setschedparam (attributes, &rt_param)) {
+                  printf("Cannot set scheduling priority %d for RT thread (%s)\n",
+                     priority, strerror(errno));
+                  }
+            }
+      
+      //pthread_attr_t* attributes = (pthread_attr_t*) malloc(sizeof(pthread_attr_t));
+      //pthread_attr_init(attributes);
       if (pthread_create(&dummyThread, attributes, ::dummyLoop, this))
             perror("creating thread failed:");
-      pthread_attr_destroy(attributes);
+      if (priority)
+        pthread_attr_destroy(attributes);
       }
 
 void DummyAudioDevice::stop ()
