@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//    $Id: jack.cpp,v 1.30.2.8 2006/02/01 23:17:38 spamatica Exp $
+//    $Id: jack.cpp,v 1.30.2.17 2009/12/20 05:00:35 terminator356 Exp $
 //  (C) Copyright 2002 Werner Schweer (ws@seh.de)
 //=========================================================
 
@@ -9,12 +9,18 @@
 #include <stdio.h>
 #include <errno.h>
 #include <stdarg.h>
+//#include <time.h> 
+#include <unistd.h>
 
 #include "audio.h"
 #include "globals.h"
 #include "song.h"
 #include "jackaudio.h"
 #include "track.h"
+#include "pos.h"
+#include "tempo.h"
+#include "sync.h"
+#include "utils.h"
 
 #define JACK_DEBUG 0
 
@@ -48,7 +54,7 @@ inline bool checkJackClient(jack_client_t* _client)
 bool checkAudioDevice()
       {
       if (audioDevice == NULL) {
-            printf("Panic! no audioDevice!\n");
+            printf("Muse:checkAudioDevice: no audioDevice\n");
             return false;
             }
       return true;
@@ -59,9 +65,10 @@ bool checkAudioDevice()
 //   jack_thread_init
 //---------------------------------------------------------
 
-static void jack_thread_init (void* /*data*/)
+static void jack_thread_init (void* )  // data
       {
       doSetuid();
+      /*
       if (jackAudio->isRealtime()) {
             struct sched_param rt_param;
             int rv;
@@ -87,8 +94,9 @@ static void jack_thread_init (void* /*data*/)
                         "======reliable RT operation not possible!!======\n");
                   else
                         fprintf(stderr, "JACK thread succesfully set to SCHED_FIFO\n");
-                  }
+                  } 
             }
+            */
 #ifdef VST_SUPPORT
       if (loadVST)
             fst_adopt_thread();
@@ -127,17 +135,68 @@ static int processSync(jack_transport_state_t state, jack_position_t* pos, void*
       if (JACK_DEBUG)
             printf("processSync()\n");
       
+      if(!useJackTransport)
+        return 1;
+        
       int audioState = Audio::STOP;
       switch (state) {
-            case JackTransportStopped:   audioState = Audio::STOP;
+            case JackTransportStopped:   
+              audioState = Audio::STOP;
+            break;  
             case JackTransportLooping:
-            case JackTransportRolling:   audioState = Audio::PLAY;
-            case JackTransportStarting:  audioState = Audio::START_PLAY;
+            case JackTransportRolling:   
+              audioState = Audio::PLAY;
+            break;  
+            case JackTransportStarting:  
+              // Added by Tim. p3.3.6
+              //printf("processSync JackTransportStarting\n");
+              
+              audioState = Audio::START_PLAY;
+            break;  
+            //case JackTransportNetStarting:  
+            // FIXME: Quick and dirty hack to support both Jack-1 and Jack-2
+            // Really need a config check of version...
+            case 4:  
+              // Added by Tim. p3.3.6
+              //printf("processSync JackTransportNetStarting\n");
+              
+              audioState = Audio::START_PLAY;
+            break;  
             }
+            
       unsigned frame = pos->frame;
+      // Added by Tim. p3.3.6
+      //printf("processSync valid:%d frame:%d\n", pos->valid, frame);
+      
       return audio->sync(audioState, frame);
       }
 
+//---------------------------------------------------------
+//   timebase_callback
+//---------------------------------------------------------
+
+static void timebase_callback(jack_transport_state_t /* state */,
+   jack_nframes_t /* nframes */,
+   jack_position_t* pos,
+   int /* new_pos */,
+   void*)
+  {
+      Pos p(pos->frame, false);
+      pos->valid = JackPositionBBT;
+      p.mbt(&pos->bar, &pos->beat, &pos->tick);
+      pos->bar++;
+      pos->beat++;
+      pos->bar_start_tick = Pos(pos->bar, 0, 0).tick();
+      //
+      //  dummy:
+      //
+      pos->beats_per_bar = 4;
+      pos->beat_type = 4;
+      pos->ticks_per_beat = 384;
+      int tempo = tempomap.tempo(p.tick());
+      pos->beats_per_minute = (60000000.0 / tempo) * tempomap.globalTempo()/100.0;
+      }
+
 //---------------------------------------------------------
 //   processShutdown
 //---------------------------------------------------------
@@ -191,6 +250,8 @@ JackAudioDevice::JackAudioDevice(jack_client_t* cl, char * name)
       //JackAudioDevice::jackStarted=false;
       strcpy(jackRegisteredName, name);
       _client = cl;
+      dummyState = Audio::STOP;
+      dummyPos = 0;
       }
 
 //---------------------------------------------------------
@@ -209,6 +270,31 @@ JackAudioDevice::~JackAudioDevice()
             }
       }
 
+//---------------------------------------------------------
+//   realtimePriority
+//      return zero if not running realtime
+//      can only be called if JACK client thread is already
+//      running
+//---------------------------------------------------------
+
+int JackAudioDevice::realtimePriority() const
+      {
+      pthread_t t = jack_client_thread_id(_client);
+      int policy;
+      struct sched_param param;
+      memset(&param, 0, sizeof(param));
+        int rv = pthread_getschedparam(t, &policy, &param);
+      if (rv) {
+            perror("MusE: JackAudioDevice::realtimePriority: Error: Get jack schedule parameter");
+            return 0;
+            }
+      if (policy != SCHED_FIFO) {
+            printf("MusE: JackAudioDevice::realtimePriority: JACK is not running realtime\n");
+            return 0;
+            }
+      return param.sched_priority;
+      }
+
 //---------------------------------------------------------
 //   getJackName()
 //---------------------------------------------------------
@@ -228,7 +314,7 @@ bool initJackAudio()
       if (JACK_DEBUG)
             printf("initJackAudio()\n");
       if (debugMsg) {
-            fprintf(stderr,"init Jack Audio\n");
+            fprintf(stderr,"initJackAudio()\n");
             jack_set_error_function(jackError);
             }
       else
@@ -240,7 +326,8 @@ bool initJackAudio()
       char jackIdString[8];
       for (i = 0; i < 5; ++i) {
             sprintf(jackIdString, "MusE-%d", i+1);
-            client = jack_client_new(jackIdString);
+            //client = jack_client_new(jackIdString);
+            client = jack_client_open(jackIdString, JackNoStartServer, 0);
             if (client)
                   break;
             }
@@ -249,21 +336,20 @@ bool initJackAudio()
             return true;
 
       if (debugMsg)
-            fprintf(stderr, "init Jack Audio: register device\n");
+            fprintf(stderr, "initJackAudio(): client %s opened.\n", jackIdString);
       if (client) {
             jack_set_error_function(jackError);
-            if (debugMsg)
-                  fprintf(stderr, "init Jack Audio: register device\n");
-
             jackAudio = new JackAudioDevice(client, jackIdString);
             if (debugMsg)
-                  fprintf(stderr, "init Jack Audio: register client\n");
+                  fprintf(stderr, "initJackAudio(): registering client...\n");
             jackAudio->registerClient();
             sampleRate  = jack_get_sample_rate(client);
             segmentSize = jack_get_buffer_size(client);
             jack_set_thread_init_callback(client, (JackThreadInitCallback) jack_thread_init, 0);
+            //jack_set_timebase_callback(client, 0, (JackTimebaseCallback) timebase_callback, 0);
             }
       undoSetuid();
+      
       if (client) {
             audioDevice = jackAudio;
             return false;
@@ -283,6 +369,8 @@ static int bufsize_callback(jack_nframes_t n, void*)
 
 static void freewheel_callback(int starting, void*)
       {
+      if (debugMsg || JACK_DEBUG)
+            printf("JACK: freewheel_callback: starting%d\n", starting);
       audio->setFreewheel(starting);
       }
 
@@ -511,6 +599,12 @@ void JackAudioDevice::registerClient()
       if(!checkJackClient(_client)) return;
       jack_set_process_callback(_client, processAudio, 0);
       jack_set_sync_callback(_client, processSync, 0);
+      // FIXME: FIXME:
+      // Added by Tim. p3.3.20
+      // Did not help. Seek during play: Jack keeps switching to STOP state after about 1-2 seconds timeout if sync is holding it up.
+      // Nothing in MusE seems to be telling it to stop.
+      //jack_set_sync_timeout(_client, 5000000); // Change default 2 to 5 second sync timeout because prefetch may be very slow esp. with resampling !
+      
       jack_on_shutdown(_client, processShutdown, 0);
       jack_set_buffer_size_callback(_client, bufsize_callback, 0);
       jack_set_sample_rate_callback(_client, srate_callback, 0);
@@ -558,6 +652,10 @@ void exitJackAudio()
             printf("exitJackAudio()\n");
       if (jackAudio)
             delete jackAudio;
+            
+      // Added by Tim. p3.3.14
+      audioDevice = NULL;      
+      
       }
 
 //---------------------------------------------------------
@@ -606,7 +704,8 @@ void JackAudioDevice::disconnect(void* src, void* dst)
 //   start
 //---------------------------------------------------------
 
-void JackAudioDevice::start()
+//void JackAudioDevice::start()
+void JackAudioDevice::start(int /*priority*/)
       {
       if (JACK_DEBUG)
             printf("JackAudioDevice::start()\n");
@@ -651,7 +750,9 @@ void JackAudioDevice::start()
                         }
                   }
             }
+      
       undoSetuid();
+      
       //MUSE_DEBUG("JackAudioDevice::start()\n");
       fflush(stdin);
       //JackAudioDevice::jackStarted=true;
@@ -672,14 +773,54 @@ void JackAudioDevice::stop()
       //JackAudioDevice::jackStarted=false;
       }
 
+//---------------------------------------------------------
+//   transportQuery
+//---------------------------------------------------------
+
+jack_transport_state_t JackAudioDevice::transportQuery(jack_position_t* pos)
+{ 
+  if (JACK_DEBUG)
+    printf("JackAudioDevice::transportQuery pos:%d\n", (unsigned int)pos->frame);
+  
+  // TODO: Compose and return a state if MusE is disengaged from Jack transport.
+  
+  return jack_transport_query(_client, pos); 
+}
+
+//---------------------------------------------------------
+//   getCurFrame
+//---------------------------------------------------------
+
+unsigned int JackAudioDevice::getCurFrame()
+{ 
+  if (JACK_DEBUG)
+    printf("JackAudioDevice::getCurFrame pos.frame:%d\n", pos.frame);
+  
+  if(!useJackTransport)
+    return (unsigned int)dummyPos;
+    
+  return pos.frame; 
+}
+
 //---------------------------------------------------------
 //   framePos
 //---------------------------------------------------------
 
 int JackAudioDevice::framePos() const
       {
+      //if(!useJackTransport)
+      //{
+      //  if (JACK_DEBUG)
+      //    printf("JackAudioDevice::framePos dummyPos:%d\n", dummyPos);
+      //  return dummyPos;
+      //}
+      
       if(!checkJackClient(_client)) return 0;
       jack_nframes_t n = jack_frame_time(_client);
+      
+      if (JACK_DEBUG)
+        printf("JackAudioDevice::framePos jack frame:%d\n", (int)n);
+      
       return (int)n;
       }
 
@@ -716,7 +857,7 @@ std::list<QString> JackAudioDevice::outputPorts()
             printf("JackAudioDevice::outputPorts()\n");
       std::list<QString> clientList;
       if(!checkJackClient(_client)) return clientList;
-      const char** ports = jack_get_ports(_client, 0, 0, 0);
+      const char** ports = jack_get_ports(_client, 0, JACK_DEFAULT_AUDIO_TYPE, 0);
       for (const char** p = ports; p && *p; ++p) {
             jack_port_t* port = jack_port_by_name(_client, *p);
             int flags = jack_port_flags(port);
@@ -741,7 +882,7 @@ std::list<QString> JackAudioDevice::inputPorts()
             printf("JackAudioDevice::inputPorts()\n");
       std::list<QString> clientList;
       if(!checkJackClient(_client)) return clientList;
-      const char** ports = jack_get_ports(_client, 0, 0, 0);
+      const char** ports = jack_get_ports(_client, 0, JACK_DEFAULT_AUDIO_TYPE, 0);
       for (const char** p = ports; p && *p; ++p) {
             jack_port_t* port = jack_port_by_name(_client, *p);
             int flags = jack_port_flags(port);
@@ -792,17 +933,46 @@ void JackAudioDevice::unregisterPort(void* p)
 
 int JackAudioDevice::getState()
       {
+      // If we're not using Jack's transport, just return current state.
+      if(!useJackTransport)
+      {
+        //pos.valid = jack_position_bits_t(0);
+        //pos.frame = audio->pos().frame();
+        //return audio->getState();
+        if (JACK_DEBUG)
+          printf("JackAudioDevice::getState dummyState:%d\n", dummyState);
+        return dummyState;
+      }
+      
       //if (JACK_DEBUG)
       //      printf("JackAudioDevice::getState ()\n");
       if(!checkJackClient(_client)) return 0;
       transportState = jack_transport_query(_client, &pos);
+      if (JACK_DEBUG)
+          printf("JackAudioDevice::getState transportState:%d\n", transportState);
+      
       switch (transportState) {
-            case JackTransportStopped:  return Audio::STOP;
+            case JackTransportStopped:  
+              return Audio::STOP;
             case JackTransportLooping:
-            case JackTransportRolling:  return Audio::PLAY;
-            case JackTransportStarting:  return Audio::START_PLAY;
+            case JackTransportRolling:  
+              return Audio::PLAY;
+            case JackTransportStarting:  
+              // Added by Tim. p3.3.6
+              //printf("JackAudioDevice::getState JackTransportStarting\n");
+              
+              return Audio::START_PLAY;
+            //case JackTransportNetStarting:  
+            // FIXME: Quick and dirty hack to support both Jack-1 and Jack-2
+            // Really need a config check of version...
+            case 4:  
+              // Added by Tim. p3.3.6
+              //printf("JackAudioDevice::getState JackTransportNetStarting\n");
+              
+              return Audio::START_PLAY;
+            break;  
             default:
-                  return Audio::STOP;
+              return Audio::STOP;
             }
       }
 
@@ -819,46 +989,193 @@ void JackAudioDevice::setFreewheel(bool f)
       jack_set_freewheel(_client, f);
       }
 
+//---------------------------------------------------------
+//   dummySync
+//---------------------------------------------------------
+
+bool JackAudioDevice::dummySync(int state)
+{
+  // Roughly segment time length.
+  //timespec ts = { 0, (1000000000 * segmentSize) / sampleRate };     // In nanoseconds.
+  unsigned int sl = (1000000 * segmentSize) / sampleRate;            // In microseconds.
+  
+  double ct = curTime();
+  // Wait for a default maximum of 5 seconds. 
+  // Similar to how Jack is supposed to wait a default of 2 seconds for slow clients.
+  // TODO: Make this timeout a 'settings' option so it can be applied both to Jack and here.
+  while((curTime() - ct) < 5.0)  
+  {
+    // Is MusE audio ready to roll?
+    if(audio->sync(state, dummyPos))
+      return true;
+    
+    // Not ready. Wait a 'segment', try again...
+    //nanosleep(&ts, NULL);  
+    usleep(sl);              // usleep is supposed to be obsolete!
+  }
+    
+  //if(JACK_DEBUG)
+    printf("JackAudioDevice::dummySync Sync timeout - audio not ready!\n");
+  
+  return false;
+}
+
 //---------------------------------------------------------
 //   startTransport
 //---------------------------------------------------------
 
 void JackAudioDevice::startTransport()
-      {
+    {
       if (JACK_DEBUG)
             printf("JackAudioDevice::startTransport()\n");
+      
+      // If we're not using Jack's transport, just pass PLAY and current frame along
+      //  as if processSync was called. 
+      if(!useJackTransport)
+      {
+        //dummyState = Audio::START_PLAY;
+        
+        // Is MusE audio ready to roll?
+        //if(dummySync(dummyState))
+        if(dummySync(Audio::START_PLAY))
+        {
+          // MusE audio is ready to roll. Let's play.
+          dummyState = Audio::PLAY;
+          return;
+        }
+          
+        // Ready or not, we gotta roll. Similar to how Jack is supposed to roll anyway.
+        dummyState = Audio::PLAY;
+        return;
+      }
+      
       if(!checkJackClient(_client)) return;
 //      printf("JACK: startTransport\n");
       jack_transport_start(_client);
-      }
+    }
 
 //---------------------------------------------------------
 //   stopTransport
 //---------------------------------------------------------
 
 void JackAudioDevice::stopTransport()
-      {
+    {
       if (JACK_DEBUG)
             printf("JackAudioDevice::stopTransport()\n");
+      
+      dummyState = Audio::STOP;
+      
+      if(!useJackTransport)
+      {
+        //dummyState = Audio::STOP;
+        return;
+      }
+      
       if(!checkJackClient(_client)) return;
       if (transportState != JackTransportStopped) {
         //      printf("JACK: stopTransport\n");
             jack_transport_stop(_client);
             transportState=JackTransportStopped;
             }
-      }
+    }
 
 //---------------------------------------------------------
 //   seekTransport
 //---------------------------------------------------------
 
 void JackAudioDevice::seekTransport(unsigned frame)
-      {
+    {
       if (JACK_DEBUG)
-            printf("JackAudioDevice::seekTransport(\n");
+            printf("JackAudioDevice::seekTransport() frame:%d\n", frame);
+      
+      dummyPos = frame;
+      if(!useJackTransport)
+      {
+        // If we're not using Jack's transport, just pass the current state and new frame along
+        //  as if processSync was called. 
+        //dummyPos = frame;
+        int tempState = dummyState;
+        //dummyState = Audio::START_PLAY;
+        
+        // Is MusE audio ready yet?
+        //audio->sync(dummyState, dummyPos);
+        //if(dummySync(dummyState))
+        if(dummySync(Audio::START_PLAY))
+        {
+          dummyState = tempState;
+          return;
+        }
+        
+        // Not ready, resume previous state anyway.
+        // FIXME: Observed: Seek during play: Jack transport STOPs on timeout. 
+        // Docs say when starting play, transport will roll anyway, ready or not (observed),
+        //  but don't mention what should happen on seek during play. 
+        // And setting the slow-sync timeout doesn't seem to do anything!
+        //dummyState = tempState;
+        dummyState = Audio::STOP;
+        return;
+      }
+      
       if(!checkJackClient(_client)) return;
 //      printf("JACK: seekTransport %d\n", frame);
       jack_transport_locate(_client, frame);
+    }
+
+//---------------------------------------------------------
+//   seekTransport
+//---------------------------------------------------------
+
+void JackAudioDevice::seekTransport(const Pos &p)
+      {
+      if (JACK_DEBUG)
+            printf("JackAudioDevice::seekTransport() frame:%d\n", p.frame());
+      
+      dummyPos = p.frame();
+      if(!useJackTransport)
+      {
+        // If we're not using Jack's transport, just pass the current state and new frame along
+        //  as if processSync was called. 
+        //dummyPos = p.frame();
+        int tempState = dummyState;
+        //dummyState = Audio::START_PLAY;
+        
+        // Is MusE audio ready yet?
+        //audio->sync(dummyState, dummyPos);
+        //if(dummySync(dummyState))
+        if(dummySync(Audio::START_PLAY))
+        {
+          dummyState = tempState;
+          return;
+        }
+        
+        // Not ready, resume previous state anyway.
+        // FIXME: See fixme in other seekTransport...
+        //dummyState = tempState;
+        dummyState = Audio::STOP;
+        return;
+      }
+      
+      if(!checkJackClient(_client)) return;
+      
+      /*
+      jack_position_t jp;
+      jp.valid = JackPositionBBT;
+      p.mbt(&jp.bar, &jp.beat, &jp.tick);
+      jp.bar++;
+      jp.beat++;
+      jp.bar_start_tick = Pos(jp.bar, 0, 0).tick();
+      //
+      //  dummy:
+      //
+      jp.beats_per_bar = 4;
+      jp.beat_type = 4;
+      jp.ticks_per_beat = 384;
+      int tempo = tempomap.tempo(p.tick());
+      jp.beats_per_minute = (60000000.0 / tempo) * tempomap.globalTempo()/100.0;
+      
+      jack_transport_reposition(_client, &jp);
+      */
+      jack_transport_locate(_client, p.frame());
       }
 
 //---------------------------------------------------------
@@ -875,3 +1192,40 @@ void* JackAudioDevice::findPort(const char* name)
       return p;
       }
 
+int JackAudioDevice::setMaster(bool f)
+{
+  if (JACK_DEBUG)
+    printf("JackAudioDevice::setMaster val:%d\n", f);
+  if(!checkJackClient(_client)) 
+    return 0;
+  
+  int r = 0;
+  if(f)
+  {
+    if(useJackTransport)
+    {
+      // Make Muse the Jack timebase master. Do it unconditionally (second param = 0).
+      r = jack_set_timebase_callback(_client, 0, (JackTimebaseCallback) timebase_callback, 0);
+      if(debugMsg || JACK_DEBUG)
+      {
+        if(r)
+          printf("JackAudioDevice::setMaster jack_set_timebase_callback failed: result:%d\n", r);
+      }      
+    }  
+    else
+    {
+      r = 1;
+      printf("JackAudioDevice::setMaster cannot set master because useJackTransport is false\n");
+    }
+  }  
+  else
+  {
+    r = jack_release_timebase(_client);
+    if(debugMsg || JACK_DEBUG)
+    {
+      if(r)
+        printf("JackAudioDevice::setMaster jack_release_timebase failed: result:%d\n", r);
+    }      
+  }
+  return r;  
+}
