@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//    $Id: jackaudio.h,v 1.20.2.2 2006/02/01 23:17:38 spamatica Exp $
+//    $Id: jackaudio.h,v 1.20.2.4 2009/12/20 05:00:35 terminator356 Exp $
 //  (C) Copyright 2002 Werner Schweer (ws@seh.de)
 //=========================================================
 
@@ -24,13 +24,19 @@ class JackAudioDevice : public AudioDevice {
       jack_transport_state_t transportState;
       jack_position_t pos;
       char jackRegisteredName[8];
+      int dummyState;
+      int dummyPos;
 
    public:
       JackAudioDevice(jack_client_t* cl, char * jack_id_string);
       virtual ~JackAudioDevice();
       virtual void nullify_client() { _client = 0; }
-      virtual void start();
+      
+      //virtual void start();
+      virtual void start(int);
       virtual void stop ();
+      virtual bool dummySync(int state); // Artificial sync when not using Jack transport.
+      
       virtual int framePos() const;
 
       virtual float* getBuffer(void* port, unsigned long nframes) {
@@ -54,15 +60,17 @@ class JackAudioDevice : public AudioDevice {
       virtual void* findPort(const char* name);
       virtual QString portName(void* port);
       virtual int getState();
-      virtual unsigned int getCurFrame() { return pos.frame; }
+      virtual unsigned int getCurFrame();
       virtual bool isRealtime()          { return jack_is_realtime(_client); }
+      virtual int realtimePriority() const;
       virtual void startTransport();
       virtual void stopTransport();
       virtual void seekTransport(unsigned frame);
+      virtual void seekTransport(const Pos &p);
       virtual void setFreewheel(bool f);
-      jack_transport_state_t transportQuery(jack_position_t* pos)
-         { return jack_transport_query(_client, pos); }
+      jack_transport_state_t transportQuery(jack_position_t* pos);
       void graphChanged();
+      virtual int setMaster(bool f);
 
       //static bool jackStarted;
       };
