@@ -1,17 +1,22 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//  $Id: event.cpp,v 1.8.2.1 2005/08/14 21:47:31 spamatica Exp $
+//  $Id: event.cpp,v 1.8.2.5 2009/12/20 05:00:35 terminator356 Exp $
 //
 //  (C) Copyright 2000-2003 Werner Schweer (ws@seh.de)
 //=========================================================
 
 #include <stdio.h>
-#include <memory.h>
+// #include <memory.h>
+#include "audioconvert.h"
 #include "event.h"
 #include "eventbase.h"
 #include "waveevent.h"
 #include "midievent.h"
+//#include "globals.h"
+
+// Added by Tim. p3.3.20
+#define USE_SAMPLERATE
 
 //---------------------------------------------------------
 //   Event
@@ -63,7 +68,17 @@ Event Event::clone()
       return Event(ev->clone());
       }
 
+Event::Event() 
+{ 
+  ev = 0; 
+  _sfCurFrame = 0;
+  _audConv = 0;
+}
+
 Event::Event(EventType t) {
+            _sfCurFrame = 0;
+            _audConv = 0;
+            
             if (t == Wave)
                   ev = new WaveEventBase(t);
             else
@@ -71,13 +86,32 @@ Event::Event(EventType t) {
             ++(ev->refCount);
             }
 Event::Event(const Event& e) {
+            _sfCurFrame = 0;
+            _audConv = 0;
+            
             ev = e.ev;
-            if (ev)
-                  ++(ev->refCount);
-            }
+            if(ev)
+              ++(ev->refCount);
+            
+            #ifdef USE_SAMPLERATE
+            //_audConv = AudioConverter::getAudioConverter(e._audConv);
+            if(e._audConv)
+              _audConv = e._audConv->reference();
+            #endif
+          }
 Event::Event(EventBase* eb) {
+            _sfCurFrame = 0;
+            _audConv = 0;
+            
             ev = eb;
             ++(ev->refCount);
+            
+            #ifdef USE_SAMPLERATE
+            if(!ev->sndFile().isNull())
+              //_audConv = AudioConverter::getAudioConverter(eb, SRC_SINC_MEDIUM_QUALITY);
+              //_audConv = new AudioConverter(ev->sndFile().channels(), SRC_SINC_MEDIUM_QUALITY);
+              _audConv = new SRCAudioConverter(ev->sndFile().channels(), SRC_SINC_MEDIUM_QUALITY);
+            #endif  
             }
 
 Event::~Event() {
@@ -85,6 +119,10 @@ Event::~Event() {
                   delete ev;
                   ev=0;
                   }
+
+            #ifdef USE_SAMPLERATE
+            AudioConverter::release(_audConv);
+            #endif
             }
 
 bool Event::empty() const      { return ev == 0; }
@@ -103,6 +141,7 @@ void Event::setType(EventType t) {
             }
 
 Event& Event::operator=(const Event& e) {
+            /*
             if (ev == e.ev)
                   return *this;
             if (ev && --(ev->refCount) == 0) {
@@ -113,7 +152,31 @@ Event& Event::operator=(const Event& e) {
             if (ev)
                   ++(ev->refCount);
             return *this;
+            */
+            
+            if (ev != e.ev)
+            {
+              if (ev && --(ev->refCount) == 0) {
+                    delete ev;
+                    ev = 0;
+                    }
+              ev = e.ev;
+              if (ev)
+                    ++(ev->refCount);
+            }      
+            
+            #ifdef USE_SAMPLERATE
+            if (_audConv != e._audConv)
+            {
+              if(_audConv)
+                AudioConverter::release(_audConv);
+              //_audConv = AudioConverter::getAudioConverter(e._audConv);
+              _audConv = e._audConv->reference();
+            }      
+            #endif
+            return *this;
             }
+
 bool Event::operator==(const Event& e) const {
             return ev == e.ev;
             }
@@ -123,8 +186,37 @@ bool Event::selected() const      { return ev->_selected; }
 void Event::setSelected(bool val) { ev->_selected = val; }
 void Event::move(int offset)      { ev->move(offset); }
 
-void Event::read(Xml& xml)            { ev->read(xml); }
-void Event::write(int a, Xml& xml, const Pos& o) const { ev->write(a, xml, o); }
+//void Event::read(Xml& xml)            { ev->read(xml); }
+void Event::read(Xml& xml)            
+{ 
+  ev->read(xml); 
+  
+  #ifdef USE_SAMPLERATE
+  if(!ev->sndFile().isNull())
+  {
+    if(_audConv)
+    {
+      _audConv->setChannels(ev->sndFile().channels());
+    }
+    else
+    {
+      //int srcerr;
+      //if(debugMsg)
+      //  printf("Event::read Creating samplerate converter with %d channels\n", ev->sndFile().channels());
+      //_src_state = src_new(SRC_SINC_MEDIUM_QUALITY, ev->sndFile().channels(), &srcerr);
+//      _audConv = new AudioConverter(ev->sndFile().channels(), SRC_SINC_MEDIUM_QUALITY);
+      _audConv = new SRCAudioConverter(ev->sndFile().channels(), SRC_SINC_MEDIUM_QUALITY);
+      //if(!_src_state)      
+      //if(!_audConv)      
+      //  printf("Event::read Creation of samplerate converter with %d channels failed:%s\n", ev->sndFile().channels(), src_strerror(srcerr));
+    }  
+  }  
+  #endif
+}
+
+
+//void Event::write(int a, Xml& xml, const Pos& o) const { ev->write(a, xml, o); }
+void Event::write(int a, Xml& xml, const Pos& o, bool forceWavePaths) const { ev->write(a, xml, o, forceWavePaths); }
 void Event::dump(int n) const     { ev->dump(n); }
 Event Event::mid(unsigned a, unsigned b) { return Event(ev->mid(a, b)); }
 
@@ -154,12 +246,33 @@ void Event::setName(const QString& s)        { ev->setName(s);     }
 int Event::spos() const                      { return ev->spos();  }
 void Event::setSpos(int s)                   { ev->setSpos(s);     }
 SndFileR Event::sndFile() const              { return ev->sndFile(); }
-void Event::setSndFile(SndFileR& sf) { ev->setSndFile(sf);   }
-void Event::read(unsigned offset, float** bpp, int channels, int nn)
-      {
-      ev->read(offset, bpp, channels, nn);
-      }
 
+//void Event::setSndFile(SndFileR& sf) { ev->setSndFile(sf);   }
+void Event::setSndFile(SndFileR& sf) 
+{ 
+  ev->setSndFile(sf);   
+  
+  #ifdef USE_SAMPLERATE
+  //if(_audConv)
+  if(_audConv && !sf.isNull())
+  { 
+    //_audConv->setSndFile(sf);
+    //if(sf.isNull())
+    //  AudioConverter::release(_audConv);
+    //else
+      _audConv->setChannels(sf.channels());
+  }  
+  #endif
+}
+
+//void Event::read(unsigned offset, float** bpp, int channels, int nn, bool overwrite)
+void Event::readAudio(unsigned offset, float** bpp, int channels, int nn, bool doSeek, bool overwrite)
+      {
+        //ev->read(offset, bpp, channels, nn, overwrite);
+        //ev->readAudio(offset, bpp, channels, nn, doSeek, overwrite);
+        //_sfCurFrame = ev->readAudio(_src_state, _sfCurFrame, offset, bpp, channels, nn, doSeek, overwrite);
+        _sfCurFrame = ev->readAudio(_audConv, _sfCurFrame, offset, bpp, channels, nn, doSeek, overwrite);
+      }
 void Event::setTick(unsigned val)       { ev->setTick(val); }
 unsigned Event::tick() const            { return ev->tick(); }
 unsigned Event::frame() const           { return ev->frame(); }
