@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//  $Id: gconfig.cpp,v 1.15.2.3 2005/08/14 21:47:31 spamatica Exp $
+//  $Id: gconfig.cpp,v 1.15.2.13 2009/12/01 03:52:40 terminator356 Exp $
 //
 //  (C) Copyright 1999-2003 Werner Schweer (ws@seh.de)
 //=========================================================
@@ -25,7 +25,7 @@ GlobalConfigValues config = {
         QColor(0xff, 0xff, 0xff),
         QColor(0xff, 0xff, 0xff),
         QColor(0xff, 0xff, 0xff),
-        QColor(0xff, 0xff, 0xff),
+        QColor(0xff, 0xff, 0xff)
         },
       {
         QColor(255,  232,  140),   // part colors
@@ -57,18 +57,21 @@ GlobalConfigValues config = {
         QFont(QString("arial"), 10, QFont::Bold),
         QFont(QString("arial"), 8,  QFont::Bold),    // timescale numbers
         QFont(QString("Lucidatypewriter"), 14,  QFont::Bold),
+        QFont(QString("arial"), 8,  QFont::Bold, true)  // Mixer strip labels. Looks and fits better with bold + italic than bold alone, 
+                                                        //  at the price of only few more pixels than Normal mode.
         },
-      QColor(0xff, 0xff, 0xff),     // trackBg;
+      QColor(84, 97, 114),     // trackBg;
       QColor(0x80, 0xff, 0x80),     // selected track Bg;
       QColor(0x00, 0x00, 0x00),     // selected track Fg;
-      QColor(0xff, 0xff, 0xff),     // midiTrackBg;
-      QColor(0xff, 0xff, 0xff),     // drumTrackBg;
-      QColor(0xff, 0xff, 0xff),     // waveTrackBg;
-      QColor(0xff, 0xff, 0xff),     // outputTrackBg;
-      QColor(0xff, 0xff, 0xff),     // inputTrackBg;
-      QColor(0xff, 0xff, 0xff),     // groupTrackBg;
-      QColor(0xff, 0xff, 0xff),     // auxTrackBg;
-      QColor(0xff, 0xff, 0xff),     // synthTrackBg;
+      QColor(220, 220, 220),     // midiTrackBg;
+      QColor(0x00, 0x00, 0xff),     // ctrlGraphFg;
+      QColor(220, 220, 220),     // drumTrackBg;
+      QColor(220, 220, 220),     // waveTrackBg;
+      QColor(189, 220, 193),     // outputTrackBg;
+      QColor(189, 220, 193),     // inputTrackBg;
+      QColor(220, 220, 220),     // groupTrackBg;
+      QColor(220, 220, 220),     // auxTrackBg;
+      QColor(220, 220, 220),     // synthTrackBg;
       QColor(98, 124, 168),         // part canvas bg
       QColor(0, 0, 0),              // mixerBg;
 
@@ -76,12 +79,15 @@ GlobalConfigValues config = {
       1024,                         // rtcTicks
       -60,                          // int minMeter;
       -60.0,                        // double minSlider;
+      false,                        // use Jack freewheel
       20,                           // int guiRefresh;
       QString(""),                  // helpBrowser
       true,                         // extendedMidi
       384,                          // division for smf export
       QString(""),                  // copyright string for smf export
       1,                            // smf export file format
+      false,                        // midi export file 2 byte timesigs instead of 4
+      true,                         // optimize midi export file note offs
       1,                            // startMode
       QString(""),                  // start song path
       384,                          // gui division
@@ -102,5 +108,10 @@ GlobalConfigValues config = {
       QString(""),                  // canvasBgPixmap;
       QString(""),                  // style
       QString("sweep"),             // externalWavEditor
-      };
+      false,                        // useOldStyleStopShortCut
+      true,                         // useDenormalBias
+      false,                        // useOutputLimiter
+      true,                         // showDidYouKnow
+      false                         // vstInPlace  Enable VST in-place processing
+    };
 
