@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//  $Id: gconfig.h,v 1.12.2.2 2005/08/14 21:47:31 spamatica Exp $
+//  $Id: gconfig.h,v 1.12.2.10 2009/12/01 03:52:40 terminator356 Exp $
 //
 //  (C) Copyright 1999-2003 Werner Schweer (ws@seh.de)
 //=========================================================
@@ -15,6 +15,7 @@
 #include <qrect.h>
 
 #define NUM_PARTCOLORS 17
+#define NUM_FONTS 7
 
 //---------------------------------------------------------
 //   GlobalConfigValues
@@ -27,11 +28,13 @@ struct GlobalConfigValues {
 	QColor bigTimeForegroundColor;
 	QColor bigTimeBackgroundColor;
       QColor waveEditBackgroundColor;
-      QFont fonts[6];
+      //QFont fonts[6];
+      QFont fonts[NUM_FONTS];
       QColor trackBg;
       QColor selectTrackBg;
       QColor selectTrackFg;
       QColor midiTrackBg;
+      QColor ctrlGraphFg;
       QColor drumTrackBg;
       QColor waveTrackBg;
       QColor outputTrackBg;
@@ -46,6 +49,7 @@ struct GlobalConfigValues {
       int rtcTicks;
       int minMeter;
       double minSlider;
+      bool freewheelMode;
       int guiRefresh;
       QString helpBrowser;
 
@@ -53,7 +57,9 @@ struct GlobalConfigValues {
       int midiDivision;       // division for smf export
       QString copyright;      // copyright string for smf export
       int smfFormat;          // smf export file type
-
+      bool exp2ByteTimeSigs;  // Export 2 byte time sigs instead of 4 bytes
+      bool expOptimNoteOffs;  // Save space by replacing note offs with note on velocity 0
+      
       int startMode;          // 0 - start with last song
                               // 1 - start with default template
                               // 2 - start with song
@@ -79,6 +85,11 @@ struct GlobalConfigValues {
       QString style;
 
       QString externalWavEditor;
+      bool useOldStyleStopShortCut;
+      bool useDenormalBias;
+      bool useOutputLimiter;
+      bool showDidYouKnow;
+      bool vstInPlace; // Enable VST in-place processing
       };
 
 extern GlobalConfigValues config;
