@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//  $Id: globals.cpp,v 1.15.2.6 2006/02/11 22:21:21 spamatica Exp $
+//  $Id: globals.cpp,v 1.15.2.11 2009/11/25 09:09:43 terminator356 Exp $
 //
 //  (C) Copyright 1999-2004 Werner Schweer (ws@seh.de)
 //=========================================================
@@ -21,6 +21,11 @@ unsigned fifoLength =  128;       // 131072/segmentSize
                                   // 131072 - magic number that gives a sufficient buffer size
 int segmentCount = 2;
 
+// denormal bias value used to eliminate the manifestation of denormals by
+// lifting the zero level slightly above zero
+// denormal problems occur when values get extremely close to zero
+const float denormalBias=1e-18;
+
 bool overrideAudioOutput = false;
 bool overrideAudioInput = false;
 
@@ -68,6 +73,8 @@ QString museUser;
 QString museProject;
 QString museProjectInitPath("./");
 QString configName;
+QString museInstruments;
+QString museUserInstruments;
 
 QString lastWavePath(".");
 QString lastMidiPath(".");
@@ -77,9 +84,13 @@ bool debugMsg = false;
 bool midiInputTrace = false;
 bool midiOutputTrace = false;
 bool realTimeScheduling = false;
-int realTimePriority = 80;
+int realTimePriority = 40;  // 80
+int midiRTPrioOverride = -1;
 bool loadPlugins = true;
 bool loadVST = true;
+bool loadDSSI = true;
+bool usePythonBridge = false;
+bool useLASH = true;
 
 const char* midi_file_pattern[] = {
       "Midi/Kar (*.mid *.MID *.kar *.KAR *.mid.gz *.mid.bz2)",
