@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//  $Id: globals.h,v 1.10.2.6 2006/02/11 22:21:21 spamatica Exp $
+//  $Id: globals.h,v 1.10.2.11 2009/11/25 09:09:43 terminator356 Exp $
 //
 //  (C) Copyright 1999/2000 Werner Schweer (ws@seh.de)
 //=========================================================
@@ -19,6 +19,8 @@
 
 #include <unistd.h>
 
+extern const float denormalBias;
+
 extern int recFileNumber;
 
 extern int sampleRate;
@@ -43,6 +45,8 @@ extern QString museUser;
 extern QString museProject;
 extern QString museProjectInitPath;
 extern QString configName;
+extern QString museInstruments;
+extern QString museUserInstruments;
 
 extern QString lastWavePath;
 extern QString lastMidiPath;
@@ -54,9 +58,14 @@ extern bool debugMsg;
 extern bool debugSync;
 extern bool loadPlugins;
 extern bool loadVST;
+extern bool loadDSSI;
+extern bool usePythonBridge;
+extern bool useLASH;
 
 extern bool realTimeScheduling;
 extern int realTimePriority;
+extern int midiRTPrioOverride;
+
 extern const char* midi_file_pattern[];  //!< File name pattern for midi files
 extern const char* med_file_pattern[];   //!< File name pattern for muse project files
 extern const char* image_file_pattern[]; //!< File name pattern for image files (gfx)
