@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//  $Id: icons.cpp,v 1.13.2.1 2005/03/29 16:37:34 qknight Exp $
+//  $Id: icons.cpp,v 1.13.2.8 2009/11/14 03:37:48 terminator356 Exp $
 //
 //  (C) Copyright 1999/2000 Werner Schweer (ws@seh.de)
 //=========================================================
@@ -46,6 +46,7 @@
 #include "xpm/delete.xpm"
 #include "xpm/play.xpm"
 
+#include "xpm/record1.xpm"
 #include "xpm/record.xpm"
 #include "xpm/record_on.xpm"
 #include "xpm/record_off.xpm"
@@ -66,6 +67,7 @@
 #include "xpm/note1.xpm"
 #include "xpm/stick.xpm"
 #include "xpm/wave.xpm"
+#include "xpm/synth.xpm"
 #include "xpm/cmark.xpm"
 #include "xpm/lmark.xpm"
 #include "xpm/rmark.xpm"
@@ -90,8 +92,12 @@
 // #include "xpm/mute.xpm"
 #include "xpm/solobutton_on.xpm"
 #include "xpm/solobutton_off.xpm"
+#include "xpm/solobutton_on_blksq.xpm"
+#include "xpm/solobutton_off_blksq.xpm"
 #include "xpm/mutebutton_on.xpm"
 #include "xpm/mutebutton_off.xpm"
+#include "xpm/rec_echo_on.xpm"
+#include "xpm/rec_echo_off.xpm"
 
 #include "xpm/up.xpm"
 #include "xpm/down.xpm"
@@ -126,6 +132,12 @@
 #include "xpm/editpasteS.xpm"
 #include "xpm/editmute.xpm"
 #include "xpm/editmuteS.xpm"
+#include "xpm/editpasteclone.xpm"
+#include "xpm/editpastecloneS.xpm"
+#include "xpm/editpaste2track.xpm"
+#include "xpm/editpaste2trackS.xpm"
+#include "xpm/editpasteclone2track.xpm"
+#include "xpm/editpasteclone2trackS.xpm"
 
 #include "xpm/speaker.xpm"
 #include "xpm/buttondown.xpm"
@@ -139,11 +151,15 @@
 #include "xpm/exit.xpm"
 #include "xpm/exit1.xpm"
 
+#include "xpm/redled.xpm"
+#include "xpm/darkredled.xpm"
 #include "xpm/greendot.xpm"
-#include "xpm/darkgreendot.xpm"
+//#include "xpm/darkgreendot.xpm"
 #include "xpm/bluedot.xpm"
 #include "xpm/graydot.xpm"
 #include "xpm/off.xpm"
+#include "xpm/blacksquare.xpm"
+#include "xpm/blacksqcheck.xpm"
 
 #include "xpm/mastertrackS.xpm"
 #include "xpm/localoffS.xpm"
@@ -170,6 +186,7 @@
 #include "xpm/addtrack_wavetrack.xpm"
 #include "xpm/edit_drumms.xpm"
 #include "xpm/edit_list.xpm"
+#include "xpm/edit_wave.xpm"
 #include "xpm/edit_mastertrack.xpm"
 #include "xpm/edit_pianoroll.xpm"
 #include "xpm/edit_score.xpm"
@@ -224,6 +241,7 @@ QPixmap* loopIcon;
 QPixmap* loop1Icon;
 QPixmap* playIcon;
 
+QPixmap* record1_Icon;
 QPixmap* record_on_Icon;
 QPixmap* record_off_Icon;
 QPixmap* recordIcon;
@@ -238,6 +256,7 @@ QPixmap* note1Icon;
 QPixmap* noteIcon;
 QPixmap* stickIcon;
 QPixmap* waveIcon;
+QPixmap* synthIcon;
 QPixmap* markIcon[3];
 QPixmap* cutIcon;
 QPixmap* steprecIcon;
@@ -285,10 +304,16 @@ QPixmap* speakerIcon;
 QPixmap* buttondownIcon;
 QPixmap* configureIcon;
 
+QPixmap* recEchoIconOn;
+QPixmap* recEchoIconOff;
 QPixmap* muteIconOn;
 QPixmap* muteIconOff;
 QPixmap* soloIconOn;
 QPixmap* soloIconOff;
+QPixmap* soloblksqIconOn;
+QPixmap* soloblksqIconOff;
+QIconSet* soloIconSet1;
+QIconSet* soloIconSet2;
 
 QPixmap* editmuteIcon;
 QPixmap* editmuteSIcon;
@@ -300,12 +325,19 @@ QIconSet* editcutIconSet;
 QIconSet* editmuteIconSet;
 QIconSet* editcopyIconSet;
 QIconSet* editpasteIconSet;
+QIconSet* editpaste2TrackIconSet;
+QIconSet* editpasteCloneIconSet;
+QIconSet* editpasteClone2TrackIconSet;
 
+QPixmap* redLedIcon;
+QPixmap* darkRedLedIcon;
 QPixmap* greendotIcon;
-QPixmap* darkgreendotIcon;
+//QPixmap* darkgreendotIcon;
 QPixmap* graydotIcon;
 QPixmap* bluedotIcon;
 QPixmap* offIcon;
+QPixmap* blacksquareIcon;
+QPixmap* blacksqcheckIcon;
 
 QPixmap* addtrack_addmiditrackIcon;
 QPixmap* addtrack_audiogroupIcon;
@@ -316,6 +348,7 @@ QPixmap* addtrack_drumtrackIcon;
 QPixmap* addtrack_wavetrackIcon;
 QPixmap* edit_drummsIcon;
 QPixmap* edit_listIcon;
+QPixmap* edit_waveIcon;
 QPixmap* edit_mastertrackIcon;
 QPixmap* edit_pianorollIcon;
 QPixmap* edit_scoreIcon;
@@ -386,8 +419,9 @@ void initIcons()
       loop1Icon    = new QPixmap(loop1_xpm);
       playIcon     = new QPixmap(play_xpm);
 
-      record_on_Icon= new QPixmap(record_on_xpm);
-      record_off_Icon= new QPixmap(record_off_xpm);
+      record1_Icon  = new QPixmap(record1_xpm);
+      record_on_Icon = new QPixmap(record_on_xpm);
+      record_off_Icon = new QPixmap(record_off_xpm);
       recordIcon   = new QPixmap(record_xpm);
       stopIcon     = new QPixmap(stop_xpm);
       startIcon    = new QPixmap(start_xpm);
@@ -400,6 +434,7 @@ void initIcons()
       note1Icon    = new QPixmap(note1_xpm);
       stickIcon    = new QPixmap(stick_xpm);
       waveIcon     = new QPixmap(wave_xpm);
+      synthIcon    = new QPixmap(synth_xpm);
       markIcon[0]  = new QPixmap(cmark_xpm);
       markIcon[1]  = new QPixmap(lmark_xpm);
       markIcon[2]  = new QPixmap(rmark_xpm);
@@ -457,6 +492,9 @@ void initIcons()
       editcopyIconSet      = new QIconSet(QPixmap(editcopyS_xpm), QPixmap(editcopy_xpm));
       editpasteIconSet     = new QIconSet(QPixmap(editpasteS_xpm), QPixmap(editpaste_xpm));
       editmuteIconSet      = new QIconSet(QPixmap(editmuteS_xpm), QPixmap(editmute_xpm));
+      editpaste2TrackIconSet = new QIconSet(QPixmap(editpaste2trackS_xpm), QPixmap(editpaste2track_xpm));
+      editpasteCloneIconSet  = new QIconSet(QPixmap(editpastecloneS_xpm), QPixmap(editpasteclone_xpm));
+      editpasteClone2TrackIconSet = new QIconSet(QPixmap(editpasteclone2trackS_xpm), QPixmap(editpasteclone2track_xpm));
 
       exitIcon             = new QPixmap(exit_xpm);
       exit1Icon            = new QPixmap(exit1_xpm);
@@ -465,15 +503,30 @@ void initIcons()
       newmuteIcon          = new QPixmap(newmutebutton_xpm);
       soloIcon             = new QPixmap(solobutton_xpm);
 
+      recEchoIconOn        = new QPixmap(rec_echo_on_xpm);
+      recEchoIconOff       = new QPixmap(rec_echo_off_xpm);
       muteIconOn           = new QPixmap(mutebutton_on_xpm);
       muteIconOff          = new QPixmap(mutebutton_off_xpm);
       soloIconOn           = new QPixmap(solobutton_on_xpm);
       soloIconOff          = new QPixmap(solobutton_off_xpm);
+      soloblksqIconOn      = new QPixmap(solobutton_on_blksq_xpm);
+      soloblksqIconOff     = new QPixmap(solobutton_off_blksq_xpm);
+      soloIconSet1         = new QIconSet(); 
+      soloIconSet2         = new QIconSet();
+      soloIconSet1->setPixmap(*soloIconOn, QIconSet::Automatic, QIconSet::Normal, QIconSet::On);
+      soloIconSet1->setPixmap(*soloIconOff, QIconSet::Automatic, QIconSet::Normal, QIconSet::Off);
+      soloIconSet2->setPixmap(*soloblksqIconOn, QIconSet::Automatic, QIconSet::Normal, QIconSet::On);
+      soloIconSet2->setPixmap(*soloblksqIconOff, QIconSet::Automatic, QIconSet::Normal, QIconSet::Off);
+      
+      redLedIcon           = new QPixmap(redled_xpm);
+      darkRedLedIcon       = new QPixmap(darkredled_xpm);
       greendotIcon         = new QPixmap(greendot_xpm);
-      darkgreendotIcon     = new QPixmap(darkgreendot_xpm);
+      //darkgreendotIcon     = new QPixmap(darkgreendot_xpm);
       bluedotIcon          = new QPixmap(bluedot_xpm);
       graydotIcon          = new QPixmap(graydot_xpm);
       offIcon              = new QPixmap(off_xpm);
+      blacksquareIcon      = new QPixmap(blacksquare_xpm);
+      blacksqcheckIcon     = new QPixmap(blacksqcheck_xpm);
 
       mastertrackSIcon     = new QPixmap(mastertrackS_xpm);
       localoffSIcon        = new QPixmap(localoffS_xpm);
@@ -500,6 +553,7 @@ void initIcons()
       addtrack_wavetrackIcon        = new QPixmap(addtrack_wavetrack_xpm);
       edit_drummsIcon               = new QPixmap(edit_drumms_xpm);
       edit_listIcon                 = new QPixmap(edit_list_xpm);
+      edit_waveIcon                 = new QPixmap(edit_wave_xpm);
       edit_mastertrackIcon          = new QPixmap(edit_mastertrack_xpm);
       edit_pianorollIcon            = new QPixmap(edit_pianoroll_xpm);
       edit_scoreIcon                = new QPixmap(edit_score_xpm);
