@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//  $Id: importmidi.cpp,v 1.26.2.3 2005/06/13 18:30:48 lunar_shuttle Exp $
+//  $Id: importmidi.cpp,v 1.26.2.10 2009/11/05 03:14:35 terminator356 Exp $
 //
 //  (C) Copyright 1999-2003 Werner Schweer (ws@seh.de)
 //=========================================================
@@ -150,7 +150,10 @@ bool MusE::importMidi(const QString name, bool merge)
                               continue;
                         MidiTrack* track = new MidiTrack();
                         if ((*t)->isDrumTrack)
+                        {
                               track->setType(Track::DRUM);
+                        }
+                              
                         track->setOutChannel(channel);
                         track->setOutPort(port);
 
@@ -159,17 +162,23 @@ bool MusE::importMidi(const QString name, bool merge)
                         mport->setInstrument(instr);
 
                         EventList* mel = track->events();
-                        buildMidiEventList(mel, el, track, division, first);
+                        //buildMidiEventList(mel, el, track, division, first);
+                        // Don't do loops.
+                        buildMidiEventList(mel, el, track, division, first, false);
                         first = false;
 
-                        for (iEvent i = mel->begin(); i != mel->end(); ++i) {
-                              Event event = i->second;
-                              if (event.type() == Controller) {
-                                    importController(channel, mport, event.dataA());
-                                    midiPorts[track->outPort()].setCtrl(channel, event.tick(), event.dataA(), event.dataB());
-                                    }
-                              }
+                        // Removed by T356. Handled by addPortCtrlEvents() below.
+                        //for (iEvent i = mel->begin(); i != mel->end(); ++i) {
+                        //      Event event = i->second;
+                        //      if (event.type() == Controller) {
+                        //            importController(channel, mport, event.dataA());
+                        //            midiPorts[track->outPort()].setCtrl(channel, event.tick(), event.dataA(), event.dataB());
+                        //            }
+                        //      }
 
+                        // Comment Added by T356.
+                        // Hmm. buildMidiEventList already takes care of this. 
+                        // But it seems to work. How? Must test. 
                         if (channel == 9 && song->mtype() != MT_UNKNOWN) {
                               track->setType(Track::DRUM);
                               //
@@ -182,9 +191,23 @@ bool MusE::importMidi(const QString name, bool merge)
                                           int pitch = drumInmap[ev.pitch()];
                                           ev.setPitch(pitch);
                                           }
+                                    else
+                                    if(ev.type() == Controller)
+                                    {
+                                      int ctl = ev.dataA();
+                                      MidiController *mc = mport->drumController(ctl);
+                                      if(mc)
+                                        ev.setA((ctl & ~0xff) | drumInmap[ctl & 0x7f]);
                                     }
+                                  }
                               }
+                              
                         processTrack(track);
+                        
+                        // Added by T356. Send all controller values to the port's controller value list.
+                        // No, done in song->insertTrack2() now.
+                        //track->addPortCtrlEvents();
+                        
                         song->insertTrack0(track, -1);
                         }
                   }
@@ -197,7 +220,9 @@ bool MusE::importMidi(const QString name, bool merge)
                   track->setOutChannel(0);
                   track->setOutPort(0);
                   EventList* mel = track->events();
-                  buildMidiEventList(mel, el, track, division, true);
+                  //buildMidiEventList(mel, el, track, division, true);
+                  // Do SysexMeta. Don't do loops.
+                  buildMidiEventList(mel, el, track, division, true, false);
                   processTrack(track);
                   song->insertTrack0(track, -1);
                   }
@@ -248,8 +273,8 @@ void MusE::processTrack(MidiTrack* track)
       //---------------------------------------------------
       //    Parts ermitteln
       //    die Midi-Spuren werden in Parts aufgebrochen;
-      //    ein neuer Part wird bei einer Lcke von einem
-      //    Takt gebildet; die Lnge wird jeweils auf
+      //    ein neuer Part wird bei einer LÃ¯Â¿Â½cke von einem
+      //    Takt gebildet; die LÃ¯Â¿Â½nge wird jeweils auf
       //    Takte aufgerundet und aligned
       //---------------------------------------------------
 
@@ -336,13 +361,13 @@ void MusE::processTrack(MidiTrack* track)
                   Event ev = i->second;
                   int ntick = ev.tick() - startTick;
                   ev.setTick(ntick);
-                  el->add(ev, ntick);
+                  el->add(ev);
                   }
             tevents->erase(r1, r2);
             }
 
       if (tevents->size())
-            printf("-----------events left: %d\n", tevents->size());
+            printf("-----------events left: %zd\n", tevents->size());
       for (iEvent i = tevents->begin(); i != tevents->end(); ++i) {
             printf("%d===\n", i->first);
             i->second.dump();
@@ -366,15 +391,16 @@ void MusE::importController(int channel, MidiPort* mport, int n)
       MidiControllerList* mcl = instr->controller();
 // printf("import Ctrl\n");
       for (iMidiController i = mcl->begin(); i != mcl->end(); ++i) {
-            int cn = (*i)->num();
+            MidiController* mc = i->second;
+            int cn = mc->num();
 // printf("   %x %x\n", n, cn);
             if (cn == n) {
-                  ctrl = *i;
+                  ctrl = mc;
                   break;
                   }
             // wildcard?
             if (((cn & 0xff) == 0xff) && ((cn & ~0xff) == (n & ~0xff))) {
-                  ctrl = *i;
+                  ctrl = i->second;
                   break;
                   }
             }
@@ -401,12 +427,15 @@ void MusE::importPart()
       for (iTrack i = tracks->begin(); i != tracks->end(); i++) {
             Track* t = *i;
             if (t->selected()) {
-                  if (t->isMidiTrack()) {
+                  // Changed by T356. Support mixed .mpt files.
+                  //if (t->isMidiTrack()) {
+                  if (t->isMidiTrack() || t->type() == Track::WAVE) {
                         track = t;
                         break;
                         }
                   else {
-                        QMessageBox::warning(this, QString("MusE"), tr("Import part is only valid for midi tracks!"));
+                        //QMessageBox::warning(this, QString("MusE"), tr("Import part is only valid for midi tracks!"));
+                        QMessageBox::warning(this, QString("MusE"), tr("Import part is only valid for midi and wave tracks!"));
                         return;
                         }
                   }
@@ -415,8 +444,21 @@ void MusE::importPart()
       if (track) {
             bool loadAll;
             QString filename = getOpenFileName(QString(""), part_file_pattern, this, tr("MusE: load part"), &loadAll);
-            if (!filename.isEmpty())
+            if (!filename.isEmpty()){
+                  // Make a backup of the current clone list, to retain any 'copy' items,
+                  //  so that pasting works properly after.
+                  CloneList copyCloneList = cloneList;
+                  // Clear the clone list to prevent any dangerous associations with
+                  //  current non-original parts.
+                  cloneList.clear();
+            
                   importPartToTrack(filename, curPos, track);
+                  
+                  // Restore backup of the clone list, to retain any 'copy' items,
+                  //  so that pasting works properly after.
+                  cloneList.clear();
+                  cloneList = copyCloneList;
+               }   
             }
       else {
             QMessageBox::warning(this, QString("MusE"), tr("No track selected for import"));
@@ -427,24 +469,124 @@ void MusE::importPart()
 //   importPartToTrack
 //---------------------------------------------------------
 void MusE::importPartToTrack(QString& filename, unsigned tick, Track* track)
-      {
+{
+      // Changed by T356
+      /*
       bool popenFlag = false;
       FILE* fp = fileOpen(this, filename, ".mpt", "r", popenFlag, false, false);
 
-      if (fp) {
-            MidiPart* importedPart = new MidiPart((MidiTrack*)track);
-            Xml tmpXml = Xml(fp);
+      if(fp) 
+      {
+        MidiPart* importedPart = new MidiPart((MidiTrack*)track);
+        Xml tmpXml = Xml(fp);
 
-            Xml::Token token = tmpXml.parse();
-            const QString& tag = tmpXml.s1();
-            if (token == Xml::TagStart && tag == "part") {
-                  importedPart->read(tmpXml);
-                  importedPart->setTick(tick);
-                  audio->msgAddPart(importedPart);
-                  }
-            else {
-                  printf("Unknown tag: %s\n", tag.latin1());
-                  }
-            fclose(fp);
-            }
+        Xml::Token token = tmpXml.parse();
+        const QString& tag = tmpXml.s1();
+        if (token == Xml::TagStart && tag == "part") 
+        {
+          // Make a backup of the current clone list, to retain any 'copy' items,
+          //  so that pasting works properly after.
+          CloneList copyCloneList = cloneList;
+          // Clear the clone list to prevent any dangerous associations with
+          //  current non-original parts.
+          cloneList.clear();
+
+          importedPart->read(tmpXml);
+          importedPart->setTick(tick);
+          
+          // Restore backup of the clone list, to retain any 'copy' items,
+          //  so that pasting works properly after.
+          cloneList.clear();
+          cloneList = copyCloneList;
+          
+          audio->msgAddPart(importedPart);
+        }
+        else 
+        {
+          printf("Unknown tag: %s\n", tag.latin1());
+        }
+        fclose(fp);
+      }      
+      return;
+      */            
+        
+      
+      bool popenFlag = false;
+      FILE* fp = fileOpen(this, filename, ".mpt", "r", popenFlag, false, false);
+
+      if(fp) 
+      {
+        Xml xml = Xml(fp);
+        bool firstPart = true;
+        int posOffset = 0;
+        int  notDone = 0;
+        int  done = 0;
+        
+        bool end = false;
+        song->startUndo();
+        for (;;) 
+        {
+          Xml::Token token = xml.parse();
+          const QString& tag = xml.s1();
+          switch (token) 
+          {
+            case Xml::Error:
+            case Xml::End:
+                  end = true;
+                  break;
+            case Xml::TagStart:
+                  if (tag == "part") {
+                        //MidiPart* p = new MidiPart((MidiTrack*)track);
+                        //p->read(xml);
+                        
+                        // Read the part.
+                        Part* p = 0;
+                        p = readXmlPart(xml, track);
+                        // If it could not be created...
+                        if(!p)
+                        {
+                          // Increment the number of parts not done and break.
+                          ++notDone;
+                          break;
+                        } 
+                        
+                        // Increment the number of parts done.
+                        ++done;
+                        
+                        if(firstPart) 
+                        {
+                          firstPart=false;
+                          posOffset = tick - p->tick();
+                        }
+                        p->setTick(p->tick() + posOffset);
+                        //finalPos=p->tick() + p->lenTick();
+                        ////pos += p->lenTick();
+                        audio->msgAddPart(p,false);
+                        }
+                  else
+                        xml.unknown("MusE::importPartToTrack");
+                  break;
+            case Xml::TagEnd:
+                  break;
+            default:
+                  end = true;
+                  break;
+          }
+          if(end)
+            break;
+        }
+        fclose(fp);
+        song->endUndo(SC_PART_INSERTED);
+        
+        if(notDone)
+        {
+          int tot = notDone + done;
+          QMessageBox::critical(this, QString("MusE"),
+            QString().setNum(notDone) + (tot > 1 ? (tr(" out of ") + QString().setNum(tot)) : QString("")) + 
+            (tot > 1 ? tr(" parts") : tr(" part")) + 
+            tr(" could not be imported.\nLikely the track is the wrong type."));
+        }
+        
+        return;
       }
+}
