@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//    $Id: listedit.cpp,v 1.11.2.5 2006/01/01 20:51:55 spamatica Exp $
+//    $Id: listedit.cpp,v 1.11.2.11 2009/05/24 21:43:44 terminator356 Exp $
 //  (C) Copyright 1999 Werner Schweer (ws@seh.de)
 //=========================================================
 
@@ -53,11 +53,43 @@ class EventListItem : public QListViewItem {
             part  = p;
             }
       virtual QString text(int col) const;
-      virtual int compare(QListViewItem*i, int, bool) const {
-            return event.tick() - ((EventListItem*)i)->event.tick();
+      virtual int compare(QListViewItem* i, int col, bool ascend) const 
+      {
+          EventListItem* eli = (EventListItem*)i;
+          switch(col)
+          {
+            case 0:
+                  return event.tick() - eli->event.tick();
+                  break;
+            case 1:
+                  return part->tick() + event.tick() - (eli->part->tick() + eli->event.tick());
+                  break;
+            case 2:
+                  return key(col, ascend).localeAwareCompare(i->key(col, ascend));
+                  break;   
+            case 3:
+                  return part->track()->outChannel() - eli->part->track()->outChannel();
+                  break;
+            case 4:
+                  return event.dataA() - eli->event.dataA();
+                  break;
+            case 5:
+                  return event.dataB() - eli->event.dataB();
+                  break;
+            case 6:
+                  return event.dataC() - eli->event.dataC();
+                  break;
+            case 7:
+                  return event.lenTick() - eli->event.lenTick();
+                  break;
+            case 8:
+                  return key(col, ascend).localeAwareCompare(i->key(col, ascend));
+                  break;
+            default:
+                  return 0;
             }
+          }
       };
-
 /*---------------------------------------------------------
  *    midi_meta_name
  *---------------------------------------------------------*/
@@ -129,7 +161,7 @@ static QString midiMetaComment(const Event& ev)
                   }
             }
       }
-
+      
 //---------------------------------------------------------
 //   closeEvent
 //---------------------------------------------------------
@@ -187,6 +219,11 @@ void ListEdit::songChanged(int type)
                         for (iEvent i = el->begin(); i != el->end(); ++i) {
                               EventListItem* item = new EventListItem(liste, i->second, part);
                               item->setSelected(i->second.selected());
+                              if (item->event.tick() == (unsigned) selectedTick) { //prevent compiler warning: comparison of signed/unsigned)
+                                    liste->setCurrentItem(item);
+                                    item->setSelected(true);
+                                    liste->ensureItemVisible(item);
+                                    }
                               }
                         }
                   }
@@ -197,7 +234,7 @@ void ListEdit::songChanged(int type)
       }
 
 //---------------------------------------------------------
-//   addEvent
+//   text
 //---------------------------------------------------------
 
 QString EventListItem::text(int col) const
@@ -224,7 +261,7 @@ QString EventListItem::text(int col) const
                               break;
                         case Controller:
                               {
-                              char* cs;
+                              const char* cs;
                               switch (midiControllerType(event.dataA())) {
                                     case MidiController::Controller7:  cs = "Ctrl7"; break;
                                     case MidiController::Controller14: cs = "Ctrl14"; break;
@@ -282,7 +319,7 @@ QString EventListItem::text(int col) const
                   s.setNum(part->track()->outChannel() + 1);
                   break;
             case 4:
-                  if (event.isNote())
+                  if (event.isNote() || event.type() == PAfter)
                         s = pitch2string(event.dataA());
                   else if (event.type() == Controller)
                         s.setNum(event.dataA() & 0xffff);  // mask off type bits
@@ -290,7 +327,23 @@ QString EventListItem::text(int col) const
                         s.setNum(event.dataA());
                   break;
             case 5:
-                  s.setNum(event.dataB());
+                  if(event.type() == Controller &&
+                     midiControllerType(event.dataA()) == MidiController::Program) 
+                  {
+                    int val = event.dataB();
+                    int hb = ((val >> 16) & 0xff) + 1;
+                    if (hb == 0x100)
+                      hb = 0;
+                    int lb = ((val >> 8) & 0xff) + 1;
+                    if (lb == 0x100)
+                      lb = 0;
+                    int pr = (val & 0xff) + 1;
+                    if (pr == 0x100)
+                      pr = 0;
+                    s.sprintf("%d-%d-%d", hb, lb, pr);
+                  }
+                  else        
+                    s.setNum(event.dataB());
                   break;
             case 6:
                   s.setNum(event.dataC());
@@ -415,23 +468,23 @@ ListEdit::ListEdit(PartList* pl)
       int n = fm.width('9');
       int b = 24;
       int c = fm.width(QString("Val B"));
+      int sortIndW = n * 3;
       liste->setAllColumnsShowFocus(true);
       liste->setSorting(0);
       liste->setSelectionMode(QListView::Extended);
-
+      liste->setShowSortIndicator(true);
       liste->addColumn(tr("Tick"),  n * 6 + b);
       liste->addColumn(tr("Bar"),   fm.width(QString("9999.99.999")) + b);
-      liste->addColumn(tr("Typ"),   fm.width(QString("SysEx")) + b);
-      liste->addColumn(tr("Ch"),    n * 2 + b);
-      liste->addColumn(tr("Val A"), c + b);
-      liste->addColumn(tr("Val B"), c + b);
-      liste->addColumn(tr("Val C"), c + b);
-      liste->addColumn(tr("Len"),   n * 4 + b);
+      liste->addColumn(tr("Type"),  fm.width(QString("Program")) + b);
+      liste->addColumn(tr("Ch"),    n * 2 + b + sortIndW);
+      liste->addColumn(tr("Val A"), c + b + sortIndW);
+      liste->addColumn(tr("Val B"), c + b + sortIndW);
+      liste->addColumn(tr("Val C"), c + b + sortIndW);
+      liste->addColumn(tr("Len"),   n * 4 + b + sortIndW);
       liste->addColumn(tr("Comment"), fm.width(QString("MainVolume")) + 70);
       liste->setResizeMode(QListView::LastColumn);
       connect(liste, SIGNAL(selectionChanged()), SLOT(selectionChanged()));
       connect(liste, SIGNAL(doubleClicked(QListViewItem*)), SLOT(doubleClicked(QListViewItem*)));
-
       //---------------------------------------------------
       //    Rest
       //---------------------------------------------------
@@ -471,7 +524,9 @@ void ListEdit::editInsertNote()
             else
                   tick-= curPart->tick();
             event.setTick(tick);
-            audio->msgAddEvent(event, curPart);
+            // Indicate do undo, and do not handle port controller values. 
+            //audio->msgAddEvent(event, curPart);
+            audio->msgAddEvent(event, curPart, true, false, false);
             }
       }
 
@@ -490,7 +545,9 @@ void ListEdit::editInsertSysEx()
             else
                   tick-= curPart->tick();
             event.setTick(tick);
-            audio->msgAddEvent(event, curPart);
+            // Indicate do undo, and do not handle port controller values. 
+            //audio->msgAddEvent(event, curPart);
+            audio->msgAddEvent(event, curPart, true, false, false);
             }
       }
 
@@ -509,7 +566,9 @@ void ListEdit::editInsertCtrl()
             else
                   tick-= curPart->tick();
             event.setTick(tick);
-            audio->msgAddEvent(event, curPart);
+            // Indicate do undo, and do port controller values and clone parts. 
+            //audio->msgAddEvent(event, curPart);
+            audio->msgAddEvent(event, curPart, true, true, true);
             }
       }
 
@@ -528,7 +587,9 @@ void ListEdit::editInsertMeta()
             else
                   tick-= curPart->tick();
             event.setTick(tick);
-            audio->msgAddEvent(event, curPart);
+            // Indicate do undo, and do not handle port controller values. 
+            //audio->msgAddEvent(event, curPart);
+            audio->msgAddEvent(event, curPart, true, false, false);
             }
       }
 
@@ -547,7 +608,9 @@ void ListEdit::editInsertCAfter()
             else
                   tick-= curPart->tick();
             event.setTick(tick);
-            audio->msgAddEvent(event, curPart);
+            // Indicate do undo, and do not handle port controller values. 
+            //audio->msgAddEvent(event, curPart);
+            audio->msgAddEvent(event, curPart, true, false, false);
             }
       }
 
@@ -567,7 +630,9 @@ void ListEdit::editInsertPAfter()
             else
                   tick-= curPart->tick();
             event.setTick(tick);
-            audio->msgAddEvent(event, curPart);
+            // Indicate do undo, and do not handle port controller values. 
+            //audio->msgAddEvent(event, curPart);
+            audio->msgAddEvent(event, curPart, true, false, false);
             }
       }
 
@@ -609,8 +674,17 @@ void ListEdit::editEvent(Event& event, MidiPart* part)
                   printf("event not in part %d - %d - %d, not changed\n", part->tick(),
                      nevent.tick(), part->tick() + part->lenTick());
             else
-                  audio->msgChangeEvent(event, nevent, part);
-            }
+            {
+              if(event.type() == Controller)
+                // Indicate do undo, and do port controller values and clone parts. 
+                //audio->msgChangeEvent(event, nevent, part);
+                audio->msgChangeEvent(event, nevent, part, true, true, true);
+              else  
+                // Indicate do undo, and do not do port controller values and clone parts. 
+                //audio->msgChangeEvent(event, nevent, part);
+                audio->msgChangeEvent(event, nevent, part, true, false, false);
+            }      
+          }
       }
 
 //---------------------------------------------------------
@@ -687,15 +761,52 @@ void ListEdit::cmd(int cmd)
       {
       switch(cmd) {
             case CMD_DELETE:
+                  bool found = false;
+                  for (QListViewItem* i = liste->firstChild(); i; i = i->nextSibling()) 
+                  {
+                    EventListItem *item = (EventListItem *) i;
+                    if (i->isSelected() || item->event.selected()) 
+                    {
+                          found = true;
+                          break;
+                    }
+                  }
+                  if(!found)
+                    break;
                   song->startUndo();
+                  
+                  EventListItem *deletedEvent=NULL;
                   for (QListViewItem* i = liste->firstChild(); i; i = i->nextSibling()) {
                         EventListItem *item = (EventListItem *) i;
 
-                        if (i->isSelected() || item->event.selected())
-                              audio->msgDeleteEvent(item->event, item->part, false);
-
+                        if (i->isSelected() || item->event.selected()) {
+                              deletedEvent=item;
+                              // Indicate no undo, and do port controller values and clone parts. 
+                              //audio->msgDeleteEvent(item->event, item->part, false);
+                              audio->msgDeleteEvent(item->event, item->part, false, true, true);
+                              }
                         }
+                  
+                  unsigned int nextTick=0;
+                  // find biggest tick
+                  for (QListViewItem* i = liste->firstChild(); i; i = i->nextSibling()) {
+                        EventListItem *item = (EventListItem *) i;
+                        if (item->event.tick() > nextTick && item != deletedEvent)
+                            nextTick=item->event.tick();
+                  }
+                  // check if there's a tick that is "just" bigger than the deleted
+                  for (QListViewItem* i = liste->firstChild(); i; i = i->nextSibling()) {
+                        EventListItem *item = (EventListItem *) i;
+                        if (item->event.tick() >= deletedEvent->event.tick() && 
+                            item->event.tick() < nextTick &&
+                            item != deletedEvent ) {
+                            nextTick=item->event.tick();
+                        }
+                  }
+                  selectedTick=nextTick;
                   song->endUndo(SC_EVENT_MODIFIED);
+                  //printf("selected tick = %d\n", selectedTick);
+                  //emit selectionChanged();
                   break;
             }
       }
