@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//    $Id: markerview.cpp,v 1.7.2.3 2006/03/26 20:38:05 spamatica Exp $
+//    $Id: markerview.cpp,v 1.7.2.6 2009/08/25 20:28:45 spamatica Exp $
 //  (C) Copyright 2000 Werner Schweer (ws@seh.de)
 //=========================================================
 
@@ -137,6 +137,7 @@ void MarkerItem::setTick(unsigned v)
 void MarkerView::closeEvent(QCloseEvent* e)
       {
       emit deleted((unsigned long)this);
+      emit closed();
       e->accept();
       }
 
@@ -242,6 +243,14 @@ MarkerView::MarkerView(QWidget* parent)
 
       connect(song, SIGNAL(songChanged(int)), SLOT(updateList()));
       updateList();
+
+      // work around for probable QT/WM interaction bug.
+      // for certain window managers, e.g xfce, this window is
+      // is displayed although not specifically set to show();
+      // bug: 2811156  	 Softsynth GUI unclosable with XFCE4 (and a few others)
+      show();
+      hide();
+
       }
 
 //---------------------------------------------------------
@@ -437,7 +446,7 @@ void MarkerView::markerChanged(int val)
 
 void MarkerView::nextMarker()
       {
-      int curPos = song->cpos();
+      unsigned int curPos = song->cpos();//prevent compiler warning: comparison of sigend/unsigned
       unsigned int nextPos = 0xFFFFFFFF;
       MarkerList* marker = song->marker();
       for (iMarker i = marker->begin(); i != marker->end(); ++i) {
@@ -452,7 +461,7 @@ void MarkerView::nextMarker()
       }
 void MarkerView::prevMarker()
       {
-      int curPos = song->cpos();
+      unsigned int curPos = song->cpos();//prevent compiler warning: comparison of sigend/unsigned
       unsigned int nextPos = 0;
       MarkerList* marker = song->marker();
       for (iMarker i = marker->begin(); i != marker->end(); ++i) {
