@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//    $Id: lmaster.cpp,v 1.2.2.5 2005/12/11 21:29:23 spamatica Exp $
+//    $Id: lmaster.cpp,v 1.2.2.8 2009/03/09 02:05:18 terminator356 Exp $
 //  (C) Copyright 2000 Werner Schweer (ws@seh.de)
 //=========================================================
 
@@ -169,14 +169,14 @@ void LMaster::insertTempo(const TEvent* ev)
 void LMaster::updateList()
       {
       LMasterLViewItem* selected = (LMasterLViewItem*) view->selectedItem();
-      LMASTER_LVTYPE type;
-      unsigned tick;
+      LMASTER_LVTYPE type = LMASTER_TEMPO;
+      unsigned tick = 0;
 
       if (selected) {
             type = selected->getType();
             tick = selected->tick();
             }
-
+      
       view->clear();
       const TempoList* t = &tempomap;
       const SigList* s   = &sigmap;
@@ -209,12 +209,15 @@ void LMaster::updateList()
             }
 
       // Try to reselect the previous selection:
-      LMasterLViewItem* tmp = getItemAtPos(tick, type);
-      if (tmp) {
-         view->clearSelection();
-         view->setSelected(tmp, true);
-         }
-      }
+      if(selected)
+      {
+        LMasterLViewItem* tmp = getItemAtPos(tick, type);
+        if (tmp) {
+           view->clearSelection();
+           view->setSelected(tmp, true);
+           }
+      }     
+    }
 
 //---------------------------------------------------------
 //   readStatus
