@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//    $Id: masteredit.cpp,v 1.4.2.1 2005/12/11 21:29:23 spamatica Exp $
+//    $Id: masteredit.cpp,v 1.4.2.5 2009/07/01 22:14:56 spamatica Exp $
 //  (C) Copyright 1999 Werner Schweer (ws@seh.de)
 //=========================================================
 
@@ -11,7 +11,6 @@
 #include "sigscale.h"
 #include "scrollscale.h"
 #include "poslabel.h"
-#include "intlabel.h"
 #include "master.h"
 #include "utils.h"
 #include "tscale.h"
@@ -22,6 +21,7 @@
 #include "sigedit.h"
 #include "globals.h"
 
+#include <values.h>
 #include <qtoolbar.h>
 #include <qtoolbutton.h>
 #include <qtooltip.h>
@@ -60,7 +60,23 @@ void MasterEdit::songChanged(int type)
       {
       if (type & SC_TEMPO) {
             int tempo = tempomap.tempo(song->cpos());
+            curTempo->blockSignals(true);
             curTempo->setValue(double(60000000.0/tempo));
+            
+            curTempo->blockSignals(false);
+            }
+      if (type & SC_SIG) {
+            int z, n;
+            sigmap.timesig(song->cpos(), z, n);
+            curSig->blockSignals(true);
+            curSig->setValue(Sig(z, n));
+            curSig->blockSignals(false);
+            sign->redraw();
+            }
+      if (type & SC_MASTER) {
+            enableButton->blockSignals(true);
+            enableButton->setOn(song->masterFlag());
+            enableButton->blockSignals(false);
             }
       }
 
@@ -88,7 +104,7 @@ MasterEdit::MasterEdit()
          PointerTool | PencilTool | RubberTool);
 
       QToolBar* enableMaster = new QToolBar(tr("EnableMaster"), this);
-      QToolButton* enableButton = new QToolButton(enableMaster);
+      enableButton = new QToolButton(enableMaster);
       enableButton->setToggleButton(true);
       enableButton->setText(tr("Enable"));
       QToolTip::add(enableButton, tr("Enable usage of master track"));
@@ -107,6 +123,7 @@ MasterEdit::MasterEdit()
             QT_TR_NOOP("Off"), "Bar", "1/2", "1/4", "1/8", "1/16"
             };
       rasterLabel = new LabelCombo(tr("Snap"), info);
+	  rasterLabel->setFocusPolicy(QWidget::NoFocus);
       for (int i = 0; i < 6; i++)
             rasterLabel->insertItem(tr(rastval[i]), i);
       rasterLabel->setCurrentItem(1);
