@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//  $Id: midi.cpp,v 1.43.2.9 2006/01/07 13:23:09 lunar_shuttle Exp $
+//  $Id: midi.cpp,v 1.43.2.22 2009/11/09 20:28:28 terminator356 Exp $
 //
 //  (C) Copyright 1999/2004 Werner Schweer (ws@seh.de)
 //=========================================================
@@ -32,12 +32,12 @@
 
 extern void dump(const unsigned char* p, int n);
 
-unsigned const char gmOnMsg[] = { 0x7e, 0x7f, 0x09, 0x01 };
-unsigned const char gsOnMsg[] = { 0x41, 0x10, 0x42, 0x12, 0x40, 0x00, 0x7f, 0x00, 0x41 };
-unsigned const char xgOnMsg[] = { 0x43, 0x10, 0x4c, 0x00, 0x00, 0x7e, 0x00 };
-unsigned const int  gmOnMsgLen = sizeof(gmOnMsg);
-unsigned const int  gsOnMsgLen = sizeof(gsOnMsg);
-unsigned const int  xgOnMsgLen = sizeof(xgOnMsg);
+const unsigned char gmOnMsg[] = { 0x7e, 0x7f, 0x09, 0x01 };
+const unsigned char gsOnMsg[] = { 0x41, 0x10, 0x42, 0x12, 0x40, 0x00, 0x7f, 0x00, 0x41 };
+const unsigned char xgOnMsg[] = { 0x43, 0x10, 0x4c, 0x00, 0x00, 0x7e, 0x00 };
+const unsigned int  gmOnMsgLen = sizeof(gmOnMsg);
+const unsigned int  gsOnMsgLen = sizeof(gsOnMsg);
+const unsigned int  xgOnMsgLen = sizeof(xgOnMsg);
 
 #define CALC_TICK(the_tick) lrintf((float(the_tick) * float(config.division) + float(div/2)) / float(div));
 /*---------------------------------------------------------
@@ -163,7 +163,7 @@ QString nameSysex(unsigned int len, const unsigned char* buf)
 //---------------------------------------------------------
 
 void buildMidiEventList(EventList* del, const MPEventList* el, MidiTrack* track,
-   int div, bool addSysexMeta)
+   int div, bool addSysexMeta, bool doLoops)
       {
       int hbank    = 0xff;
       int lbank    = 0xff;
@@ -183,6 +183,49 @@ void buildMidiEventList(EventList* del, const MPEventList* el, MidiTrack* track,
                || ((ev.channel() == track->outChannel()) && (ev.port() == track->outPort()))))
                   continue;
             unsigned tick = ev.time();
+            // Added by Tim. p3.3.8
+            
+            // Added by T356.
+            if(doLoops)
+            {
+              if(tick >= song->lPos().tick() && tick < song->rPos().tick())
+              {
+                int loopn = ev.loopNum();
+                int loopc = audio->loopCount();
+                int cmode = song->cycleMode(); // CYCLE_NORMAL, CYCLE_MIX, CYCLE_REPLACE
+                // If we want REPLACE and the event was recorded in a previous loop, 
+                //  just ignore it. This will effectively ignore ALL previous loop events inside
+                //  the left and right markers, regardless of where recording was started or stopped.
+                // We want to keep any loop 0 note-offs from notes which crossed over the left marker. 
+                // To avoid more searching here, just keep ALL note-offs from loop 0, and let code below 
+                //  sort out and keep which ones had note-ons.
+                if(!(ev.isNoteOff() && loopn == 0))
+                {
+                  if(cmode == Song::CYCLE_REPLACE && loopn < loopc)
+                  {
+                    // Added by Tim. p3.3.8
+                    //printf("buildMidiEventList: CYCLE_REPLACE t:%d type:%d A:%d B:%d ln:%d lc:%d\n", tick, ev.type(), ev.dataA(), ev.dataB(), loopn, loopc);
+                    
+                    continue;
+                  }  
+                  // If we want NORMAL, same as REPLACE except keep all events from the previous loop
+                  //  from rec stop position to right marker (and beyond).
+                  if(cmode == Song::CYCLE_NORMAL)
+                  {
+                    // Not sure of accuracy here. Adjust? Adjusted when used elsewhere?
+                    unsigned endRec = audio->getEndRecordPos().tick();
+                    if((tick < endRec && loopn < loopc) || (tick >= endRec && loopn < (loopc - 1)))
+                    {
+                      // Added by Tim. p3.3.8
+                      //printf("buildMidiEventList: CYCLE_NORMAL t:%d type:%d A:%d B:%d ln:%d lc:%d\n", tick, ev.type(), ev.dataA(), ev.dataB(), loopn, loopc);
+                    
+                      continue;
+                    }  
+                  } 
+                }  
+              }
+            }
+            
             Event e;
             switch(ev.type()) {
                   case ME_NOTEON:
@@ -193,8 +236,10 @@ void buildMidiEventList(EventList* del, const MPEventList* el, MidiTrack* track,
                               e.setPitch(instr);
                               }
                         else
+                        {
                               e.setPitch(ev.dataA());
-
+                        }
+                        
                         e.setVelo(ev.dataB());
                         e.setLenTick(0);
                         break;
@@ -299,7 +344,18 @@ void buildMidiEventList(EventList* del, const MPEventList* el, MidiTrack* track,
 
                               default:
                                     e.setType(Controller);
-                                    e.setA(ev.dataA());
+                                    int ctl = ev.dataA();
+                                    e.setA(ctl);
+                                    
+                                    if(track->type() == Track::DRUM) 
+                                    {
+                                      // Is it a drum controller event, according to the track port's instrument?
+                                      MidiController *mc = midiPorts[track->outPort()].drumController(ctl);
+                                      if(mc)
+                                        // Store an index into the drum map.
+                                        e.setA((ctl & ~0xff) | drumInmap[ctl & 0x7f]);
+                                    }
+                                          
                                     e.setB(val);
                                     break;
                               }
@@ -387,6 +443,9 @@ void buildMidiEventList(EventList* del, const MPEventList* el, MidiTrack* track,
                   }   // switch(ev.type()
             if (!e.empty()) {
                   e.setTick(tick);
+                  // Added by Tim. p3.3.8
+                  //printf("buildMidiEventList: mel adding t:%d type:%d A:%d B:%d C:%d\n", tick, e.type(), e.dataA(), e.dataB(), e.dataC());
+                  
                   mel.add(e);
                   }
             }  // i != el->end()
@@ -401,77 +460,113 @@ void buildMidiEventList(EventList* del, const MPEventList* el, MidiTrack* track,
 //                  event.setLenTick(0);
 //            }
 
-      for (iEvent i = mel.begin(); i != mel.end(); ++i) {
-            Event ev  = i->second;
-            if (ev.isNote()) {
-                  if (ev.isNoteOff()) {
-                        iEvent k;
-                        bool found = false;
-                        for (k = i; k != mel.end(); ++k) {
-                              Event event = k->second;
-                              if (event.tick() > ev.tick())
-                                    break;
-                              if (event.isNoteOff(ev)) {
-                                    ev.setLenTick(1);
-                                    ev.setVelo(event.velo());
-                                    ev.setVeloOff(0);
-                                    found = true;
-                                    break;
-                                    }
-                              }
-                        if (!found) {
-                              printf("NOTE OFF without Note ON tick %d type %d  %d %d\n",
-                                    ev.tick(), ev.type(), ev.pitch(), ev.velo());
-                              }
-                        else {
-                              mel.erase(k);
-                              continue;
-                              }
-                        }
-                  iEvent k;
-                  for (k = mel.lower_bound(ev.tick()); k != mel.end(); ++k) {
-                        Event event = k->second;
-                        if (ev.isNoteOff(event)) {
-                              int t = k->first - i->first;
-                              if (t <= 0) {
-                                    if (debugMsg) {
-                                          printf("Note len is (%d-%d)=%d, set to 1\n",
-                                            k->first, i->first, k->first - i->first);
-                                          ev.dump();
-                                          event.dump();
-                                          }
-                                    t = 1;
-                                    }
-                              ev.setLenTick(t);
-                              ev.setVeloOff(event.veloOff());
-                              break;
-                              }
-                        }
-                  if (k == mel.end()) {
-                        printf("-no note-off! %d pitch %d velo %d\n",
-                           ev.tick(), ev.pitch(), ev.velo());
-                        //
-                        // switch off at end of measure
-                        //
-                        int endTick = song->roundUpBar(ev.tick()+1);
-                        ev.setLenTick(endTick-ev.tick());
-                        }
-                  else {
-                        mel.erase(k);
-                        }
-                  }
-            }
+      // Added by Tim. p3.3.8 
+      
+      // The loop is a safe way to delete while iterating.
+      bool loop;
+      do
+      {
+        loop = false;
+        
+        for (iEvent i = mel.begin(); i != mel.end(); ++i) {
+              Event ev  = i->second;
+              if (ev.isNote()) {
+                    if (ev.isNoteOff()) {
+                          iEvent k;
+                          bool found = false;
+                          for (k = i; k != mel.end(); ++k) {
+                                Event event = k->second;
+                                if (event.tick() > ev.tick())
+                                      break;
+                                if (event.isNoteOff(ev)) {
+                                      ev.setLenTick(1);
+                                      ev.setVelo(event.velo());
+                                      ev.setVeloOff(0);
+                                      // Added by Tim. p3.3.8
+                                      //printf("buildMidiEventList: found note off: event t:%d len:%d type:%d A:%d B:%d C:%d  ev t:%d len:%d type:%d A:%d B:%d C:%d\n", event.tick(), event.lenTick(), event.type(), event.dataA(), event.dataB(), event.dataC(), ev.tick(), ev.lenTick(), ev.type(), ev.dataA(), ev.dataB(), ev.dataC());
+                                      
+                                      found = true;
+                                      break;
+                                      }
+                                }
+                          if (!found) {
+                                printf("NOTE OFF without Note ON tick %d type %d  %d %d\n",
+                                      ev.tick(), ev.type(), ev.pitch(), ev.velo());
+                                }
+                          else {
+                                mel.erase(k);
+                                
+                                // Changed by Tim. p3.3.8
+                                //continue;
+                                loop = true;
+                                break;
+                                
+                                }
+                          }
+                    // Added by Tim. p3.3.8
+                    
+                    // If the event length is not zero, it means the event and its 
+                    //  note on/off have already been taken care of. So ignore it.
+                    if(ev.lenTick() != 0)
+                    {
+                      continue;
+                    }
+                    
+                    iEvent k;
+                    for (k = mel.lower_bound(ev.tick()); k != mel.end(); ++k) {
+                          Event event = k->second;
+                          if (ev.isNoteOff(event)) {
+                                int t = k->first - i->first;
+                                if (t <= 0) {
+                                      if (debugMsg) {
+                                            printf("Note len is (%d-%d)=%d, set to 1\n",
+                                              k->first, i->first, k->first - i->first);
+                                            ev.dump();
+                                            event.dump();
+                                            }
+                                      t = 1;
+                                      }
+                                ev.setLenTick(t);
+                                ev.setVeloOff(event.veloOff());
+                                // Added by Tim. p3.3.8
+                                //printf("buildMidiEventList: set len and velOff: event t:%d len:%d type:%d A:%d B:%d C:%d  ev t:%d len:%d type:%d A:%d B:%d C:%d\n", event.tick(), event.lenTick(), event.type(), event.dataA(), event.dataB(), event.dataC(), ev.tick(), ev.lenTick(), ev.type(), ev.dataA(), ev.dataB(), ev.dataC());
+                                      
+                                break;
+                                }
+                          }
+                    if (k == mel.end()) {
+                          printf("-no note-off! %d pitch %d velo %d\n",
+                            ev.tick(), ev.pitch(), ev.velo());
+                          //
+                          // switch off at end of measure
+                          //
+                          int endTick = song->roundUpBar(ev.tick()+1);
+                          ev.setLenTick(endTick-ev.tick());
+                          }
+                    else {
+                          mel.erase(k);
+                          // Added by Tim. p3.3.8
+                          loop = true;
+                          break;
+                          
+                          }
+                    }
+              }
+        }     
+        while (loop);
 
 // DEBUG: any note offs left?
 
-      for (iEvent i = mel.begin(); i != mel.end(); ++i) {
-            Event ev  = i->second;
-            if (ev.isNoteOff()) {
-                  printf("+extra note-off! %d pitch %d velo %d\n",
-                           i->first, ev.pitch(), ev.velo());
+      // Removed by Tim. p3.3.8
+      //for (iEvent i = mel.begin(); i != mel.end(); ++i) {
+      //      Event ev  = i->second;
+      //      if (ev.isNoteOff()) {
+      //            printf("+extra note-off! %d pitch %d velo %d\n",
+      //                     i->first, ev.pitch(), ev.velo());
 //                  ev.dump();
-                  }
-            }
+      //            }
+      //      }
+      
       for (iEvent i = mel.begin(); i != mel.end(); ++i) {
             Event ev  = i->second;
             if (ev.isNoteOff()) {
@@ -577,9 +672,11 @@ void Audio::initDevices()
       //
       // damit Midi-Devices, die mehrere Ports besitzen, wie z.B.
       // das Korg NS5R, nicht mehrmals zwischen GM und XG/GS hin und
-      // hergeschaltet werden, wird zunchst auf allen Ports GM
+      // hergeschaltet werden, wird zunÃÂ¯ÃÂ¿ÃÂ½hst auf allen Ports GM
       // initialisiert, und dann erst XG/GS
       //
+      
+      // Standard initialization...
       for (int i = 0; i < MIDI_PORTS; ++i) {
             if (!activePorts[i])
                   continue;
@@ -602,12 +699,15 @@ void Audio::initDevices()
                   case MT_UNKNOWN:
                         break;
                   case MT_GM:
+                        port->sendGmInitValues();
                         break;
                   case MT_GS:
                         port->sendGsOn();
+                        port->sendGsInitValues();
                         break;
                   case MT_XG:
                         port->sendXgOn();
+                        port->sendXgInitValues();
                         break;
                   }
             }
@@ -618,7 +718,7 @@ void Audio::initDevices()
 //    collect events for next audio segment
 //---------------------------------------------------------
 
-void Audio::collectEvents(MidiTrack* track, int cts, int nts)
+void Audio::collectEvents(MidiTrack* track, unsigned int cts, unsigned int nts)
       {
       int port    = track->outPort();
       int channel = track->outChannel();
@@ -636,6 +736,7 @@ void Audio::collectEvents(MidiTrack* track, int cts, int nts)
                   continue;
             EventList* events = part->events();
             unsigned partTick = part->tick();
+            unsigned partLen  = part->lenTick();
             int delay         = track->delay;
 
             if (cts > nts) {
@@ -648,6 +749,10 @@ void Audio::collectEvents(MidiTrack* track, int cts, int nts)
                   continue;
             unsigned stick = (offset > cts) ? 0 : cts - offset;
             unsigned etick = nts - offset;
+            // By T356. Do not play events which are past the end of this part. 
+            if(etick > partLen)
+              continue;
+              
             iEvent ie   = events->lower_bound(stick);
             iEvent iend = events->lower_bound(etick);
 
@@ -672,6 +777,7 @@ void Audio::collectEvents(MidiTrack* track, int cts, int nts)
                               {
                               int len   = ev.lenTick();
                               int pitch = ev.pitch();
+                              int velo  = ev.velo();
                               if (track->type() == Track::DRUM)  {
                                     //
                                     // Map drum-notes to the drum-map values
@@ -680,6 +786,7 @@ void Audio::collectEvents(MidiTrack* track, int cts, int nts)
                                    pitch     = drumMap[instr].anote;
                                    port      = drumMap[instr].port; //This changes to non-default port
                                    channel   = drumMap[instr].channel;
+                                   velo      = int(double(velo) * (double(drumMap[instr].vol) / 100.0)) ;
                                    }
                               else {
                                     //
@@ -692,7 +799,6 @@ void Audio::collectEvents(MidiTrack* track, int cts, int nts)
                                     pitch = 127;
                               if (pitch < 0)
                                     pitch = 0;
-                              int velo  = ev.velo();
                               velo += track->velocity;
                               velo = (velo * track->compression) / 100;
                               if (velo > 127)
@@ -709,21 +815,53 @@ void Audio::collectEvents(MidiTrack* track, int cts, int nts)
                                     playEvents->add(MidiPlayEvent(frame, port, channel, 0x90, pitch, velo));
                                     stuckNotes->add(MidiPlayEvent(tick + len, port, channel,
                                        veloOff ? 0x80 : 0x90, pitch, veloOff));
-                                          track->addActivity(velo);
                                     }
                               else { //Handle events to different port than standard.
-                                    //printf("** Adding to spec device!: frame=%d port=%d channel=%d pitch=%d velo=%d\n",frame, port, channel, pitch, velo);
                                     MidiDevice* mdAlt = midiPorts[port].device();
                                     if (mdAlt) {
                                         mdAlt->playEvents()->add(MidiPlayEvent(frame, port, channel, 0x90, pitch, velo));
                                         mdAlt->stuckNotes()->add(MidiPlayEvent(tick + len, port, channel,
                                           veloOff ? 0x80 : 0x90, pitch, veloOff));
-                                              track->addActivity(velo);
-                                        }
+                                      }
                                     }
+                              
+                              if(velo > track->activity())
+                                track->setActivity(velo);
                               }
                               break;
 
+                        // Added by T356.
+                        case Controller:
+                              {
+                                //int len   = ev.lenTick();
+                                //int pitch = ev.pitch();
+                                if (track->type() == Track::DRUM)  
+                                {
+                                  int ctl   = ev.dataA();
+                                  // Is it a drum controller event, according to the track port's instrument?
+                                  MidiController *mc = midiPorts[defaultPort].drumController(ctl);
+                                  if(mc)
+                                  {
+                                    int instr = ctl & 0x7f;
+                                    ctl &=  ~0xff;
+                                    int pitch = drumMap[instr].anote & 0x7f;
+                                    port      = drumMap[instr].port; //This changes to non-default port
+                                    channel   = drumMap[instr].channel;
+                                    MidiDevice* mdAlt = midiPorts[port].device();
+                                    if(mdAlt) 
+                                    {
+                                      //playEvents->add(MidiPlayEvent(frame, port, channel, ev));
+                                      mdAlt->playEvents()->add(MidiPlayEvent(frame, port, channel, 
+                                                                             ME_CONTROLLER, ctl | pitch, ev.dataB()));
+                                    }  
+                                    break;
+                                  }  
+                                }
+                                playEvents->add(MidiPlayEvent(frame, port, channel, ev));
+                              }     
+                              break;
+                                   
+                                   
                         default:
                               playEvents->add(MidiPlayEvent(frame, port, channel, ev));
                               break;
@@ -743,6 +881,7 @@ void Audio::collectEvents(MidiTrack* track, int cts, int nts)
 
 void Audio::processMidi()
       {
+      midiBusy=true;
       //
       // TODO: syntis should directly write into recordEventList
       //
@@ -770,62 +909,96 @@ void Audio::processMidi()
       iMPEvent nextPlayEvent  = metronome->nextPlayEvent();
       playEvents->erase(playEvents->begin(), nextPlayEvent);
 
-      for (iMidiTrack t = song->midis()->begin(); t != song->midis()->end(); ++t) {
+      for (iMidiTrack t = song->midis()->begin(); t != song->midis()->end(); ++t) 
+      {
             MidiTrack* track = *t;
             int port = track->outPort();
             MidiDevice* md = midiPorts[port].device();
-            if (track->isMute() || md == 0)
-                  continue;
-
-            MPEventList* playEvents = md->playEvents();
-
-            if (isPlaying() && (curTickPos < nextTickPos))
+            
+            // Changed by Tim. p3.3.8
+            //if(md == 0)
+            //  continue;
+            //MPEventList* playEvents = md->playEvents();
+            //if (playEvents == 0)
+            //    continue;
+            //if (!track->isMute()) 
+            MPEventList* playEvents = 0;
+            if(md)
+            {
+              playEvents = md->playEvents();
+              
+              // only add track events if the track is unmuted
+              if(!track->isMute()) 
+              {
+                if(isPlaying() && (curTickPos < nextTickPos))
                   collectEvents(track, curTickPos, nextTickPos);
+              }
+            }  
 
             //
             //----------midi recording
             //
-            if (track->recordFlag()) {
-                  int portMask    = track->inPortMask();
+            if (track->recordFlag()) 
+            {
+                  //int portMask    = track->inPortMask();
+                  unsigned int portMask = track->inPortMask();
                   int channelMask = track->inChannelMask();
                   MPEventList* rl = track->mpevents();
+                  MidiPort* tport = &midiPorts[port];
 
-                  for (iMidiDevice id = midiDevices.begin(); id != midiDevices.end(); ++id) {
+                  for (iMidiDevice id = midiDevices.begin(); id != midiDevices.end(); ++id) 
+                  {
                         MidiDevice* dev = *id;
 
-                        int port = dev->midiPort();
+                        int devport = dev->midiPort();
+                  
                         // record only from ports marked in portMask:
-                        if (port == -1 || !(portMask & (1 << port)))
+                        if (devport == -1 || !(portMask & (1 << devport)))
                               continue;
+                              
                         MREventList* el = dev->recordEvents();
-                        for (iMREvent ie = el->begin(); ie != el->end(); ++ie) {
+                        for (iMREvent ie = el->begin(); ie != el->end(); ++ie) 
+                        {
                               int channel = ie->channel();
-                              int defaultPort = port;
+                              int defaultPort = devport;
                               if (!(channelMask & (1 << channel)))
+                              {
                                     continue;
+                              }      
 
                               MidiPlayEvent event(*ie);
-                              int drumRecPitch; //Hmmm, hehhh... TODO: Clean up a bit around here when it comes to separate events for rec & for playback. But not before 0.7 (ml)
+                              int drumRecPitch=0; //prevent compiler warning: variable used without initialization
+                              MidiController *mc = 0;
+                              int ctl = 0;
+                              
+                              //Hmmm, hehhh... 
+                              // TODO: Clean up a bit around here when it comes to separate events for rec & for playback. 
+                              // But not before 0.7 (ml)
+
                               int prePitch = 0, preVelo = 0;
 
                               event.setChannel(track->outChannel());
-                              if (event.isNote() || event.isNoteOff()) {
+                              
+                              if (event.isNote() || event.isNoteOff()) 
+                              {
                                     //
                                     // apply track values
                                     //
 
                                     //Apply drum inkey:
-                                    if (track->type() == Track::DRUM) {
+                                    if (track->type() == Track::DRUM) 
+                                    {
                                           int pitch = event.dataA();
                                           //Map note that is played according to drumInmap
-                                          drumRecPitch = drumMap[drumInmap[pitch]].enote;
-                                          port = drumMap[drumInmap[pitch]].port;
-                                          event.setPort(port);
-                                          channel = drumMap[drumInmap[pitch]].channel;
-                                          event.setA(drumMap[drumInmap[pitch]].anote);
+                                          drumRecPitch = drumMap[(unsigned int)drumInmap[pitch]].enote;
+                                          devport = drumMap[(unsigned int)drumInmap[pitch]].port;
+                                          event.setPort(devport);
+                                          channel = drumMap[(unsigned int)drumInmap[pitch]].channel;
+                                          event.setA(drumMap[(unsigned int)drumInmap[pitch]].anote);
                                           event.setChannel(channel);
-                                          }
-                                    else { //Track transpose if non-drum
+                                    }
+                                    else 
+                                    { //Track transpose if non-drum
                                           prePitch = event.dataA();
                                           int pitch = prePitch + track->transposition;
                                           if (pitch > 127)
@@ -833,9 +1006,10 @@ void Audio::processMidi()
                                           if (pitch < 0)
                                                 pitch = 0;
                                           event.setA(pitch);
-                                          }
+                                    }
 
-                                    if (!event.isNoteOff()) {
+                                    if (!event.isNoteOff()) 
+                                    {
                                           preVelo = event.dataB();
                                           int velo = preVelo + track->velocity;
                                           velo = (velo * track->compression) / 100;
@@ -844,58 +1018,127 @@ void Audio::processMidi()
                                           if (velo < 1)
                                                 velo = 1;
                                           event.setB(velo);
-                                          }
                                     }
+                              }
+                              // Added by T356.
+                              else
+                              if(event.type() == ME_CONTROLLER)
+                              {
+                                if(track->type() == Track::DRUM) 
+                                {
+                                  ctl = event.dataA();
+                                  // Regardless of what port the event came from, is it a drum controller event 
+                                  //  according to the track port's instrument?
+                                  mc = tport->drumController(ctl);
+                                  if(mc)
+                                  {
+                                    int pitch = ctl & 0x7f;
+                                    ctl &= ~0xff;
+                                    int dmindex = drumInmap[pitch] & 0x7f;
+                                    //Map note that is played according to drumInmap
+                                    drumRecPitch = drumMap[dmindex].enote;
+                                    devport = drumMap[dmindex].port;
+                                    event.setPort(devport);
+                                    channel = drumMap[dmindex].channel;
+                                    event.setA(ctl | drumMap[dmindex].anote);
+                                    event.setChannel(channel);
+                                  }  
+                                }
+                              }
+                              
                               unsigned time = event.time() + segmentSize*(segmentCount-1);
                               event.setTime(time);
 
                               // dont't echo controller changes back to software
                               // synthesizer:
 
-                              if (!dev->isSynti()) {
-                                    //Check if we're outputting to another port than default:
-                                    if (port == defaultPort) {
-                                          event.setPort(track->outPort());
+                              if (!dev->isSynti()) 
+                              {
+                                // Added by Tim. p3.3.8
+                                if(track->recEcho())
+                                {
+                                
+                                  //Check if we're outputting to another port than default:
+                                  if (devport == defaultPort) {
+                                        event.setPort(port);
+                                        if(md)
                                           playEvents->add(event);
-                                          }
-                                    else {
-                                          // Hmm, this appears to work, but... Will this induce trouble with md->setNextPlayEvent??
-                                          MidiDevice* mdAlt = midiPorts[port].device();
+                                        }
+                                  else {
+                                        // Hmm, this appears to work, but... Will this induce trouble with md->setNextPlayEvent??
+                                        MidiDevice* mdAlt = midiPorts[devport].device();
+                                        if(mdAlt)
                                           mdAlt->playEvents()->add(event);
-                                          }
-                                    }
-
+                                        }
+                                }
+                                // Shall we activate meters even while rec echo is off? Sure, why not...
+                                if(event.isNote() && event.dataB() > track->activity())
+                                  track->setActivity(event.dataB());
+                              }
+                              
                               time = tempomap.frame2tick(event.time());
                               event.setTime(time);  // set tick time
 
                               // Special handling of events stored in rec-lists. a bit hACKish. TODO: Clean up (after 0.7)! :-/ (ml)
-                              if (recording) {
-                                    if (track->type() == Track::DRUM) {
+                              if (recording) 
+                              {
+                                    // In these next steps, it is essential to set the recorded event's port 
+                                    //  to the track port so buildMidiEventList will accept it. Even though 
+                                    //  the port may have no device "<none>".
+                                    //
+                                    if (track->type() == Track::DRUM) 
+                                    {
+                                      // Is it a drum controller event?
+                                      if(mc)
+                                      {    
+                                          MidiPlayEvent drumRecEvent = event;
+                                          drumRecEvent.setA(ctl | drumRecPitch);
+                                          // In this case, preVelo is simply the controller value.
+                                          drumRecEvent.setB(preVelo);
+                                          drumRecEvent.setPort(port); //rec-event to current port
+                                          drumRecEvent.setChannel(track->outChannel()); //rec-event to current channel
+                                          rl->add(drumRecEvent);
+                                      }
+                                      else
+                                      {
+                                      
                                           MidiPlayEvent drumRecEvent = event;
                                           drumRecEvent.setA(drumRecPitch);
                                           drumRecEvent.setB(preVelo);
-                                          drumRecEvent.setPort(port);
+                                          // Changed by T356. 
+                                          // Tested: Events were not being recorded for a drum map entry pointing to a 
+                                          //  different port. This must have been wrong - buildMidiEventList would ignore this.
+                                          //drumRecEvent.setPort(devport);
+                                          drumRecEvent.setPort(port);  //rec-event to current port
+                                          
                                           drumRecEvent.setChannel(track->outChannel()); //rec-event to current channel
                                           rl->add(drumRecEvent);
-                                          }
-                                    else {
+                                      }    
+                                    }
+                                    else 
+                                    {
                                           // Restore record-pitch to non-transposed value since we don't want the note transposed twice next
                                           MidiPlayEvent recEvent = event;
                                           if (prePitch)
                                                 recEvent.setA(prePitch);
                                           if (preVelo)
                                                 recEvent.setB(preVelo);
-                                          recEvent.setPort(track->outPort());
+                                          recEvent.setPort(port);
                                           recEvent.setChannel(track->outChannel());
                                                 
                                           rl->add(recEvent);
-                                          }
                                     }
                               }
                         }
                   }
-            md->setNextPlayEvent(playEvents->begin());
             }
+            // Added by Tim. p3.3.8
+            if(md)
+            {  
+              
+              md->setNextPlayEvent(playEvents->begin());
+            }
+      }
 
       //
       // clear all recorded events in midiDevices
@@ -905,6 +1148,10 @@ void Audio::processMidi()
             MidiDevice* md = *id;
             md->recordEvents()->clear();
 
+            // By T356. Done processing this rec buffer, now flip to the other one.
+            md->flipRecBuffer();
+            
+            
             MPEventList* stuckNotes = md->stuckNotes();
             MPEventList* playEvents = md->playEvents();
 
@@ -929,14 +1176,12 @@ void Audio::processMidi()
       if (midiClickFlag)
             md = midiPorts[clickPort].device();
       if (song->click() && (isPlaying() || state == PRECOUNT)) {
-            MPEventList* playEvents;
-            MPEventList* stuckNotes;
-
+            MPEventList* playEvents = 0;
+            MPEventList* stuckNotes = 0;
             if (md) {
                   playEvents = md->playEvents();
                   stuckNotes = md->stuckNotes();
                   }
-
             int bar, beat;
             unsigned tick;
             bool isMeasure = false;
@@ -1006,5 +1251,6 @@ void Audio::processMidi()
                   stuckNotes->clear();
                   }
             }
+      midiBusy=false;
       }
 
