@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//    $Id: dcanvas.cpp,v 1.16.2.4 2005/06/20 20:04:01 lunar_shuttle Exp $
+//    $Id: dcanvas.cpp,v 1.16.2.10 2009/10/15 22:45:50 terminator356 Exp $
 //  (C) Copyright 1999 Werner Schweer (ws@seh.de)
 //=========================================================
 
@@ -50,8 +50,23 @@ DEvent::DEvent(Event e, Part* p)
 
 void DrumCanvas::addItem(Part* part, Event& event)
       {
+      if (signed(event.tick())<0) {
+            printf("ERROR: trying to add event before current part!\n");
+            return;
+      }
+      
       DEvent* ev = new DEvent(event, part);
       items.add(ev);
+      
+      int diff = event.endTick()-part->lenTick();
+      if (diff > 0)  {// too short part? extend it
+            //printf("addItem - this code should not be run!\n");
+            //Part* newPart = part->clone();
+            //newPart->setLenTick(newPart->lenTick()+diff);
+            //audio->msgChangePart(part, newPart,false);
+            //part = newPart;
+            part->setLenTick(part->lenTick()+diff);
+            }
       }
 
 //---------------------------------------------------------
@@ -66,14 +81,207 @@ DrumCanvas::DrumCanvas(MidiEditor* pr, QWidget* parent, int sx,
       songChanged(SC_TRACK_INSERTED);
       }
 
+//---------------------------------------------------------
+//   moveCanvasItems
+//---------------------------------------------------------
+
+void DrumCanvas::moveCanvasItems(CItemList& items, int dp, int dx, DragType dtype, int* pflags)
+{      
+  if(editor->parts()->empty())
+    return;
+    
+  PartsToChangeMap parts2change;
+  
+  int modified = 0;
+  for(iPart ip = editor->parts()->begin(); ip != editor->parts()->end(); ++ip)
+  {
+    Part* part = ip->second;
+    if(!part)
+      continue;
+    
+    int npartoffset = 0;
+    for(iCItem ici = items.begin(); ici != items.end(); ++ici) 
+    {
+      CItem* ci = ici->second;
+      if(ci->part() != part)
+        continue;
+      
+      int x = ci->pos().x() + dx;
+      int y = pitch2y(y2pitch(ci->pos().y()) + dp);
+      QPoint newpos = raster(QPoint(x, y));
+      
+      // Test moving the item...
+      DEvent* nevent = (DEvent*) ci;
+      Event event    = nevent->event();
+      x              = newpos.x();
+      if(x < 0)
+        x = 0;
+      int ntick = editor->rasterVal(x) - part->tick();
+      if(ntick < 0)
+        ntick = 0;
+      int diff = ntick + event.lenTick() - part->lenTick();
+      
+      // If moving the item would require a new part size...
+      if(diff > npartoffset)
+        npartoffset = diff;
+    }
+        
+    if(npartoffset > 0)
+    {    
+      // Create new part...
+      // if there are several events that are moved outside the part, it will be recreated for each
+      // so the part _in_ the event will not be valid, ask the authority.
+//      Part* newPart = part->clone();
+      //Part* newPart = Canvas::part()->clone();
+
+//      newPart->setLenTick(newPart->lenTick() + npartoffset);
+      //audio->msgChangePart(part, newPart,false);
+
+//      modified = SC_PART_MODIFIED;
+
+      // BUG FIX: #1650953
+      // Added by T356.
+      // Fixes posted "select and drag past end of part - crashing" bug
+//      for(iPart ip = editor->parts()->begin(); ip != editor->parts()->end(); ++ip)
+//      {
+//        if(ip->second == part)
+//        {
+//          editor->parts()->erase(ip);
+//          break;
+//        }
+//      }
+      
+//      editor->parts()->add(newPart);
+//      audio->msgChangePart(part, newPart,false);
+      
+//      if(parts2change.find(part) == parts2change.end())
+//       parts2change.insert(std::pair<Part*, Part*> (part, newPart));
+      iPartToChange ip2c = parts2change.find(part);
+      if(ip2c == parts2change.end())
+      {
+        PartToChange p2c = {0, npartoffset};
+        parts2change.insert(std::pair<Part*, PartToChange> (part, p2c));
+      }
+      else
+        ip2c->second.xdiff = npartoffset;
+      
+      
+      //part = newPart; // reassign
+      //item->setPart(part);
+      //item->setEvent(newEvent);
+      //curPart = part;
+      //curPartId = curPart->sn();
+
+    }
+  }
+    
+  for(iPartToChange ip2c = parts2change.begin(); ip2c != parts2change.end(); ++ip2c)
+  {
+    Part* opart = ip2c->first;
+    int diff = ip2c->second.xdiff;
+    
+    Part* newPart = opart->clone();
+    
+    newPart->setLenTick(newPart->lenTick() + diff);
+    
+    modified = SC_PART_MODIFIED;
+    
+    // BUG FIX: #1650953
+    // Added by T356.
+    // Fixes posted "select and drag past end of part - crashing" bug
+    for(iPart ip = editor->parts()->begin(); ip != editor->parts()->end(); ++ip)
+    {
+      if(ip->second == opart)
+      {
+        editor->parts()->erase(ip);
+        break;
+      }
+    }
+      
+    editor->parts()->add(newPart);
+    // Indicate no undo, and do port controller values but not clone parts. 
+    //audio->msgChangePart(opart, newPart, false);
+    audio->msgChangePart(opart, newPart, false, true, false);
+    
+    ip2c->second.npart = newPart;
+    
+  }
+    
+  iPartToChange icp = parts2change.find(curPart);
+  if(icp != parts2change.end())
+  {
+    curPart = icp->second.npart;
+    curPartId = curPart->sn();
+  }  
+    
+  std::vector< CItem* > doneList;
+  typedef std::vector< CItem* >::iterator iDoneList;
+  
+  for(iCItem ici = items.begin(); ici != items.end(); ++ici) 
+  {
+    CItem* ci = ici->second;
+    
+    // If this item's part is in the parts2change list, change the item's part to the new part.
+    Part* pt = ci->part();
+    iPartToChange ip2c = parts2change.find(pt);
+    if(ip2c != parts2change.end())
+      ci->setPart(ip2c->second.npart);
+    
+    int x = ci->pos().x();
+    int y = ci->pos().y();
+    int nx = x + dx;
+    int ny = pitch2y(y2pitch(y) + dp);
+    QPoint newpos = raster(QPoint(nx, ny));
+    selectItem(ci, true);
+    
+    iDoneList idl;
+    for(idl = doneList.begin(); idl != doneList.end(); ++idl)
+      // This compares EventBase pointers to see if they're the same...
+      if((*idl)->event() == ci->event())
+        break;
+      
+    // Do not process if the event has already been processed (meaning it's an event in a clone part)...
+    //if(moveItem(ci, newpos, dtype))
+    if(idl != doneList.end())
+      // Just move the canvas item.
+      ci->move(newpos);
+    else
+    {
+      // Currently moveItem always returns true.
+      if(moveItem(ci, newpos, dtype))
+      {
+        // Add the canvas item to the list of done items.
+        doneList.push_back(ci);
+        // Move the canvas item.
+        ci->move(newpos);
+      }  
+    }
+          
+    if(moving.size() == 1) {
+          itemReleased(curItem, newpos);
+          }
+    if(dtype == MOVE_COPY || dtype == MOVE_CLONE)
+          selectItem(ci, false);
+  }  
+      
+  if(pflags)
+    *pflags = modified;
+}
+      
 //---------------------------------------------------------
 //   moveItem
 //---------------------------------------------------------
 
+// Changed by T356.
+//bool DrumCanvas::moveItem(CItem* item, const QPoint& pos, DragType dtype, int* pflags)
 bool DrumCanvas::moveItem(CItem* item, const QPoint& pos, DragType dtype)
       {
       DEvent* nevent   = (DEvent*) item;
-      MidiPart* part   = (MidiPart*)nevent->part();
+      
+      // Changed by T356.
+      //MidiPart* part   = (MidiPart*)Canvas::part(); // part can be dynamically recreated, ask the authority
+      MidiPart* part   = (MidiPart*)nevent->part();   
+      
       Event event      = nevent->event();
       int x            = pos.x();
       if (x < 0)
@@ -87,12 +295,63 @@ bool DrumCanvas::moveItem(CItem* item, const QPoint& pos, DragType dtype)
       newEvent.setPitch(npitch);
       newEvent.setTick(ntick);
 
+      // Removed by T356.
+      /*
+      // Added by T356.
+      int modified = 0;
+      //song->startUndo();
+      int diff = newEvent.endTick()-part->lenTick();
+      if (diff > 0)  // too short part? extend it
+      {
+        // if there are several events that are moved outside the part, it will be recreated for each
+        // so the part _in_ the event will not be valid, ask the authority.
+        //Part* newPart = part->clone();
+        MidiPart* newPart = (MidiPart*)Canvas::part()->clone();
+        newPart->setLenTick(newPart->lenTick()+diff);
+        audio->msgChangePart(Canvas::part(), newPart,false);
+        
+        modified = SC_PART_MODIFIED;
+        part = newPart; // reassign
+        for(iPart i = editor->parts()->begin(); i != editor->parts()->end(); ++i)
+        {  
+          if(i->second == Canvas::part())
+          {
+            editor->parts()->erase(i); 
+            break;
+          }    
+        }
+        editor->parts()->add(part);
+        item->setPart(part);
+        item->setEvent(newEvent);
+        curPart = part;
+        curPartId = curPart->sn();
+      }
+      */
+      
+      // Added by T356. 
+      // msgAddEvent and msgChangeEvent (below) will set these, but set them here first?
+      //item->setPart(part);
+      item->setEvent(newEvent);
+      
+      // Added by T356. 
+      if(((int)newEvent.endTick() - (int)part->lenTick()) > 0)  
+        printf("DrumCanvas::moveItem Error! New event end:%d exceeds length:%d of part:%s\n", newEvent.endTick(), part->lenTick(), part->name().latin1());
+      
       if (dtype == MOVE_COPY || dtype == MOVE_CLONE) {
-            audio->msgAddEvent(newEvent, part, false);
+            // Indicate no undo, and do not do port controller values and clone parts. 
+            //audio->msgAddEvent(newEvent, part, false);
+            audio->msgAddEvent(newEvent, part, false, false, false);
             }
       else {
-            audio->msgChangeEvent(event, newEvent, part, false);
+            // Indicate no undo, and do not do port controller values and clone parts. 
+            //audio->msgChangeEvent(event, newEvent, part, false);
+            audio->msgChangeEvent(event, newEvent, part, false, false, false);
             }
+        
+      // Removed by T356.
+      //if(pflags)
+      //  *pflags = modified;
+      
       return true;
       }
 
@@ -128,7 +387,9 @@ void DrumCanvas::resizeItem(CItem* item, bool)
       {
       DEvent* nevent = (DEvent*) item;
       Event ev = nevent->event();
-      audio->msgDeleteEvent(ev, nevent->part());
+      // Indicate do undo, and do not do port controller values and clone parts. 
+      //audio->msgDeleteEvent(ev, nevent->part());
+      audio->msgDeleteEvent(ev, nevent->part(), true, false, false);
       }
 
 //---------------------------------------------------------
@@ -157,13 +418,39 @@ void DrumCanvas::newItem(CItem* item, bool noSnap)
 
       for (iEvent i = lower; i != upper; ++i) {
             Event ev = i->second;
+            // Added by T356. Only do notes.
+            if(!ev.isNote())
+              continue;
+              
             if (ev.pitch() == npitch) {
-                  audio->msgDeleteEvent(ev, nevent->part());
+                  // Indicate do undo, and do not do port controller values and clone parts. 
+                  //audio->msgDeleteEvent(ev, nevent->part());
+                  audio->msgDeleteEvent(ev, nevent->part(), true, false, false);
                   return;
                   }
             }
 
-      audio->msgAddEvent(event, nevent->part());
+      // Added by T356.
+      Part* part = nevent->part();
+      song->startUndo();
+      int modified=SC_EVENT_MODIFIED;
+      int diff = event.endTick()-part->lenTick();
+      if (diff > 0)  {// too short part? extend it
+            //printf("extend Part!\n");
+            Part* newPart = part->clone();
+            newPart->setLenTick(newPart->lenTick()+diff);
+            // Indicate no undo, and do port controller values but not clone parts. 
+            //audio->msgChangePart(part, newPart,false);
+            audio->msgChangePart(part, newPart, false, true, false);
+            modified=modified|SC_PART_MODIFIED;
+            part = newPart; // reassign
+            }
+      // Indicate no undo, and do not do port controller values and clone parts. 
+      //audio->msgAddEvent(event, part,false); 
+      audio->msgAddEvent(event, part, false, false, false); 
+      song->endUndo(modified);
+      
+      //audio->msgAddEvent(event, nevent->part());
       }
 
 //---------------------------------------------------------
@@ -173,7 +460,9 @@ void DrumCanvas::newItem(CItem* item, bool noSnap)
 bool DrumCanvas::deleteItem(CItem* item)
       {
       Event ev = ((DEvent*)item)->event();
-      audio->msgDeleteEvent(ev, ((DEvent*)item)->part());
+      // Indicate do undo, and do not do port controller values and clone parts. 
+      //audio->msgDeleteEvent(ev, ((DEvent*)item)->part());
+      audio->msgDeleteEvent(ev, ((DEvent*)item)->part(), true, false, false);
       return false;
       }
 
@@ -181,55 +470,87 @@ bool DrumCanvas::deleteItem(CItem* item)
 //   drawItem
 //---------------------------------------------------------
 
-void DrumCanvas::drawItem(QPainter&p, const CItem*item, const QRect&)
+void DrumCanvas::drawItem(QPainter&p, const CItem*item, const QRect& rect)
       {
-      p.setPen(black);
       DEvent* e   = (DEvent*) item;
-      Event me    = e->event();
-
-      int x = mapx(item->pos().x());
-      int y = mapy(item->pos().y());
-
+      int x = 0, y = 0;
+        x = mapx(item->pos().x());
+        y = mapy(item->pos().y());
       QPointArray pa(4);
       pa.setPoint(0, x - CARET2, y);
       pa.setPoint(1, x,          y - CARET2);
       pa.setPoint(2, x + CARET2, y);
       pa.setPoint(3, x,          y + CARET2);
-
-      int velo    = me.velo();
-      DrumMap* dm = &drumMap[y2pitch(y)]; //Get the drum item
-
-      QColor color;
-      if (velo < dm->lv1)
-            color.setRgb(240, 240, 255);
-      else if (velo < dm->lv2)
-            color.setRgb(200, 200, 255);
-      else if (velo < dm->lv3)
-            color.setRgb(170, 170, 255);
-      else
-            color.setRgb(0, 0, 255);
-
+      QRect r(pa.boundingRect());
+      r = r.intersect(rect);
+      if(!r.isValid())
+        return;
+      
+      p.setPen(black);
+      
       if (e->part() != curPart)
-            p.setBrush(lightGray);
+      {
+            if(item->isMoving()) 
+              p.setBrush(gray);
+            else if(item->isSelected()) 
+              p.setBrush(black);
+            else  
+              p.setBrush(lightGray);
+      }      
       else if (item->isMoving()) {
-            p.setBrush(gray);
-            p.drawPolygon(pa);
-            p.setBrush(black);
-            QPoint p = map(item->mp());
-            x = p.x();
-            y = p.y();
-            pa.setPoint(0, x-CARET2,  y + TH/2);
-            pa.setPoint(1, x,         y + TH/2+CARET2);
-            pa.setPoint(2, x+CARET2,  y + TH/2);
-            pa.setPoint(3, x,         y + (TH-CARET)/2);
+              p.setBrush(gray);
             }
       else if (item->isSelected())
+      {
             p.setBrush(black);
+      }
       else
+      {
+            int velo    = e->event().velo();
+            DrumMap* dm = &drumMap[y2pitch(y)]; //Get the drum item
+            QColor color;
+            if (velo < dm->lv1)
+                  color.setRgb(240, 240, 255);
+            else if (velo < dm->lv2)
+                  color.setRgb(200, 200, 255);
+            else if (velo < dm->lv3)
+                  color.setRgb(170, 170, 255);
+            else
+                  color.setRgb(0, 0, 255);
             p.setBrush(color);
+      }
+            
       p.drawPolygon(pa);
       }
 
+//---------------------------------------------------------
+//   drawMoving
+//    draws moving items
+//---------------------------------------------------------
+
+void DrumCanvas::drawMoving(QPainter& p, const CItem* item, const QRect& rect)
+    {
+      //if(((DEvent*)item)->part() != curPart)
+      //  return;
+      //if(!item->isMoving()) 
+      //  return;
+      QPointArray pa(4);
+      QPoint pt = map(item->mp());
+      int x = pt.x();
+      int y = pt.y();
+      pa.setPoint(0, x-CARET2,  y + TH/2);
+      pa.setPoint(1, x,         y + TH/2+CARET2);
+      pa.setPoint(2, x+CARET2,  y + TH/2);
+      pa.setPoint(3, x,         y + (TH-CARET)/2);
+      QRect mr(pa.boundingRect());
+      mr = mr.intersect(rect);
+      if(!mr.isValid())
+        return;
+      p.setPen(black);
+      p.setBrush(black);
+      p.drawPolygon(pa);
+    }
+
 //---------------------------------------------------------
 //   drawCanvas
 //---------------------------------------------------------
@@ -296,7 +617,9 @@ void DrumCanvas::cmd(int cmd)
                               continue;
                         DEvent* e = (DEvent*)(i->second);
                         Event event = e->event();
-                        audio->msgDeleteEvent(event, e->part(), false);
+                        // Indicate no undo, and do not do port controller values and clone parts. 
+                        //audio->msgDeleteEvent(event, e->part(), false);
+                        audio->msgDeleteEvent(event, e->part(), false, false, false);
                         }
                   song->endUndo(SC_EVENT_REMOVED);
                   break;
@@ -351,7 +674,9 @@ void DrumCanvas::cmd(int cmd)
                               if (!i->second->isSelected())
                                     continue;
                               Event ev = i->second->event();
-                              audio->msgDeleteEvent(ev, i->second->part(), false);
+                              // Indicate no undo, and do not do port controller values and clone parts. 
+                              //audio->msgDeleteEvent(ev, i->second->part(), false);
+                              audio->msgDeleteEvent(ev, i->second->part(), false, false, false);
                               }
                         song->endUndo(SC_EVENT_REMOVED);
                         }
@@ -372,7 +697,9 @@ void DrumCanvas::cmd(int cmd)
                               Event event    = devent->event();
                               Event newEvent = event.clone();
                               newEvent.setLenTick(drumMap[event.pitch()].len);
-                              audio->msgChangeEvent(event, newEvent, devent->part() , false);
+                              // Indicate no undo, and do not do port controller values and clone parts. 
+                              //audio->msgChangeEvent(event, newEvent, devent->part() , false);
+                              audio->msgChangeEvent(event, newEvent, devent->part(), false, false, false);
                               }
                         }
                   song->endUndo(SC_EVENT_MODIFIED);
@@ -429,7 +756,9 @@ void DrumCanvas::cmd(int cmd)
                               if (event.velo() != velo) {
                                     Event newEvent = event.clone();
                                     newEvent.setVelo(velo);
-                                    audio->msgChangeEvent(event, newEvent, devent->part(), false);
+                                    // Indicate no undo, and do not do port controller values and clone parts. 
+                                    //audio->msgChangeEvent(event, newEvent, devent->part(), false);
+                                    audio->msgChangeEvent(event, newEvent, devent->part(), false, false, false);
                                     }
                               }
                         }
@@ -437,6 +766,7 @@ void DrumCanvas::cmd(int cmd)
                   }
                   break;
             }
+      updateSelection();
       redraw();
       }
 
@@ -522,6 +852,9 @@ int DrumCanvas::pasteAt(const QString& pt, int pos)
       const char* p = pt.latin1();
       Xml xml(p);
 
+      // Added by T356. 
+      int modified = SC_EVENT_INSERTED;
+      
       song->startUndo();
       for (;;) {
             Xml::Token token = xml.parse();
@@ -529,17 +862,42 @@ int DrumCanvas::pasteAt(const QString& pt, int pos)
             switch (token) {
                   case Xml::Error:
                   case Xml::End:
-                        song->endUndo(SC_EVENT_INSERTED);
+                        //song->endUndo(SC_EVENT_INSERTED); By T356
+                        song->endUndo(modified);
                         return pos;
                   case Xml::TagStart:
                         if (tag == "event") {
                               Event e(Note);
                               e.read(xml);
-                              e.setTick(e.tick() + pos - curPart->tick());
-                              audio->msgAddEvent(e, curPart, false);
+                              
+                              // Added by T356. 
+                              int tick = e.tick() + pos - curPart->tick();
+                              if (tick<0) { 
+                                      printf("DrumCanvas::pasteAt ERROR: trying to add event before current part!\n");
+                                      song->endUndo(SC_EVENT_INSERTED);
+                                      //delete el;
+                                      return pos;
+                                      }
+                              e.setTick(tick);
+                              int diff = e.endTick() - curPart->lenTick();
+                              if (diff > 0)  {// too short part? extend it
+                                      Part* newPart = curPart->clone();
+                                      newPart->setLenTick(newPart->lenTick()+diff);
+                                      // Indicate no undo, and do port controller values but not clone parts. 
+                                      //audio->msgChangePart(curPart, newPart,false);
+                                      audio->msgChangePart(curPart, newPart, false, true, false);
+                                      
+                                      modified=modified|SC_PART_MODIFIED;
+                                      curPart = newPart; // reassign
+                                      }
+                              
+                              // Indicate no undo, and do not do port controller values and clone parts. 
+                              //audio->msgAddEvent(e, curPart, false);
+                              audio->msgAddEvent(e, curPart, false, false, false);
                               }
                         else
-                              xml.unknown("EventCanvas::paste");
+                              //xml.unknown("EventCanvas::paste"); By T356
+                              xml.unknown("DCanvas::pasteAt");
                         break;
                   case Xml::TagEnd:
                   default:
@@ -672,53 +1030,110 @@ void DrumCanvas::mapChanged(int spitch, int dpitch)
       // Other solution: implement a specific undo-event for this (SC_DRUMMAP_MODIFIED or something) which undoes movement of
       // dlist-items (ml)
 
-      std::vector< std::pair<Part*, Event*> > spitch_events;
-      std::vector< std::pair<Part*, Event*> > dpitch_events;
-
-      TrackList* tracks = song->tracks();
-      for (iTrack t = tracks->begin(); t != tracks->end(); t++) {
-            Track* curTrack = *t;
+      std::vector< std::pair<Part*, Event*> > delete_events;
+      std::vector< std::pair<Part*, Event> > add_events;
+      
+      typedef std::vector< std::pair<Part*, Event*> >::iterator idel_ev;
+      typedef std::vector< std::pair<Part*, Event> >::iterator iadd_ev;
+      
+      /*
+      class delete_events : public std::vector< Part*, Event* > 
+      {
+        public:
+          idel_ev find(Part* p, Event* e)
+          {
+          
+          };
+      };
+      class add_events : public std::vector< Part*, Event > 
+      {
+        public:
+          iadd_ev find(Part* p, Event& e)
+          {
+          
+          };
+      };
+      */
+      
+      MidiTrackList* tracks = song->midis();
+      for (ciMidiTrack t = tracks->begin(); t != tracks->end(); t++) {
+            MidiTrack* curTrack = *t;
             if (curTrack->type() != Track::DRUM)
                   continue;
 
+            MidiPort* mp = &midiPorts[curTrack->outPort()];
             PartList* parts= curTrack->parts();
             for (iPart part = parts->begin(); part != parts->end(); ++part) {
                   EventList* events = part->second->events();
                   Part* thePart = part->second;
                   for (iEvent i = events->begin(); i != events->end(); ++i) {
                         Event event = i->second;
+                        if(event.type() != Controller && event.type() != Note)
+                          continue;
                         int pitch = event.pitch();
+                        bool drc = false;
+                        // Is it a drum controller event, according to the track port's instrument?
+                        if(event.type() == Controller && mp->drumController(event.dataA()))
+                        {
+                          drc = true;
+                          pitch = event.dataA() & 0x7f;
+                        }
+                        
                         if (pitch == spitch) {
                               Event* spitch_event = &(i->second);
-                              spitch_events.push_back(std::pair<Part*, Event*>(thePart, spitch_event));
+                              delete_events.push_back(std::pair<Part*, Event*>(thePart, spitch_event));
+                              Event newEvent = spitch_event->clone();
+                              if(drc)
+                                newEvent.setA((newEvent.dataA() & ~0xff) | dpitch);
+                              else
+                                newEvent.setPitch(dpitch);
+                              add_events.push_back(std::pair<Part*, Event>(thePart, newEvent));
                               }
                         else if (pitch == dpitch) {
                               Event* dpitch_event = &(i->second);
-                              dpitch_events.push_back(std::pair<Part*, Event*>(thePart, dpitch_event));
+                              delete_events.push_back(std::pair<Part*, Event*>(thePart, dpitch_event));
+                              Event newEvent = dpitch_event->clone();
+                              if(drc)
+                                newEvent.setA((newEvent.dataA() & ~0xff) | spitch);
+                              else
+                                newEvent.setPitch(spitch);
+                              add_events.push_back(std::pair<Part*, Event>(thePart, newEvent));
                               }
                         }
                   }
             }
 
       song->startUndo();
-      for (std::vector< std::pair<Part*, Event*> >::iterator i = spitch_events.begin(); i != spitch_events.end(); i++) {
-            std::pair<Part*, Event*> pair = *i;
-            Part*  thePart = pair.first;
-            Event* theEvent = pair.second;
-            Event newEvent = theEvent->clone();
-            newEvent.setPitch(dpitch);
-            audio->msgChangeEvent(*theEvent, newEvent, thePart, false);
+      for (idel_ev i = delete_events.begin(); i != delete_events.end(); i++) {
+            //std::pair<Part*, Event*> pair = *i;
+            //Part*  thePart = pair.first;
+            //Event* theEvent = pair.second;
+            Part*  thePart = (*i).first;
+            Event* theEvent = (*i).second;
+            // Indicate no undo, and do port controller values but not clone parts. 
+            //audio->msgDeleteEvent(*theEvent, thePart, false);
+            audio->msgDeleteEvent(*theEvent, thePart, false, true, false);
             }
 
-      for (std::vector< std::pair<Part*, Event*> >::iterator i = dpitch_events.begin(); i != dpitch_events.end(); i++) {
-            std::pair<Part*, Event*> pair = *i;
-            Part*  thePart = pair.first;
-            Event* theEvent = pair.second;
-            Event newEvent = theEvent->clone();
-            newEvent.setPitch(spitch);
-            audio->msgChangeEvent(*theEvent, newEvent, thePart, false);
+      DrumMap dm = drumMap[spitch];
+      drumMap[spitch] = drumMap[dpitch];
+      drumMap[dpitch] = dm;
+      drumInmap[int(drumMap[spitch].enote)]  = spitch;
+      drumOutmap[int(drumMap[int(spitch)].anote)] = spitch;
+      drumInmap[int(drumMap[int(dpitch)].enote)]  = dpitch;
+      drumOutmap[int(drumMap[int(dpitch)].anote)] = dpitch;
+            
+      for (iadd_ev i = add_events.begin(); i != add_events.end(); i++) {
+            //std::pair<Part*, Event> pair = *i;
+            //Part*  thePart = pair.first;
+            //Event& theEvent = pair.second;
+            Part*  thePart = (*i).first;
+            Event& theEvent = (*i).second;
+            // Indicate no undo, and do port controller values but not clone parts. 
+            //audio->msgAddEvent(theEvent, thePart, false);
+            audio->msgAddEvent(theEvent, thePart, false, true, false);
             }
-
+      
       song->endUndo(SC_EVENT_MODIFIED);
       song->update(SC_DRUMMAP);
       }
@@ -812,8 +1227,20 @@ void DrumCanvas::modifySelected(NoteInfo::ValType type, int delta)
                         break;
                   }
             song->changeEvent(event, newEvent, part);
-            song->undoOp(UndoOp::ModifyEvent, newEvent, event, part);
+            // Indicate do not do port controller values and clone parts. 
+            //song->undoOp(UndoOp::ModifyEvent, newEvent, event, part);
+            song->undoOp(UndoOp::ModifyEvent, newEvent, event, part, false, false);
             }
       song->endUndo(SC_EVENT_MODIFIED);
       audio->msgIdle(false);
       }
+
+//---------------------------------------------------------
+//   curPartChanged
+//---------------------------------------------------------
+
+void DrumCanvas::curPartChanged()
+      {
+      editor->setCaption(getCaption());
+      }
+
