@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//    $Id: drumedit.h,v 1.9.2.1 2005/12/11 21:29:24 spamatica Exp $
+//    $Id: drumedit.h,v 1.9.2.7 2009/11/16 11:29:33 lunar_shuttle Exp $
 //  (C) Copyright 1999 Werner Schweer (ws@seh.de)
 //=========================================================
 
@@ -13,6 +13,7 @@
 #include <qtooltip.h>
 #include <qwhatsthis.h>
 
+#include <values.h>
 #include "midieditor.h"
 #include "noteinfo.h"
 #include "cobject.h"
@@ -61,6 +62,7 @@ class DWhatsThis : public QWhatsThis {
 class DHeaderTip : public QToolTip {
    public:
       DHeaderTip(QWidget * parent) : QToolTip(parent) {}
+      virtual ~DHeaderTip(){}	
    protected:
       void maybeTip(const QPoint &);
       };
@@ -83,7 +85,7 @@ class DrumEdit : public MidiEditor {
       Toolbar1* toolbar;
       Splitter* split1;
       Splitter* split2;
-
+      QWidget* split1w1;
       DList* dlist;
       Header* header;
       QToolBar* tools;
@@ -114,21 +116,26 @@ class DrumEdit : public MidiEditor {
       void selectionChanged(); // enable/disable "Copy" & "Paste"
       void load();
       void save();
+      void reset();
       void setTime(unsigned);
       void follow(int);
       void newCanvasWidth(int);
       void configChanged();
+      void songChanged1(int);
 
    public slots:
       void setSelection(int, Event&, Part*);
       void soloChanged(bool);       // called by Solo button
-
+      void execDeliveredScript(int);
+      void execUserScript(int);
+      
+      virtual void updateHScrollRange();
    signals:
       void deleted(unsigned long);
 
    public:
-      DrumEdit(PartList*, QWidget* parent = 0, const char* name = 0);
-      ~DrumEdit();
+      DrumEdit(PartList*, QWidget* parent = 0, const char* name = 0, unsigned initPos = MAXINT);
+      virtual ~DrumEdit();
       virtual void readStatus(Xml&);
       virtual void writeStatus(int, Xml&) const;
       static void readConfiguration(Xml& xml);
