@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//  $Id: drummap.h,v 1.3 2004/06/14 19:59:24 lunar_shuttle Exp $
+//  $Id: drummap.h,v 1.3.2.3 2009/10/29 02:14:37 terminator356 Exp $
 //
 //  (C) Copyright 1999/2000 Werner Schweer (ws@seh.de)
 //=========================================================
@@ -19,6 +19,7 @@ class Xml;
 
 struct DrumMap {
       QString name;
+      unsigned char vol;            // playback volume, percent.
       int quant;
       int len;                      // len of event in ticks
       int channel;                  // midi channel
@@ -28,7 +29,8 @@ struct DrumMap {
       bool mute;
 //      bool selected;
 
-      bool const operator==(const DrumMap& map) const;
+      //bool const operator==(const DrumMap& map) const;
+      bool operator==(const DrumMap& map) const;
       };
 
 #define DRUM_MAPSIZE  128
@@ -39,6 +41,7 @@ extern DrumMap drumMap[DRUM_MAPSIZE];
 extern void initDrumMap();
 extern void writeDrumMap(int level, Xml& xml, bool external);
 extern void readDrumMap(Xml& xml, bool external);
+extern void resetGMDrumMap();
 
 #endif
 
