@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//    $Id: ecanvas.cpp,v 1.8.2.2 2006/01/02 23:15:08 spamatica Exp $
+//    $Id: ecanvas.cpp,v 1.8.2.6 2009/05/03 04:14:00 terminator356 Exp $
 //  (C) Copyright 2001 Werner Schweer (ws@seh.de)
 //=========================================================
 
@@ -98,10 +98,10 @@ void EventCanvas::startUndo(DragType)
 //   endUndo
 //---------------------------------------------------------
 
-void EventCanvas::endUndo(DragType dtype)
+void EventCanvas::endUndo(DragType dtype, int flags)
       {
-      song->endUndo((dtype == MOVE_COPY || dtype == MOVE_CLONE)
-         ? SC_EVENT_INSERTED : SC_EVENT_MODIFIED);
+      song->endUndo(flags | ((dtype == MOVE_COPY || dtype == MOVE_CLONE)
+         ? SC_EVENT_INSERTED : SC_EVENT_MODIFIED));
       }
 
 //---------------------------------------------------------
@@ -130,6 +130,10 @@ void EventCanvas::updateSelection()
 
 void EventCanvas::songChanged(int flags)
       {
+      // Is it simply a midi controller value adjustment? Forget it.
+      if(flags == SC_MIDI_CONTROLLER)
+        return;
+    
       if (flags & ~SC_SELECTION) {
             items.clear();
             start_tick  = MAXINT;
@@ -140,7 +144,8 @@ void EventCanvas::songChanged(int flags)
                   if (part->sn() == curPartId)
                         curPart = part;
                   unsigned stick = part->tick();
-                  unsigned etick = stick + part->lenTick();
+                  unsigned len = part->lenTick();
+                  unsigned etick = stick + len;
                   if (stick < start_tick)
                         start_tick = stick;
                   if (etick > end_tick)
@@ -149,6 +154,11 @@ void EventCanvas::songChanged(int flags)
                   EventList* el = part->events();
                   for (iEvent i = el->begin(); i != el->end(); ++i) {
                         Event e = i->second;
+                        // Added by T356. Do not add events which are either past, or extend past the end of the part.
+                        //if(e.tick() > len)
+                        if(e.endTick() > len)
+                          break;
+                        
                         if (e.isNote()) {
                               addItem(part, e);
                               }
@@ -195,16 +205,29 @@ void EventCanvas::songChanged(int flags)
       }
 
 //---------------------------------------------------------
-//   selectFirst
+//   selectAtTick
 //---------------------------------------------------------
-
-void EventCanvas::selectFirst()
+void EventCanvas::selectAtTick(unsigned int tick)
       {
-      //Select leftmost note if none selected and there are any
+      //Select note nearest tick, if none selected and there are any
       if (!items.empty() && selectionSize() == 0) {
             iCItem i = items.begin();
-            if (!i->second->isSelected()) {
-                  selectItem(i->second, true);
+            CItem* nearest = i->second;
+
+            while (i != items.end()) {
+                CItem* cur=i->second;                
+                unsigned int curtk=abs(cur->x() + cur->part()->tick() - tick);
+                unsigned int neartk=abs(nearest->x() + nearest->part()->tick() - tick);
+
+                if (curtk < neartk) {
+                    nearest=cur;
+                    }
+
+                i++;
+                }
+
+            if (!nearest->isSelected()) {
+                  selectItem(nearest, true);
                   songChanged(SC_SELECTION);
                   }
             }
