@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//    $Id: pianoroll.cpp,v 1.25.2.4 2005/12/11 21:29:24 spamatica Exp $
+//    $Id: pianoroll.cpp,v 1.25.2.15 2009/11/16 11:29:33 lunar_shuttle Exp $
 //  (C) Copyright 1999 Werner Schweer (ws@seh.de)
 //=========================================================
 
@@ -18,6 +18,7 @@
 #include <qmenubar.h>
 #include <qapplication.h>
 #include <qclipboard.h>
+#include <qdir.h>
 
 #include <stdio.h>
 
@@ -56,11 +57,14 @@ static const int yscale = 1;
 static const int pianoWidth = 40;
 static int pianorollTools = PointerTool | PencilTool | RubberTool | DrawTool;
 
+
+
+
 //---------------------------------------------------------
 //   PianoRoll
 //---------------------------------------------------------
 
-PianoRoll::PianoRoll(PartList* pl, QWidget* parent, const char* name)
+PianoRoll::PianoRoll(PartList* pl, QWidget* parent, const char* name, unsigned initPos)
    : MidiEditor(_quantInit, _rasterInit, pl, parent, name)
       {
       deltaMode = false;
@@ -73,7 +77,6 @@ PianoRoll::PianoRoll(PartList* pl, QWidget* parent, const char* name)
       _quantLen      = _quantLenInit;
       _to            = _toInit;
       colorMode      = colorModeInit;
-
       //---------Men----------------------------------
       menuEdit = new QPopupMenu(this);
       menuBar()->insertItem(tr("&Edit"), menuEdit);
@@ -143,6 +146,7 @@ PianoRoll::PianoRoll(PartList* pl, QWidget* parent, const char* name)
       menuFunctions->insertItem(tr("Delete Measure"),               PianoCanvas::CMD_DELETE_MEASURE);
       menuFunctions->insertItem(tr("Create Measure"),               PianoCanvas::CMD_CREATE_MEASURE);
       menuFunctions->insertItem(tr("Set fixed length"),             PianoCanvas::CMD_FIXED_LEN);
+      menuFunctions->insertItem(tr("Delete overlaps"),              PianoCanvas::CMD_DELETE_OVERLAPS);
 
       menuFunctions->setItemEnabled(PianoCanvas::CMD_CRESCENDO, false);
       menuFunctions->setItemEnabled(PianoCanvas::CMD_TRANSPOSE, false);
@@ -155,6 +159,9 @@ PianoRoll::PianoRoll(PartList* pl, QWidget* parent, const char* name)
       menuFunctions->setItemEnabled(PianoCanvas::CMD_DELETE_MEASURE, false);
       menuFunctions->setItemEnabled(PianoCanvas::CMD_CREATE_MEASURE, false);
 
+      menuPlugins = new QPopupMenu(this);
+      song->populateScriptMenu(menuPlugins, this);
+      menuBar()->insertItem(tr("&Plugins"), menuPlugins);
       connect(menuEdit, SIGNAL(activated(int)),      SLOT(cmd(int)));
       connect(menuSelect, SIGNAL(activated(int)),    SLOT(cmd(int)));
       connect(menuFunctions, SIGNAL(activated(int)), SLOT(cmd(int)));
@@ -234,8 +241,8 @@ PianoRoll::PianoRoll(PartList* pl, QWidget* parent, const char* name)
       gridS1->addMultiCellWidget(hLine(split1), 1, 1, 0, 2);
       gridS1->addWidget(piano,                  2,    0);
       gridS1->addWidget(canvas,                 2,    1);
+      
       gridS1->addWidget(vscroll,                2,    2);
-
 //      gridS1->addWidget(time,                   0,    1);
 //      gridS1->addWidget(hLine(split1),          1,    1);
 //      gridS1->addWidget(piano,                  2,    0);
@@ -261,6 +268,8 @@ PianoRoll::PianoRoll(PartList* pl, QWidget* parent, const char* name)
 
       connect(canvas, SIGNAL(pitchChanged(int)), piano, SLOT(setPitch(int)));
       connect(canvas, SIGNAL(verticalScroll(unsigned)), vscroll, SLOT(setPos(unsigned)));
+      connect(canvas,  SIGNAL(horizontalScroll(unsigned)),hscroll, SLOT(setPos(unsigned)));
+      connect(canvas,  SIGNAL(horizontalScrollNoLimit(unsigned)),hscroll, SLOT(setPosNoLimit(unsigned))); 
       connect(canvas, SIGNAL(selectionChanged(int, Event&, Part*)), this,
          SLOT(setSelection(int, Event&, Part*)));
 
@@ -271,6 +280,7 @@ PianoRoll::PianoRoll(PartList* pl, QWidget* parent, const char* name)
       connect(speaker, SIGNAL(toggled(bool)), SLOT(setSpeaker(bool)));
       connect(canvas, SIGNAL(followEvent(int)), SLOT(follow(int)));
 
+      connect(hscroll, SIGNAL(scaleChanged(int)),  SLOT(updateHScrollRange()));
       piano->setYPos(KH * 30);
       canvas->setYPos(KH * 30);
       vscroll->setPos(KH * 30);
@@ -280,11 +290,8 @@ PianoRoll::PianoRoll(PartList* pl, QWidget* parent, const char* name)
       connect(song, SIGNAL(songChanged(int)), SLOT(songChanged1(int)));
 
       setCaption(canvas->getCaption());
-      int s, e;
-      canvas->range(&s, &e);
-      e += sigmap.ticksMeasure(e);  // show one more measure
-      hscroll->setRange(s, e);
-
+      
+      updateHScrollRange();
       // connect to toolbar
       connect(canvas,   SIGNAL(pitchChanged(int)), toolbar, SLOT(setPitch(int)));
       connect(canvas,   SIGNAL(timeChanged(unsigned)),  SLOT(setTime(unsigned)));
@@ -304,7 +311,23 @@ PianoRoll::PianoRoll(PartList* pl, QWidget* parent, const char* name)
       clipboardChanged(); // enable/disable "Paste"
       selectionChanged(); // enable/disable "Copy" & "Paste"
       initShortcuts(); // initialize shortcuts
-      canvas->selectFirst();
+
+      const Pos cpos=song->cPos();
+      canvas->setPos(0, cpos.tick(), true);
+      canvas->selectAtTick(cpos.tick());
+      //canvas->selectFirst();
+//      
+      if(canvas->track())
+        toolbar->setSolo(canvas->track()->solo());
+        
+      unsigned pos;
+      if(initPos >= MAXINT)
+        pos = song->cpos();
+      else
+        pos = initPos;
+      if(pos > MAXINT)
+        pos = MAXINT;
+      hscroll->setOffset((int)pos);
       }
 
 //---------------------------------------------------------
@@ -313,9 +336,13 @@ PianoRoll::PianoRoll(PartList* pl, QWidget* parent, const char* name)
 
 void PianoRoll::songChanged1(int bits)
       {
-      if (bits & SC_SOLO)
+        
+        if (bits & SC_SOLO)
+        {
             toolbar->setSolo(canvas->track()->solo());
-      songChanged(bits);
+            return;
+        }      
+        songChanged(bits);
       }
 
 //---------------------------------------------------------
@@ -327,6 +354,26 @@ void PianoRoll::configChanged()
       initShortcuts();
       }
 
+//---------------------------------------------------------
+//   updateHScrollRange
+//---------------------------------------------------------
+
+void PianoRoll::updateHScrollRange()
+{
+      int s, e;
+      canvas->range(&s, &e);
+      // Show one more measure.
+      e += sigmap.ticksMeasure(e);  
+      // Show another quarter measure due to imprecise drawing at canvas end point.
+      e += sigmap.ticksMeasure(e) / 4;
+      // Compensate for the fixed piano and vscroll widths. 
+      e += canvas->rmapxDev(pianoWidth - vscroll->width()); 
+      int s1, e1;
+      hscroll->range(&s1, &e1);
+      if(s != s1 || e != e1) 
+        hscroll->setRange(s, e);
+}
+
 //---------------------------------------------------------
 //   follow
 //---------------------------------------------------------
@@ -444,7 +491,9 @@ void PianoRoll::noteinfoChanged(NoteInfo::ValType type, int val)
                         event.setPitch(val);
                         break;
                   }
-            audio->msgChangeEvent(selEvent, event, selPart);
+            // Indicate do undo, and do not do port controller values and clone parts. 
+            //audio->msgChangeEvent(selEvent, event, selPart);
+            audio->msgChangeEvent(selEvent, event, selPart, true, false, false);
             }
       else {
             // multiple events are selected; treat noteinfo values
@@ -595,7 +644,8 @@ void PianoRoll::writeConfiguration(int level, Xml& xml)
 
 void PianoRoll::soloChanged(bool flag)
       {
-      canvas->track()->setSolo(flag);
+      audio->msgSetSolo(canvas->track(), flag);
+      song->update(SC_SOLO);
       }
 
 //---------------------------------------------------------
@@ -802,6 +852,46 @@ void PianoRoll::keyPressEvent(QKeyEvent* event)
             pc->pianoCmd(CMD_DELETE);
             return;
             }
+      else if (key == shortcuts[SHRT_ZOOM_IN].key) {
+            int mag = hscroll->mag();
+            int zoomlvl = ScrollScale::getQuickZoomLevel(mag);
+            if (zoomlvl < 23)
+                  zoomlvl++;
+
+            int newmag = ScrollScale::convertQuickZoomLevelToMag(zoomlvl);
+            hscroll->setMag(newmag);
+            //printf("mag = %d zoomlvl = %d newmag = %d\n", mag, zoomlvl, newmag);
+            return;
+            }
+      else if (key == shortcuts[SHRT_ZOOM_OUT].key) {
+            int mag = hscroll->mag();
+            int zoomlvl = ScrollScale::getQuickZoomLevel(mag);
+            if (zoomlvl > 1)
+                  zoomlvl--;
+
+            int newmag = ScrollScale::convertQuickZoomLevelToMag(zoomlvl);
+            hscroll->setMag(newmag);
+            //printf("mag = %d zoomlvl = %d newmag = %d\n", mag, zoomlvl, newmag);
+            return;
+            }
+      else if (key == shortcuts[SHRT_GOTO_CPOS].key) {
+            PartList* p = this->parts();
+            Part* first = p->begin()->second;
+            hscroll->setPos(song->cpos() - first->tick() );
+            return;
+            }
+      else if (key == shortcuts[SHRT_SCROLL_LEFT].key) {
+            int pos = hscroll->pos() - config.division;
+            if (pos < 0)
+                  pos = 0;
+            hscroll->setPos(pos);
+            return;
+            }
+      else if (key == shortcuts[SHRT_SCROLL_RIGHT].key) {
+            int pos = hscroll->pos() + config.division;
+            hscroll->setPos(pos);
+            return;
+            }
       else if (key == shortcuts[SHRT_SET_QUANT_1].key)
             val = rasterTable[8 + off];
       else if (key == shortcuts[SHRT_SET_QUANT_2].key)
@@ -978,4 +1068,25 @@ void PianoRoll::initShortcuts()
       menuFunctions->setAccel(shortcuts[SHRT_CREATE_MEASURE].key,   PianoCanvas::CMD_CREATE_MEASURE);
       menuFunctions->setAccel(shortcuts[SHRT_CONFIG_QUANT].key, menu_ids[CMD_CONFIG_QUANT]);
       menuFunctions->setAccel(shortcuts[SHRT_FIXED_LEN].key,        PianoCanvas::CMD_FIXED_LEN);
+      menuFunctions->setAccel(shortcuts[SHRT_DELETE_OVERLAPS].key,  PianoCanvas::CMD_DELETE_OVERLAPS);
       }
+
+//---------------------------------------------------------
+//   execDeliveredScript
+//---------------------------------------------------------
+void PianoRoll::execDeliveredScript(int id)
+{
+      //QString scriptfile = QString(INSTPREFIX) + SCRIPTSSUFFIX + deliveredScriptNames[id];
+      QString scriptfile = song->getScriptPath(id, true);
+      song->executeScript(scriptfile.latin1(), parts(), quant(), true); 
+}
+
+//---------------------------------------------------------
+//   execUserScript
+//---------------------------------------------------------
+void PianoRoll::execUserScript(int id)
+{
+      QString scriptfile = song->getScriptPath(id, false);
+      song->executeScript(scriptfile.latin1(), parts(), quant(), true);
+}
+
