@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//    $Id: pianoroll.h,v 1.5.2.1 2005/12/11 21:29:24 spamatica Exp $
+//    $Id: pianoroll.h,v 1.5.2.4 2009/11/16 11:29:33 lunar_shuttle Exp $
 //  (C) Copyright 1999 Werner Schweer (ws@seh.de)
 //=========================================================
 
@@ -11,6 +11,7 @@
 #include <qwidget.h>
 #include <qmainwindow.h>
 
+#include <values.h>
 #include "noteinfo.h"
 #include "cobject.h"
 #include "midieditor.h"
@@ -48,7 +49,7 @@ class PianoRoll : public MidiEditor {
 
       enum { CMD_EVENT_COLOR, CMD_CONFIG_QUANT, CMD_LAST };
       int menu_ids[CMD_LAST];
-      QPopupMenu *menuEdit, *menuFunctions, *menuSelect, *menuConfig;
+      QPopupMenu *menuEdit, *menuFunctions, *menuSelect, *menuConfig, *menuPlugins;
 
       int tickOffset;
       int lenOffset;
@@ -86,9 +87,10 @@ class PianoRoll : public MidiEditor {
       bool _quantLen;
       QuantConfig* quantConfig;
       bool _playEvents;
-      void initShortcuts();
+
 
       Q_OBJECT
+      void initShortcuts();
       QWidget* genToolbar(QWidget* parent);
       virtual void closeEvent(QCloseEvent*);
       virtual void keyPressEvent(QKeyEvent*);
@@ -120,9 +122,13 @@ class PianoRoll : public MidiEditor {
 
    signals:
       void deleted(unsigned long);
-
+   
+   public slots:
+      virtual void updateHScrollRange();
+      void execDeliveredScript(int id);
+      void execUserScript(int id);
    public:
-      PianoRoll(PartList*, QWidget* parent = 0, const char* name = 0);
+      PianoRoll(PartList*, QWidget* parent = 0, const char* name = 0, unsigned initPos = MAXINT);
       ~PianoRoll();
       virtual void readStatus(Xml&);
       virtual void writeStatus(int, Xml&) const;
