@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//    $Id: prcanvas.cpp,v 1.20.2.6 2006/01/05 18:14:39 spamatica Exp $
+//    $Id: prcanvas.cpp,v 1.20.2.19 2009/11/16 11:29:33 lunar_shuttle Exp $
 //  (C) Copyright 1999-2004 Werner Schweer (ws@seh.de)
 //=========================================================
 
@@ -9,13 +9,18 @@
 #include <qclipboard.h>
 #include <qpainter.h>
 #include <qdragobject.h>
+#include <qmessagebox.h>
 
 #include <values.h>
 #include <stdio.h>
 #include <math.h>
 #include <errno.h>
 #include <sys/stat.h>
+#include <sys/types.h>
+#include <sys/stat.h>
 #include <sys/mman.h>
+#include <fcntl.h>
+#include <dirent.h>
 
 #include "xml.h"
 #include "prcanvas.h"
@@ -51,10 +56,10 @@ void PianoCanvas::addItem(Part* part, Event& event)
             printf("ERROR: trying to add event before current part!\n");
             return;
       }
-      
+
       NEvent* ev = new NEvent(event, part, pitch2y(event.pitch()));
       items.add(ev);
-      
+
       int diff = event.endTick()-part->lenTick();
       if (diff > 0)  {// too short part? extend it
             //printf("addItem - this code should not be run!\n");
@@ -128,11 +133,15 @@ int PianoCanvas::y2pitch(int y) const
 //---------------------------------------------------------
 
 void PianoCanvas::drawItem(QPainter& p, const CItem* item,
-   const QRect&)
+   const QRect& rect)
       {
-      p.setPen(black);
       QRect r = item->bbox();
-
+      if(!virt())
+        r.moveCenter(map(item->pos()));
+      r = r.intersect(rect);
+      if(!r.isValid())
+        return;
+      p.setPen(black);
       struct Triple {
             int r, g, b;
             };
@@ -153,22 +162,23 @@ void PianoCanvas::drawItem(QPainter& p, const CItem* item,
 
       NEvent* nevent   = (NEvent*) item;
       Event event = nevent->event();
-
-      QColor color;
-      if (nevent->part() != curPart)
-            p.setBrush(lightGray);
+      if (nevent->part() != curPart){
+            if(item->isMoving()) 
+              p.setBrush(gray);
+            else if(item->isSelected()) 
+              p.setBrush(black);
+            else  
+              p.setBrush(lightGray);
+      }      
       else {
             if (item->isMoving()) {
-                  p.setBrush(gray);
-                  p.drawRect(r);
-                  p.setBrush(NoBrush);
-                  p.drawRect(item->mp().x(), item->mp().y() - item->height()/2, item->width(), item->height());
-                  return;
-                  }
+                    p.setBrush(gray);
+                }
             else if (item->isSelected()) {
                   p.setBrush(black);
                   }
             else {
+                  QColor color;
                   color.setRgb(0, 0, 255);
                   switch(colorMode) {
                         case 0:
@@ -195,6 +205,27 @@ void PianoCanvas::drawItem(QPainter& p, const CItem* item,
       p.drawRect(r);
       }
 
+
+//---------------------------------------------------------
+//   drawMoving
+//    draws moving items
+//---------------------------------------------------------
+
+void PianoCanvas::drawMoving(QPainter& p, const CItem* item, const QRect& rect)
+    {
+      //if(((NEvent*)item)->part() != curPart)
+      //  return;
+      //if(!item->isMoving()) 
+      //  return;
+      QRect mr = QRect(item->mp().x(), item->mp().y() - item->height()/2, item->width(), item->height());
+      mr = mr.intersect(rect);
+      if(!mr.isValid())
+        return;
+      p.setPen(black);
+      p.setBrush(NoBrush);
+      p.drawRect(mr);
+    }
+
 //---------------------------------------------------------
 //   viewMouseDoubleClickEvent
 //---------------------------------------------------------
@@ -207,11 +238,197 @@ void PianoCanvas::viewMouseDoubleClickEvent(QMouseEvent* event)
             }
       }
 
+//---------------------------------------------------------
+//   moveCanvasItems
+//---------------------------------------------------------
+
+void PianoCanvas::moveCanvasItems(CItemList& items, int dp, int dx, DragType dtype, int* pflags)
+{      
+  if(editor->parts()->empty())
+    return;
+    
+  PartsToChangeMap parts2change;
+  
+  int modified = 0;
+  for(iPart ip = editor->parts()->begin(); ip != editor->parts()->end(); ++ip)
+  {
+    Part* part = ip->second;
+    if(!part)
+      continue;
+    
+    int npartoffset = 0;
+    for(iCItem ici = items.begin(); ici != items.end(); ++ici) 
+    {
+      CItem* ci = ici->second;
+      if(ci->part() != part)
+        continue;
+      
+      int x = ci->pos().x() + dx;
+      int y = pitch2y(y2pitch(ci->pos().y()) + dp);
+      QPoint newpos = raster(QPoint(x, y));
+      
+      // Test moving the item...
+      NEvent* nevent = (NEvent*) ci;
+      Event event    = nevent->event();
+      x              = newpos.x();
+      if(x < 0)
+        x = 0;
+      int ntick = editor->rasterVal(x) - part->tick();
+      if(ntick < 0)
+        ntick = 0;
+      int diff = ntick + event.lenTick() - part->lenTick();
+      
+      // If moving the item would require a new part size...
+      if(diff > npartoffset)
+        npartoffset = diff;
+    }
+        
+    if(npartoffset > 0)
+    {    
+      // Create new part...
+      // if there are several events that are moved outside the part, it will be recreated for each
+      // so the part _in_ the event will not be valid, ask the authority.
+//      Part* newPart = part->clone();
+      //Part* newPart = Canvas::part()->clone();
+
+//      newPart->setLenTick(newPart->lenTick() + npartoffset);
+      //audio->msgChangePart(part, newPart,false);
+
+//      modified = SC_PART_MODIFIED;
+
+      // BUG FIX: #1650953
+      // Added by T356.
+      // Fixes posted "select and drag past end of part - crashing" bug
+//      for(iPart ip = editor->parts()->begin(); ip != editor->parts()->end(); ++ip)
+//      {
+//        if(ip->second == part)
+//        {
+//          editor->parts()->erase(ip);
+//          break;
+//        }
+//      }
+      
+//      editor->parts()->add(newPart);
+//      audio->msgChangePart(part, newPart,false);
+      
+      //if(parts2change.find(part) == parts2change.end())
+      //  parts2change.insert(std::pair<Part*, Part*> (part, newPart));
+      iPartToChange ip2c = parts2change.find(part);
+      if(ip2c == parts2change.end())
+      {
+        PartToChange p2c = {0, npartoffset};
+        parts2change.insert(std::pair<Part*, PartToChange> (part, p2c));
+      }
+      else
+        ip2c->second.xdiff = npartoffset;
+      
+      //part = newPart; // reassign
+      //item->setPart(part);
+      //item->setEvent(newEvent);
+      //curPart = part;
+      //curPartId = curPart->sn();
+
+    }
+  }
+  
+  for(iPartToChange ip2c = parts2change.begin(); ip2c != parts2change.end(); ++ip2c)
+  {
+    Part* opart = ip2c->first;
+    int diff = ip2c->second.xdiff;
+    
+    Part* newPart = opart->clone();
+    
+    newPart->setLenTick(newPart->lenTick() + diff);
+    
+    modified = SC_PART_MODIFIED;
+    
+    // BUG FIX: #1650953
+    // Added by T356.
+    // Fixes posted "select and drag past end of part - crashing" bug
+    for(iPart ip = editor->parts()->begin(); ip != editor->parts()->end(); ++ip)
+    {
+      if(ip->second == opart)
+      {
+        editor->parts()->erase(ip);
+        break;
+      }
+    }
+      
+    editor->parts()->add(newPart);
+    // Indicate no undo, and do port controller values but not clone parts. 
+    audio->msgChangePart(opart, newPart, false, true, false);
+    
+    ip2c->second.npart = newPart;
+    
+  }
+    
+  iPartToChange icp = parts2change.find(curPart);
+  if(icp != parts2change.end())
+  {
+    curPart = icp->second.npart;
+    curPartId = curPart->sn();
+  }  
+    
+  std::vector< CItem* > doneList;
+  typedef std::vector< CItem* >::iterator iDoneList;
+  
+  for(iCItem ici = items.begin(); ici != items.end(); ++ici) 
+  {
+    CItem* ci = ici->second;
+    
+    // If this item's part is in the parts2change list, change the item's part to the new part.
+    Part* pt = ci->part();
+    iPartToChange ip2c = parts2change.find(pt);
+    if(ip2c != parts2change.end())
+      ci->setPart(ip2c->second.npart);
+    
+    int x = ci->pos().x();
+    int y = ci->pos().y();
+    int nx = x + dx;
+    int ny = pitch2y(y2pitch(y) + dp);
+    QPoint newpos = raster(QPoint(nx, ny));
+    selectItem(ci, true);
+    
+    iDoneList idl;
+    for(idl = doneList.begin(); idl != doneList.end(); ++idl)
+      // This compares EventBase pointers to see if they're the same...
+      if((*idl)->event() == ci->event())
+        break;
+      
+    // Do not process if the event has already been processed (meaning it's an event in a clone part)...
+    //if(moveItem(ci, newpos, dtype))
+    if(idl != doneList.end())
+      // Just move the canvas item.
+      ci->move(newpos);
+    else
+    {
+      // Currently moveItem always returns true.
+      if(moveItem(ci, newpos, dtype))
+      {
+        // Add the canvas item to the list of done items.
+        doneList.push_back(ci);
+        // Move the canvas item.
+        ci->move(newpos);
+      }  
+    }
+          
+    if(moving.size() == 1) 
+          itemReleased(curItem, newpos);
+    if(dtype == MOVE_COPY || dtype == MOVE_CLONE)
+          selectItem(ci, false);
+  }  
+      
+  if(pflags)
+    *pflags = modified;
+}
+      
 //---------------------------------------------------------
 //   moveItem
 //    called after moving an object
 //---------------------------------------------------------
 
+// Changed by T356. 
+//bool PianoCanvas::moveItem(CItem* item, const QPoint& pos, DragType dtype, int* pflags)
 bool PianoCanvas::moveItem(CItem* item, const QPoint& pos, DragType dtype)
       {
       NEvent* nevent = (NEvent*) item;
@@ -221,7 +438,6 @@ bool PianoCanvas::moveItem(CItem* item, const QPoint& pos, DragType dtype)
       int x          = pos.x();
       if (x < 0)
             x = 0;
-
       if (event.pitch() != npitch && _playEvents) {
             int port    = track()->outPort();
             int channel = track()->outChannel();
@@ -231,8 +447,11 @@ bool PianoCanvas::moveItem(CItem* item, const QPoint& pos, DragType dtype)
             MidiPlayEvent ev2(0, port, channel, 0x90, npitch + track()->transposition, event.velo());
             audio->msgPlayMidiEvent(&ev2);
             }
-      //Part* part = nevent->part(); //
-      Part * part = Canvas::part();  // part can be dynamically recreated, ask the authority
+      
+      // Changed by T356. 
+      Part* part = nevent->part(); //
+      //Part * part = Canvas::part();  // part can be dynamically recreated, ask the authority
+      
       newEvent.setPitch(npitch);
       int ntick = editor->rasterVal(x) - part->tick();
       if (ntick < 0)
@@ -240,29 +459,66 @@ bool PianoCanvas::moveItem(CItem* item, const QPoint& pos, DragType dtype)
       newEvent.setTick(ntick);
       newEvent.setLenTick(event.lenTick());
 
-
-
-      int modified=SC_EVENT_MODIFIED;
-      song->startUndo();
+      // Removed by T356. 
+      /*
+      int modified=0;
+      //song->startUndo();
       int diff = newEvent.endTick()-part->lenTick();
-      if (diff > 0)  {// too short part? extend it
+      if (diff > 0){// too short part? extend it
             // if there are several events that are moved outside the part, it will be recreated for each
             // so the part _in_ the event will not be valid, ask the authority.
             //Part* newPart = part->clone();
             Part* newPart = Canvas::part()->clone();
-            
+
             newPart->setLenTick(newPart->lenTick()+diff);
             audio->msgChangePart(Canvas::part(), newPart,false);
-            modified=modified|SC_PART_MODIFIED;
+      
+            modified = SC_PART_MODIFIED;
             part = newPart; // reassign
+
+            // BUG FIX: #1650953
+            // Added by T356.
+            // Fixes posted "select and drag past end of part - crashing" bug
+            for(iPart i = editor->parts()->begin(); i != editor->parts()->end(); ++i)
+            {
+              if(i->second == Canvas::part())
+              {
+                editor->parts()->erase(i);
+                break;
+              }
             }
+            editor->parts()->add(part);
+            item->setPart(part);
+            item->setEvent(newEvent);
+            curPart = part;
+            curPartId = curPart->sn();
 
+            }
+      */
+
+      // Added by T356. 
+      // msgAddEvent and msgChangeEvent (below) will set these, but set them here first?
+      //item->setPart(part);
+      item->setEvent(newEvent);
+      
+      // Added by T356. 
+      if(((int)newEvent.endTick() - (int)part->lenTick()) > 0)  
+        printf("PianoCanvas::moveItem Error! New event end:%d exceeds length:%d of part:%s\n", newEvent.endTick(), part->lenTick(), part->name().latin1());
+      
       if (dtype == MOVE_COPY || dtype == MOVE_CLONE)
-            audio->msgAddEvent(newEvent, part, false); 
+            // Indicate no undo, and do not do port controller values and clone parts. 
+            //audio->msgAddEvent(newEvent, part, false);
+            audio->msgAddEvent(newEvent, part, false, false, false);
       else
-            audio->msgChangeEvent(event, newEvent, part, false); 
-      song->endUndo(modified);
+            // Indicate no undo, and do not do port controller values and clone parts. 
+            //audio->msgChangeEvent(event, newEvent, part, false);
+            audio->msgChangeEvent(event, newEvent, part, false, false, false);
+      //song->endUndo(modified);
 
+      // Removed by T356. 
+      //if(pflags)
+      //  *pflags = modified;
+      
       return true;
       }
 
@@ -277,7 +533,7 @@ CItem* PianoCanvas::newItem(const QPoint& p, int)
       int tick  = editor->rasterVal1(p.x());
       int len   = p.x() - tick;
       tick     -= curPart->tick();
-      if (tick < 0) 
+      if (tick < 0)
             tick=0;
       Event e =  Event(Note);
       e.setTick(tick);
@@ -307,7 +563,7 @@ void PianoCanvas::newItem(CItem* item, bool noSnap)
       event.setTick(x - part->tick());
       event.setLenTick(w);
       event.setPitch(y2pitch(item->y()));
-      
+
       song->startUndo();
       int modified=SC_EVENT_MODIFIED;
       int diff = event.endTick()-part->lenTick();
@@ -315,11 +571,15 @@ void PianoCanvas::newItem(CItem* item, bool noSnap)
             //printf("extend Part!\n");
             Part* newPart = part->clone();
             newPart->setLenTick(newPart->lenTick()+diff);
-            audio->msgChangePart(part, newPart,false);
+            // Indicate no undo, and do port controller values but not clone parts. 
+            //audio->msgChangePart(part, newPart,false);
+            audio->msgChangePart(part, newPart, false, true, false);
             modified=modified|SC_PART_MODIFIED;
             part = newPart; // reassign
             }
-      audio->msgAddEvent(event, part,false); 
+      // Indicate no undo, and do not do port controller values and clone parts. 
+      //audio->msgAddEvent(event, part,false);
+      audio->msgAddEvent(event, part, false, false, false);
       song->endUndo(modified);
       }
 
@@ -354,13 +614,17 @@ void PianoCanvas::resizeItem(CItem* item, bool noSnap)         // experimental c
             //printf("extend Part!\n");
             Part* newPart = part->clone();
             newPart->setLenTick(newPart->lenTick()+diff);
-            audio->msgChangePart(part, newPart,false);
+            // Indicate no undo, and do port controller values but not clone parts. 
+            //audio->msgChangePart(part, newPart,false);
+            audio->msgChangePart(part, newPart, false, true, false);
             modified=modified|SC_PART_MODIFIED;
             part = newPart; // reassign
             }
 
       newEvent.setLenTick(len);
-      audio->msgChangeEvent(event, newEvent, nevent->part(),false); 
+      // Indicate no undo, and do not do port controller values and clone parts. 
+      //audio->msgChangeEvent(event, newEvent, nevent->part(),false);
+      audio->msgChangeEvent(event, newEvent, nevent->part(), false, false, false);
       song->endUndo(modified);
       }
 
@@ -373,7 +637,9 @@ bool PianoCanvas::deleteItem(CItem* item)
       NEvent* nevent = (NEvent*) item;
       if (nevent->part() == curPart) {
             Event ev = nevent->event();
-            audio->msgDeleteEvent(ev, curPart);
+            // Indicate do undo, and do not do port controller values and clone parts. 
+            //audio->msgDeleteEvent(ev, curPart);
+            audio->msgDeleteEvent(ev, curPart, true, false, false);
             return true;
             }
       return false;
@@ -421,7 +687,9 @@ void PianoCanvas::pianoCmd(int cmd)
                         Event event = *i;
                         Event newEvent = event.clone();
                         newEvent.setTick(event.tick() + editor->raster());// - part->tick());
-                        audio->msgChangeEvent(event, newEvent, part, false);
+                        // Indicate no undo, and do not do port controller values and clone parts. 
+                        //audio->msgChangeEvent(event, newEvent, part, false);
+                        audio->msgChangeEvent(event, newEvent, part, false, false, false);
                         }
                   song->endUndo(SC_EVENT_MODIFIED);
                   Pos p(editor->rasterVal(pos[0] + editor->rasterStep(pos[0])), true);
@@ -445,7 +713,9 @@ void PianoCanvas::pianoCmd(int cmd)
                         Event event = *i;
                         Event newEvent = event.clone();
                         newEvent.setTick(event.tick() - editor->raster() - part->tick());
-                        audio->msgChangeEvent(event, newEvent, part, false);
+                        // Indicate no undo, and do not do port controller values and clone parts. 
+                        //audio->msgChangeEvent(event, newEvent, part, false);
+                        audio->msgChangeEvent(event, newEvent, part, false, false, false);
                         }
                   song->endUndo(SC_EVENT_MODIFIED);
                   Pos p(editor->rasterVal(pos[0] - editor->rasterStep(pos[0])), true);
@@ -482,7 +752,9 @@ void PianoCanvas::pianoPressed(int pitch, int velocity, bool shift)
             e.setPitch(pitch);
             e.setVelo(127);
             e.setLenTick(len);
-            audio->msgAddEvent(e, curPart);
+            // Indicate do undo, and do not do port controller values and clone parts. 
+            //audio->msgAddEvent(e, curPart);
+            audio->msgAddEvent(e, curPart, true, false, false);
             tick += editor->rasterStep(tick) + curPart->tick();
             if (tick != song->cpos()) {
                   Pos p(tick, true);
@@ -615,7 +887,9 @@ void PianoCanvas::cmd(int cmd, int quantStrength,
                               continue;
                         NEvent* e = (NEvent*)(i->second);
                         Event ev  = e->event();
-                        audio->msgDeleteEvent(ev, e->part(), false);
+                        // Indicate no undo, and do not do port controller values and clone parts. 
+                        //audio->msgDeleteEvent(ev, e->part(), false);
+                        audio->msgDeleteEvent(ev, e->part(), false, false, false);
                         }
                   song->endUndo(SC_EVENT_REMOVED);
                   break;
@@ -632,7 +906,9 @@ void PianoCanvas::cmd(int cmd, int quantStrength,
                               if (!i->second->isSelected())
                                     continue;
                               Event ev = i->second->event();
-                              audio->msgDeleteEvent(ev, i->second->part(), false);
+                              // Indicate no undo, and do not do port controller values and clone parts. 
+                              //audio->msgDeleteEvent(ev, i->second->part(), false);
+                              audio->msgDeleteEvent(ev, i->second->part(), false, false, false);
                               }
                         song->endUndo(SC_EVENT_REMOVED);
                         }
@@ -666,8 +942,9 @@ void PianoCanvas::cmd(int cmd, int quantStrength,
             case CMD_SELECT_ILOOP:     // select inside loop
                   for (iCItem k = items.begin(); k != items.end(); ++k) {
                         NEvent* nevent = (NEvent*)(k->second);
+                        Part* part     = nevent->part();
                         Event event    = nevent->event();
-                        unsigned tick  = event.tick();
+                        unsigned tick  = event.tick() + part->tick();
                         if (tick < song->lpos() || tick >= song->rpos())
                               selectItem(k->second, false);
                         else
@@ -677,8 +954,9 @@ void PianoCanvas::cmd(int cmd, int quantStrength,
             case CMD_SELECT_OLOOP:     // select outside loop
                   for (iCItem k = items.begin(); k != items.end(); ++k) {
                         NEvent* nevent = (NEvent*)(k->second);
+                        Part* part     = nevent->part();
                         Event event    = nevent->event();
-                        unsigned tick  = event.tick();
+                        unsigned tick  = event.tick() + part->tick();
                         if (tick < song->lpos() || tick >= song->rpos())
                               selectItem(k->second, true);
                         else
@@ -709,17 +987,19 @@ void PianoCanvas::cmd(int cmd, int quantStrength,
                            || (range == 1 && selected)
                            || (range == 2 && inLoop)
                            || (range == 3 && selected && inLoop)) {
-                              int len   = event.lenTick();
+                              unsigned int len   = event.lenTick(); //prevent compiler warning: comparison singed/unsigned
 
                               len = rate ? (len * 100) / rate : 1;
                               len += offset;
-                              if (len <= 1)
+                              if (len < 1)
                                     len = 1;
 
                               if (event.lenTick() != len) {
                                     Event newEvent = event.clone();
                                     newEvent.setLenTick(len);
-                                    audio->msgChangeEvent(event, newEvent, nevent->part(), false);
+                                    // Indicate no undo, and do not do port controller values and clone parts. 
+                                    //audio->msgChangeEvent(event, newEvent, nevent->part(), false);
+                                    audio->msgChangeEvent(event, newEvent, nevent->part(), false, false, false);
                                     }
                               }
                         }
@@ -764,7 +1044,9 @@ void PianoCanvas::cmd(int cmd, int quantStrength,
                               if (event.velo() != velo) {
                                     Event newEvent = event.clone();
                                     newEvent.setVelo(velo);
-                                    audio->msgChangeEvent(event, newEvent, nevent->part(), false);
+                                    // Indicate no undo, and do not do port controller values and clone parts. 
+                                    //audio->msgChangeEvent(event, newEvent, nevent->part(), false);
+                                    audio->msgChangeEvent(event, newEvent, nevent->part(), false, false, false);
                                     }
                               }
                         }
@@ -782,12 +1064,84 @@ void PianoCanvas::cmd(int cmd, int quantStrength,
                               Event event = nevent->event();
                               Event newEvent = event.clone();
                               newEvent.setLenTick(editor->raster());
-                              audio->msgChangeEvent(event, newEvent, nevent->part() , false);
+                              // Indicate no undo, and do not do port controller values and clone parts. 
+                              //audio->msgChangeEvent(event, newEvent, nevent->part() , false);
+                              audio->msgChangeEvent(event, newEvent, nevent->part(), false, false, false);
                               }
                         }
                   song->endUndo(SC_EVENT_MODIFIED);
                   break;
 
+            case CMD_DELETE_OVERLAPS:
+                  if (!selectionSize())
+                        break;
+
+                  song->startUndo();
+                  for (iCItem k = items.begin(); k != items.end(); k++) {
+                      if (k->second->isSelected() == false)
+                          continue;
+
+                      NEvent* e1 = (NEvent*) (k->second); // first note
+                      NEvent* e2 = NULL; // ptr to next selected note (which will be checked for overlap)
+                      Event ce1  = e1->event();
+                      Event ce2;
+
+                      if (ce1.type() != Note)
+                          continue;
+
+                      // Find next selected item on the same pitch
+                      iCItem l = k; l++;
+                      for (; l != items.end(); l++) {
+                          if (l->second->isSelected() == false)
+                              continue;
+
+                          e2 = (NEvent*) l->second;
+                          ce2  = e2->event();
+
+                          // Same pitch?
+                          if (ce1.dataA() == ce2.dataA())
+                              break;
+
+                          // If the note has the same len and place we treat it as a duplicate note and not a following note
+                          // The best thing to do would probably be to delete the duplicate note, we just want to avoid
+                          // matching against the same note
+                          if (   ce1.tick() + e1->part()->tick() == ce2.tick() + e2->part()->tick() 
+                              && ce1.lenTick() + e1->part()->tick() == ce2.lenTick() + e2->part()->tick())
+                          {
+                              e2 = NULL; // this wasn't what we were looking for
+                              continue;
+                          }
+
+                          }
+
+                      if (e2 == NULL) // None found
+                          break;
+
+                      Part* part1 = e1->part();
+                      Part* part2 = e2->part();
+                      if (ce2.type() != Note)
+                          continue;
+
+
+                      unsigned event1pos = ce1.tick() + part1->tick();
+                      unsigned event1end = event1pos + ce1.lenTick();
+                      unsigned event2pos = ce2.tick() + part2->tick();
+
+                      //printf("event1pos %u event1end %u event2pos %u\n", event1pos, event1end, event2pos);
+                      if (event1end > event2pos) {
+                          Event newEvent = ce1.clone();
+                          unsigned newlen = ce1.lenTick() - (event1end - event2pos);
+                          //printf("newlen: %u\n", newlen);
+                          newEvent.setLenTick(newlen);
+                          // Indicate no undo, and do not do port controller values and clone parts. 
+                          //audio->msgChangeEvent(ce1, newEvent, e1->part(), false);
+                          audio->msgChangeEvent(ce1, newEvent, e1->part(), false, false, false);
+                          }
+                      }
+                  song->endUndo(SC_EVENT_MODIFIED);
+                  break;
+
+
             case CMD_CRESCENDO:
             case CMD_TRANSPOSE:
             case CMD_THIN_OUT:
@@ -830,7 +1184,7 @@ void PianoCanvas::quantize(int strength, int limit, bool quantLen)
                && ((tick < song->lpos() || tick >= song->rpos())))
                   continue;
 
-            int len   = event.lenTick();
+            unsigned int len   = event.lenTick(); //prevent compiler warning: comparison singed/unsigned
             int tick2 = tick + len;
 
             // quant start position
@@ -848,7 +1202,9 @@ void PianoCanvas::quantize(int strength, int limit, bool quantLen)
                   Event newEvent = event.clone();
                   newEvent.setTick(tick - part->tick());
                   newEvent.setLenTick(len);
-                  audio->msgChangeEvent(event, newEvent, part, false);
+                  // Indicate no undo, and do not do port controller values and clone parts. 
+                  //audio->msgChangeEvent(event, newEvent, part, false);
+                  audio->msgChangeEvent(event, newEvent, part, false, false, false);
                   }
             }
       song->endUndo(SC_EVENT_MODIFIED);
@@ -864,7 +1220,7 @@ void PianoCanvas::midiNote(int pitch, int velo)
          && !audio->isPlaying() && velo && pos[0] >= start_tick
          && pos[0] < end_tick
          && !(globalKeyState & AltButton)) {
-            int len            = editor->quant();
+            unsigned int len   = editor->quant();//prevent compiler warning: comparison singed/unsigned
             unsigned tick      = pos[0]; //CDW
             unsigned starttick = tick;
             if (globalKeyState & ShiftButton)
@@ -879,10 +1235,12 @@ void PianoCanvas::midiNote(int pitch, int velo)
                         Event ev = i->second;
                         if (!ev.isNote())
                               continue;
-                        if (ev.pitch() == pitch && ((ev.tick() + ev.lenTick()) == (int)starttick)) {
+                        if (ev.pitch() == pitch && ((ev.tick() + ev.lenTick()) == /*(int)*/starttick)) {
                               Event e = ev.clone();
                               e.setLenTick(ev.lenTick() + editor->rasterStep(starttick));
-                              audio->msgChangeEvent(ev, e, curPart);
+                              // Indicate do undo, and do not do port controller values and clone parts. 
+                              //audio->msgChangeEvent(ev, e, curPart);
+                              audio->msgChangeEvent(ev, e, curPart, true, false, false);
                               tick += editor->rasterStep(tick);
                               if (tick != song->cpos()) {
                                     Pos p(tick, true);
@@ -900,7 +1258,9 @@ void PianoCanvas::midiNote(int pitch, int velo)
             for (iEvent i = range.first; i != range.second; ++i) {
                   Event ev = i->second;
                   if (ev.isNote() && ev.pitch() == pitch) {
-                        audio->msgDeleteEvent(ev, curPart);
+                        // Indicate do undo, and do not do port controller values and clone parts. 
+                        //audio->msgDeleteEvent(ev, curPart);
+                        audio->msgDeleteEvent(ev, curPart, true, false, false);
                         if (globalKeyState & ShiftButton)
                               tick += editor->rasterStep(tick);
                         return;
@@ -911,7 +1271,9 @@ void PianoCanvas::midiNote(int pitch, int velo)
             e.setPitch(pitch);
             e.setVelo(velo);
             e.setLenTick(len);
-            audio->msgAddEvent(e, curPart);
+            // Indicate do undo, and do not do port controller values and clone parts. 
+            //audio->msgAddEvent(e, curPart);
+            audio->msgAddEvent(e, curPart, true, false, false);
             tick += editor->rasterStep(tick);
             if (tick != song->cpos()) {
                   Pos p(tick, true);
@@ -1019,22 +1381,27 @@ void PianoCanvas::pasteAt(const QString& pt, int pos)
                               for (iEvent i = el->begin(); i != el->end(); ++i) {
                                     Event e = i->second;
                                     int tick = e.tick() + pos - curPart->tick();
-                                    if (tick<0) { 
+                                    if (tick<0) {
                                             printf("ERROR: trying to add event before current part!\n");
                                             song->endUndo(SC_EVENT_INSERTED);
                                             delete el;
                                             return;
                                             }
+
+                                    e.setTick(tick);
                                     int diff = e.endTick()-curPart->lenTick();
                                     if (diff > 0)  {// too short part? extend it
                                             Part* newPart = curPart->clone();
                                             newPart->setLenTick(newPart->lenTick()+diff);
-                                            audio->msgChangePart(curPart, newPart,false);
+                                            // Indicate no undo, and do port controller values but not clone parts. 
+                                            //audio->msgChangePart(curPart, newPart,false);
+                                            audio->msgChangePart(curPart, newPart, false, true, false);
                                             modified=modified|SC_PART_MODIFIED;
                                             curPart = newPart; // reassign
                                             }
-                                    e.setTick(tick);
-                                    audio->msgAddEvent(e, curPart, false);
+                                    // Indicate no undo, and do not do port controller values and clone parts. 
+                                    //audio->msgAddEvent(e, curPart, false);
+                                    audio->msgAddEvent(e, curPart, false, false, false);
                                     }
                               song->endUndo(modified);
                               delete el;
@@ -1050,7 +1417,6 @@ void PianoCanvas::pasteAt(const QString& pt, int pos)
                   }
             }
       }
-
 //---------------------------------------------------------
 //   paste
 //    paste events
@@ -1272,7 +1638,9 @@ void PianoCanvas::modifySelected(NoteInfo::ValType type, int delta)
                         break;
                   }
             song->changeEvent(event, newEvent, part);
-            song->undoOp(UndoOp::ModifyEvent, newEvent, event, part);
+            // Indicate do not do port controller values and clone parts. 
+            //song->undoOp(UndoOp::ModifyEvent, newEvent, event, part);
+            song->undoOp(UndoOp::ModifyEvent, newEvent, event, part, false, false);
             }
       song->endUndo(SC_EVENT_MODIFIED);
       audio->msgIdle(false);
