@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//  $Id: midiseq.cpp,v 1.30.2.12 2006/03/02 22:27:23 spamatica Exp $
+//  $Id: midiseq.cpp,v 1.30.2.21 2009/12/20 05:00:35 terminator356 Exp $
 //
 //    high priority task for scheduling midi events
 //
@@ -12,6 +12,7 @@
 #include <fcntl.h>
 #include <sys/ioctl.h>
 #include <poll.h>
+#include <math.h>
 
 #include "globals.h"
 #include "midi.h"
@@ -28,6 +29,8 @@
 
 MidiSeq* midiSeq;
 int MidiSeq::ticker = 0;
+volatile bool midiBusy=false;
+
 
 //---------------------------------------------------------
 //   readMsg
@@ -84,7 +87,26 @@ void MidiSeq::processMsg(const ThreadMsg* m)
                   song->cmdRemovePart((Part*)msg->p1);
                   break;
             case SEQM_CHANGE_PART:
-                  song->cmdChangePart((Part*)msg->p1, (Part*)msg->p2);
+                  //song->cmdChangePart((Part*)msg->p1, (Part*)msg->p2);
+                  song->cmdChangePart((Part*)msg->p1, (Part*)msg->p2, msg->a, msg->b);
+                  break;
+            case SEQM_SET_TRACK_OUT_CHAN:
+                  {
+                  MidiTrack* track = (MidiTrack*)(msg->p1);
+                  track->setOutChanAndUpdate(msg->a);
+                  }
+                  break;
+            case SEQM_SET_TRACK_OUT_PORT:
+                  {
+                  MidiTrack* track = (MidiTrack*)(msg->p1);
+                  track->setOutPortAndUpdate(msg->a);
+                  }
+                  break;
+            case SEQM_REMAP_PORT_DRUM_CTL_EVS:
+                    song->remapPortDrumCtrlEvents(msg->ival, msg->a, msg->b, msg->c);
+                  break;
+            case SEQM_CHANGE_ALL_PORT_DRUM_CTL_EVS:      
+                    song->changeAllPortDrumCtrlEvents((bool)msg->a, (bool)msg->b);
                   break;
             case SEQM_SET_MIDI_DEVICE:
                   ((MidiPort*)(msg->p1))->setMidiDevice((MidiDevice*)(msg->p2));
@@ -162,14 +184,24 @@ void MidiSeq::processSeek()
                   }
             else
                   el->erase(el->begin(), dev->nextPlayEvent());
+            
             for (iMidiCtrlValList ivl = cll->begin(); ivl != cll->end(); ++ivl) {
                   MidiCtrlValList* vl = ivl->second;
-                  int val = vl->value(pos);
-                  if (val != CTRL_VAL_UNKNOWN) {
-                        int channel = ivl->first >> 24;
-                        el->add(MidiPlayEvent(0, port, channel, ME_CONTROLLER, vl->num(), val));
-                        }
+                  //int val = vl->value(pos);
+                  //if (val != CTRL_VAL_UNKNOWN) {
+                  //      int channel = ivl->first >> 24;
+                  //      el->add(MidiPlayEvent(0, port, channel, ME_CONTROLLER, vl->num(), val));
+                  //      }
+                  iMidiCtrlVal imcv = vl->iValue(pos);
+                  if(imcv != vl->end()) 
+                  {
+                    Part* p = imcv->second.part;
+                    unsigned t = (unsigned)imcv->first;
+                    // Do not add values that are outside of the part.
+                    if(p && t >= p->tick() && t < (p->tick() + p->lenTick()) )
+                      el->add(MidiPlayEvent(0, port, ivl->first >> 24, ME_CONTROLLER, vl->num(), imcv->second.val));
                   }
+                }
             dev->setNextPlayEvent(el->begin());
             }
       }
@@ -178,10 +210,15 @@ void MidiSeq::processSeek()
 //   MidiSeq
 //---------------------------------------------------------
 
-MidiSeq::MidiSeq(int priority, const char* name)
-   : Thread(priority, name)
+//MidiSeq::MidiSeq(int priority, const char* name)
+//   : Thread(priority, name)
+MidiSeq::MidiSeq(const char* name)
+   : Thread(name)
       {
-      prio = priority;
+      // Changed by Tim. p3.3.17
+      //prio = priority;
+      prio = 0;
+      
       idle = false;
       midiClock = 0;
       mclock1 = 0.0;
@@ -243,6 +280,8 @@ signed int MidiSeq::selectTimer()
 
 void MidiSeq::threadStart(void*)
       {
+      // Removed by Tim. p3.3.17
+      /*
       struct sched_param rt_param;
       memset(&rt_param, 0, sizeof(rt_param));
       int prio_min = sched_get_priority_min(SCHED_FIFO);
@@ -255,6 +294,7 @@ void MidiSeq::threadStart(void*)
       int rv = pthread_setschedparam(pthread_self(), SCHED_FIFO, &rt_param);
       if (rv != 0)
             perror("set realtime scheduler");
+      */
 
       int policy;
       if ((policy = sched_getscheduler (0)) < 0) {
@@ -336,8 +376,10 @@ void MidiSeq::updatePollFd()
             int port = dev->midiPort();
             if (port == -1)
                   continue;
-            if ((dev->rwFlags()&0x2) || (extSyncFlag.value()
-               && (rxSyncPort == port || rxSyncPort == -1))) {
+            if ((dev->rwFlags() & 0x2) || (extSyncFlag.value()
+               //&& (rxSyncPort == port || rxSyncPort == -1))) {
+               //&& (dev->syncInfo().MCIn()))) {
+               && (midiPorts[port].syncInfo().MCIn()))) {
                   addPollFd(dev->selectRfd(), POLLIN, ::midiRead, this, dev);
                   }
             if (dev->bytesToWrite())
@@ -383,8 +425,13 @@ bool MidiSeq::setRtcTicks()
 //    return true on error
 //---------------------------------------------------------
 
-bool MidiSeq::start()
+//bool MidiSeq::start()
+void MidiSeq::start(int priority)
       {
+      // Changed by Tim. p3.3.17
+      
+      prio = priority;
+      
       //timerFd = -1;
 
       doSetuid();
@@ -393,8 +440,9 @@ bool MidiSeq::start()
       //printf("timerFd=%d\n",timerFd);
       setRtcTicks();
       undoSetuid();
-      Thread::start();
-      return false;
+      //Thread::start();
+      Thread::start(priority);
+      //return false;
       }
 
 //---------------------------------------------------------
@@ -403,9 +451,10 @@ bool MidiSeq::start()
 
 void MidiSeq::processMidiClock()
       {
-      if (genMCSync) {
-            midiPorts[txSyncPort].sendClock();
-      }
+//      if (genMCSync) {
+//            midiPorts[txSyncPort].sendClock();
+//      }
+
 /*      if (state == START_PLAY) {
             // start play on sync
             state      = PLAY;
@@ -429,7 +478,7 @@ void MidiSeq::processMidiClock()
             startRecordPos.setPosTick(playTickPos);
             }
 */
-      midiClock += config.division/24;
+//      midiClock += config.division/24;
       }
 
 //---------------------------------------------------------
@@ -456,13 +505,15 @@ void MidiSeq::midiTick(void* p, void*)
 
 void MidiSeq::processTimerTick()
       {
-      extern int watchMidi;
-      ++watchMidi;      // make a simple watchdog happy
+      // Disabled by Tim. p3.3.22
+//      extern int watchMidi;
+//      ++watchMidi;      // make a simple watchdog happy
 
       //---------------------------------------------------
       //    read elapsed rtc timer ticks
       //---------------------------------------------------
 
+      // This is required otherwise it freezes.
       unsigned long nn;
       if (timerFd != -1) {
             nn = timer->getTimerTicks();
@@ -471,23 +522,143 @@ void MidiSeq::processTimerTick()
             }
 
       if (idle) {
-        //printf("IDLE\n");
+//        printf("IDLE\n");
             return;
       }
+      if (midiBusy) {
+            // we hit audio: midiSeq->msgProcess
+            // miss this timer tick
+            return;
+            }
 
       unsigned curFrame = audio->curFrame();
-
+      
+      // Keep the sync detectors running... 
+      // No, done in Song::beat(), (at the much slower heartbeat rate).
+      //
+      //for(int port = 0; port < MIDI_PORTS; ++port)
+      //{
+        // Must keep them running even if there's no device...
+        //if(midiPorts[port].device())
+      //    midiPorts[port].syncInfo().setCurFrame(curFrame);
+      //}  
+      //for(iMidiDevice imd = midiDevices.begin(); imd != midiDevices.end(); ++imd) 
+      //  (*imd)->syncInfo().setCurFrame(curFrame);
+      
       if (!extSyncFlag.value()) {
-            int curTick = tempomap.frame2tick(curFrame);
-            if ( midiClock > curTick + 100) // reinitialize
+            //int curTick = tempomap.frame2tick(curFrame);
+            // Copied from Tempomap.
+            //int curTick = lrint((double(curFrame)/double(sampleRate)) * tempomap.globalTempo() * config.division * 10000.0 / double(tempomap.tempo(song->cpos())));
+            //int curTick = lrint((double(curFrame)/double(sampleRate)) * tempomap.globalTempo() * 240000.0 / double(tempomap.tempo(song->cpos())));
+            int curTick = lrint((double(curFrame)/double(sampleRate)) * double(tempomap.globalTempo()) * double(config.division) * 10000.0 / double(tempomap.tempo(song->cpos())));
+            //int curTick = int((double(curFrame)/double(sampleRate)) * double(tempomap.globalTempo()) * double(config.division * 10000.0) / double(tempomap.tempo(song->cpos())));
+            
+/*            if ( midiClock > curTick + 100) // reinitialize
+                {
               midiClock = curTick;
+                }
             else if( curTick > midiClock + 100) // reinitialize
+                {
               midiClock = curTick;
+                }*/
               
-            //printf("curTick=%d >= midiClock=%d\n",curTick,midiClock);
-            if (curTick >= midiClock)  {
-                  processMidiClock();
-                 }
+            if(midiClock > curTick)
+              midiClock = curTick;
+            
+            int div = config.division/24;
+            if(curTick >= midiClock + div)  {
+            //if(curTick >= midiClock)  {
+                  //processMidiClock();
+                  int perr = (curTick - midiClock) / div;
+                  //int perr = curTick - midiClock;
+                  
+                  bool used = false;
+                  
+                  //if(genMCSync)
+                  //{ 
+                    //midiPorts[txSyncPort].sendClock();
+                    for(int port = 0; port < MIDI_PORTS; ++port)
+                    {
+                      MidiPort* mp = &midiPorts[port];
+                      
+                      // No device? Clock out not turned on?
+                      //MidiDevice* dev = mp->device();
+                      //if(!dev || !mp->syncInfo().MCOut())
+                      if(!mp->device() || !mp->syncInfo().MCOut())
+                        continue;
+                        
+                      // Shall we check open flags?
+                      //if(!(dev->rwFlags() & 0x1) || !(dev->openFlags() & 1))
+                      //if(!(dev->openFlags() & 1))
+                      //  continue;
+        
+                      used = true;
+                      
+                      mp->sendClock();
+                    }
+                    
+                    /*
+                    for(iMidiDevice imd = midiDevices.begin(); imd != midiDevices.end(); ++imd) 
+                    {
+                      MidiDevice* dev = *imd;
+                      
+                      if(!dev->syncInfo().MCOut())
+                        continue;
+                        
+                      // Shall we check open flags?
+                      //if(!(dev->rwFlags() & 0x1) || !(dev->openFlags() & 1))
+                      //if(!(dev->openFlags() & 1))
+                      //  continue;
+        
+                      int port = dev->midiPort();
+                      // Without this -1 check, interesting sync things can be done by the user without ever
+                      //  assigning any devices to ports ! 
+                      //if(port < 0 || port > MIDI_PORTS)
+                      if(port < -1 || port > MIDI_PORTS)
+                        continue;
+                        
+                      if(port == -1)
+                      // Send straight to the device... Copied from MidiPort.
+                      {
+                        MidiPlayEvent event(0, 0, 0, ME_CLOCK, 0, 0);
+                        dev->putEvent(event);
+                      }  
+                      else
+                        // Go through the port...
+                        midiPorts[port].sendClock();
+                    }
+                    */
+                    
+                    if(debugMsg && used && perr > 1)
+                      printf("Dropped %d midi out clock(s). curTick:%d midiClock:%d div:%d\n", perr, curTick, midiClock, div);
+                  //}
+                    
+                  // Keeping in mind how (receiving end) Phase Locked Loops (usually) operate...
+                  // Increment as if we had caught the timer exactly on the mark, even if the timer
+                  //  has passed beyond the mark, or even beyond 2 * div.
+                  // If we missed some chances to send clock, resume the count where it would have been, 
+                  //  had we not missed chances.
+                  // We can't do anything about missed chances except send right away, and make up
+                  //  for gained time by losing time in the next count...
+                  // In other words, use equalization periods to counter gained/lost time, so that
+                  //  ultimately, over time, the receiver remains in phase, despite any short dropouts / phase glitches.
+                  // (midiClock only increments by div units).
+                  //
+                  // Tested: With midi thread set to high priority, very few clock dropouts ocurred (P4 1.6Ghz).
+                  // But target device tick drifts out of phase with muse tick slowly over time, say 20 bars or so.
+                  // May need more tweaking, possibly use round with/instead of lrint (above), and/or
+                  //  do not use equalization periods - set midiClock to fractions of div.
+                  // Tested: With RTC resolution at 1024, stability was actually better than with 8192!
+                  // It stayed in sync more than 64 bars...
+                  //
+                  //
+                  // Using equalization periods...
+                  midiClock += (perr * div);
+                  //midiClock += perr;
+                  //
+                  // No equalization periods... TODO:
+                  //midiClock += (perr * div);
+               }
             }
 
 //      if (genMTCSync) {
@@ -508,12 +679,11 @@ void MidiSeq::processTimerTick()
                   continue;
             iMPEvent i = md->nextPlayEvent();
             for (; i != el->end(); ++i) {
-/*                  if (i->time() > curFrame) {
-                        printf("  curT %d  frame %d\n", i->time(), curFrame);
-                        break;
+                  if (i->time() > curFrame) {
+                        //printf("  curT %d  frame %d\n", i->time(), curFrame);
+                        break; // skip this event
                         }
- */
-                  // break if event cannot be delivered
+
                   if (mp) {
                         if (mp->sendEvent(*i))
                               break;
