@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//  $Id: midiseq.h,v 1.6.2.9 2006/03/02 22:27:23 spamatica Exp $
+//  $Id: midiseq.h,v 1.6.2.11 2009/12/20 05:00:35 terminator356 Exp $
 //
 //  (C) Copyright 2003 Werner Schweer (ws@seh.de)
 //=========================================================
@@ -65,9 +65,14 @@ class MidiSeq : public Thread {
       void nonRealtimeSystemSysex(const unsigned char* p, int n);
 
    public:
-      MidiSeq(int prio, const char* name);
+      //MidiSeq(int prio, const char* name);
+      MidiSeq(const char* name);
+      
       ~MidiSeq();
-      bool start();
+      
+      //bool start();
+      virtual void start(int);
+      
       virtual void threadStop();
       virtual void threadStart(void*);
 
@@ -89,5 +94,6 @@ class MidiSeq : public Thread {
       };
 
 extern MidiSeq* midiSeq;
+extern volatile bool midiBusy;
 #endif
 
