@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//  $Id: miditransform.cpp,v 1.8.2.1 2005/01/09 00:26:46 lunar_shuttle Exp $
+//  $Id: miditransform.cpp,v 1.8.2.3 2009/05/24 21:43:44 terminator356 Exp $
 //
 //  (C) Copyright 2001 Werner Schweer (ws@seh.de)
 //=========================================================
@@ -208,18 +208,8 @@ MidiTransformerDialog::MidiTransformerDialog(QWidget* parent,
       //  populate preset list
       //---------------------------------------------------
 
-      for (iMidiTransformation i = mtlist.begin(); i != mtlist.end(); ++i) {
-            presetList->insertItem((*i)->name);
-            if (data->cmt == 0)
-                  data->cmt = *i;
-            }
-      if (data->cmt == 0) {
-            data->cmt = new MidiTransformation(tr("New"));
-            mtlist.push_back(data->cmt);
-            presetList->insertItem(tr("New"));
-            }
-      data->cindex = 0;
-      presetList->setCurrentItem(0);
+      updatePresetList();
+      connect(song, SIGNAL(songChanged(int)), SLOT(songChanged(int)));
       }
 
 //---------------------------------------------------------
@@ -231,6 +221,44 @@ MidiTransformerDialog::~MidiTransformerDialog()
       delete data;
       }
 
+//---------------------------------------------------------
+//   songChanged
+//---------------------------------------------------------
+
+void MidiTransformerDialog::songChanged(int flags)
+{
+  // Whenever a song is loaded, flags is -1. Since transforms are part of configuration, 
+  //  use SC_CONFIG here, to filter unwanted song change events.
+  if(flags & SC_CONFIG)
+    updatePresetList();
+}
+
+//---------------------------------------------------------
+//   updatePresetList
+//---------------------------------------------------------
+
+void MidiTransformerDialog::updatePresetList()
+{
+      data->cmt = 0;
+      data->cindex = 0;
+      presetList->clear();
+      for (iMidiTransformation i = mtlist.begin(); i != mtlist.end(); ++i) {
+            presetList->insertItem((*i)->name);
+            if (data->cmt == 0)
+                  data->cmt = *i;
+            }
+      if (data->cmt == 0) {
+            data->cmt = new MidiTransformation(tr("New"));
+            mtlist.push_back(data->cmt);
+            presetList->insertItem(tr("New"));
+            presetList->setCurrentItem(0);
+            }
+      
+      //data->cindex = 0;
+      //presetList->setCurrentItem(0);
+      
+}
+
 //---------------------------------------------------------
 //   writeMidiTransforms
 //---------------------------------------------------------
@@ -393,6 +421,14 @@ void readMidiTransform(Xml& xml)
                         break;
                   case Xml::TagEnd:
                         if (xml.s1() == "midiTransform") {
+                              // By T356. A previous bug caused some .med files to grow very large
+                              //  with duplicate transforms. Here we can eliminate those duplicates.
+                              for(iMidiTransformation i = mtlist.begin(); i != mtlist.end(); ++i) 
+                              {
+                                if((*i)->name == trans.name)
+                                  return;
+                              }
+                              
                               MidiTransformation* t = new MidiTransformation(trans);
                               mtlist.push_back(t);
                               return;
@@ -403,6 +439,21 @@ void readMidiTransform(Xml& xml)
             }
       }
 
+//---------------------------------------------------------
+//   clearMidiTransforms
+//---------------------------------------------------------
+
+void clearMidiTransforms()
+{
+  for (iMidiTransformation i = mtlist.begin(); i != mtlist.end(); ++i) 
+  {
+    MidiTransformation* t = *i;
+    if(t)
+      delete t;
+  }
+  mtlist.clear();
+}
+
 //---------------------------------------------------------
 //   accept
 //---------------------------------------------------------
@@ -608,17 +659,31 @@ void MidiTransformerDialog::transformEvent(Event& event, MidiPart* part,
       Event dummy;
       switch(data->cmt->funcOp) {
             case Transform:
+                  // Indicate do clone parts. 
+                  removePortCtrlEvents(event, part, true);
                   song->changeEvent(event, newEvent, part);
-                  song->undoOp(UndoOp::ModifyEvent, newEvent, event, part);
+                  // Indicate do clone parts. 
+                  addPortCtrlEvents(newEvent, part, true);
+                  // Indicate do port controller values and clone parts. 
+                  //song->undoOp(UndoOp::ModifyEvent, newEvent, event, part);
+                  song->undoOp(UndoOp::ModifyEvent, newEvent, event, part, true, true);
                   song->addUpdateFlags(SC_EVENT_MODIFIED);
                   break;
             case Insert:
-                  song->undoOp(UndoOp::AddEvent, dummy, newEvent, part);
+                  // Indicate do port controller values and clone parts. 
+                  //song->undoOp(UndoOp::AddEvent, dummy, newEvent, part);
+                  song->undoOp(UndoOp::AddEvent, dummy, newEvent, part, true, true);
                   song->addEvent(newEvent, part);
+                  // Indicate do clone parts. 
+                  addPortCtrlEvents(newEvent, part, true);
                   song->addUpdateFlags(SC_EVENT_INSERTED);
                   break;
             case Extract:
-                  song->undoOp(UndoOp::DeleteEvent, dummy, event, part);
+                  // Indicate do port controller values and clone parts. 
+                  //song->undoOp(UndoOp::DeleteEvent, dummy, event, part);
+                  song->undoOp(UndoOp::DeleteEvent, dummy, event, part, true, true);
+                  // Indicate do clone parts. 
+                  removePortCtrlEvents(event, part, true);
                   song->deleteEvent(event, part);
                   song->addUpdateFlags(SC_EVENT_REMOVED);
             case Copy:
@@ -643,10 +708,16 @@ void MidiTransformerDialog::processEvent(Event& event, MidiPart* part, MidiPart*
                   int tick = event.tick();
                   int rt = sigmap.raster(tick, data->cmt->quantVal) - tick;
                   if (tick != rt) {
+                        // Indicate do clone parts. 
+                        removePortCtrlEvents(event, part, true);
                         Event newEvent = event.clone();
                         newEvent.setTick(rt);
                         song->changeEvent(event, newEvent, part);
-                        song->undoOp(UndoOp::ModifyEvent, newEvent, event, part);
+                        // Indicate do clone parts. 
+                        addPortCtrlEvents(newEvent, part, true);
+                        // Indicate do port controller values and clone parts. 
+                        //song->undoOp(UndoOp::ModifyEvent, newEvent, event, part);
+                        song->undoOp(UndoOp::ModifyEvent, newEvent, event, part, true, true);
                         song->addUpdateFlags(SC_EVENT_MODIFIED);
                         }
                   }
@@ -654,7 +725,11 @@ void MidiTransformerDialog::processEvent(Event& event, MidiPart* part, MidiPart*
             case Delete:
                   {
                   Event ev;
-                  song->undoOp(UndoOp::DeleteEvent, ev, event, part);
+                  // Indicate do port controller values and clone parts. 
+                  //song->undoOp(UndoOp::DeleteEvent, ev, event, part, true, true);
+                  song->undoOp(UndoOp::DeleteEvent, ev, event, part, true, true);
+                  // Indicate do clone parts. 
+                  removePortCtrlEvents(event, part, true);
                   song->deleteEvent(event, part);
                   song->addUpdateFlags(SC_EVENT_REMOVED);
                   }
@@ -842,6 +917,10 @@ void MidiTransformerDialog::apply()
       bool copyExtract = (data->cmt->funcOp == Copy)
                          || (data->cmt->funcOp == Extract);
 
+      std::vector< EventList* > doneList;
+      typedef std::vector< EventList* >::iterator iDoneList;
+      iDoneList idl;
+      
       MidiTrackList* tracks = song->midis();
       MidiTrackList tl;
       for (iMidiTrack t = tracks->begin(); t != tracks->end(); ++t) {
@@ -854,6 +933,14 @@ void MidiTransformerDialog::apply()
                   for (iPart p = pl->begin(); p != pl->end(); ++p) {
                         MidiPart* part = (MidiPart *) p->second;
                         EventList* el = part->events();
+                        // Check if the event list has already been done. Skip repeated clones.
+                        for(idl = doneList.begin(); idl != doneList.end(); ++idl)
+                          if(*idl == el)
+                            break;
+                        if(idl != doneList.end())
+                          break;
+                        doneList.push_back(el);
+                        
                         for (iEvent i = el->begin(); i != el->end(); ++i) {
                               Event event = i->second;
                               unsigned tick = event.tick();
@@ -874,6 +961,14 @@ void MidiTransformerDialog::apply()
                   MidiPart* part = (MidiPart *) p->second;
                   MidiPart* newPart = 0;
                   EventList* el = part->events();
+                  // Check if the event list has already been done. Skip repeated clones.
+                  for(idl = doneList.begin(); idl != doneList.end(); ++idl)
+                    if(*idl == el)
+                      break;
+                  if(idl != doneList.end())
+                    break;
+                  doneList.push_back(el);
+                  
                   if (copyExtract) {
                         // check wether we must generate a new part
                         for (iEvent i = el->begin(); i != el->end(); ++i) {
