@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//  $Id: astrip.cpp,v 1.23.2.2 2006/02/01 23:17:38 spamatica Exp $
+//  $Id: astrip.cpp,v 1.23.2.17 2009/11/16 01:55:55 terminator356 Exp $
 //
 //  (C) Copyright 2000-2004 Werner Schweer (ws@seh.de)
 //=========================================================
@@ -41,7 +41,9 @@
 #include "icons.h"
 #include "gconfig.h"
 #include "ttoolbutton.h"
+#include "menutitleitem.h"
 
+/*
 //---------------------------------------------------------
 //   MenuTitleItem
 //---------------------------------------------------------
@@ -57,6 +59,7 @@ class MenuTitleItem : public QCustomMenuItem {
    public:
       MenuTitleItem(QString s);
       };
+*/
 
 //---------------------------------------------------------
 //   MenuTitleItem
@@ -83,26 +86,47 @@ void MenuTitleItem::paint(QPainter* p, const QColorGroup&, bool,
       p->drawText(x, y, w, h, AlignCenter, s);
       }
 
+//---------------------------------------------------------
+//   minimumSizeHint
+//---------------------------------------------------------
+
+QSize AudioStrip::minimumSizeHint () const
+{
+    // We force the width of the size hint to be what we want
+    //return QWidget::minimumSizeHint();
+    return QSize(66,QWidget::minimumSizeHint().height());
+}
+
 //---------------------------------------------------------
 //   heartBeat
 //---------------------------------------------------------
 
 void AudioStrip::heartBeat()
       {
-      Strip::heartBeat();
-      AutomationType t = track->automationType();
-
-      if (automation && ((t == AUTO_READ) || (t == AUTO_TOUCH))) {
-            //
-            //  dont automate controller if in AUTO_TOUCH mode and
-            //  user has touched controller
-            //
-            if ((t == AUTO_READ) || !volumeTouched)
+        for (int ch = 0; ch < track->channels(); ++ch) {
+          if (meter[ch]) {
+            //int meterVal = track->meter(ch);
+            //int peak  = track->peak(ch);
+            //meter[ch]->setVal(meterVal, peak, false);
+            meter[ch]->setVal(track->meter(ch), track->peak(ch), false);
+          }
+        }
+        Strip::heartBeat();
                   updateVolume();
-            if ((t == AUTO_READ) || !panTouched)
                   updatePan();
             }
-      }
+
+//---------------------------------------------------------
+//   configChanged
+//---------------------------------------------------------
+
+void AudioStrip::configChanged()    
+{ 
+  // Added by Tim. p3.3.6
+  //printf("AudioStrip::configChanged\n");
+      
+  songChanged(SC_CONFIG); 
+}
 
 //---------------------------------------------------------
 //   songChanged
@@ -110,28 +134,81 @@ void AudioStrip::heartBeat()
 
 void AudioStrip::songChanged(int val)
       {
+      // Is it simply a midi controller value adjustment? Forget it.
+      if(val == SC_MIDI_CONTROLLER)
+        return;
+    
       AudioTrack* src = (AudioTrack*)track;
+      
+      // Do channels before config...
+      if (val & SC_CHANNELS)
+        updateChannels();
+      // Catch when label font, or configuration min slider and meter values change.
+      if (val & SC_CONFIG)
+      {
+        // Added by Tim. p3.3.9
+        
+        // Set the strip label's font.
+        //label->setFont(config.fonts[1]);
+        setLabelFont();
+        
+        // Adjust minimum volume slider and label values.
+        slider->setRange(config.minSlider-0.1, 10.0);
+        sl->setRange(config.minSlider, 10.0);
+        
+        // Adjust minimum aux knob and label values.
+        int n = auxKnob.size();
+        for (int idx = 0; idx < n; ++idx) 
+        {
+          auxKnob[idx]->blockSignals(true);
+          auxLabel[idx]->blockSignals(true);
+          auxKnob[idx]->setRange(config.minSlider-0.1, 10.0);
+          auxLabel[idx]->setRange(config.minSlider, 10.1);
+          auxKnob[idx]->blockSignals(false);
+          auxLabel[idx]->blockSignals(false);
+        }
+        
+        // Adjust minimum meter values.
+        for(int c = 0; c < channel; ++c) 
+          meter[c]->setRange(config.minMeter, 10.0);
+      }
+      
       if (mute && (val & SC_MUTE)) {      // mute && off
             mute->blockSignals(true);
-            mute->setOn(src->isMute());
+            mute->setOn(src->mute());
             mute->blockSignals(false);
             updateOffState();
             }
       if (solo && (val & SC_SOLO)) {
+            if((bool)track->internalSolo())
+            {
+              if(!useSoloIconSet2)
+              {
+                solo->setIconSet(*soloIconSet2);
+                useSoloIconSet2 = true;
+              }  
+            }  
+            else if(useSoloIconSet2)
+            {
+              solo->setIconSet(*soloIconSet1);
+              useSoloIconSet2 = false;
+            }  
+            
             solo->blockSignals(true);
-            solo->setOn(src->solo());
+            solo->setOn(track->solo());
             solo->blockSignals(false);
             }
       if (val & SC_RECFLAG)
             setRecordFlag(track->recordFlag());
       if (val & SC_TRACK_MODIFIED)
+      {
             setLabelText();
-      if (val & SC_MIXER_VOLUME)
-            updateVolume();
-      if (val & SC_MIXER_PAN)
-            updatePan();
-      if (val & SC_CHANNELS)
-            updateChannels();
+            // Added by Tim. p3.3.9
+            setLabelFont();
+            
+      }      
+      //if (val & SC_CHANNELS)
+      //      updateChannels();
       if (val & SC_ROUTE) {
             if (pre) {
                   pre->blockSignals(true);
@@ -154,6 +231,11 @@ void AudioStrip::songChanged(int val)
       if (autoType && (val & SC_AUTOMATION)) {
             autoType->blockSignals(true);
             autoType->setCurrentItem(track->automationType());
+            if(track->automationType() == AUTO_TOUCH || track->automationType() == AUTO_WRITE)
+              autoType->setPaletteBackgroundColor(red);
+            else  
+              autoType->setPaletteBackgroundColor(qApp->palette().active().background());
+      
             autoType->blockSignals(false);
             }
       }
@@ -163,9 +245,13 @@ void AudioStrip::songChanged(int val)
 //---------------------------------------------------------
 
 void AudioStrip::updateVolume()
-      {
+{
       double vol = ((AudioTrack*)track)->volume();
-      if (vol != volume) {
+        if (vol != volume) 
+        {
+            // Added by Tim. p3.3.6
+            //printf("AudioStrip::updateVolume setting slider and label\n");
+          
             slider->blockSignals(true);
             sl->blockSignals(true);
             double val = fast_log10(vol) * 20.0;
@@ -175,16 +261,20 @@ void AudioStrip::updateVolume()
             slider->blockSignals(false);
             volume = vol;
             }
-      }
+}
 
 //---------------------------------------------------------
 //   updatePan
 //---------------------------------------------------------
 
 void AudioStrip::updatePan()
-      {
+{
       double v = ((AudioTrack*)track)->pan();
-      if (v != panVal) {
+        if (v != panVal) 
+        {
+            // Added by Tim. p3.3.6
+            //printf("AudioStrip::updatePan setting slider and label\n");
+            
             pan->blockSignals(true);
             panl->blockSignals(true);
             pan->setValue(v);
@@ -193,7 +283,7 @@ void AudioStrip::updatePan()
             pan->blockSignals(false);
             panVal = v;
             }
-      }
+}       
 
 //---------------------------------------------------------
 //   offToggled
@@ -213,10 +303,22 @@ void AudioStrip::updateOffState()
       {
       bool val = !track->off();
       slider->setEnabled(val);
+      sl->setEnabled(val);
       pan->setEnabled(val);
+      panl->setEnabled(val);
       if (track->type() != Track::AUDIO_SOFTSYNTH)
             stereo->setEnabled(val);
       label->setEnabled(val);
+      
+      int n = auxKnob.size();
+      for (int i = 0; i < n; ++i) 
+      {
+        auxKnob[i]->setEnabled(val);
+        auxLabel[i]->setEnabled(val);
+      }
+            
+      if (pre)
+            pre->setEnabled(val);
       if (record)
             record->setEnabled(val);
       if (solo)
@@ -279,12 +381,27 @@ void AudioStrip::auxChanged(double val, int idx)
       song->update(SC_AUX);
       }
 
+//---------------------------------------------------------
+//   auxLabelChanged
+//---------------------------------------------------------
+
+void AudioStrip::auxLabelChanged(double val, unsigned int idx) 
+      {
+        if(idx >= auxKnob.size())
+          return;    
+        auxKnob[idx]->setValue(val);
+      }
+
 //---------------------------------------------------------
 //   volumeChanged
 //---------------------------------------------------------
 
 void AudioStrip::volumeChanged(double val)
       {
+      AutomationType at = ((AudioTrack*)track)->automationType();
+      if(at == AUTO_WRITE || (audio->isPlaying() && at == AUTO_TOUCH))
+        track->enableVolumeController(false);
+      
       double vol;
       if (val <= config.minSlider) {
             vol = 0.0;
@@ -295,7 +412,6 @@ void AudioStrip::volumeChanged(double val)
       volume = vol;
       audio->msgSetVolume((AudioTrack*)track, vol);
       ((AudioTrack*)track)->recordAutomation(AC_VOLUME, vol);
-      song->update(SC_MIXER_VOLUME);
       }
 
 //---------------------------------------------------------
@@ -304,8 +420,21 @@ void AudioStrip::volumeChanged(double val)
 
 void AudioStrip::volumePressed()
       {
-      ((AudioTrack*)track)->startAutoRecord(AC_VOLUME);
-      volumeTouched = true;
+      AutomationType at = ((AudioTrack*)track)->automationType();
+      if(at == AUTO_WRITE || (at == AUTO_READ || at == AUTO_TOUCH))
+        track->enableVolumeController(false);
+      
+      double val = slider->value();
+      double vol;
+      if (val <= config.minSlider) {
+            vol = 0.0;
+            //val -= 1.0; // display special value "off"
+            }
+      else
+            vol = pow(10.0, val/20.0);
+      volume = vol;
+      audio->msgSetVolume((AudioTrack*)track, volume);
+      ((AudioTrack*)track)->startAutoRecord(AC_VOLUME, volume);
       }
 
 //---------------------------------------------------------
@@ -314,8 +443,41 @@ void AudioStrip::volumePressed()
 
 void AudioStrip::volumeReleased()
       {
-      ((AudioTrack*)track)->stopAutoRecord(AC_VOLUME);
-      volumeTouched = false;
+      if(track->automationType() != AUTO_WRITE)
+        track->enableVolumeController(true);
+      
+      ((AudioTrack*)track)->stopAutoRecord(AC_VOLUME, volume);
+      }
+
+//---------------------------------------------------------
+//   volumeRightClicked
+//---------------------------------------------------------
+void AudioStrip::volumeRightClicked(const QPoint &p)
+{
+  song->execAutomationCtlPopup((AudioTrack*)track, p, AC_VOLUME);
+}
+
+//---------------------------------------------------------
+//   volLabelChanged
+//---------------------------------------------------------
+
+void AudioStrip::volLabelChanged(double val)
+      {
+      AutomationType at = ((AudioTrack*)track)->automationType();
+      if(at == AUTO_WRITE || (audio->isPlaying() && at == AUTO_TOUCH))
+        track->enableVolumeController(false);
+      
+      double vol;
+      if (val <= config.minSlider) {
+            vol = 0.0;
+            val -= 1.0; // display special value "off"
+            }
+      else
+            vol = pow(10.0, val/20.0);
+      volume = vol;
+      slider->setValue(val);
+      audio->msgSetVolume((AudioTrack*)track, vol);
+      ((AudioTrack*)track)->startAutoRecord(AC_VOLUME, vol);
       }
 
 //---------------------------------------------------------
@@ -324,10 +486,13 @@ void AudioStrip::volumeReleased()
 
 void AudioStrip::panChanged(double val)
       {
+      AutomationType at = ((AudioTrack*)track)->automationType();
+      if(at == AUTO_WRITE || (audio->isPlaying() && at == AUTO_TOUCH))
+        track->enablePanController(false);
+      
+      panVal = val;  
       audio->msgSetPan(((AudioTrack*)track), val);
-      panVal = val;
       ((AudioTrack*)track)->recordAutomation(AC_PAN, val);
-      song->update(SC_MIXER_PAN);
       }
 
 //---------------------------------------------------------
@@ -336,8 +501,13 @@ void AudioStrip::panChanged(double val)
 
 void AudioStrip::panPressed()
       {
-      ((AudioTrack*)track)->startAutoRecord(AC_PAN);
-      panTouched = true;
+      AutomationType at = ((AudioTrack*)track)->automationType();
+      if(at == AUTO_WRITE || (at == AUTO_READ || at == AUTO_TOUCH))
+        track->enablePanController(false);
+      
+      panVal = pan->value();  
+      audio->msgSetPan(((AudioTrack*)track), panVal);
+      ((AudioTrack*)track)->startAutoRecord(AC_PAN, panVal);
       }
 
 //---------------------------------------------------------
@@ -346,22 +516,51 @@ void AudioStrip::panPressed()
 
 void AudioStrip::panReleased()
       {
-      ((AudioTrack*)track)->stopAutoRecord(AC_PAN);
-      panTouched = false;
+      if(track->automationType() != AUTO_WRITE)
+        track->enablePanController(true);
+      ((AudioTrack*)track)->stopAutoRecord(AC_PAN, panVal);
+      }
+
+//---------------------------------------------------------
+//   panRightClicked
+//---------------------------------------------------------
+void AudioStrip::panRightClicked(const QPoint &p)
+{
+  song->execAutomationCtlPopup((AudioTrack*)track, p, AC_PAN);
+}
+
+//---------------------------------------------------------
+//   panLabelChanged
+//---------------------------------------------------------
+
+void AudioStrip::panLabelChanged(double val)
+      {
+      AutomationType at = ((AudioTrack*)track)->automationType();
+      if(at == AUTO_WRITE || (audio->isPlaying() && at == AUTO_TOUCH))
+        track->enablePanController(false);
+      
+      panVal = val;
+      pan->setValue(val);
+      audio->msgSetPan((AudioTrack*)track, val);
+      ((AudioTrack*)track)->startAutoRecord(AC_PAN, val);
       }
 
 //---------------------------------------------------------
 //   updateChannels
 //---------------------------------------------------------
-
+                                       
 void AudioStrip::updateChannels()
       {
       AudioTrack* t = (AudioTrack*)track;
       int c = t->channels();
+      // Added by Tim. p3.3.6
+      //printf("AudioStrip::updateChannels track channels:%d current channels:%d\n", c, channel);
+      
       if (c > channel) {
             for (int cc = channel; cc < c; ++cc) {
                   meter[cc] = new Meter(this);
-                  meter[cc]->setRange(config.minSlider, 10.0);
+                  //meter[cc]->setRange(config.minSlider, 10.0);
+                  meter[cc]->setRange(config.minMeter, 10.0);
                   meter[cc]->setFixedWidth(15);
                   connect(meter[cc], SIGNAL(mousePress()), this, SLOT(resetPeaks()));
                   sliderGrid->addWidget(meter[cc], 0, cc+1, AlignHCenter);
@@ -407,9 +606,11 @@ Knob* AudioStrip::addKnob(int type, int id, DoubleLabel** dlabel)
       if (type == 0)
             pl = new DoubleLabel(0, -1.0, +1.0, this);
       else
-            pl = new DoubleLabel(0.0, config.minSlider, 10.0, this);
+            pl = new DoubleLabel(0.0, config.minSlider, 10.1, this);
+            
       if (dlabel)
             *dlabel = pl;
+      pl->setSlider(knob);
       pl->setFont(config.fonts[1]);
       pl->setBackgroundMode(PaletteMid);
       pl->setFrame(true);
@@ -417,7 +618,6 @@ Knob* AudioStrip::addKnob(int type, int id, DoubleLabel** dlabel)
             pl->setPrecision(2);
       else {
             pl->setPrecision(0);
-            pl->setSpecialText(tr("off"));
             pl->setPrecision(0);
             }
       pl->setFixedWidth(STRIP_WIDTH/2);
@@ -440,15 +640,22 @@ Knob* AudioStrip::addKnob(int type, int id, DoubleLabel** dlabel)
       layout->addLayout(pangrid);
 
       connect(knob, SIGNAL(valueChanged(double,int)), pl, SLOT(setValue(double)));
-      connect(pl, SIGNAL(valueChanged(double, int)), knob,  SLOT(setValue(double)));
+      //connect(pl, SIGNAL(valueChanged(double, int)), SLOT(panChanged(double)));
 
       if (type == 0) {
+            connect(pl, SIGNAL(valueChanged(double, int)), SLOT(panLabelChanged(double)));
             connect(knob, SIGNAL(sliderMoved(double,int)), SLOT(panChanged(double)));
-            connect(knob, SIGNAL(sliderPressed()), SLOT(panPressed()));
-            connect(knob, SIGNAL(sliderReleased()), SLOT(panReleased()));
+            connect(knob, SIGNAL(sliderPressed(int)), SLOT(panPressed()));
+            connect(knob, SIGNAL(sliderReleased(int)), SLOT(panReleased()));
+            connect(knob, SIGNAL(sliderRightClicked(const QPoint &, int)), SLOT(panRightClicked(const QPoint &)));
             }
       else {
             knob->setId(id);
+            
+            connect(pl, SIGNAL(valueChanged(double, int)), knob,  SLOT(setValue(double)));
+            // Not used yet. Switch if/when necessary.
+            //connect(pl, SIGNAL(valueChanged(double, int)), SLOT(auxLabelChanged(double, int)));
+            
             connect(knob, SIGNAL(sliderMoved(double, int)), SLOT(auxChanged(double, int)));
             }
       return knob;
@@ -462,6 +669,7 @@ AudioStrip::~AudioStrip()
       {
       }
 
+
 //---------------------------------------------------------
 //   AudioStrip
 //    create mixer strip
@@ -473,13 +681,16 @@ AudioStrip::AudioStrip(QWidget* parent, AudioTrack* at)
       iR            = 0;
       oR            = 0;
       off           = 0;
+      
       volume        = -1.0;
       panVal        = 0;
+      
       record        = 0;
-      panTouched    = false;
-      volumeTouched = false;
+      
       AudioTrack* t = (AudioTrack*)track;
       channel       = at->channels();
+      setFixedWidth(STRIP_WIDTH);
+      setMinimumWidth(STRIP_WIDTH);
 
       int ch = 0;
       for (; ch < channel; ++ch)
@@ -542,7 +753,7 @@ AudioStrip::AudioStrip(QWidget* parent, AudioTrack* at)
                   Knob* ak = addKnob(1, idx, &al);
                   auxKnob.push_back(ak);
                   auxLabel.push_back(al);
-                  float val = fast_log10(t->auxSend(idx))*20.0;
+                  double val = fast_log10(t->auxSend(idx))*20.0;
                   ak->setValue(val);
                   al->setValue(val);
                   }
@@ -559,7 +770,10 @@ AudioStrip::AudioStrip(QWidget* parent, AudioTrack* at)
       sliderGrid = new QGridLayout;
       sliderGrid->setRowStretch(0, 100);
 
-      slider = new Slider(this);
+      //slider = new Slider(this);
+      slider = new Slider(this, "vol", Slider::Vertical, Slider::None,
+         Slider::BgTrough | Slider::BgSlot);
+      slider->setCursorHoming(true);
       slider->setRange(config.minSlider-0.1, 10.0);
       slider->setFixedWidth(20);
       slider->setFont(config.fonts[1]);
@@ -568,7 +782,8 @@ AudioStrip::AudioStrip(QWidget* parent, AudioTrack* at)
       sliderGrid->addWidget(slider, 0, 0, AlignHCenter);
 
       for (int i = 0; i < channel; ++i) {
-            meter[i]->setRange(config.minSlider, 10.0);
+            //meter[i]->setRange(config.minSlider, 10.0);
+            meter[i]->setRange(config.minMeter, 10.0);
             meter[i]->setFixedWidth(15);
             connect(meter[i], SIGNAL(mousePress()), this, SLOT(resetPeaks()));
             sliderGrid->addWidget(meter[i], 0, i+1, AlignHCenter);
@@ -577,20 +792,22 @@ AudioStrip::AudioStrip(QWidget* parent, AudioTrack* at)
       layout->addLayout(sliderGrid);
 
       sl = new DoubleLabel(0.0, config.minSlider, 10.0, this);
+      sl->setSlider(slider);
       sl->setFont(config.fonts[1]);
       sl->setBackgroundMode(PaletteMid);
-      sl->setSpecialText(tr("off"));
       sl->setSuffix(tr("dB"));
       sl->setFrame(true);
       sl->setPrecision(0);
       sl->setFixedWidth(STRIP_WIDTH);
       sl->setValue(fast_log10(t->volume()) * 20.0);
 
-      connect(sl, SIGNAL(valueChanged(double,int)), slider, SLOT(setValue(double)));
+      connect(sl, SIGNAL(valueChanged(double,int)), SLOT(volLabelChanged(double)));
+      //connect(sl, SIGNAL(valueChanged(double,int)), SLOT(volumeChanged(double)));
       connect(slider, SIGNAL(valueChanged(double,int)), sl, SLOT(setValue(double)));
       connect(slider, SIGNAL(sliderMoved(double,int)), SLOT(volumeChanged(double)));
-      connect(slider, SIGNAL(sliderPressed()), SLOT(volumePressed()));
-      connect(slider, SIGNAL(sliderReleased()), SLOT(volumeReleased()));
+      connect(slider, SIGNAL(sliderPressed(int)), SLOT(volumePressed()));
+      connect(slider, SIGNAL(sliderReleased(int)), SLOT(volumeReleased()));
+      connect(slider, SIGNAL(sliderRightClicked(const QPoint &, int)), SLOT(volumeRightClicked(const QPoint &)));
       layout->addWidget(sl);
 
       //---------------------------------------------------
@@ -637,12 +854,21 @@ AudioStrip::AudioStrip(QWidget* parent, AudioTrack* at)
       smBox2->addWidget(mute);
 
       solo  = new QToolButton(this);
-      QIconSet soloSet;
-      soloSet.setPixmap(*soloIconOn,   QIconSet::Automatic, QIconSet::Normal, QIconSet::On);
-      soloSet.setPixmap(*soloIconOff, QIconSet::Automatic, QIconSet::Normal, QIconSet::Off);
-      solo->setIconSet(soloSet);
-//      solo->setToggleButton(true);
-      solo->setDisabled(true);
+      
+      if((bool)t->internalSolo())
+      {
+        solo->setIconSet(*soloIconSet2);
+        useSoloIconSet2 = true;
+      }  
+      else  
+      {
+        solo->setIconSet(*soloIconSet1);
+        useSoloIconSet2 = false;
+      }  
+              
+      solo->setToggleButton(true);
+      solo->setOn(t->solo());
+      
       solo->setFixedWidth(STRIP_WIDTH/2-2);
       smBox2->addWidget(solo);
       connect(solo, SIGNAL(toggled(bool)), SLOT(soloToggled(bool)));
@@ -686,7 +912,7 @@ AudioStrip::AudioStrip(QWidget* parent, AudioTrack* at)
             iR->setFixedWidth((STRIP_WIDTH-4)/2);
             iR->setText(tr("iR"));
             iR->setToggleButton(false);
-            QToolTip::add(iR, tr("intput routing"));
+            QToolTip::add(iR, tr("input routing"));
             rBox->addWidget(iR);
             connect(iR, SIGNAL(pressed()), SLOT(iRoutePressed()));
             }
@@ -715,6 +941,12 @@ AudioStrip::AudioStrip(QWidget* parent, AudioTrack* at)
       autoType->insertItem(tr("Touch"), AUTO_TOUCH);
       autoType->insertItem(tr("Write"), AUTO_WRITE);
       autoType->setCurrentItem(t->automationType());
+      
+      if(t->automationType() == AUTO_TOUCH || t->automationType() == AUTO_WRITE)
+        autoType->setPaletteBackgroundColor(red);
+      else  
+        autoType->setPaletteBackgroundColor(qApp->palette().active().background());
+      
       QToolTip::add(autoType, tr("automation type"));
       connect(autoType, SIGNAL(activated(int,int)), SLOT(setAutomationType(int,int)));
       layout->addWidget(autoType);
@@ -943,6 +1175,7 @@ void AudioStrip::iRoutePressed()
                   // connect
                   audio->msgAddRoute(srcRoute, dstRoute);
                   }
+            audio->msgUpdateSoloStates();
             song->update(SC_ROUTE);
             }
       delete pup;
@@ -1027,6 +1260,7 @@ void AudioStrip::oRoutePressed()
                   // connect if route does not exist
                   audio->msgAddRoute(srcRoute, dstRoute);
                   }
+            audio->msgUpdateSoloStates();
             song->update(SC_ROUTE);
             }
       delete pup;
