@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//  $Id: mstrip.h,v 1.4 2004/05/06 15:08:07 wschweer Exp $
+//  $Id: mstrip.h,v 1.4.2.4 2009/10/25 19:26:29 lunar_shuttle Exp $
 //
 //  (C) Copyright 2000-2004 Werner Schweer (ws@seh.de)
 //=========================================================
@@ -45,7 +45,9 @@ class MidiStrip : public Strip {
 
       void addKnob(int idx, const QString&, const QString&, const char*, bool);
       void ctrlChanged(int num, int val);
-
+      void updateControls();
+      void updateOffState();
+   
    private slots:
       void routeClicked();
       void setVolume(double);
@@ -53,6 +55,9 @@ class MidiStrip : public Strip {
       void setChorusSend(double);
       void setVariSend(double);
       void setReverbSend(double);
+      void labelDoubleClicked(int);
+      void volLabelChanged(double);
+      void controlRightClicked(const QPoint&, int);
 
    protected slots:
       virtual void heartBeat();
