@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//  $Id: rack.cpp,v 1.7.2.4 2006/02/11 22:21:21 spamatica Exp $
+//  $Id: rack.cpp,v 1.7.2.7 2007/01/27 14:52:43 spamatica Exp $
 //
 //  (C) Copyright 2000-2003 Werner Schweer (ws@seh.de)
 //=========================================================
@@ -149,7 +149,7 @@ void EffectRack::menuRequested(QListBoxItem* it, const QPoint& pt)
             mute  = pipe->isOn(idx);
             }
 
-      enum { NEW, UP, DOWN, REMOVE, BYPASS, SHOW, SAVE };
+      enum { NEW, CHANGE, UP, DOWN, REMOVE, BYPASS, SHOW, SAVE };
       QPopupMenu* menu = new QPopupMenu;
       menu->insertItem(QIconSet(*upIcon), tr("move up"),   UP, UP);
       menu->insertItem(QIconSet(*downIcon), tr("move down"), DOWN, DOWN);
@@ -170,7 +170,7 @@ void EffectRack::menuRequested(QListBoxItem* it, const QPoint& pt)
             menu->setItemEnabled(SAVE, false);
             }
       else {
-            menu->insertItem(tr("change"), NEW, NEW);
+            menu->insertItem(tr("change"), CHANGE, CHANGE);
             menu->insertItem(tr("save preset"),  SAVE, SAVE);
             if (idx == 0)
                   menu->setItemEnabled(UP, false);
@@ -190,7 +190,7 @@ void EffectRack::menuRequested(QListBoxItem* it, const QPoint& pt)
                   if (plugin) {
                         PluginI* plugi = new PluginI();
                         if (plugi->initPluginInstance(plugin, track->channels())) {
-                              printf("cannot intantiate plugin <%s>\n",
+                              printf("cannot instantiate plugin <%s>\n",
                                  plugin->name().latin1());
                               delete plugi;
                               break;
@@ -199,6 +199,22 @@ void EffectRack::menuRequested(QListBoxItem* it, const QPoint& pt)
                         }
                   break;
                   }
+            case CHANGE:
+                  {
+                  Plugin* plugin = PluginDialog::getPlugin(this);
+                  if (plugin) {
+                        PluginI* plugi = new PluginI();
+                        if (plugi->initPluginInstance(plugin, track->channels())) {
+                              printf("cannot instantiate plugin <%s>\n",
+                                 plugin->name().latin1());
+                              delete plugi;
+                              break;
+                              }
+                        audio->msgAddPlugin(track, idx, 0);
+                        audio->msgAddPlugin(track, idx, plugi);
+                        }
+                  break;
+                  }
             case REMOVE:
                   audio->msgAddPlugin(track, idx, 0);
                   break;
@@ -318,7 +334,7 @@ void EffectRack::startDrag(int idx)
       drag->drag();
       }
 
-void EffectRack::contentsDropEvent(QDropEvent *event)
+void EffectRack::contentsDropEvent(QDropEvent * /*event*/)// prevent of compiler warning: unsued variable
       {
       }
 
@@ -331,6 +347,22 @@ void EffectRack::dropEvent(QDropEvent *event)
       Pipeline* pipe = track->efxPipe();
       if (pipe) {
             if ((*pipe)[idx] != NULL) {
+                QWidget *sw = event->source();
+                if(sw)
+                {
+                  if(strcmp(sw->className(), "EffectRack") == 0) 
+                  { 
+                    EffectRack *ser = (EffectRack*)sw;
+                    Pipeline* spipe = ser->getTrack()->efxPipe();
+                    if(!spipe)
+                      return;
+                    QListBoxItem *i = ser->itemAt(contentsToViewport(ser->getDragPos()));
+                    int idx0 = ser->index(i);
+                    if (!(*spipe)[idx0] || 
+                        (idx == idx0 && (ser == this || ser->getTrack()->name() == track->name())))
+                      return; 
+                  }
+                }
                 if(!QMessageBox::question(this, tr("Replace effect"),tr("Do you really want to replace the effect %1?").arg(pipe->name(idx)),
                       tr("&Yes"), tr("&No"),
                       QString::null, 0, 1 ))
@@ -375,7 +407,7 @@ void EffectRack::dragEnterEvent(QDragEnterEvent *event)
       }
 
 
-void EffectRack::contentsDragEnterEvent(QDragEnterEvent *event)
+void EffectRack::contentsDragEnterEvent(QDragEnterEvent * /*event*/)// prevent of compiler warning: unused parameter
       {
       }
 
@@ -390,6 +422,14 @@ void EffectRack::contentsMousePressEvent(QMouseEvent *event)
 void EffectRack::contentsMouseMoveEvent(QMouseEvent *event)
       {
       if (event->state() & LeftButton) {
+            Pipeline* pipe = track->efxPipe();
+            if(!pipe)
+              return;
+            QListBoxItem *i = itemAt(contentsToViewport(dragPos));
+            int idx0 = index(i);
+            if (!(*pipe)[idx0])
+              return; 
+            
             int distance = (dragPos-event->pos()).manhattanLength();
             if (distance > QApplication::startDragDistance()) {
                   QListBoxItem *i = itemAt( contentsToViewport(event->pos()) );
@@ -414,7 +454,7 @@ void EffectRack::initPlugin(Xml xml, int idx)
                         if (tag == "plugin") {
                               PluginI* plugi = new PluginI();
                               if (plugi->readConfiguration(xml, false)) {
-                                  printf("cannot intantiate plugin\n");
+                                  printf("cannot instantiate plugin\n");
                                   delete plugi;
                                   }
                               else {
