@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//  $Id: routedialog.cpp,v 1.5.2.1 2006/02/01 23:17:38 spamatica Exp $
+//  $Id: routedialog.cpp,v 1.5.2.2 2007/01/04 00:35:17 terminator356 Exp $
 //
 //  (C) Copyright 2004 Werner Schweer (ws@seh.de)
 //=========================================================
@@ -122,6 +122,8 @@ void RouteDialog::removeRoute()
       if (item == 0)
             return;
       audio->msgRemoveRoute(Route(item->text(0), false, -1), Route(item->text(1), true, -1));
+      audio->msgUpdateSoloStates();
+      song->update(SC_SOLO);
       delete item;
       }
 
@@ -136,6 +138,8 @@ void RouteDialog::addRoute()
       if (srcItem == 0 || dstItem == 0)
             return;
       audio->msgAddRoute(Route(srcItem->text(), false, -1), Route(dstItem->text(), true, -1));
+      audio->msgUpdateSoloStates();
+      song->update(SC_SOLO);
       new QListViewItem(routeList, srcItem->text(), dstItem->text());
       }
 
