@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//  $Id: strip.h,v 1.3 2004/04/17 11:30:43 wschweer Exp $
+//  $Id: strip.h,v 1.3.2.2 2009/11/14 03:37:48 terminator356 Exp $
 //
 //  (C) Copyright 2000-2004 Werner Schweer (ws@seh.de)
 //=========================================================
@@ -10,6 +10,7 @@
 #define __STRIP_H__
 
 #include <qframe.h>
+#include <qiconset.h>
 
 #include "globaldefs.h"
 
@@ -35,6 +36,7 @@ class Strip : public QFrame {
       QLabel* label;
       QVBoxLayout* layout;
       Meter* meter[MAX_CHANNELS];
+      bool useSoloIconSet2;
 
       QToolButton* record;
       QToolButton* solo;
@@ -61,6 +63,7 @@ class Strip : public QFrame {
       ~Strip();
       void setRecordFlag(bool flag);
       Track* getTrack() const { return track; }
+      void setLabelFont();
       };
 
 #endif
