@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//  $Id: route.cpp,v 1.18.2.1 2006/02/01 23:17:38 spamatica Exp $
+//  $Id: route.cpp,v 1.18.2.3 2008/05/21 00:28:52 terminator356 Exp $
 //
 //  (C) Copyright 2003-2004 Werner Schweer (ws@seh.de)
 //=========================================================
@@ -128,9 +128,10 @@ void addRoute(Route src, Route dst)
 
 void removeRoute(Route src, Route dst)
       {
-//    printf("removeRoute %d.%d:<%s> %d.%d:<%s>\n",
-//         src.type, src.channel, src.name().latin1(),
-//         dst.type, dst.channel, dst.name().latin1());
+    //printf("removeRoute %d.%d:<%s> %d.%d:<%s>\n",
+    //     src.type, src.channel, src.name().latin1(),
+    //     dst.type, dst.channel, dst.name().latin1());
+         
       if (src.type == JACK_ROUTE) {
             if (dst.type != TRACK_ROUTE) {
                   fprintf(stderr, "removeRoute: bad route 1\n");
@@ -347,9 +348,18 @@ void Song::readRoute(Xml& xml)
                         break;
                   case Xml::TagEnd:
                         if (xml.s1() == "Route") {
-                              Route s = name2route(src, false);
-                              Route d = name2route(dst, true);
-                              addRoute(s, d);
+                              
+                              if(!src.isEmpty() && !dst.isEmpty())
+                              {
+                                
+                                Route s = name2route(src, false);
+                                Route d = name2route(dst, true);
+                                addRoute(s, d);
+                              
+                              }  
+                              else
+                                printf("  Warning - route name missing. Ignoring route!\n");
+                              
                               return;
                               }
                   default:
