@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//  $Id: seqmsg.cpp,v 1.32.2.4 2006/02/01 23:17:38 spamatica Exp $
+//  $Id: seqmsg.cpp,v 1.32.2.17 2009/12/20 05:00:35 terminator356 Exp $
 //
 //  (C) Copyright 2001 Werner Schweer (ws@seh.de)
 //=========================================================
@@ -259,6 +259,138 @@ void Audio::msgSetChannels(AudioTrack* node, int n)
       sendMsg(&msg);
       }
 
+//---------------------------------------------------------
+//   msgSetPluginCtrlVal
+//---------------------------------------------------------
+
+void Audio::msgSetPluginCtrlVal(PluginI* plugin, int param, double val)
+{
+      AudioMsg msg;
+      
+      msg.id     = AUDIO_SET_PLUGIN_CTRL_VAL;
+      msg.ival   = param;
+      msg.dval = val;
+      msg.plugin = plugin;
+      sendMsg(&msg);
+}
+
+//---------------------------------------------------------
+//   msgSwapControllerIDX
+//---------------------------------------------------------
+
+void Audio::msgSwapControllerIDX(AudioTrack* node, int idx1, int idx2)
+{
+      AudioMsg msg;
+      
+      msg.id     = AUDIO_SWAP_CONTROLLER_IDX;
+      msg.snode  = node;
+      msg.a      = idx1;
+      msg.b      = idx2;
+      sendMsg(&msg);
+}
+
+//---------------------------------------------------------
+//   msgClearControllerEvents
+//---------------------------------------------------------
+
+void Audio::msgClearControllerEvents(AudioTrack* node, int acid)
+{
+      AudioMsg msg;
+      
+      msg.id     = AUDIO_CLEAR_CONTROLLER_EVENTS;
+      msg.snode  = node;
+      msg.ival   = acid;
+      sendMsg(&msg);
+}
+
+//---------------------------------------------------------
+//   msgSeekPrevACEvent
+//---------------------------------------------------------
+
+void Audio::msgSeekPrevACEvent(AudioTrack* node, int acid)
+{
+      AudioMsg msg;
+      
+      msg.id     = AUDIO_SEEK_PREV_AC_EVENT;
+      msg.snode  = node;
+      msg.ival   = acid;
+      sendMsg(&msg);
+}
+
+//---------------------------------------------------------
+//   msgSeekNextACEvent
+//---------------------------------------------------------
+
+void Audio::msgSeekNextACEvent(AudioTrack* node, int acid)
+{
+      AudioMsg msg;
+      
+      msg.id     = AUDIO_SEEK_NEXT_AC_EVENT;
+      msg.snode  = node;
+      msg.ival   = acid;
+      sendMsg(&msg);
+}
+
+//---------------------------------------------------------
+//   msgEraseACEvent
+//---------------------------------------------------------
+
+void Audio::msgEraseACEvent(AudioTrack* node, int acid, int frame)
+{
+      AudioMsg msg;
+      
+      msg.id     = AUDIO_ERASE_AC_EVENT;
+      msg.snode  = node;
+      msg.ival   = acid;
+      msg.a      = frame; 
+      sendMsg(&msg);
+}
+
+//---------------------------------------------------------
+//   msgEraseRangeACEvents
+//---------------------------------------------------------
+
+void Audio::msgEraseRangeACEvents(AudioTrack* node, int acid, int frame1, int frame2)
+{
+      AudioMsg msg;
+      
+      msg.id     = AUDIO_ERASE_RANGE_AC_EVENTS;
+      msg.snode  = node;
+      msg.ival   = acid;
+      msg.a      = frame1; 
+      msg.b      = frame2; 
+      sendMsg(&msg);
+}
+
+//---------------------------------------------------------
+//   msgAddACEvent
+//---------------------------------------------------------
+
+void Audio::msgAddACEvent(AudioTrack* node, int acid, int frame, double val)
+{
+      AudioMsg msg;
+      
+      msg.id     = AUDIO_ADD_AC_EVENT;
+      msg.snode  = node;
+      msg.ival   = acid;
+      msg.a      = frame; 
+      msg.dval   = val;
+      sendMsg(&msg);
+}
+
+//---------------------------------------------------------
+//   msgSetSolo
+//---------------------------------------------------------
+
+void Audio::msgSetSolo(Track* track, bool val)
+{
+      AudioMsg msg;
+      msg.id     = AUDIO_SET_SOLO;
+      msg.track  = track;
+      msg.ival   = int(val);
+      sendMsg(&msg);
+}
+
 //---------------------------------------------------------
 //   msgSetSegSize
 //---------------------------------------------------------
@@ -279,7 +411,8 @@ void Audio::msgSetSegSize(int bs, int sr)
 void Audio::msgSeek(const Pos& pos)
       {
       if (!checkAudioDevice()) return;
-      audioDevice->seekTransport(pos.frame());
+      //audioDevice->seekTransport(pos.frame());
+      audioDevice->seekTransport(pos);
       }
 
 //---------------------------------------------------------
@@ -312,7 +445,15 @@ void Audio::msgPlay(bool val)
       {
       if (val) {
             if (audioDevice)
+            {
+                unsigned sfr = song->cPos().frame();
+                unsigned dcfr = audioDevice->getCurFrame();
+                if(dcfr != sfr)
+                  //audioDevice->seekTransport(sfr);
+                  audioDevice->seekTransport(song->cPos());
                 audioDevice->startTransport();
+            }
+              
       }else {
             if (audioDevice)
                 audioDevice->stopTransport();
@@ -371,22 +512,56 @@ void Audio::msgRemoveTrack(Track* track, bool doUndoFlag)
 //---------------------------------------------------------
 
 void Audio::msgRemoveTracks()
-      {
+{
       bool loop;
-      do {
-            loop = false;
+      do 
+      {
+        loop = false;
+        TrackList* tl = song->tracks();
+        for (iTrack t = tl->begin(); t != tl->end(); ++t) 
+        {
+          Track* tr = *t;
+          if (tr->selected()) 
+          {
+            song->removeTrack1(tr);
+            msgRemoveTrack(tr, false);
+            song->removeTrack3(tr);
+            loop = true;
+            break;
+          }
+        }
+      } 
+      while (loop);
+            
+     /*       
+            // TESTED: DIDN'T WORK: It still skipped some selected tracks !
+            // Quote from SGI STL: "Erasing an element from a map also does not invalidate any iterators, 
+            //                      except, of course, for iterators that actually point to the element 
+            //                      that is being erased."
+            // Well that doesn't seem true here...
+            
             TrackList* tl = song->tracks();
-            for (iTrack t = tl->begin(); t != tl->end(); ++t) {
-                  if ((*t)->selected()) {
-                        song->removeTrack1(*t);
-                        msgRemoveTrack(*t, false);
-                        song->removeTrack3(*t);
-                        loop = true;
-                        break;
-                        }
+            for(ciTrack t = tl->begin(); t != tl->end() ; ) 
+            {
+                  if((*t)->selected()) 
+                  {
+                        // Changed 20070102: - Iterator t becomes invalid after msgRemoveTrack.
+                        ciTrack tt = t;
+                        ++t;
+                        Track* tr = *tt;
+                        
+                        song->removeTrack1(tr);
+                        msgRemoveTrack(tr, false);
+                        song->removeTrack3(tr);
+                        
                   }
-            } while (loop);
-      }
+                  else
+                    ++t;
+                        
+            }
+  */          
+            
+}
 
 //---------------------------------------------------------
 //   msgChangeTrack
@@ -410,6 +585,8 @@ void Audio::msgChangeTrack(Track* oldTrack, Track* newTrack, bool doUndoFlag)
 
 void Audio::msgMoveTrack(int idx1, int idx2, bool doUndoFlag)
       {
+      if (idx1 < 0 || idx2 < 0)   // sanity check
+            return;
       int n = song->tracks()->size();
       if (idx1 >= n || idx2 >= n)   // sanity check
             return;
@@ -484,12 +661,15 @@ bool Song::msgRemoveParts()
 //   msgChangePart
 //---------------------------------------------------------
 
-void Audio::msgChangePart(Part* oldPart, Part* newPart, bool doUndoFlag)
+//void Audio::msgChangePart(Part* oldPart, Part* newPart, bool doUndoFlag)
+void Audio::msgChangePart(Part* oldPart, Part* newPart, bool doUndoFlag, bool doCtrls, bool doClones)
       {
       AudioMsg msg;
       msg.id = SEQM_CHANGE_PART;
       msg.p1 = oldPart;
       msg.p2 = newPart;
+      msg.a = doCtrls;
+      msg.b = doClones;
       sendMessage(&msg, doUndoFlag);
       }
 
@@ -497,12 +677,15 @@ void Audio::msgChangePart(Part* oldPart, Part* newPart, bool doUndoFlag)
 //   msgAddEvent
 //---------------------------------------------------------
 
-void Audio::msgAddEvent(Event& event, Part* part, bool doUndoFlag)
+//void Audio::msgAddEvent(Event& event, Part* part, bool doUndoFlag)
+void Audio::msgAddEvent(Event& event, Part* part, bool doUndoFlag, bool doCtrls, bool doClones)
       {
       AudioMsg msg;
       msg.id = SEQM_ADD_EVENT;
       msg.ev1 = event;
       msg.p2 = part;
+      msg.a = doCtrls;
+      msg.b = doClones;
       sendMessage(&msg, doUndoFlag);
       }
 
@@ -510,13 +693,15 @@ void Audio::msgAddEvent(Event& event, Part* part, bool doUndoFlag)
 //   msgDeleteEvent
 //---------------------------------------------------------
 
-void Audio::msgDeleteEvent(Event& event, Part* part, bool doUndoFlag)
+//void Audio::msgDeleteEvent(Event& event, Part* part, bool doUndoFlag)
+void Audio::msgDeleteEvent(Event& event, Part* part, bool doUndoFlag, bool doCtrls, bool doClones)
       {
       AudioMsg msg;
       msg.id = SEQM_REMOVE_EVENT;
       msg.ev1 = event;
       msg.p2 = part;
-      
+      msg.a = doCtrls;
+      msg.b = doClones;
       sendMessage(&msg, doUndoFlag);
       }
 
@@ -524,13 +709,16 @@ void Audio::msgDeleteEvent(Event& event, Part* part, bool doUndoFlag)
 //   msgChangeEvent
 //---------------------------------------------------------
 
-void Audio::msgChangeEvent(Event& oe, Event& ne, Part* part, bool doUndoFlag)
+//void Audio::msgChangeEvent(Event& oe, Event& ne, Part* part, bool doUndoFlag)
+void Audio::msgChangeEvent(Event& oe, Event& ne, Part* part, bool doUndoFlag, bool doCtrls, bool doClones)
       {
       AudioMsg msg;
       msg.id = SEQM_CHANGE_EVENT;
       msg.ev1 = oe;
       msg.ev2 = ne;
       msg.p3 = part;
+      msg.a = doCtrls;
+      msg.b = doClones;
       sendMessage(&msg, doUndoFlag);
       }
 
@@ -669,6 +857,17 @@ void Audio::msgLocalOff()
       sendMessage(&msg, false);
       }
 
+//---------------------------------------------------------
+//   msgUpdateSoloStates
+//---------------------------------------------------------
+
+void Audio::msgUpdateSoloStates()
+      {
+      AudioMsg msg;
+      msg.id    = SEQM_UPDATE_SOLO_STATES;
+      sendMsg(&msg);
+      }
+
 //---------------------------------------------------------
 //   msgSetAux
 //---------------------------------------------------------
@@ -695,6 +894,104 @@ void Audio::msgPlayMidiEvent(const MidiPlayEvent* event)
       sendMessage(&msg, false);
       }
 
+//---------------------------------------------------------
+//   msgSetHwCtrlState
+//---------------------------------------------------------
+
+void Audio::msgSetHwCtrlState(MidiPort* port, int ch, int ctrl, int val)
+      {
+      AudioMsg msg;
+      msg.id = SEQM_SET_HW_CTRL_STATE;
+      msg.p1 = port;
+      msg.a = ch;
+      msg.b = ctrl;
+      msg.c = val;
+      sendMessage(&msg, false);
+      }
+
+//---------------------------------------------------------
+//   msgSetHwCtrlState
+//---------------------------------------------------------
+
+void Audio::msgSetHwCtrlStates(MidiPort* port, int ch, int ctrl, int val, int lastval)
+      {
+      AudioMsg msg;
+      msg.id = SEQM_SET_HW_CTRL_STATE;
+      msg.p1 = port;
+      msg.a = ch;
+      msg.b = ctrl;
+      msg.c = val;
+      msg.ival = lastval;
+      sendMessage(&msg, false);
+      }
+
+//---------------------------------------------------------
+//   msgSetTrackOutChannel
+//---------------------------------------------------------
+
+void Audio::msgSetTrackOutChannel(MidiTrack* track, int ch)
+{
+      AudioMsg msg;
+      msg.id = SEQM_SET_TRACK_OUT_CHAN;
+      msg.p1 = track;
+      msg.a = ch;
+      sendMessage(&msg, false);
+}
+
+//---------------------------------------------------------
+//   msgSetTrackOutPort
+//---------------------------------------------------------
+
+void Audio::msgSetTrackOutPort(MidiTrack* track, int port)
+{
+      AudioMsg msg;
+      msg.id = SEQM_SET_TRACK_OUT_PORT;
+      msg.p1 = track;
+      msg.a = port;
+      sendMessage(&msg, false);
+}
+
+//---------------------------------------------------------
+//   msgRemapPortDrumCtlEvents
+//---------------------------------------------------------
+
+void Audio::msgRemapPortDrumCtlEvents(int mapidx, int newnote, int newchan, int newport)
+{
+      AudioMsg msg;
+      msg.id = SEQM_REMAP_PORT_DRUM_CTL_EVS;
+      msg.ival = mapidx;
+      msg.a = newnote;
+      msg.b = newchan;
+      msg.c = newport;
+      sendMessage(&msg, false);
+}
+
+//---------------------------------------------------------
+//   msgChangeAllPortDrumCtlEvents
+//---------------------------------------------------------
+
+void Audio::msgChangeAllPortDrumCtrlEvents(bool add, bool drumonly)
+{
+      AudioMsg msg;
+      msg.id = SEQM_CHANGE_ALL_PORT_DRUM_CTL_EVS;
+      msg.a = (int)add;
+      msg.b = (int)drumonly;
+      sendMessage(&msg, false);
+}
+
+//---------------------------------------------------------
+//   msgSetSendMetronome
+//---------------------------------------------------------
+
+void Audio::msgSetSendMetronome(AudioTrack* track, bool b)
+{
+      AudioMsg msg;
+      msg.id    = AUDIO_SET_SEND_METRONOME;
+      msg.snode = track;
+      msg.ival  = (int)b;
+      sendMessage(&msg, false);
+}
+
 //---------------------------------------------------------
 //   msgBounce
 //    start bounce operation
@@ -704,7 +1001,8 @@ void Audio::msgBounce()
       {
       _bounce = true;
       if (!checkAudioDevice()) return;
-      audioDevice->seekTransport(song->lPos().frame());
+      //audioDevice->seekTransport(song->lPos().frame());
+      audioDevice->seekTransport(song->lPos());
       }
 
 //---------------------------------------------------------
