@@ -29,6 +29,7 @@ void initShortCuts()
       defShrt(SHRT_PLAY_SONG,     Qt::Key_Enter, "Transport: Start playback from current location", GLOBAL_SHRT, "play");
       defShrt(SHRT_TOGGLE_METRO,  Qt::Key_C,     "Transport: Toggle metronome", GLOBAL_SHRT,"toggle_metro");
       defShrt(SHRT_STOP,          Qt::Key_Insert,"Transport: Stop Playback", GLOBAL_SHRT, "stop");
+      defShrt(SHRT_GOTO_START,    Qt::Key_W,      "Transport: Goto Start", GLOBAL_SHRT, "goto_start");
       defShrt(SHRT_PLAY_TOGGLE,   Qt::Key_Space, "Transport: Play, Stop, Rewind", GLOBAL_SHRT, "play_toggle");
       defShrt(SHRT_GOTO_LEFT,     Qt::Key_End,   "Transport: Goto left marker"  , GLOBAL_SHRT, "goto_left");
       defShrt(SHRT_GOTO_RIGHT,    Qt::Key_PageDown,  "Transport: Goto right marker" , GLOBAL_SHRT, "goto_right");
@@ -62,6 +63,7 @@ void initShortCuts()
       defShrt(SHRT_OPEN_PIANO,            Qt::CTRL + Qt::Key_E, "Open pianoroll", ARRANG_SHRT, "open_pianoroll");
       defShrt(SHRT_OPEN_DRUMS,            Qt::CTRL + Qt::Key_D, "Open drumeditor", ARRANG_SHRT, "open_drumedit");
       defShrt(SHRT_OPEN_LIST,             Qt::CTRL + Qt::Key_L, "Open listeditor", ARRANG_SHRT, "open_listedit");
+      defShrt(SHRT_OPEN_WAVE,             Qt::CTRL + Qt::Key_W, "Open waveeditor", ARRANG_SHRT, "open_waveedit");
       defShrt(SHRT_OPEN_GRAPHIC_MASTER,   Qt::CTRL + Qt::Key_M, "Open graphical mastertrack editor", ARRANG_SHRT, "open_graph_master");
       defShrt(SHRT_OPEN_LIST_MASTER,      Qt::CTRL + Qt::SHIFT + Qt::Key_M, "Open list mastertrack editor", ARRANG_SHRT, "open_list_master");
       defShrt(SHRT_OPEN_MIDI_TRANSFORM,   Qt::CTRL + Qt::Key_T, "Open midi transformer", ARRANG_SHRT, "open_midi_transform");
@@ -144,6 +146,11 @@ void initShortCuts()
       defShrt(SHRT_DEC_PITCH,        Qt::CTRL + Qt::Key_Down, "Edit: Decrease pitch", PROLL_SHRT + DEDIT_SHRT, "sel_dec_pitch");
       defShrt(SHRT_INC_POS,          Qt::CTRL + Qt::Key_Right,  "Edit: Increase event position", PROLL_SHRT + DEDIT_SHRT, "sel_inc_pos");
       defShrt(SHRT_DEC_POS,          Qt::CTRL + Qt::Key_Left,   "Edit: Decrease event position", PROLL_SHRT + DEDIT_SHRT, "sel_dec_pos");
+      defShrt(SHRT_ZOOM_IN,          Qt::CTRL + Qt::Key_PageUp,   "View: Zoom in",  PROLL_SHRT + DEDIT_SHRT, "zoom_in");
+      defShrt(SHRT_ZOOM_OUT,         Qt::CTRL + Qt::Key_PageDown, "View: Zoom out", PROLL_SHRT + DEDIT_SHRT, "zoom_out");
+      defShrt(SHRT_GOTO_CPOS,        Qt::Key_C, "View: Goto current position", PROLL_SHRT + DEDIT_SHRT, "goto_cpos");
+      defShrt(SHRT_SCROLL_LEFT,      Qt::Key_H, "View: Scroll left", PROLL_SHRT + DEDIT_SHRT, "scroll_left");
+      defShrt(SHRT_SCROLL_RIGHT,     Qt::Key_L, "View: Scroll left", PROLL_SHRT + DEDIT_SHRT, "scroll_right");
 
       //-----------------------------------------------------------
       //Drum:
@@ -165,6 +172,7 @@ void initShortCuts()
       defShrt(SHRT_CRESCENDO,        0, "Edit: Crescendo", PROLL_SHRT, "midi_crescendo");
       defShrt(SHRT_THIN_OUT,         0, "Edit: Thin Out", PROLL_SHRT, "midi_thin_out");
       defShrt(SHRT_ERASE_EVENT,      0, "Edit: Erase Event", PROLL_SHRT, "midi_erase_event");
+      defShrt(SHRT_DELETE_OVERLAPS,  0, "Edit: Delete overlaps", PROLL_SHRT, "midi_delete_overlaps");
       defShrt(SHRT_NOTE_SHIFT,       0, "Edit: Note Shift", PROLL_SHRT, "midi_note_shift");
       defShrt(SHRT_MOVE_CLOCK,       0, "Edit: Move Clock", PROLL_SHRT, "midi_move_clock");
       defShrt(SHRT_COPY_MEASURE,     0, "Edit: Copy Measure", PROLL_SHRT, "midi_copy_measure");
@@ -174,7 +182,6 @@ void initShortCuts()
       defShrt(SHRT_EVENT_COLOR,      Qt::Key_E, "Edit: Change event color", PROLL_SHRT, "change_event_color");
 
 
-
       // Shortcuts for tools
       // global
       defShrt(SHRT_TOOL_POINTER,     Qt::Key_A, "Tool: Pointer", ARRANG_SHRT + PROLL_SHRT + DEDIT_SHRT, "pointer_tool");
