@@ -54,6 +54,7 @@ enum {
       SHRT_PLAY_SONG, //Enter
       SHRT_PLAY_TOGGLE, //Space
       SHRT_STOP, //Insert
+      SHRT_GOTO_START, // W
       SHRT_GOTO_LEFT, //End-keypad
       SHRT_GOTO_RIGHT, //Cursordown-keypad
       SHRT_POS_INC, // Plus
@@ -93,6 +94,7 @@ enum {
       SHRT_OPEN_SCORE, //Ctrl+R
       SHRT_OPEN_DRUMS, //Ctrl+D
       SHRT_OPEN_LIST, //Ctrl+L
+      SHRT_OPEN_WAVE, //Ctrl+W
       SHRT_OPEN_GRAPHIC_MASTER, //Ctrl+M
       SHRT_OPEN_LIST_MASTER, //Ctrl+Shift+M
       SHRT_OPEN_MIDI_TRANSFORM, //Ctrl+T
@@ -175,8 +177,6 @@ enum {
       SHRT_INC_POS,
       SHRT_DEC_POS,
       SHRT_LOCATORS_TO_SELECTION, //Alt+P, currently in arranger & pianoroll
-      SHRT_ZOOM_IN, //H - pianoroll
-      SHRT_ZOOM_OUT, //G - pianoroll
       SHRT_INSERT_AT_LOCATION, //Shift+CrsrRight
 
       SHRT_TOOL_1,//Shift+1 Pointer
@@ -188,6 +188,11 @@ enum {
       SHRT_TRANSPOSE, //Default: undefined
 
       //Shortcuts to be in pianoroll & drumeditor
+      SHRT_ZOOM_IN,  // PgUp
+      SHRT_ZOOM_OUT, // PgDown
+      SHRT_GOTO_CPOS, // c
+      SHRT_SCROLL_LEFT, // h
+      SHRT_SCROLL_RIGHT, // l
       SHRT_FIXED_LEN, //Alt+L, currently only drumeditor
       SHRT_QUANTIZE, //q
       SHRT_OVER_QUANTIZE, //Default: undefined
@@ -198,6 +203,7 @@ enum {
       SHRT_MODIFY_GATE_TIME, //Default: undefined
       SHRT_MODIFY_VELOCITY,
       SHRT_CRESCENDO,
+      SHRT_DELETE_OVERLAPS,
 
       SHRT_THIN_OUT,
       SHRT_ERASE_EVENT,
