@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//  $Id: sync.cpp,v 1.6.2.8 2006/03/02 22:27:23 spamatica Exp $
+//  $Id: sync.cpp,v 1.6.2.12 2009/06/20 22:20:41 terminator356 Exp $
 //
 //  (C) Copyright 2003 Werner Schweer (ws@seh.de)
 //=========================================================
@@ -17,22 +17,26 @@
 #include "audio.h"
 #include "audiodev.h"
 #include "gconfig.h"
+#include "xml.h"
 
-int rxSyncPort = -1;         // receive from all ports
-int txSyncPort = 1;
-int rxDeviceId = 0x7f;       // any device
-int txDeviceId = 0x7f;       // any device
+//int rxSyncPort = -1;         // receive from all ports
+//int txSyncPort = 1;
+//int rxDeviceId = 0x7f;       // any device
+//int txDeviceId = 0x7f;       // any device
+//MidiSyncPort midiSyncPorts[MIDI_PORTS];
+int curMidiSyncInPort = -1;
 
 bool debugSync = false;
 int mtcType     = 1;
 MTC mtcOffset;
 BValue extSyncFlag(0, "extSync");       // false - MASTER, true - SLAVE
-bool genMTCSync = false;      // output MTC Sync
-bool genMCSync  = false;      // output MidiClock Sync
-bool genMMC     = false;      // output Midi Machine Control
-bool acceptMTC  = false;
-bool acceptMC   = true;
-bool acceptMMC  = true;
+//bool genMTCSync = false;      // output MTC Sync
+//bool genMCSync  = false;      // output MidiClock Sync
+//bool genMMC     = false;      // output Midi Machine Control
+//bool acceptMTC  = false;
+//bool acceptMC   = true;
+//bool acceptMMC  = true;
+bool useJackTransport = true;
 
 static MTC mtcCurTime;
 static int mtcState;    // 0-7 next expected quarter message
@@ -40,8 +44,280 @@ static bool mtcValid;
 static int mtcLost;
 static bool mtcSync;    // receive complete mtc frame?
 
- static bool mcStart = false;
- static int mcStartTick;
+// Not used yet.
+// static bool mcStart = false;
+// static int mcStartTick;
+
+//---------------------------------------------------------
+//  MidiSyncInfo
+//---------------------------------------------------------
+
+MidiSyncInfo::MidiSyncInfo()
+{
+  _port          = -1;
+  _idOut         = 127;
+  _idIn          = 127;
+  _sendMC        = false;
+  _sendMMC       = false;
+  _sendMTC       = false;
+  _recMC         = false;
+  _recMMC        = false;
+  _recMTC        = false;
+  
+  _lastClkTime   = 0.0;
+  _lastTickTime  = 0.0;
+  _clockTrig     = false;
+  _tickTrig      = false;
+  _clockDetect   = false;
+  _tickDetect    = false;
+  _actDetectBits = 0;
+  for(int i = 0; i < MIDI_CHANNELS; ++i)
+  {
+    _lastActTime[i] = 0.0;
+    _actTrig[i]     = false;
+    _actDetect[i]   = false;
+  }
+}
+
+//---------------------------------------------------------
+//   operator =
+//---------------------------------------------------------
+
+MidiSyncInfo& MidiSyncInfo::operator=(const MidiSyncInfo &sp)
+{
+  //_port          = sp._port;
+  
+  copyParams(sp);
+  
+  _lastClkTime   = sp._lastClkTime;
+  _lastTickTime  = sp._lastTickTime;
+  _clockTrig     = sp._clockTrig;
+  _tickTrig      = sp._tickTrig;
+  _clockDetect   = sp._clockDetect;
+  _tickDetect    = sp._tickDetect;
+  _actDetectBits = sp._actDetectBits;
+  for(int i = 0; i < MIDI_CHANNELS; ++i)
+  {
+    _lastActTime[i] = sp._lastActTime[i];
+    _actTrig[i]     = sp._actTrig[i];
+    _actDetect[i]   = sp._actDetect[i];
+  }
+  return *this;
+}
+
+//---------------------------------------------------------
+//   copyParams
+//---------------------------------------------------------
+
+MidiSyncInfo& MidiSyncInfo::copyParams(const MidiSyncInfo &sp)
+{
+  //_port          = sp._port;
+  
+  _idOut         = sp._idOut;
+  _idIn          = sp._idIn;
+  _sendMC        = sp._sendMC;
+  _sendMMC       = sp._sendMMC;
+  _sendMTC       = sp._sendMTC;
+  setMCIn(sp._recMC);
+  _recMMC        = sp._recMMC;
+  _recMTC        = sp._recMTC;
+  return *this;
+}
+
+//---------------------------------------------------------
+//  setTime
+//---------------------------------------------------------
+
+void MidiSyncInfo::setTime() 
+{ 
+  // Note: CurTime() makes a system call to gettimeofday(), 
+  //  which apparently can be slow in some cases. So I avoid calling this function
+  //  too frequently by calling it (at the heartbeat rate) in Song::beat().  T356
+  double t = curTime();
+  
+  if(_clockTrig)
+  {
+    _clockTrig = false;
+    _lastClkTime = t;  
+  }
+  else
+  if(_clockDetect && (t - _lastClkTime >= 1.0)) // Set detect indicator timeout to about 1 second.
+  {
+    _clockDetect = false;
+    if(curMidiSyncInPort == _port)
+      curMidiSyncInPort = -1;
+  }
+  
+  if(_tickTrig)
+  {
+    _tickTrig = false;
+    _lastTickTime = t;  
+  }
+  else
+  if(_tickDetect && (t - _lastTickTime) >= 1.0) // Set detect indicator timeout to about 1 second.
+    _tickDetect = false;
+    
+  for(int i = 0; i < MIDI_CHANNELS; i++)
+  {
+    if(_actTrig[i])
+    {
+      _actTrig[i] = false;
+      _lastActTime[i] = t;  
+    }
+    else
+    if(_actDetect[i] && (t - _lastActTime[i]) >= 1.0) // Set detect indicator timeout to about 1 second.
+    {
+      _actDetect[i] = false;
+      _actDetectBits &= ~bitShiftLU[i];
+    }  
+  }
+}
+
+//---------------------------------------------------------
+//  setMCIn
+//---------------------------------------------------------
+
+void MidiSyncInfo::setMCIn(const bool v) 
+{ 
+  _recMC = v; 
+  // If sync receive was turned off, clear the current midi sync in port number so another port can grab it.
+  if(!_recMC && _port != -1 && curMidiSyncInPort == _port)
+    curMidiSyncInPort = -1;
+}
+
+//---------------------------------------------------------
+//  trigMCSyncDetect
+//---------------------------------------------------------
+
+void MidiSyncInfo::trigMCSyncDetect() 
+{ 
+  _clockDetect = true;
+  _clockTrig = true;
+  // Set the current midi sync in port number if it's not taken...
+  if(_recMC && curMidiSyncInPort == -1)
+    curMidiSyncInPort = _port;
+}
+
+//---------------------------------------------------------
+//  trigTickDetect
+//---------------------------------------------------------
+
+void MidiSyncInfo::trigTickDetect()   
+{ 
+  _tickDetect = true;
+  _tickTrig = true;
+}
+    
+//---------------------------------------------------------
+//  actDetect
+//---------------------------------------------------------
+
+bool MidiSyncInfo::actDetect(const int ch) const
+{ 
+  if(ch < 0 || ch >= MIDI_CHANNELS)
+    return false;
+    
+  return _actDetect[ch]; 
+}           
+
+//---------------------------------------------------------
+//  trigActDetect
+//---------------------------------------------------------
+
+void MidiSyncInfo::trigActDetect(const int ch)   
+{ 
+  if(ch < 0 || ch >= MIDI_CHANNELS)
+    return;
+    
+  _actDetectBits |= bitShiftLU[ch];
+  _actDetect[ch] = true;
+  _actTrig[ch] = true;
+}
+    
+//---------------------------------------------------------
+//   read
+//---------------------------------------------------------
+
+void MidiSyncInfo::read(Xml& xml)
+      {
+      for (;;) {
+            Xml::Token token(xml.parse());
+            const QString& tag(xml.s1());
+            switch (token) {
+                  case Xml::Error:
+                  case Xml::End:
+                        return;
+                  case Xml::TagStart:
+                             if (tag == "idOut")
+                              _idOut = xml.parseInt();
+                        else if (tag == "idIn")
+                              _idIn = xml.parseInt();
+                        else if (tag == "sendMC")
+                              _sendMC = xml.parseInt();
+                        else if (tag == "sendMMC")
+                              _sendMMC = xml.parseInt();
+                        else if (tag == "sendMTC")
+                              _sendMTC = xml.parseInt();
+                        else if (tag == "recMC")
+                              _recMC = xml.parseInt();
+                        else if (tag == "recMMC")
+                              _recMMC = xml.parseInt();
+                        else if (tag == "recMTC")
+                              _recMTC = xml.parseInt();
+                        else
+                              xml.unknown("midiSyncInfo");
+                        break;
+                  case Xml::TagEnd:
+                        if (tag == "midiSyncInfo")
+                            return;
+                  default:
+                        break;
+                  }
+            }
+      }
+
+//---------------------------------------------------------
+//  write
+//---------------------------------------------------------
+
+//void MidiSyncInfo::write(int level, Xml& xml, MidiDevice* md)
+void MidiSyncInfo::write(int level, Xml& xml)
+{
+  //if(!md)
+  //  return;
+  
+  // All defaults? Nothing to write.
+  if(_idOut == 127 && _idIn == 127 && !_sendMC && !_sendMMC && !_sendMTC && !_recMC && !_recMMC && !_recMTC)
+    return;
+  
+  xml.tag(level++, "midiSyncInfo");
+  //xml.intTag(level, "idx", idx);
+  //xml.intTag(level++, "midiSyncPort", idx);
+  //xml.tag(level++, "midiSyncInfo idx=\"%d\"", idx);
+  
+  //xml.strTag(level, "device", md->name());
+  
+  if(_idOut != 127)
+    xml.intTag(level, "idOut", _idOut);
+  if(_idIn != 127)
+    xml.intTag(level, "idIn", _idIn);
+  
+  if(_sendMC)
+    xml.intTag(level, "sendMC", true);
+  if(_sendMMC)
+    xml.intTag(level, "sendMMC", true);
+  if(_sendMTC)
+    xml.intTag(level, "sendMTC", true);
+  
+  if(_recMC)
+    xml.intTag(level, "recMC", true);
+  if(_recMMC)
+    xml.intTag(level, "recMMC", true);
+  if(_recMTC)
+    xml.intTag(level, "recMTC", true);
+  
+  xml.etag(level, "midiSyncInfo");
+}
 
 //---------------------------------------------------------
 //  mmcInput
@@ -53,7 +329,8 @@ void MidiSeq::mmcInput(const unsigned char* p, int n)
       if (debugSync)
             printf("mmcInput: n:%d %02x %02x %02x %02x\n",
                n, p[2], p[3], p[4], p[5]);
-      if (!(extSyncFlag.value() && acceptMMC))
+      //if (!(extSyncFlag.value() && acceptMMC))
+      if(!extSyncFlag.value())
             return;
 
       switch(p[3]) {
@@ -109,7 +386,8 @@ void MidiSeq::mmcInput(const unsigned char* p, int n)
 
                         Pos tp(mmcPos, true);
                         if (!checkAudioDevice()) return;
-                        audioDevice->seekTransport(tp.frame());
+                        //audioDevice->seekTransport(tp.frame());
+                        audioDevice->seekTransport(tp);
                         alignAllTicks();
                         //seek(tp);
                         if (debugSync) {
@@ -259,12 +537,25 @@ void MidiSeq::setSongPosition(int port, int midiBeat)
       {
       if (midiInputTrace)
             printf("set song position port:%d %d\n", port, midiBeat);
-      if (!extSyncFlag.value())
+      
+      midiPorts[port].syncInfo().trigMCSyncDetect();
+      
+      //if (!extSyncFlag.value())
+      // External sync not on? Clock in not turned on? 
+      if(!extSyncFlag.value() || !midiPorts[port].syncInfo().MCIn())
             return;
+            
       Pos pos((config.division * midiBeat) / 4, true);
+      
+      // Re-transmit song position to other devices if clock out turned on.
+      for(int p = 0; p < MIDI_PORTS; ++p)
+        if(p != port && midiPorts[p].syncInfo().MCOut())
+          midiPorts[p].sendSongpos(midiBeat);
+                  
       if (!checkAudioDevice()) return;
 
-      audioDevice->seekTransport(pos.frame());
+      //audioDevice->seekTransport(pos.frame());
+      audioDevice->seekTransport(pos);
       alignAllTicks(pos.frame());
       if (debugSync)
             printf("setSongPosition %d\n", pos.tick());
@@ -289,12 +580,14 @@ void MidiSeq::alignAllTicks(int frameOverride)
 
       // use the last old values to give start values for the tripple buffering
       int recTickSpan = recTick1 - recTick2;
-      int songTickSpan = songtick1 - songtick2;
+      int songTickSpan = (int)(songtick1 - songtick2);    //prevent compiler warning:  casting double to int
       storedtimediffs = 0; // pretend there is no sync history
 
       mclock2=mclock1=0.0; // set all clock values to "in sync"
-      recTick = (double(curFrame)/double(sampleRate)) *
-                                           double(config.division * 1000000.0) / double(tempo);
+
+      recTick = (int) ((double(curFrame)/double(sampleRate)) *
+                        double(config.division * 1000000.0) / double(tempo) //prevent compiler warning:  casting double to int
+                );
       songtick1 = recTick - songTickSpan;
       if (songtick1 < 0)
         songtick1 = 0;
@@ -312,8 +605,6 @@ void MidiSeq::alignAllTicks(int frameOverride)
 
       }
 
-
-
 //---------------------------------------------------------
 //   realtimeSystemInput
 //    real time message received
@@ -324,17 +615,41 @@ void MidiSeq::realtimeSystemInput(int port, int c)
       if (midiInputTrace)
             printf("realtimeSystemInput port:%d 0x%x\n", port+1, c);
 
-      if (midiInputTrace && (rxSyncPort != port) && rxSyncPort != -1) {
-            if (debugSync)
-                  printf("rxSyncPort configured as %d; received sync from port %d\n",
-                     rxSyncPort, port);
-            return;
-            }
-      if (!extSyncFlag.value())
-            return;
+      //if (midiInputTrace && (rxSyncPort != port) && rxSyncPort != -1) {
+      //      if (debugSync)
+      //            printf("rxSyncPort configured as %d; received sync from port %d\n",
+      //               rxSyncPort, port);
+      //      return;
+      //      }
+      
+      MidiPort* mp = &midiPorts[port];
+      
+      // Trigger on any tick, clock, or realtime command. 
+      if(c == 0xf9)
+        mp->syncInfo().trigTickDetect();
+      else
+        mp->syncInfo().trigMCSyncDetect();
+       
+      // External sync not on? Clock in not turned on? 
+      if(!extSyncFlag.value() || !mp->syncInfo().MCIn())
+        return;
+      
       switch(c) {
             case 0xf8:  // midi clock (24 ticks / quarter note)
                   {
+                  // Not for the current in port? Forget it.
+                  if(port != curMidiSyncInPort)
+                    break;
+                  
+                  // Re-transmit clock to other devices if clock out turned on.
+                  // Must be careful not to allow more than one clock input at a time.
+                  // Would re-transmit mixture of multiple clocks - confusing receivers.
+                  // Solution: Added curMidiSyncInPort. 
+                  // Maybe in MidiSeq::processTimerTick(), call sendClock for the other devices, instead of here.
+                  for(int p = 0; p < MIDI_PORTS; ++p)
+                    if(p != port && midiPorts[p].syncInfo().MCOut())
+                      midiPorts[p].sendClock();
+                  
                   double mclock0 = curTime();
                   // Difference in time last 2 rounds:
                   double tdiff0   = mclock0 - mclock1;
@@ -422,7 +737,7 @@ void MidiSeq::realtimeSystemInput(int port, int c)
                   // Calculate tempo depending on time per pulse
                   //
                   if (mclock1 == 0.0) {
-                        midiPorts[port].device()->discardInput();
+                        mp->device()->discardInput();
                         if (debugSync)
                            printf("Discarding input from port %d\n", port);
                         }
@@ -445,18 +760,25 @@ void MidiSeq::realtimeSystemInput(int port, int c)
                   }
                   break;
             case 0xf9:  // midi tick  (every 10 msec)
-                  if (mcStart) {
-                        song->setPos(0, mcStartTick);
-                        mcStart = false;
-                        return;
-                        }
+                  // FIXME: Unfinished? mcStartTick is uninitialized and Song::setPos doesn't set it either. Dangerous to allow this.
+                  //if (mcStart) {
+                  //      song->setPos(0, mcStartTick);
+                  //      mcStart = false;
+                  //      return;
+                  //      }
                   break;
             case 0xfa:  // start
+                  // Re-transmit start to other devices if clock out turned on.
+                  for(int p = 0; p < MIDI_PORTS; ++p)
+                    if(p != port && midiPorts[p].syncInfo().MCOut())
+                      midiPorts[p].sendStart();
+                  
                   if (debugSync)
                         printf("   start\n");
                   if (1 /* !audio->isPlaying()*/ /*state == IDLE*/) {
                         if (!checkAudioDevice()) return;
-                        audioDevice->seekTransport(0);
+                        //audioDevice->seekTransport(0);
+                        audioDevice->seekTransport(Pos(0, false));
                         unsigned curFrame = audio->curFrame();
                         if (debugSync)
                               printf("       curFrame=%d\n", curFrame);
@@ -473,6 +795,11 @@ void MidiSeq::realtimeSystemInput(int port, int c)
                         }
                   break;
             case 0xfb:  // continue
+                  // Re-transmit continue to other devices if clock out turned on.
+                  for(int p = 0; p < MIDI_PORTS; ++p)
+                    if(p != port && midiPorts[p].syncInfo().MCOut())
+                      midiPorts[p].sendContinue();
+                  
                   if (debugSync)
                         printf("   continue\n");
                   if (1 /* !audio->isPlaying() */ /*state == IDLE */) {
@@ -484,6 +811,11 @@ void MidiSeq::realtimeSystemInput(int port, int c)
                         }
                   break;
             case 0xfc:  // stop
+                  // Re-transmit stop to other devices if clock out turned on.
+                  for(int p = 0; p < MIDI_PORTS; ++p)
+                    if(p != port && midiPorts[p].syncInfo().MCOut())
+                      midiPorts[p].sendStop();
+                  
                   if (debugSync)
                         printf("   stop\n");
                   if (audio->isPlaying() /*state == PLAY*/) {
