@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//    $Id: synth.cpp,v 1.43.2.7 2006/03/25 21:13:15 spamatica Exp $
+//    $Id: synth.cpp,v 1.43.2.23 2009/12/15 03:39:58 terminator356 Exp $
 //  (C) Copyright 2000-2003 Werner Schweer (ws@seh.de)
 //=========================================================
 
@@ -38,13 +38,14 @@ std::vector<Synth*> synthis;  // array of available synthis
 
 extern void connectNodes(AudioTrack*, AudioTrack*);
 
+/*
 //---------------------------------------------------------
 //   description
 //---------------------------------------------------------
 
 const char* MessSynth::description() const
       {
-      return descr ? descr->description : "";
+      return _descr ? _descr->description : "";
       }
 
 //---------------------------------------------------------
@@ -53,8 +54,9 @@ const char* MessSynth::description() const
 
 const char* MessSynth::version() const
       {
-      return descr ? descr->version : "";
+      return _descr ? _descr->version : "";
       }
+*/
 
 bool MessSynthIF::guiVisible() const
       {
@@ -107,14 +109,19 @@ void MessSynthIF::setGeometry(int x, int y, int w, int h)
 //    search for synthesizer base class
 //---------------------------------------------------------
 
-static Synth* findSynth(const QString& sclass)
+//static Synth* findSynth(const QString& sclass)
+static Synth* findSynth(const QString& sclass, const QString& label)
       {
       for (std::vector<Synth*>::iterator i = synthis.begin();
-         i != synthis.end(); ++i) {
-            if ((*i)->baseName() == sclass)
+         i != synthis.end(); ++i) 
+         {
+            //if ((*i)->baseName() == sclass)
+            //if ((*i)->name() == sclass)
+            if ( ((*i)->baseName() == sclass) && (label.isEmpty() || ((*i)->name() == label)) )
+                  
                   return *i;
-            }
-      printf("synthi class <%s> not found\n", sclass.latin1());
+         }
+      printf("synthi class:%s label:%s not found\n", sclass.latin1(), label.latin1());
       return 0;
       }
 
@@ -123,22 +130,26 @@ static Synth* findSynth(const QString& sclass)
 //    create a synthesizer instance of class "label"
 //---------------------------------------------------------
 
-static SynthI* createSynthI(const QString& sclass)
+//static SynthI* createSynthI(const QString& sclass)
+static SynthI* createSynthI(const QString& sclass, const QString& label)
       {
-      Synth* s = findSynth(sclass);
+      //Synth* s = findSynth(sclass);
+      Synth* s = findSynth(sclass, label);
       SynthI* si = 0;
       if (s) {
             si = new SynthI();
             QString n;
             n.setNum(s->instances());
-            QString instance_name = s->baseName() + "-" + n;
+            //QString instance_name = s->baseName() + "-" + n;
+            QString instance_name = s->name() + "-" + n;
+            
             if (si->initInstance(s, instance_name)) {
                   delete si;
                   return 0;
                   }
             }
       else
-            printf("synthi class <%s> not found\n", sclass.latin1());
+            printf("synthi class:%s label:%s not found\n", sclass.latin1(), label.latin1());
       return si;
       }
 
@@ -146,8 +157,12 @@ static SynthI* createSynthI(const QString& sclass)
 //   Synth
 //---------------------------------------------------------
 
-Synth::Synth(const QFileInfo& fi)
-   : info(fi)
+//Synth::Synth(const QFileInfo& fi)
+//   : info(fi)
+//Synth::Synth(const QFileInfo& fi, QString label)
+//   : info(fi), _name(label)
+Synth::Synth(const QFileInfo& fi, QString label, QString descr, QString maker, QString ver)
+   : info(fi), _name(label), _description(descr), _maker(maker), _version(ver)
       {
       _instances = 0;
       }
@@ -156,12 +171,15 @@ Synth::Synth(const QFileInfo& fi)
 //   instantiate
 //---------------------------------------------------------
 
-void* MessSynth::instantiate()
+//void* MessSynth::instantiate()
+void* MessSynth::instantiate(const QString& instanceName)
       {
       ++_instances;
-      QString n;
-      n.setNum(_instances);
-      QString instanceName = baseName() + "-" + n;
+      
+      //QString n;
+      //n.setNum(_instances);
+      //QString instanceName = baseName() + "-" + n;
+      
       doSetuid();
       const char* path = info.filePath().latin1();
 
@@ -188,13 +206,13 @@ void* MessSynth::instantiate()
                   return 0;
                   }
             }
-      descr = msynth();
-      if (descr == 0) {
+      _descr = msynth();
+      if (_descr == 0) {
             fprintf(stderr, "Synth::instantiate: no MESS descr found\n");
             undoSetuid();
             return 0;
             }
-      Mess* mess = descr->instantiate(sampleRate, muse, instanceName.latin1());
+      Mess* mess = _descr->instantiate(sampleRate, muse, &museProject, instanceName.latin1());
       undoSetuid();
       return mess;
       }
@@ -207,13 +225,57 @@ SynthI::SynthI()
    : AudioTrack(AUDIO_SOFTSYNTH)
       {
       synthesizer = 0;
-      sif         = 0;
+      _sif        = 0;
       _rwFlags    = 1;
       _openFlags  = 1;
+      _readEnable = false;
+      _writeEnable = false;
       setVolume(1.0);
       setPan(0.0);
       }
 
+//---------------------------------------------------------
+//   open
+//---------------------------------------------------------
+
+QString SynthI::open() 
+{ 
+  // Make it behave like a regular midi device.
+  _readEnable = false;
+  _writeEnable = (_openFlags & 0x01);
+    
+  return QString("OK");
+}
+
+//---------------------------------------------------------
+//   close
+//---------------------------------------------------------
+
+void SynthI::close()
+{
+  _readEnable = false;
+  _writeEnable = false;
+}
+
+//---------------------------------------------------------
+//   putMidiEvent
+//---------------------------------------------------------
+
+bool SynthI::putEvent(const MidiPlayEvent& ev) 
+//bool SynthI::putMidiEvent(const MidiPlayEvent& ev) 
+{
+  if(_writeEnable)
+    return _sif->putEvent(ev);
+  
+  // Hmm, act as if the event went through? 
+  //return true;  
+  return false;  
+}
+
+//---------------------------------------------------------
+//   setName
+//---------------------------------------------------------
+
 void SynthI::setName(const QString& s)
       {
       AudioTrack::setName(s);
@@ -224,9 +286,12 @@ void SynthI::setName(const QString& s)
 //   init
 //---------------------------------------------------------
 
-bool MessSynthIF::init(Synth* s)
+//bool MessSynthIF::init(Synth* s)
+bool MessSynthIF::init(Synth* s, SynthI* si)
       {
-      _mess = (Mess*)s->instantiate();
+      //_mess = (Mess*)s->instantiate();
+      _mess = (Mess*)((MessSynth*)s)->instantiate(si->name());
+      
       return (_mess == 0);
       }
 
@@ -235,9 +300,14 @@ int MessSynthIF::channels() const
       return _mess->channels();
       }
 
-SynthIF* MessSynth::createSIF() const
+//SynthIF* MessSynth::createSIF() const
+SynthIF* MessSynth::createSIF(SynthI* si)
       {
-      return new MessSynthIF();
+      //return new MessSynthIF(si);
+      
+      MessSynthIF* sif = new MessSynthIF(si);
+      sif->init(this, si);
+      return sif;
       }
 
 //---------------------------------------------------------
@@ -248,12 +318,16 @@ SynthIF* MessSynth::createSIF() const
 bool SynthI::initInstance(Synth* s, const QString& instanceName)
       {
       synthesizer = s;
-      sif         = s->createSIF();
-      sif->init(s);
+      //sif         = s->createSIF();
+      //_sif        = s->createSIF(this);
+      
+      //sif->init(s);
 
       setName(instanceName);    // set midi device name
       setIName(instanceName);   // set instrument name
-      AudioTrack::setChannels(sif->channels());
+      _sif        = s->createSIF(this);
+      
+      AudioTrack::setChannels(_sif->channels());
 
       //---------------------------------------------------
       //  read available controller from synti
@@ -266,21 +340,40 @@ bool SynthI::initInstance(Synth* s, const QString& instanceName)
             int ctrl;
             int min;
             int max;
-            id = sif->getControllerInfo(id, &name, &ctrl, &min, &max);
+            int initval = CTRL_VAL_UNKNOWN;
+            id = _sif->getControllerInfo(id, &name, &ctrl, &min, &max, &initval);
 //            printf("looking for params\n");
             if (id == 0)
                   break;
 //             printf("got parameter:: %s\n", name);
-            MidiController* c = new MidiController(QString(name), ctrl, min, max, 0);
-            cl->push_back(c);
-            }
+            
+            
+            // Added by T356. Override existing program controller.
+            iMidiController i = cl->end();
+            if(ctrl == CTRL_PROGRAM)
+            {
+              for(i = cl->begin(); i != cl->end(); ++i) 
+              {
+                if(i->second->num() == CTRL_PROGRAM)
+                {
+                  delete i->second;
+                  cl->erase(i);
+                  
+                  break;
+                }
+              }
+            }  
+            
+            MidiController* c = new MidiController(QString(name), ctrl, min, max, initval);
+            cl->add(c);
+          }
 
       EventList* iel = midiState();
       if (!iel->empty()) {
             for (iEvent i = iel->begin(); i != iel->end(); ++i) {
                   Event ev = i->second;
                   MidiPlayEvent pev(0, 0, 0, ev);
-                  if (sif->putEvent(pev))
+                  if (_sif->putEvent(pev))
                         break;   // try later
                   }
             iel->clear();
@@ -288,7 +381,7 @@ bool SynthI::initInstance(Synth* s, const QString& instanceName)
 
       int idx = 0;
       for (std::vector<float>::iterator i = initParams.begin(); i != initParams.end(); ++i, ++idx)
-            sif->setParameter(idx, *i);
+            _sif->setParameter(idx, *i);
       return false;
       }
 
@@ -296,9 +389,9 @@ bool SynthI::initInstance(Synth* s, const QString& instanceName)
 //   getControllerInfo
 //---------------------------------------------------------
 
-int MessSynthIF::getControllerInfo(int id, const char** name, int* ctrl, int* min, int* max)
+int MessSynthIF::getControllerInfo(int id, const char** name, int* ctrl, int* min, int* max, int* initval)
       {
-      return _mess->getControllerInfo(id, name, ctrl, min, max);
+      return _mess->getControllerInfo(id, name, ctrl, min, max, initval);
       }
 
 //---------------------------------------------------------
@@ -320,7 +413,19 @@ void SynthI::deactivate2()
 
 void SynthI::deactivate3()
       {
-      sif->deactivate3();
+      _sif->deactivate3();
+      // Moved below by Tim. p3.3.14
+      //synthesizer->incInstances(-1);
+      
+      if(debugMsg)
+        printf("SynthI::deactivate3 deleting _sif...\n");
+      
+      delete _sif;
+      _sif = 0;
+      
+      if(debugMsg)
+        printf("SynthI::deactivate3 decrementing synth instances...\n");
+      
       synthesizer->incInstances(-1);
       }
 
@@ -359,11 +464,62 @@ void initMidiSynth()
             QFileInfoListIterator it(*list);
             QFileInfo* fi;
             while((fi = it.current())) {
-                  synthis.push_back(new MessSynth(*fi));
+            
+            
+                  //doSetuid();
+                  const char* path = fi->filePath().latin1();
+            
+                  // load Synti dll
+                  void* handle = dlopen(path, RTLD_NOW);
+                  if (handle == 0) {
+                        //fprintf(stderr, "initMidiSynth: dlopen(%s) failed: %s\n",
+                        //  path, dlerror());
+                        //undoSetuid();
+                        //return 0;
+                        ++it;
+                        continue;
+                        }
+                  typedef const MESS* (*MESS_Function)();
+                  MESS_Function msynth = (MESS_Function)dlsym(handle, "mess_descriptor");
+            
+                  if (!msynth) {
+                        //const char *txt = dlerror();
+                        //if (txt) {
+                        //      fprintf(stderr,
+                        //        "Unable to find msynth_descriptor() function in plugin "
+                        //        "library file \"%s\": %s.\n"
+                        //        "Are you sure this is a MESS plugin file?\n",
+                        //        info.filePath().ascii(), txt);
+                              //undoSetuid();
+                              //return 0;
+                        //      }
+                          dlclose(handle);
+                          ++it;
+                          continue;
+                        }
+                  const MESS* descr = msynth();
+                  if (descr == 0) {
+                        //fprintf(stderr, "Synth::instantiate: no MESS descr found\n");
+                        //undoSetuid();
+                        //return 0;
+                        dlclose(handle);
+                        ++it;
+                        continue;
+                        }
+                  //Mess* mess = descr->instantiate(sampleRate, muse, &museProject, instanceName.latin1());
+                  //undoSetuid();
+                  
+            
+            
+            
+                  //synthis.push_back(new MessSynth(*fi));
+                  synthis.push_back(new MessSynth(*fi, QString(descr->name), QString(descr->description), QString(""), QString(descr->version)));
+                  
+                  dlclose(handle);
                   ++it;
                   }
             if (debugMsg)
-                  printf("%d soft synth found\n", synthis.size());
+                  printf("%zd soft synth found\n", synthis.size());
             }
       }
 
@@ -372,18 +528,38 @@ void initMidiSynth()
 //    create a synthesizer instance of class "label"
 //---------------------------------------------------------
 
-SynthI* Song::createSynthI(const QString& sclass)
+//SynthI* Song::createSynthI(const QString& sclass)
+SynthI* Song::createSynthI(const QString& sclass, const QString& label)
       {
-      SynthI* si = ::createSynthI(sclass);
+      // Added by Tim. p3.3.13
+      //printf("Song::createSynthI calling ::createSynthI class:%s\n", sclass.latin1());
+      
+      //SynthI* si = ::createSynthI(sclass);
+      SynthI* si = ::createSynthI(sclass, label);
+      if(!si)
+        return 0;
+      // Added by Tim. p3.3.13
+      //printf("Song::createSynthI created SynthI. Before insertTrack1...\n");
+      
       insertTrack1(si, -1);
+      // Added by Tim. p3.3.13
+      //printf("Song::createSynthI after insertTrack1. Before msgInsertTrack...\n");
+      
       msgInsertTrack(si, -1, true);       // add to instance list
+      // Added by Tim. p3.3.13
+      //printf("Song::createSynthI after msgInsertTrack. Before insertTrack3...\n");
+      
       insertTrack3(si, -1);
 
+      // Added by Tim. p3.3.13
+      //printf("Song::createSynthI after insertTrack3. Adding default routes...\n");
+      
       OutputList* ol = song->outputs();
       // add default route to master (first audio output)
       if (!ol->empty()) {
             AudioOutput* ao = ol->front();
             audio->msgAddRoute(Route(si, -1), Route(ao, -1));
+            audio->msgUpdateSoloStates();
             }
       return si;
       }
@@ -397,6 +573,10 @@ void SynthI::write(int level, Xml& xml) const
       xml.tag(level++, "SynthI");
       AudioTrack::writeProperties(level, xml);
       xml.strTag(level, "class", synth()->baseName());
+      
+      // To support plugins like dssi-vst where all the baseNames are the same 'dssi-vst' and the label is the name of the dll file.
+      // Added by Tim. p3.3.16
+      xml.strTag(level, "label", synth()->name());
 
       //---------------------------------------------
       // if soft synth is attached to a midi port,
@@ -416,7 +596,7 @@ void SynthI::write(int level, Xml& xml) const
                   xml.qrectTag(level, "geometry", QRect(x, y, w, h));
             }
 
-      sif->write(level, xml);
+      _sif->write(level, xml);
       xml.etag(level, "SynthI");
       }
 
@@ -454,6 +634,8 @@ void MessSynthIF::write(int level, Xml& xml) const
 void SynthI::read(Xml& xml)
       {
       QString sclass;
+      QString label;
+      
       int port = -1;
       bool startGui = false;
       QRect r;
@@ -468,6 +650,8 @@ void SynthI::read(Xml& xml)
                   case Xml::TagStart:
                         if (tag == "class")
                               sclass = xml.parse1();
+                        else if (tag == "label")
+                              label  = xml.parse1();
                         else if (tag == "port")
                               port  = xml.parseInt();
                         else if (tag == "guiVisible")
@@ -485,7 +669,8 @@ void SynthI::read(Xml& xml)
                         break;
                   case Xml::TagEnd:
                         if (tag == "SynthI") {
-                              Synth* s = findSynth(sclass);
+                              //Synth* s = findSynth(sclass);
+                              Synth* s = findSynth(sclass, label);
                               if (s == 0)
                                     return;
                               if (initInstance(s, name()))
@@ -495,91 +680,134 @@ void SynthI::read(Xml& xml)
                                     midiPorts[port].setMidiDevice(this);
                               showGui(startGui);
                               setGeometry(r.x(), r.y(), r.width(), r.height());
+                              
+                              mapRackPluginsToControllers();
+                              
                               return;
                               }
                   default:
                         break;
                   }
             }
+      AudioTrack::mapRackPluginsToControllers();
       }
 
 //---------------------------------------------------------
 //   getPatchName
 //---------------------------------------------------------
 
-const char* MessSynthIF::getPatchName(int channel, int prog, MType type)
+const char* MessSynthIF::getPatchName(int channel, int prog, MType type, bool drum)
       {
-      if (_mess)
-            return _mess->getPatchName(channel, prog, type);
-      return "";
+        if (_mess)
+        {
+              //return _mess->getPatchName(channel, prog, type, drum);
+              const char* s = _mess->getPatchName(channel, prog, type, drum);
+              if(s)
+                return s;
+        }      
+        return "";
       }
 
 //---------------------------------------------------------
 //   populatePatchPopup
 //---------------------------------------------------------
 
-void MessSynthIF::populatePatchPopup(QPopupMenu* menu, int ch, MType)
+void MessSynthIF::populatePatchPopup(QPopupMenu* menu, int ch, MType, bool)
       {
       menu->clear();
       const MidiPatch* mp = _mess->getPatchInfo(ch, 0);
       while (mp) {
             int id = ((mp->hbank & 0xff) << 16)
                       + ((mp->lbank & 0xff) << 8) + mp->prog;
+            /*
+            int pgid = ((mp->hbank & 0xff) << 8) | (mp->lbank & 0xff) | 0x40000000;          
+            int itemnum = menu->indexOf(pgid);
+            if(itemnum == -1)
+            {
+              QPopupMenu* submenu = new QPopupMenu(menu);
+              itemnum = 
+            }
+            */  
             menu->insertItem(QString(mp->name), id);
             mp = _mess->getPatchInfo(ch, mp);
             }
       }
+
+//---------------------------------------------------------
+//   preProcessAlways
+//---------------------------------------------------------
+
+void SynthI::preProcessAlways()
+{
+  if(_sif)
+    _sif->preProcessAlways();
+  _processed = false;  
+}
+
+void MessSynthIF::preProcessAlways()
+{
+  if(_mess)
+    _mess->processMessages();
+}
+
 //---------------------------------------------------------
 //   getData
 //---------------------------------------------------------
 
 bool SynthI::getData(unsigned pos, int ports, unsigned n, float** buffer)
       {
+      for (int k = 0; k < ports; ++k)
+            memset(buffer[k], 0, n * sizeof(float));
+
       int p = midiPort();
       MidiPort* mp = (p != -1) ? &midiPorts[p] : 0;
       MPEventList* el = playEvents();
-
-      for (int i = 0; i < ports; ++i)
-            memset(buffer[i], 0, n * sizeof(float));
-
+               
       iMPEvent ie = nextPlayEvent();
-      ie = sif->getData(mp, el, ie, pos, ports, n, buffer);
+      
+      ie = _sif->getData(mp, el, ie, pos, ports, n, buffer);
+      
       setNextPlayEvent(ie);
       return true;
       }
 
-iMPEvent MessSynthIF::getData(MidiPort* mp, MPEventList* el, iMPEvent i, unsigned pos, int ports, unsigned n, float** buffer)
-      {
+iMPEvent MessSynthIF::getData(MidiPort* mp, MPEventList* el, iMPEvent i, unsigned pos, int /*ports*/, unsigned n, float** buffer)
+{
+      //prevent compiler warning: comparison of signed/unsigned
       int curPos      = pos;
       int endPos      = pos + n;
       int off         = pos;
       int frameOffset = audio->getFrameOffset();
 
-
       for (; i != el->end(); ++i) {
           int evTime = i->time(); 
-          //if (evTime == 0) {
+          if (evTime == 0) {
           //      printf("MessSynthIF::getData - time is 0!\n");
           //      continue;
-          //      }
+                evTime=frameOffset; // will cause frame to be zero, problem?
+                }
           int frame = evTime - frameOffset;
 
-          if (frame > 0) // robert: ugly fix, don't really know what is going on here
-                         // makes PPC work much better.
-              if (frame >= endPos) {
-                  printf("frame > endPos!! frame = %d >= endPos %d, i->time() %d, frameOffset %d curPos=%d\n", frame, endPos, i->time(), frameOffset,curPos);
-                  continue;
-                  }
+//TODO           if (frame > 0) // robert: ugly fix, don't really know what is going on here
+//                          // makes PPC work much better.
+
+               if (frame >= endPos) {
+                   printf("frame > endPos!! frame = %d >= endPos %d, i->time() %d, frameOffset %d curPos=%d\n", frame, endPos, i->time(), frameOffset,curPos);
+                   continue;
+                   }
 
             if (frame > curPos) {
                   if (frame < pos)
                         printf("should not happen: missed event %d\n", pos -frame);
-                  else {
+                  else 
+                  {
                         if (!_mess)
                               printf("should not happen - no _mess\n");
                         else
-                              _mess->process(buffer, curPos-pos, frame - curPos);
-                        }
+                        {
+                                _mess->process(buffer, curPos-pos, frame - curPos);
+                        }      
+                  }
                   curPos = frame;
                   }
             if (mp)
@@ -589,14 +817,17 @@ iMPEvent MessSynthIF::getData(MidiPort* mp, MPEventList* el, iMPEvent i, unsigne
                         break;
                   }
             }
-      if (endPos - curPos) {
+      if (endPos - curPos) 
+      {
             if (!_mess)
                   printf("should not happen - no _mess\n");
             else
-                  _mess->process(buffer, curPos - off, endPos - curPos);
-            }
-      return i;
+            {
+                    _mess->process(buffer, curPos - off, endPos - curPos);
+            }      
       }
+      return i;
+}
 
 //---------------------------------------------------------
 //   putEvent
