@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//  $Id: thread.h,v 1.1.1.1.2.1 2005/11/07 18:45:19 wschweer Exp $
+//  $Id: thread.h,v 1.1.1.1.2.2 2009/12/20 05:00:35 terminator356 Exp $
 //
 //  (C) Copyright 2001 Werner Schweer (ws@seh.de)
 //=========================================================
@@ -51,7 +51,7 @@ struct ThreadMsg {
 class Thread {
       const char* _name;
       volatile bool _running;
-      int realTimePriority;
+      //int realTimePriority;
       int _pollWait;    // poll timeout in msec (-1 = infinite)
 
       pthread_t thread;
@@ -64,6 +64,7 @@ class Thread {
       void* userPtr;
 
    protected:
+      int _realTimePriority;
       int fromThreadFdr;   // message from thread (seq read)
       int fromThreadFdw;   // message from thread (app write)
       int toThreadFdr;     // message to thread (seq read)
@@ -75,10 +76,15 @@ class Thread {
       virtual void defaultTick() {}
 
    public:
-      Thread(int prio, const char* name);
+      //Thread(int prio, const char* name);
+      Thread(const char* name);
+      
       virtual ~Thread();
       const char* name() const { return _name; }
-      virtual void start(void* ptr=0);
+      
+      //virtual void start(void* ptr=0);
+      virtual void start(int priority, void* ptr=0);
+      
       void stop(bool);
       void clearPollFd() {    plist.clear(); npfd = 0; }
       void addPollFd(int fd, int action, void (*handler)(void*,void*), void*, void*);
