@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//  $Id: track.h,v 1.39.2.2 2006/01/02 23:15:09 spamatica Exp $
+//  $Id: track.h,v 1.39.2.17 2009/12/20 05:00:35 terminator356 Exp $
 //
 //  (C) Copyright 1999-2004 Werner Schweer (ws@seh.de)
 //=========================================================
@@ -46,17 +46,31 @@ class Track {
       void init();
 
    protected:
+      static unsigned int _soloRefCnt;
+      static Track* _tmpSoloChainTrack;
+      static bool _tmpSoloChainDoIns;
+      static bool _tmpSoloChainNoDec;
+      
+      
       QString _name;
       bool _recordFlag;
       bool _mute;
       bool _solo;
+      unsigned int _internalSolo;
       bool _off;
       int _channels;          // 1 - mono, 2 - stereo
 
+      bool _volumeEnCtrl;
+      bool _volumeEn2Ctrl;
+      bool _panEnCtrl;
+      bool _panEn2Ctrl;
+      
       int _activity;
       int _lastActivity;
-      int _meter[MAX_CHANNELS];
-      int _peak[MAX_CHANNELS];
+      //int _meter[MAX_CHANNELS];
+      //int _peak[MAX_CHANNELS];
+      double _meter[MAX_CHANNELS];
+      double _peak[MAX_CHANNELS];
 
       int _y;
       int _height;            // visual height in arranger
@@ -68,8 +82,10 @@ class Track {
 
    public:
       Track(TrackType);
-      Track(const Track&);
-      virtual ~Track() {}
+      //Track(const Track&);
+      Track(const Track&, bool cloneParts);
+      virtual ~Track() {};
+      virtual Track& operator=(const Track& t);
       QString comment() const         { return _comment; }
       void setComment(const QString& s) { _comment = s; }
 
@@ -83,6 +99,16 @@ class Track {
       bool locked() const             { return _locked; }
       void setLocked(bool b)          { _locked = b; }
 
+      bool volumeControllerEnabled()  const  { return _volumeEnCtrl; }
+      bool volumeControllerEnabled2() const  { return _volumeEn2Ctrl; }
+      bool panControllerEnabled()     const  { return _panEnCtrl; }
+      bool panControllerEnabled2()    const  { return _panEn2Ctrl; }
+      void enableVolumeController(bool b)    { _volumeEnCtrl = b; }
+      void enable2VolumeController(bool b)   { _volumeEn2Ctrl = b; }
+      void enablePanController(bool b)       { _panEnCtrl = b; }
+      void enable2PanController(bool b)      { _panEn2Ctrl = b; }
+      void clearRecAutomation(bool clearList);
+      
       const QString& name() const     { return _name; }
       virtual void setName(const QString& s)  { _name = s; }
 
@@ -97,7 +123,8 @@ class Track {
       virtual void write(int, Xml&) const = 0;
 
       virtual Track* newTrack() const = 0;
-      virtual Track* clone() const    = 0;
+      //virtual Track* clone() const    = 0;
+      virtual Track* clone(bool CloneParts) const    = 0;
 
       virtual bool setRecordFlag1(bool f) = 0;
       virtual void setRecordFlag2(bool f) = 0;
@@ -108,9 +135,16 @@ class Track {
 
       virtual void setMute(bool val);
       virtual void setOff(bool val);
+      virtual void updateSoloStates(bool noDec) = 0;
+      virtual void updateInternalSoloStates();
+      void updateSoloState();
+      void setInternalSolo(unsigned int val);
+      static void clearSoloRefCounts();
       virtual void setSolo(bool val) = 0;
       virtual bool isMute() const = 0;
-
+      
+      unsigned int internalSolo() const  { return _internalSolo; }
+      bool soloMode() const              { return _soloRefCnt; }
       bool solo() const                  { return _solo;         }
       bool mute() const                  { return _mute;         }
       bool off() const                   { return _off;          }
@@ -123,8 +157,10 @@ class Track {
       void addActivity(int v)            { _activity += v;       }
       void resetPeaks();
       static void resetAllMeter();
-      int meter(int ch) const  { return _meter[ch]; }
-      int peak(int ch) const   { return _peak[ch]; }
+      //int meter(int ch) const  { return _meter[ch]; }
+      //int peak(int ch) const   { return _peak[ch]; }
+      double meter(int ch) const  { return _meter[ch]; }
+      double peak(int ch) const   { return _peak[ch]; }
       void resetMeter();
 
       bool readProperty(Xml& xml, const QString& tag);
@@ -142,18 +178,23 @@ class Track {
 //---------------------------------------------------------
 
 class MidiTrack : public Track {
-      static bool _soloMode;
+      //friend class AudioTrack;
+      //static unsigned int _soloRefCnt;
+      
       int _outPort;
       int _outChannel;
-      int _inPortMask;        // bitmask of accepted record ports
+      //int _inPortMask;        
+      unsigned int _inPortMask; // bitmask of accepted record ports
       int _inChannelMask;     // bitmask of accepted record channels
+      bool _recEcho;          // For midi (and audio). Whether to echo incoming record events to output device.
 
       EventList* _events;     // tmp Events during midi import
       MPEventList* _mpevents; // tmp Events druring recording
 
    public:
       MidiTrack();
-      MidiTrack(const MidiTrack&);
+      //MidiTrack(const MidiTrack&);
+      MidiTrack(const MidiTrack&, bool cloneParts);
       virtual ~MidiTrack();
 
       void init();
@@ -177,21 +218,32 @@ class MidiTrack : public Track {
       virtual void write(int, Xml&) const;
 
       virtual MidiTrack* newTrack() const { return new MidiTrack(); }
-      virtual MidiTrack* clone() const { return new MidiTrack(*this); }
+      //virtual MidiTrack* clone() const { return new MidiTrack(*this); }
+      virtual MidiTrack* clone(bool cloneParts) const { return new MidiTrack(*this, cloneParts); }
       virtual Part* newPart(Part*p=0, bool clone=false);
 
       void setOutChannel(int i)       { _outChannel = i; }
       void setOutPort(int i)          { _outPort = i; }
-      void setInPortMask(int i)       { _inPortMask = i; }
+      void setOutChanAndUpdate(int i);
+      void setOutPortAndUpdate(int i);
+      //void setInPortMask(int i)       { _inPortMask = i; }
+      void setInPortMask(unsigned int i) { _inPortMask = i; }
       void setInChannelMask(int i)    { _inChannelMask = i; }
+      void setRecEcho(bool b)         { _recEcho = b; }
       int outPort() const             { return _outPort;     }
-      int inPortMask() const          { return _inPortMask;  }
+      //int inPortMask() const          { return _inPortMask;  }
+      unsigned int inPortMask() const { return _inPortMask;  }
       int outChannel() const          { return _outChannel;  }
       int inChannelMask() const       { return _inChannelMask; }
+      bool recEcho() const            { return _recEcho; }
 
       virtual bool isMute() const;
       virtual void setSolo(bool val);
-      bool soloMode() const           { return _soloMode; }
+      virtual void updateSoloStates(bool noDec);
+      virtual void updateInternalSoloStates();
+      
+      //bool soloMode() const           { return _soloRefCnt; }
+      
       virtual bool canRecord() const  { return true; }
       };
 
@@ -203,7 +255,11 @@ class MidiTrack : public Track {
 //---------------------------------------------------------
 
 class AudioTrack : public Track {
-      static bool _soloMode;
+      //friend class MidiTrack;
+      //static unsigned int _soloRefCnt;
+      
+      bool _haveData;
+      
       CtrlListList _controller;
       CtrlRecList _recEvents;     // recorded automation events
 
@@ -215,32 +271,44 @@ class AudioTrack : public Track {
 
       RouteList _inRoutes;
       RouteList _outRoutes;
-
-      void readRecfile(Xml& xml);
+      
+      bool _sendMetronome;
+      
+      //void readRecfile(Xml& xml);
       void readAuxSend(Xml& xml);
 
    protected:
-      float** outBuffers;
+      //float** outBuffers;
+      float* outBuffers[MAX_CHANNELS];  
       unsigned bufferPos;
       virtual bool getData(unsigned, int, unsigned, float**);
       SndFile* _recFile;
       Fifo fifo;                    // fifo -> _recFile
+      bool _processed;
 
    public:
       AudioTrack(TrackType t);
-      AudioTrack(const AudioTrack&);
+      //AudioTrack(const AudioTrack&);
+      AudioTrack(const AudioTrack&, bool cloneParts);
       virtual ~AudioTrack();
 
       virtual bool setRecordFlag1(bool f);
       virtual void setRecordFlag2(bool f);
+      
+      bool processed() { return _processed; }
+      //void setProcessed(bool v) { _processed = v; }
 
       void addController(CtrlList*);
       void removeController(int id);
+      void swapControllerIDX(int idx1, int idx2);
 
       bool readProperties(Xml&, const QString&);
       void writeProperties(int, Xml&) const;
 
-      virtual AudioTrack* clone() const = 0;
+      void mapRackPluginsToControllers();
+
+      //virtual AudioTrack* clone() const = 0;
+      virtual AudioTrack* clone(bool cloneParts) const = 0;
       virtual Part* newPart(Part*p=0, bool clone=false);
 
       SndFile* recFile() const           { return _recFile; }
@@ -252,7 +320,10 @@ class AudioTrack : public Track {
 
       virtual bool isMute() const;
       virtual void setSolo(bool val);
-      bool soloMode() const               { return _soloMode; }
+      virtual void updateSoloStates(bool noDec);
+      virtual void updateInternalSoloStates();
+      
+      //bool soloMode() const               { return _soloRefCnt; }
 
       void putFifo(int channels, unsigned long n, float** bp);
 
@@ -261,6 +332,9 @@ class AudioTrack : public Track {
       virtual void setMute(bool val);
       virtual void setOff(bool val);
 
+      void setSendMetronome(bool val) { _sendMetronome = val; }
+      bool sendMetronome() const { return _sendMetronome; }
+      
       double volume() const;
       void setVolume(double val);
       double pan() const;
@@ -273,8 +347,13 @@ class AudioTrack : public Track {
 
       void setPrefader(bool val);
       Pipeline* efxPipe()                { return _efxPipe;  }
+      void deleteAllEfxGuis();
+      void clearEfxList();
       void addPlugin(PluginI* plugin, int idx);
 
+      double pluginCtrlVal(int ctlID) const;
+      void setPluginCtrlVal(int param, double val);
+      
       void readVolume(Xml& xml);
       void writeRouting(int, Xml&) const;
 
@@ -284,17 +363,26 @@ class AudioTrack : public Track {
       bool noInRoute() const   { return _inRoutes.empty();  }
       bool noOutRoute() const  { return _outRoutes.empty(); }
 
+      virtual void preProcessAlways() { _processed = false; }
       virtual void addData(unsigned, int, unsigned, float**);
       virtual void copyData(unsigned, int, unsigned, float**);
       virtual bool hasAuxSend() const { return false; }
 
       // automation
       virtual AutomationType automationType() const    { return _automationType; }
-      virtual void setAutomationType(AutomationType t) { _automationType = t; }
+      virtual void setAutomationType(AutomationType t);
+      void processAutomationEvents();
       CtrlRecList* recEvents()                         { return &_recEvents; }
-      void recordAutomation(int n, float v);
-      void startAutoRecord(int);
-      void stopAutoRecord(int);
+      void recordAutomation(int n, double v);
+      void startAutoRecord(int, double);
+      void stopAutoRecord(int, double);
+      void setControllerMode(int, CtrlList::Mode m); 
+      void clearControllerEvents(int);
+      void seekPrevACEvent(int);
+      void seekNextACEvent(int);
+      void eraseACEvent(int, int);
+      void eraseRangeACEvents(int, int, int);
+      void addACEvent(int, int, double);
       };
 
 //---------------------------------------------------------
@@ -307,9 +395,11 @@ class AudioInput : public AudioTrack {
 
    public:
       AudioInput();
-      AudioInput(const AudioInput&);
+      //AudioInput(const AudioInput&);
+      AudioInput(const AudioInput&, bool cloneParts);
       virtual ~AudioInput();
-      AudioInput* clone() const { return new AudioInput(*this); }
+      //AudioInput* clone() const { return new AudioInput(*this); }
+      AudioInput* clone(bool cloneParts) const { return new AudioInput(*this, cloneParts); }
       virtual AudioInput* newTrack() const { return new AudioInput(); }
       virtual void read(Xml&);
       virtual void write(int, Xml&) const;
@@ -334,9 +424,11 @@ class AudioOutput : public AudioTrack {
 
    public:
       AudioOutput();
-      AudioOutput(const AudioOutput&);
+      //AudioOutput(const AudioOutput&);
+      AudioOutput(const AudioOutput&, bool cloneParts);
       virtual ~AudioOutput();
-      AudioOutput* clone() const { return new AudioOutput(*this); }
+      //AudioOutput* clone() const { return new AudioOutput(*this); }
+      AudioOutput* clone(bool cloneParts) const { return new AudioOutput(*this, cloneParts); }
       virtual AudioOutput* newTrack() const { return new AudioOutput(); }
       virtual void read(Xml&);
       virtual void write(int, Xml&) const;
@@ -344,7 +436,7 @@ class AudioOutput : public AudioTrack {
       void* jackPort(int channel) { return jackPorts[channel]; }
       void setJackPort(int channel, void*p) { jackPorts[channel] = p; }
       virtual void setChannels(int n);
-      virtual bool isMute() const;
+//      virtual bool isMute() const;
 
       void processInit(unsigned);
       void process(unsigned pos, unsigned offset, unsigned);
@@ -361,8 +453,9 @@ class AudioOutput : public AudioTrack {
 
 class AudioGroup : public AudioTrack {
    public:
-      AudioGroup() : AudioTrack(AUDIO_GROUP) {}
-      AudioGroup* clone() const { return new AudioGroup(*this); }
+      AudioGroup() : AudioTrack(AUDIO_GROUP) {  }
+      //AudioGroup* clone() const { return new AudioGroup(*this); }
+      AudioGroup* clone(bool /*cloneParts*/) const { return new AudioGroup(*this); }
       virtual AudioGroup* newTrack() const { return new AudioGroup(); }
       virtual void read(Xml&);
       virtual void write(int, Xml&) const;
@@ -378,7 +471,8 @@ class AudioAux : public AudioTrack {
 
    public:
       AudioAux();
-      AudioAux* clone() const { return new AudioAux(*this); }
+      //AudioAux* clone() const { return new AudioAux(*this); }
+      AudioAux* clone(bool /*cloneParts*/) const { return new AudioAux(*this); }
       ~AudioAux();
       virtual AudioAux* newTrack() const { return new AudioAux(); }
       virtual void read(Xml&);
@@ -398,17 +492,21 @@ class WaveTrack : public AudioTrack {
    public:
       static bool firstWaveTrack;
 
-      WaveTrack() : AudioTrack(Track::WAVE) {}
-      WaveTrack(const WaveTrack& wt) : AudioTrack(wt) {}
+      WaveTrack() : AudioTrack(Track::WAVE) {  }
+      //WaveTrack(const WaveTrack& wt) : AudioTrack(wt) {}
+      WaveTrack(const WaveTrack& wt, bool cloneParts) : AudioTrack(wt, cloneParts) {}
 
-      virtual WaveTrack* clone() const    { return new WaveTrack(*this); }
+      //virtual WaveTrack* clone() const    { return new WaveTrack(*this); }
+      virtual WaveTrack* clone(bool cloneParts) const    { return new WaveTrack(*this, cloneParts); }
       virtual WaveTrack* newTrack() const { return new WaveTrack(); }
       virtual Part* newPart(Part*p=0, bool clone=false);
 
       virtual void read(Xml&);
       virtual void write(int, Xml&) const;
 
-      virtual void fetchData(unsigned pos, unsigned frames, float** bp);
+      //virtual void fetchData(unsigned pos, unsigned frames, float** bp);
+      virtual void fetchData(unsigned pos, unsigned frames, float** bp, bool doSeek);
+      
       virtual bool getData(unsigned, int ch, unsigned, float** bp);
 
       void clearPrefetchFifo()      { _prefetchFifo.clear(); }
@@ -546,6 +644,8 @@ typedef tracklist<SynthI*>::iterator iSynthI;
 typedef tracklist<SynthI*>::const_iterator ciSynthI;
 typedef tracklist<SynthI*> SynthIList;
 
+extern void addPortCtrlEvents(MidiTrack* t);
+extern void removePortCtrlEvents(MidiTrack* t);
 
 #endif
 
