@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//  $Id: transport.cpp,v 1.8 2004/05/29 13:50:08 qknight Exp $
+//  $Id: transport.cpp,v 1.8.2.3 2009/07/01 10:39:42 spamatica Exp $
 //
 //  (C) Copyright 1999/2000 Werner Schweer (ws@seh.de)
 //=========================================================
@@ -99,7 +99,7 @@ void Handle::mousePressEvent(QMouseEvent* ev)
 
 //---------------------------------------------------------
 //   TempoSig
-//    Widget fr Tempo + Signature
+//    Widget fï¿½r Tempo + Signature
 //---------------------------------------------------------
 
 TempoSig::TempoSig(QWidget* parent)
@@ -116,6 +116,7 @@ TempoSig::TempoSig(QWidget* parent)
       vb2->setAutoAdd(true);
 
       l1 = new DoubleLabel(120.0, 20.0, 400.0, f);
+      
       l1->setSpecialText(QString("extern"));
       l2 = new SigLabel(4, 4, f);
 
@@ -161,7 +162,12 @@ void TempoSig::setTempo(double t)
 
 void TempoSig::setTempo(int tempo)
       {
-      double t = (1000000.0 * 60.0)/tempo;
+      double t;
+      if(tempo == 0)
+        t = l1->off() - 1.0;
+      else  
+        t = (1000000.0 * 60.0)/tempo;
+      
       l1->blockSignals(true);
       l1->setValue(t);
       l1->blockSignals(false);
@@ -192,22 +198,25 @@ void Transport::setRecord(bool flag)
 //---------------------------------------------------------
 
 Transport::Transport(QWidget*, const char* name)
-   : QWidget(0, name, WStyle_Customize | WType_TopLevel | WStyle_Tool
-  | WStyle_NoBorder | WStyle_StaysOnTop)
+  // : QWidget(0, name, WStyle_Customize | WType_TopLevel | WStyle_Tool
+  //| WStyle_NoBorder | WStyle_StaysOnTop)
+   : QWidget(0, name, WStyle_Customize | WType_TopLevel | WStyle_NoBorder | WStyle_StaysOnTop)
       {
+      setCaption(QString("Muse: Transport"));
       setSizePolicy(QSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum));
 
       QHBoxLayout* hbox = new QHBoxLayout(this, 2, 5);
 
       hbox->setAutoAdd(true);
       lefthandle = new Handle(this, this);
-
+  
       //-----------------------------------------------------
       //    Record & Cycle Mode
       //-----------------------------------------------------
 
       QVBox* box1 = new QVBox(this);
       recMode     = new QComboBox(box1);
+      recMode->setFocusPolicy(QWidget::NoFocus);
       recMode->insertItem(tr("Overdub"), Song::REC_OVERDUP);
       recMode->insertItem(tr("Replace"), Song::REC_REPLACE);
       recMode->setCurrentItem(song->recMode());
@@ -217,6 +226,7 @@ Transport::Transport(QWidget*, const char* name)
       connect(recMode, SIGNAL(activated(int)), SLOT(setRecMode(int)));
 
       cycleMode = new QComboBox(box1);
+      cycleMode->setFocusPolicy(QWidget::NoFocus);
       cycleMode->insertItem(tr("Normal"),  Song::CYCLE_NORMAL);
       cycleMode->insertItem(tr("Mix"),     Song::CYCLE_MIX);
       cycleMode->insertItem(tr("Replace"), Song::CYCLE_REPLACE);
@@ -565,6 +575,10 @@ void Transport::setCycleMode(int id)
 
 void Transport::songChanged(int flags)
       {
+      // Is it simply a midi controller value adjustment? Forget it.
+      if(flags == SC_MIDI_CONTROLLER)
+        return;
+    
       slider->setRange(0, song->len());
       int cpos  = song->cpos();
       int t = tempomap.tempo(cpos);
