@@ -1,7 +1,7 @@
 //===================================================================
 //  MusE
 //  Linux Music Editor
-//  $Id: vst.cpp,v 1.5.2.3 2005/04/29 21:17:17 spamatica Exp $
+//  $Id: vst.cpp,v 1.5.2.6 2009/12/06 10:05:00 terminator356 Exp $
 //
 //   This code is based on jack_fst:
 //   Copyright (C) 2004 Paul Davis <paul@linuxaudiosystems.com>
@@ -281,7 +281,8 @@ static void scanVstDir(const QString& s)
                   FSTInfo* info = fst_get_info(path);
                   if (info) {
                         if (info->numInputs == 0 && info->numOutputs)
-                              synthis.push_back(new VstSynth(*fi));
+                              //synthis.push_back(new VstSynth(*fi));
+                              synthis.push_back(new VstSynth(*fi, fi->baseName(), QString(), QString(), QString()));
                         fst_free_info(info);
                         }
                   free(path);
@@ -324,10 +325,12 @@ void jfst_reserve_mem (int bufsize)
 void initVST()
       {
       jfst_reserve_mem(1000000);
+      
       if (fst_init(fstSignalHandler)) {
             printf("initVST failed\n");
             return;
             }
+      
       char* vstPath = getenv("VST_PATH");
       if (vstPath == 0)
             vstPath = "/usr/lib/vst:/usr/local/lib/vst";
@@ -456,7 +459,7 @@ void* VstSynth::instantiate()
 
 bool VstSynthIF::init(Synth* s)
       {
-      _fst = (FST*)s->instantiate();
+      _fst = (FST*)((VstSynth*)s)->instantiate();
       return (_fst == 0);
       }
 
@@ -474,9 +477,14 @@ int VstSynthIF::channels() const
 //   createSIF
 //---------------------------------------------------------
 
-SynthIF* VstSynth::createSIF() const
+//SynthIF* VstSynth::createSIF() const
+SynthIF* VstSynth::createSIF(SynthI* s)
       {
-      return new VstSynthIF();
+      //return new VstSynthIF();
+      
+      VstSynthIF* sif = new VstSynthIF(s);
+      sif->init(this, s);
+      return sif;
       }
 
 //---------------------------------------------------------
