@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//  $Id: wave.cpp,v 1.19.2.8 2005/12/28 22:17:55 spamatica Exp $
+//  $Id: wave.cpp,v 1.19.2.20 2009/12/20 05:00:35 terminator356 Exp $
 //
 //  (C) Copyright 2000-2004 Werner Schweer (ws@seh.de)
 //=========================================================
@@ -24,10 +24,14 @@
 #include "globals.h"
 #include "event.h"
 #include "audio.h"
+#include "sig.h"
+
+//#define WAVE_DEBUG
+//#define WAVE_DEBUG_PRC
 
 const char* audioFilePattern[] = {
-      "Wave/Binary (*.wav *.bin)",
-      "Wave (*.wav)",
+      "Wave/Binary (*.wav *.ogg *.bin)",
+      "Wave (*.wav *.ogg)",
       "Binary (*.bin)",
       "All Files (*)",
       0
@@ -46,10 +50,12 @@ SndFile::SndFile(const QString& name)
       {
       finfo = new QFileInfo(name);
       sf    = 0;
+      sfUI  = 0;
       csize = 0;
       cache = 0;
       openFlag = false;
       sndFiles.push_back(this);
+      refCount=0;
       }
 
 SndFile::~SndFile()
@@ -81,10 +87,14 @@ bool SndFile::openRead()
             printf("SndFile:: alread open\n");
             return false;
             }
-	QString p = path();
+      QString p = path();
+      sfinfo.format = 0;
       sf = sf_open(p.latin1(), SFM_READ, &sfinfo);
-      if (sf == 0)
+      sfinfo.format = 0;
+      sfUI = sf_open(p.latin1(), SFM_READ, &sfinfo);
+      if (sf == 0 || sfUI == 0)
             return true;
+            
       writeFlag = false;
       openFlag  = true;
       QString cacheName = finfo->dirPath(true) + QString("/") + finfo->baseName(true) + QString(".wca");
@@ -106,7 +116,7 @@ void SndFile::update()
          QString("/") + finfo->baseName(true) + QString(".wca");
       ::remove(cacheName.latin1());
       if (openRead()) {
-            printf("SndFile::openRead(%s) failed: %s\n", path().latin1(), strerror().latin1());
+            printf("SndFile::update openRead(%s) failed: %s\n", path().latin1(), strerror().latin1());
             }
       }
 
@@ -159,6 +169,9 @@ void SndFile::readCache(const QString& path, bool showProgress)
       for (unsigned k = 0; k < channels(); ++k)
             fp[k] = &data[k][0];
       int interval = csize / 10;
+      
+      if(!interval)
+        interval = 1;
       for (int i = 0; i < csize; i++) {
             if (showProgress && ((i % interval) == 0))
                   progress->setProgress(i);
@@ -208,12 +221,14 @@ void SndFile::writeCache(const QString& path)
 //   read
 //---------------------------------------------------------
 
-void SndFile::read(SampleV* s, int mag, unsigned pos)
+void SndFile::read(SampleV* s, int mag, unsigned pos, bool overwrite)
       {
-      for (unsigned ch = 0; ch < channels(); ++ch) {
+      if(overwrite)
+        for (unsigned ch = 0; ch < channels(); ++ch) {
             s[ch].peak = 0;
             s[ch].rms = 0;
             }
+            
       if (pos > samples()) {
 //            printf("%p pos %d > samples %d\n", this, pos, samples());
             return;
@@ -224,11 +239,56 @@ void SndFile::read(SampleV* s, int mag, unsigned pos)
             float* fp[channels()];
             for (unsigned i = 0; i < channels(); ++i)
                   fp[i] = &data[i][0];
-            seek(pos, 0);
-            read(channels(), fp, mag);
+            
+            sf_count_t ret = 0;
+            if(sfUI)
+              ret = sf_seek(sfUI, pos, SEEK_SET);
+            else
+              ret = sf_seek(sf, pos, SEEK_SET);
+            if(ret == -1)
+              return;  
+            {
+            int srcChannels = channels();
+            int dstChannels = sfinfo.channels;
+            size_t n        = mag;
+            float** dst     = fp;
+            float buffer[n * dstChannels];
+            
+            size_t rn = 0;
+            if(sfUI)
+              rn = sf_readf_float(sfUI, buffer, n);
+            else  
+              rn = sf_readf_float(sf, buffer, n);
+            if(rn != n)
+              return;
+            float* src = buffer;
+
+            if (srcChannels == dstChannels) {
+                  for (size_t i = 0; i < rn; ++i) {
+                        for (int ch = 0; ch < srcChannels; ++ch)
+                              *(dst[ch]+i) = *src++;
+                        }
+                  }
+            else if ((srcChannels == 1) && (dstChannels == 2)) {
+                  // stereo to mono
+                  for (size_t i = 0; i < rn; ++i)
+                        *(dst[0] + i) = src[i + i] + src[i + i + 1];
+                  }
+            else if ((srcChannels == 2) && (dstChannels == 1)) {
+                  // mono to stereo
+                  for (size_t i = 0; i < rn; ++i) {
+                        float data = *src++;
+                        *(dst[0]+i) = data;
+                        *(dst[1]+i) = data;
+                        }
+                  }
+            }
 
             for (unsigned ch = 0; ch < channels(); ++ch) {
-                  s[ch].peak = 0;
+                  
+                  if(overwrite)
+                    s[ch].peak = 0;
+                  
                   float rms = 0.0;
                   for (int i = 0; i < mag; i++) {
                         float fd = data[ch][i];
@@ -239,7 +299,8 @@ void SndFile::read(SampleV* s, int mag, unsigned pos)
                         if (s[ch].peak < idata)
                               s[ch].peak = idata;
                         }
-                  s[ch].rms = 0;    // TODO rms / mag;
+                  
+                    s[ch].rms = 0;    // TODO rms / mag;
                   }
             }
       else {
@@ -257,7 +318,12 @@ void SndFile::read(SampleV* s, int mag, unsigned pos)
                         if (s[ch].peak < cache[ch][offset].peak)
                               s[ch].peak = cache[ch][offset].peak;
                               }
-                  s[ch].rms = rms / mag;
+      
+                  if(overwrite)
+                    s[ch].rms = rms / mag;
+                  
+                  else  
+                    s[ch].rms += rms / mag;
                   }
             }
       }
@@ -272,8 +338,9 @@ bool SndFile::openWrite()
             printf("SndFile:: alread open\n");
             return false;
             }
-	QString p = path();
+  QString p = path();
       sf = sf_open(p.latin1(), SFM_RDWR, &sfinfo);
+      sfUI = 0;
       if (sf) {
             openFlag  = true;
             writeFlag = true;
@@ -295,6 +362,8 @@ void SndFile::close()
             return;
             }
       sf_close(sf);
+      if (sfUI)
+            sf_close(sfUI);
       openFlag = false;
       }
 
@@ -335,6 +404,8 @@ QString SndFile::name() const
 
 unsigned SndFile::samples() const
       {
+      if (!writeFlag) // if file is read only sfinfo is reliable
+          return sfinfo.frames;
       sf_count_t curPos = sf_seek(sf, 0, SEEK_CUR);
       int frames = sf_seek(sf, 0, SEEK_END);
       sf_seek(sf, curPos, SEEK_SET);
@@ -373,38 +444,57 @@ void SndFile::setFormat(int fmt, int ch, int rate)
 //   read
 //---------------------------------------------------------
 
-size_t SndFile::read(int srcChannels, float** dst, size_t n)
+size_t SndFile::read(int srcChannels, float** dst, size_t n, bool overwrite)
       {
-      float *buffer = new float[n * sfinfo.channels];
-      //float buffer[n * sfinfo.channels];
-      size_t rn       = sf_readf_float(sf, buffer, n);
+      // Changed by Tim. p3.3.17
+      //float *buffer = new float[n * sfinfo.channels];
+      float buffer[n * sfinfo.channels];
+      size_t rn = sf_readf_float(sf, buffer, n);
+      
       float* src      = buffer;
       int dstChannels = sfinfo.channels;
-
       if (srcChannels == dstChannels) {
-            for (size_t i = 0; i < rn; ++i) {
+            if(overwrite)
+              for (size_t i = 0; i < rn; ++i) {
                   for (int ch = 0; ch < srcChannels; ++ch)
                         *(dst[ch]+i) = *src++;
                   }
+              else
+              for (size_t i = 0; i < rn; ++i) {
+                  for (int ch = 0; ch < srcChannels; ++ch)
+                        *(dst[ch]+i) += *src++;
+                  }
             }
       else if ((srcChannels == 1) && (dstChannels == 2)) {
             // stereo to mono
-            for (size_t i = 0; i < rn; ++i)
+            if(overwrite)
+              for (size_t i = 0; i < rn; ++i)
                   *(dst[0] + i) = src[i + i] + src[i + i + 1];
+            else  
+              for (size_t i = 0; i < rn; ++i)
+                  *(dst[0] + i) += src[i + i] + src[i + i + 1];
             }
       else if ((srcChannels == 2) && (dstChannels == 1)) {
             // mono to stereo
-            for (size_t i = 0; i < rn; ++i) {
+            if(overwrite)
+              for (size_t i = 0; i < rn; ++i) {
                   float data = *src++;
                   *(dst[0]+i) = data;
                   *(dst[1]+i) = data;
                   }
+            else  
+              for (size_t i = 0; i < rn; ++i) {
+                  float data = *src++;
+                  *(dst[0]+i) += data;
+                  *(dst[1]+i) += data;
+                  }
             }
       else {
             printf("SndFile:read channel mismatch %d -> %d\n",
                srcChannels, dstChannels);
             }
-      delete buffer;
+      
+      //delete buffer;
       return rn;
       }
 
@@ -424,7 +514,7 @@ size_t SndFile::write(int srcChannels, float** src, size_t n)
       float *buffer = new float[n * dstChannels];
       float *dst      = buffer;
 
-      const float limitValue=0.99;
+      const float limitValue=0.9999;
 
 
       if (srcChannels == dstChannels) {
@@ -508,7 +598,7 @@ SndFile* SndFileList::search(const QString& name)
 //   getSnd
 //---------------------------------------------------------
 
-SndFile* getWave(const QString& inName, bool writeFlag)
+SndFile* getWave(const QString& inName, bool readOnlyFlag)
       {
       QString name = inName;
 
@@ -523,6 +613,8 @@ SndFile* getWave(const QString& inName, bool writeFlag)
                   }
             }
 // printf("=====%s %s\n", inName.latin1(), name.latin1());
+
+      // only open one instance of wave file
       SndFile* f = SndFile::sndFiles.search(name);
       if (f == 0) {
             if (!QFile::exists(name)) {
@@ -532,31 +624,54 @@ SndFile* getWave(const QString& inName, bool writeFlag)
                   }
             f = new SndFile(name);
             bool error;
-            if (writeFlag)
+            if (readOnlyFlag)
                   error = f->openRead();
-            else
+            else {
                   error = f->openWrite();
+                  // if peak cache is older than wave file we reaquire the cache
+                  QFileInfo wavinfo(name);
+                  QString cacheName = wavinfo.dirPath(true) + QString("/") + wavinfo.baseName(true) + QString(".wca");
+                  QFileInfo wcainfo(cacheName);
+                  if (!wcainfo.exists() || wcainfo.lastModified() < wavinfo.lastModified()) {
+                        //printf("wcafile is older or does not exist!\n");
+                        QFile(cacheName).remove();
+                        f->readCache(cacheName,true);
+                        }
+                  
+            }
             if (error) {
                   fprintf(stderr, "open wave file(%s) for %s failed: %s\n",
                      name.latin1(),
-                     writeFlag ? "writing" : "reading",
+                     readOnlyFlag ? "writing" : "reading",
                      f->strerror().latin1());
                      QMessageBox::critical(NULL, "MusE import error.", 
                                       "MusE failed to import the file.\n"
                                       "Possibly this wasn't a sound file?\n"
                                       "If it was check the permissions, MusE\n"
-                                      "often needs write access to the file.");
+                                      "sometimes requires write access to the file.");
 
                   delete f;
                   f = 0;
                   }
             }
       else {
-            if (writeFlag && ! f->isWritable()) {
+            if (!readOnlyFlag && ! f->isWritable()) {
                   if (f->isOpen())
                         f->close();
                   f->openWrite();
                   }
+            else {
+                  // if peak cache is older than wave file we reaquire the cache
+                  QFileInfo wavinfo(name);
+                  QString cacheName = wavinfo.dirPath(true) + QString("/") + wavinfo.baseName(true) + QString(".wca");
+                  QFileInfo wcainfo(cacheName);
+                  if (!wcainfo.exists() || wcainfo.lastModified() < wavinfo.lastModified()) {
+                        //printf("wcafile is older or does not exist!\n");
+                        QFile(cacheName).remove();
+                        f->readCache(cacheName,true);
+                        }
+                  
+                  }
             }
       return f;
       }
@@ -667,7 +782,7 @@ void MusE::importWave()
       Track* track = arranger->curTrack();
       if (track == 0 || track->type() != Track::WAVE) {
             QMessageBox::critical(this, QString("MusE"),
-              tr("to import a audio file you have first to select"
+              tr("to import an audio file you have first to select"
               "a wave track"));
             return;
             }
@@ -675,58 +790,47 @@ void MusE::importWave()
          tr("Import Wave File"), 0);
       if (!fn.isEmpty()) {
             lastWavePath = fn;
-            importWave(fn);
+            importWaveToTrack(fn);
             }
       }
 
 //---------------------------------------------------------
-//   importWave
+//   importWaveToTrack
 //---------------------------------------------------------
 
-bool MusE::importWave(const QString& name)
-      {
-      WaveTrack* track = (WaveTrack*)(arranger->curTrack());
-      SndFile* f = getWave(name, false);
-
-      if (f == 0) {
-            printf("import audio file failed\n");
-            return true;
-            }
-      int samples = f->samples();
-      track->setChannels(f->channels());
-
-      WavePart* part = new WavePart(track);
-      part->setTick(song->cpos());
-      part->setLenFrame(samples);
-
-      Event event(Wave);
-      SndFileR sf(f);
-      event.setSndFile(sf);
-      event.setSpos(0);
-      event.setLenFrame(samples);
-      part->addEvent(event);
-
-      part->setName(QFileInfo(name).baseName(true));
-      audio->msgAddPart(part);
-      unsigned endTick = part->tick() + part->lenTick();
-      if (song->len() < endTick)
-            song->setLen(endTick);
-      return false;
-      }
-
 bool MusE::importWaveToTrack(QString& name, unsigned tick, Track* track)
       {
-      SndFile* f = getWave(name, false);
+      if (track==NULL)
+            track = (WaveTrack*)(arranger->curTrack());
+
+      SndFile* f = getWave(name, true);
 
       if (f == 0) {
             printf("import audio file failed\n");
             return true;
             }
       int samples = f->samples();
+      if ((unsigned)sampleRate !=f->samplerate()) {
+            if(QMessageBox::question(this, tr("Import Wavefile"),
+                  tr("This wave file has a samplerate of %1,\n"
+                  "as opposed to current setting %2.\n"
+                  "Do you still want to import it?").arg(f->samplerate()).arg(sampleRate),
+                  tr("&Yes"), tr("&No"),
+                  QString::null, 0, 1 ))
+                  {
+                  //printf("why won't muse let me delete the file object? %d\n", f->getRefCount());
+                  if (f->getRefCount() == 0)
+                        delete f;
+                  return true;
+                  }
+            }
       track->setChannels(f->channels());
 
       WavePart* part = new WavePart((WaveTrack *)track);
-      part->setTick(tick);
+      if (tick)
+          part->setTick(tick);
+      else
+          part->setTick(song->cpos());
       part->setLenFrame(samples);
 
       Event event(Wave);
@@ -917,7 +1021,8 @@ int ClipList::idx(const Clip& clip) const
 //   cmdAddRecordedWave
 //---------------------------------------------------------
 
-void Song::cmdAddRecordedWave(WaveTrack* track, const Pos& s, const Pos& e)
+//void Song::cmdAddRecordedWave(WaveTrack* track, const Pos& s, const Pos& e)
+void Song::cmdAddRecordedWave(WaveTrack* track, Pos s, Pos e)
       {
       SndFile* f = track->recFile();
       if (f == 0) {
@@ -925,9 +1030,37 @@ void Song::cmdAddRecordedWave(WaveTrack* track, const Pos& s, const Pos& e)
                track->name().latin1());
             return;
             }
-
-      unsigned startTick = roundDownBar(s.tick());
-      unsigned endTick   = roundUpBar(e.tick());
+      
+      // Removed by Tim. p3.3.8
+      //unsigned startTick = roundDownBar(s.tick());
+      //unsigned endTick   = roundUpBar(e.tick());
+      
+      // Added by Tim. p3.3.8
+      
+      if((audio->loopCount() > 0 && s.tick() > lPos().tick()) || (punchin() && s.tick() < lPos().tick()))
+        s.setTick(lPos().tick());
+      // If we are looping, just set the end to the right marker, since we don't know how many loops have occurred.
+      // (Fixed: Added Audio::loopCount)
+      // Otherwise if punchout is on, limit the end to the right marker.
+      //if(loop() || (punchout() && e.tick() > rPos().tick()) )
+      if((audio->loopCount() > 0) || (punchout() && e.tick() > rPos().tick()) )
+        e.setTick(rPos().tick());
+      // No part to be created? Delete the rec sound file.
+      if(s.tick() >= e.tick())
+      {
+        QString s = f->path();
+        delete f;
+        // The function which calls this function already does this immediately after. But do it here anyway.
+        track->setRecFile(0);
+        remove(s.latin1());
+        if(debugMsg)
+          printf("Song::cmdAddRecordedWave: remove file %s\n", s.latin1());
+        return;
+      }
+      // Round the start down using the Arranger part snap raster value. 
+      unsigned startTick = sigmap.raster1(s.tick(), song->recRaster());
+      // Round the end up using the Arranger part snap raster value. 
+      unsigned endTick   = sigmap.raster2(e.tick(), song->recRaster());
 
       f->update();
 
@@ -940,7 +1073,18 @@ void Song::cmdAddRecordedWave(WaveTrack* track, const Pos& s, const Pos& e)
       Event event(Wave);
       SndFileR sf(f);
       event.setSndFile(sf);
+      // We are done with the _recFile member. Set to zero. The function which 
+      //  calls this function already does this immediately after. But do it here anyway.
+      track->setRecFile(0);
+      
       event.setSpos(0);
+      
+      // Since the part start was snapped down, we must apply the difference so that the
+      //  wave event tick lines up with when the user actually started recording.
+      // Added by Tim. p3.3.8
+      event.setTick(s.tick() - startTick);
+      
+      
       event.setLenFrame(e.frame() - s.frame());
       part->addEvent(event);
 
