@@ -1,19 +1,19 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//    $Id: waveedit.cpp,v 1.5.2.8 2005/12/11 21:29:24 spamatica Exp $
+//    $Id: waveedit.cpp,v 1.5.2.12 2009/04/06 01:24:54 terminator356 Exp $
 //  (C) Copyright 2000 Werner Schweer (ws@seh.de)
 //=========================================================
 
+#include "app.h"
 #include "xml.h"
 #include "waveedit.h"
 #include "mtscale.h"
 #include "scrollscale.h"
-#include "intlabel.h"
 #include "waveview.h"
 #include "ttoolbar.h"
 #include "globals.h"
-#include "waveview.h"
+#include "audio.h"
 #include "utils.h"
 #include "song.h"
 #include "poslabel.h"
@@ -106,7 +106,7 @@ WaveEdit::WaveEdit(PartList* pl)
       QAccel* qa = new QAccel(this);
       qa->connectItem(qa->insertItem(CTRL+Key_Z), song, SLOT(undo()));
       qa->connectItem(qa->insertItem(CTRL+Key_Y), song, SLOT(redo()));
-      connect(song, SIGNAL(configChanged()), SLOT(configChanged()));
+      connect(muse, SIGNAL(configChanged()), SLOT(configChanged()));
 
 
       //--------------------------------------------------
@@ -178,10 +178,17 @@ WaveEdit::WaveEdit(PartList* pl)
 //      connect(time,    SIGNAL(timeChanged(unsigned)),  SLOT(setTime(unsigned)));
       connect(view,    SIGNAL(timeChanged(unsigned)),  SLOT(setTime(unsigned)));
 
-      int s, e;
-      view->range(&s, &e);
-      hscroll->setRange(s, e);
+      connect(hscroll, SIGNAL(scaleChanged(int)),  SLOT(updateHScrollRange()));
+      connect(song, SIGNAL(songChanged(int)), SLOT(songChanged1(int)));
+      
+      updateHScrollRange();
       configChanged();
+      
+      if(!parts()->empty())
+      {
+        WavePart* part = (WavePart*)(parts()->begin()->second);
+        solo->setOn(part->track()->solo());
+      }
       }
 
 //---------------------------------------------------------
@@ -195,6 +202,25 @@ void WaveEdit::configChanged()
       select->setAccel(shortcuts[SHRT_SELECT_NONE].key, CMD_SELECT_NONE);
       }
 
+//---------------------------------------------------------
+//   updateHScrollRange
+//---------------------------------------------------------
+void WaveEdit::updateHScrollRange()
+{
+      int s, e;
+      wview->range(&s, &e);
+      // Show one more measure.
+      e += sigmap.ticksMeasure(e);  
+      // Show another quarter measure due to imprecise drawing at canvas end point.
+      e += sigmap.ticksMeasure(e) / 4;
+      // Compensate for the vscroll width. 
+      //e += wview->rmapxDev(-vscroll->width()); 
+      int s1, e1;
+      hscroll->range(&s1, &e1);
+      if(s != s1 || e != e1) 
+        hscroll->setRange(s, e);
+}
+
 //---------------------------------------------------------
 //   setTime
 //---------------------------------------------------------
@@ -204,7 +230,8 @@ void WaveEdit::setTime(unsigned samplepos)
 //    printf("setTime %d %x\n", samplepos, samplepos);
       unsigned tick = tempomap.frame2tick(samplepos);
       pos1->setValue(tick);
-      pos2->setValue(tick);
+      //pos2->setValue(tick);
+      pos2->setValue(samplepos);
       time->setPos(3, tick, false);
       }
 
@@ -331,16 +358,25 @@ void WaveEdit::resizeEvent(QResizeEvent* ev)
       }
 
 //---------------------------------------------------------
-//   soloChanged
+//   songChanged1
 //    signal from "song"
 //---------------------------------------------------------
 
-void WaveEdit::soloChanged(SNode*/* s*/)
+void WaveEdit::songChanged1(int bits)
       {
-      WavePart* part = (WavePart*)(parts()->begin()->second);
-      solo->setOn(part->track()->solo());
+        
+        if (bits & SC_SOLO)
+        {
+          WavePart* part = (WavePart*)(parts()->begin()->second);
+          solo->blockSignals(true);
+          solo->setOn(part->track()->solo());
+          solo->blockSignals(false);
+        }  
+        
+        songChanged(bits);
       }
 
+
 //---------------------------------------------------------
 //   soloChanged
 //    signal from solo button
@@ -349,7 +385,8 @@ void WaveEdit::soloChanged(SNode*/* s*/)
 void WaveEdit::soloChanged(bool flag)
       {
       WavePart* part = (WavePart*)(parts()->begin()->second);
-      part->track()->setSolo(flag);
+      audio->msgSetSolo(part->track(), flag);
+      song->update(SC_SOLO);
       }
 
 //---------------------------------------------------------
