@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//    $Id: waveedit.h,v 1.3.2.6 2005/12/11 21:29:24 spamatica Exp $
+//    $Id: waveedit.h,v 1.3.2.8 2008/01/26 07:23:21 terminator356 Exp $
 //  (C) Copyright 2000 Werner Schweer (ws@seh.de)
 //=========================================================
 
@@ -45,11 +45,13 @@ class WaveEdit : public MidiEditor {
    private slots:
       void cmd(int);
       void setTime(unsigned t);
-      void soloChanged(SNode* s);
+      void songChanged1(int);
       void soloChanged(bool flag);
 
    public slots:
       void configChanged();
+   
+      virtual void updateHScrollRange();
 
    signals:
       void deleted(unsigned long);
