@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//    $Id: waveview.cpp,v 1.10.2.8 2005/12/28 22:17:54 spamatica Exp $
+//    $Id: waveview.cpp,v 1.10.2.16 2009/11/14 03:37:48 terminator356 Exp $
 //  (C) Copyright 2000 Werner Schweer (ws@seh.de)
 //=========================================================
 
@@ -24,6 +24,7 @@
 #include "audio.h"
 #include "gconfig.h"
 
+bool modifyWarnedYet = false;
 //---------------------------------------------------------
 //   WaveView
 //---------------------------------------------------------
@@ -86,6 +87,11 @@ void WaveView::pdraw(QPainter& p, const QRect& rr)
       int h  = hh/2;
       int y  = rr.y() + h;
 
+      // Added by T356.
+      int xScale = xmag;
+      if (xScale < 0)
+            xScale = -xScale;
+      
       for (iPart ip = editor->parts()->begin(); ip != editor->parts()->end(); ++ip) {
             WavePart* wp = (WavePart*)(ip->second);
             int channels = wp->track()->channels();
@@ -99,13 +105,12 @@ void WaveView::pdraw(QPainter& p, const QRect& rr)
                   SndFileR f = event.sndFile();
                   if (f.isNull())
                         continue;
-                  int xScale = xmag;
-                  if (xScale < 0)
-                        xScale = -xScale;
-
+                  
+                  unsigned peoffset = px + event.frame() - event.spos();
                   int sx, ex;
-                  sx = event.frame() + px + xScale/2 - startSample;
-                  ex = sx + px + event.lenFrame();
+                  
+                  sx = event.frame() + px + xScale/2;
+                  ex = sx + event.lenFrame();
                   sx = sx / xScale - xpos;
                   ex = ex / xScale - xpos;
 
@@ -115,8 +120,10 @@ void WaveView::pdraw(QPainter& p, const QRect& rr)
                         ex = x2;
 
                   int pos = (xpos + sx) * xScale + event.spos() - event.frame() - px;
+                  
                   //printf("pos=%d xpos=%d sx=%d ex=%d xScale=%d event.spos=%d event.frame=%d px=%d\n",
                   //      pos, xpos, sx, ex, xScale, event.spos(), event.frame(), px);
+                  
                   h       = hh / (channels * 2);
                   int cc  = hh % (channels * 2) ? 0 : 1;
 
@@ -128,12 +135,9 @@ void WaveView::pdraw(QPainter& p, const QRect& rr)
                         if (pos < event.spos())
                               continue;
 
-                        unsigned peoffset = px + event.frame() - event.spos();
                         int selectionStartPos = selectionStart - peoffset; // Offset transformed to event coords
                         int selectionStopPos  = selectionStop  - peoffset;
 
-
-
                         for (int k = 0; k < channels; ++k) {
                               int kk = k % f.channels();
                               int peak = (sa[kk].peak * (h - 1)) / yScale;
@@ -144,7 +148,11 @@ void WaveView::pdraw(QPainter& p, const QRect& rr)
                                     rms = h;
                               QColor peak_color = QColor(darkGray);
                               QColor rms_color  = QColor(black);
-                              if (pos > selectionStartPos && pos < selectionStopPos) {
+                              
+                              // Changed by T356. Reduces (but not eliminates) drawing artifacts.
+                              //if (pos > selectionStartPos && pos < selectionStopPos) {
+                              if (pos > selectionStartPos && pos <= selectionStopPos) {
+                                    
                                     peak_color = QColor(lightGray);
                                     rms_color  = QColor(white);
                                     // Draw inverted
@@ -191,7 +199,12 @@ void WaveView::draw(QPainter& p, const QRect& r)
       if (pos[2] >= x && pos[2] < x2)
             p.drawLine(pos[2], y, pos[2], y2);
 
-      int n  = curPart->track()->channels();
+      // Changed by T356. Support multiple (or none) selected parts.
+      //int n  = curPart->track()->channels();
+      int n = 1;
+      if(curPart)
+        n = curPart->track()->channels();
+      
       int hn = h / n;
       int hh = hn / 2;
       for (int i = 0; i < n; ++i) {
@@ -210,7 +223,14 @@ void WaveView::draw(QPainter& p, const QRect& r)
 
 QString WaveView::getCaption() const
       {
-      return QString("Part ") + curPart->name();
+      
+      // Changed by T356. Support multiple (or none) selected parts.
+      //return QString("Part ") + curPart->name();
+      if(curPart)
+        return QString("Part ") + curPart->name();
+      else  
+        return QString("Part ");
+        
       }
 
 //---------------------------------------------------------
@@ -219,6 +239,10 @@ QString WaveView::getCaption() const
 
 void WaveView::songChanged(int flags)
       {
+      // Is it simply a midi controller value adjustment? Forget it.
+      if(flags == SC_MIDI_CONTROLLER)
+        return;
+    
       if (flags & SC_SELECTION) {
             startSample  = MAXINT;
             endSample    = 0;
@@ -271,7 +295,7 @@ void WaveView::setPos(int idx, unsigned val, bool adjustScrollbar)
                         break;
                   case Song::JUMP:
                         if (npos >= width()) {
-                              int ppos =  val - rmapxDev(width()/4);
+                              int ppos =  val - xorg - rmapxDev(width()/4);
                               if (ppos < 0)
                                     ppos = 0;
                               emit followEvent(ppos);
@@ -279,7 +303,7 @@ void WaveView::setPos(int idx, unsigned val, bool adjustScrollbar)
                               npos = mapx(val);
                               }
                         else if (npos < 0) {
-                              int ppos =  val - rmapxDev(width()*3/4);
+                              int ppos =  val - xorg - rmapxDev(width()*3/4);
                               if (ppos < 0)
                                     ppos = 0;
                               emit followEvent(ppos);
@@ -289,7 +313,7 @@ void WaveView::setPos(int idx, unsigned val, bool adjustScrollbar)
                         break;
                   case Song::CONTINUOUS:
                         if (npos > (width()*5)/8) {
-                              int ppos =  pos[idx] - rmapxDev(width()*5/8);
+                              int ppos =  pos[idx] - xorg - rmapxDev(width()*5/8);
                               if (ppos < 0)
                                     ppos = 0;
                               emit followEvent(ppos);
@@ -297,7 +321,7 @@ void WaveView::setPos(int idx, unsigned val, bool adjustScrollbar)
                               npos = mapx(val);
                               }
                         else if (npos < (width()*3)/8) {
-                              int ppos =  pos[idx] - rmapxDev(width()*3/8);
+                              int ppos =  pos[idx] - xorg - rmapxDev(width()*3/8);
                               if (ppos < 0)
                                     ppos = 0;
                               emit followEvent(ppos);
@@ -410,11 +434,28 @@ void WaveView::viewMouseMoveEvent(QMouseEvent* event)
 
 void WaveView::range(int* s, int *e)
       {
-      *s = curPart->frame();
-      int etick = curPart->tick() + curPart->lenTick();
-      *e = tempomap.tick2frame(etick);
+      
+      PartList* lst = editor->parts();
+      if(lst->empty())
+      {
+        *s = 0;
+        *e = tempomap.tick2frame(song->len());
+        return;  
+      }
+      int ps = song->len(), pe = 0;
+      int tps, tpe;
+      for(iPart ip = lst->begin(); ip != lst->end(); ++ip) 
+      {
+        tps = ip->second->tick();
+        if(tps < ps)
+          ps = tps;
+        tpe = tps + ip->second->lenTick();
+        if(tpe > pe)
+          pe = tpe;
+      }
+      *s = tempomap.tick2frame(ps);
+      *e = tempomap.tick2frame(pe);
       }
-
 
 //---------------------------------------------------------
 //   cmd
@@ -512,6 +553,17 @@ void WaveView::cmd(int n)
                   printf("No selection. Ignoring\n"); //@!TODO: Disable menu options when no selection
                   return;
                   }
+            
+            //if(!modifyWarnedYet)
+            //{
+            //  modifyWarnedYet = true;
+            //  if(QMessageBox::warning(this, QString("Muse"),
+            //     tr("Warning! Muse currently operates directly on the sound file.\n"
+            //        "Undo is supported, but NOT after exit, WITH OR WITHOUT A SAVE!\n"
+            //        "If you are stuck, try deleting the associated .wca file and reloading."), tr("&Ok"), tr("&Cancel"),
+            //     QString::null, 0, 1 ) != 0)
+            //   return;
+            //}
             modifySelection(modifyoperation, selectionStart, selectionStop, paramA);
             }
       }
@@ -527,7 +579,9 @@ WaveSelectionList WaveView::getSelection(unsigned startpos, unsigned stoppos)
       for (iPart ip = editor->parts()->begin(); ip != editor->parts()->end(); ++ip) {
             WavePart* wp = (WavePart*)(ip->second);
             unsigned part_offset = wp->frame();
+            
             EventList* el = wp->events();
+            //printf("eventlist length=%d\n",el->size());
 
             for (iEvent e = el->begin(); e != el->end(); ++e) {
                   Event event  = e->second;
@@ -536,14 +590,14 @@ WaveSelectionList WaveView::getSelection(unsigned startpos, unsigned stoppos)
                   SndFileR file = event.sndFile();
                   if (file.isNull())
                         continue;
-
+                  
                   unsigned event_offset = event.frame() + part_offset;
                   unsigned event_startpos  = event.spos();
                   unsigned event_length = event.lenFrame() + event.spos();
                   unsigned event_end    = event_offset + event_length;
                   //printf("startpos=%d stoppos=%d part_offset=%d event_offset=%d event_startpos=%d event_length=%d event_end=%d\n", startpos, stoppos, part_offset, event_offset, event_startpos, event_length, event_end);
 
-                  if (!(event_end <= startpos || event_offset >= stoppos)) {
+                  if (!(event_end <= startpos || event_offset > stoppos)) {
                         int tmp_sx = startpos - event_offset + event_startpos;
                         int tmp_ex = stoppos  - event_offset + event_startpos;
                         unsigned sx;
@@ -556,7 +610,8 @@ WaveSelectionList WaveView::getSelection(unsigned startpos, unsigned stoppos)
                         WaveEventSelection s;
                         s.file = file;
                         s.startframe = sx;
-                        s.endframe   = ex;
+                        s.endframe   = ex+1;
+                        //printf("sx=%d ex=%d\n",sx,ex);
                         selection.push_back(s);
                         }
                   }
@@ -732,16 +787,17 @@ void WaveView::fadeOutSelection(unsigned channels, float** data, unsigned length
 //---------------------------------------------------------
 void WaveView::reverseSelection(unsigned channels, float** data, unsigned length)
       {
+      if(length <= 1)    
+        return;
       for (unsigned i=0; i<channels; i++) {
             for (unsigned j=0; j<length/2; j++) {
                   float tmpl = data[i][j];
-                  float tmpr = data[i][length - j];
+                  float tmpr = data[i][length - j - 1];
                   data[i][j] = tmpr;
-                  data[i][length - j] = tmpl;
+                  data[i][length - j - 1] = tmpl;
                   }
             }
       }
-
 //---------------------------------------------------------
 //   applyGain
 //---------------------------------------------------------
@@ -779,7 +835,7 @@ void WaveView::editExternal(unsigned file_format, unsigned file_samplerate, unsi
       // Forkaborkabork
       int pid = fork();
       if (pid == 0) {
-            if (execlp(config.externalWavEditor.latin1(), config.externalWavEditor.latin1(), exttmpFileName.latin1(), 0) == -1) {
+            if (execlp(config.externalWavEditor.latin1(), config.externalWavEditor.latin1(), exttmpFileName.latin1(), NULL) == -1) {
                   perror("Failed to launch external editor");
                   // Get out of here
                   
