@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//  $Id: waveevent.h,v 1.6 2004/05/03 20:30:39 wschweer Exp $
+//  $Id: waveevent.h,v 1.6.2.4 2009/12/20 05:00:35 terminator356 Exp $
 //
 //  (C) Copyright 1999-2004 Werner Schweer (ws@seh.de)
 //=========================================================
@@ -9,8 +9,13 @@
 #ifndef __WAVE_EVENT_H__
 #define __WAVE_EVENT_H__
 
+//#include <samplerate.h>
+#include <sys/types.h>
+
 #include "eventbase.h"
 
+class AudioConverter;
+
 //---------------------------------------------------------
 //   WaveEvent
 //---------------------------------------------------------
@@ -28,19 +33,27 @@ class WaveEventBase : public EventBase {
       virtual ~WaveEventBase() {}
 
       virtual void read(Xml&);
-      virtual void write(int, Xml&, const Pos& offset) const;
+      //virtual void write(int, Xml&, const Pos& offset) const;
+      virtual void write(int, Xml&, const Pos& offset, bool forcePath = false) const;
       virtual EventBase* mid(unsigned, unsigned);
+      
       virtual void dump(int n = 0) const;
 
-
       virtual const QString name() const       { return _name;  }
       virtual void setName(const QString& s)   { _name = s;     }
       virtual int spos() const                 { return _spos;  }
       virtual void setSpos(int s)              { _spos = s;     }
       virtual SndFileR sndFile() const         { return f;      }
       virtual void setSndFile(SndFileR& sf)    { f = sf;        }
-      virtual void read(unsigned offset, float** bpp, int channels, int nn);
+      
+      // Changed by Tim. p3.3.17
+      //virtual void read(unsigned offset, float** bpp, int channels, int nn, bool overwrite = true);
+      //virtual void readAudio(unsigned /*offset*/, float** /*bpp*/, int /*channels*/, int /*nn*/, bool /*doSeek*/, bool /*overwrite*/);
+      //virtual off_t readAudio(SRC_STATE* /*src_state*/, off_t /*sfCurFrame*/, unsigned /*offset*/, 
+      //                       float** /*bpp*/, int /*channels*/, int /*nn*/, bool /*doSeek*/, bool /*overwrite*/);
+      virtual off_t readAudio(AudioConverter* /*audConv*/, off_t /*sfCurFrame*/, unsigned /*offset*/, 
+                             float** /*bpp*/, int /*channels*/, int /*nn*/, bool /*doSeek*/, bool /*overwrite*/);
       };
-
+      
 #endif
 
