@@ -4,6 +4,7 @@
 #include <qwidget.h>
 #include <qlabel.h>
 
+#include <qcheckbox.h>
 class MusE;
 
 //---------------------------------------------------------
@@ -17,12 +18,19 @@ class BigTime : public QWidget {
 
       bool setString(unsigned);
 
+      QWidget *dwin;
+      QCheckBox *fmtButton;
+      QLabel *absTickLabel;
+      QLabel *absFrameLabel;
       QLabel *barLabel, *beatLabel, *tickLabel,
-             *hourLabel, *minLabel, *secLabel, *frameLabel,
+             //*hourLabel, *minLabel, *secLabel, *frameLabel,
+             *minLabel, *secLabel, *frameLabel, *subFrameLabel, 
              *sep1, *sep2, *sep3, *sep4, *sep5;
-      int oldbar, oldbeat, oldhour, oldmin, oldsec, oldframe;
+      
+      //int oldbar, oldbeat, oldhour, oldmin, oldsec, oldframe;
+      int oldbar, oldbeat, oldmin, oldsec, oldframe, oldsubframe;
       unsigned oldtick;
-
+      unsigned oldAbsTick, oldAbsFrame;
       void setFgColor(QColor c);
       void setBgColor(QColor c);
 
@@ -33,7 +41,7 @@ class BigTime : public QWidget {
    public slots:
       void setPos(int, unsigned, bool);
       void configChanged();
-
+      void fmtButtonToggled(bool);
    signals:
       void closed();
 
