@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//    $Id: canvas.cpp,v 1.10.2.4 2006/03/02 22:34:01 spamatica Exp $
+//    $Id: canvas.cpp,v 1.10.2.17 2009/05/03 04:14:01 terminator356 Exp $
 //  (C) Copyright 1999 Werner Schweer (ws@seh.de)
 //=========================================================
 
@@ -11,6 +11,7 @@
 #include <qpainter.h>
 #include <qpopupmenu.h>
 #include <qcursor.h>
+#include <qtimer.h>
 
 #include "song.h"
 #include "event.h"
@@ -30,6 +31,17 @@ Canvas::Canvas(QWidget* parent, int sx, int sy, const char* name)
       {
       canvasTools = 0;
       itemPopupMenu = 0;
+      
+      canScrollLeft = true;
+      canScrollRight = true;
+      canScrollUp = true;
+      canScrollDown = true;
+      hscrollDir = HSCROLL_NONE;
+      vscrollDir = VSCROLL_NONE;
+      scrollTimer=NULL;
+      
+      scrollSpeed=10;    // hardcoded scroll jump
+
       drag    = DRAG_OFF;
       _tool   = PointerTool;
       pos[0]  = song->cpos();
@@ -50,9 +62,9 @@ Canvas::Canvas(QWidget* parent, int sx, int sy, const char* name)
 
 void Canvas::setPos(int idx, unsigned val, bool adjustScrollbar)
       {
-      if (pos[idx] == val)
-            return;
-
+      //if (pos[idx] == val) // Seems to be some refresh problems here, pos[idx] might be val but the gui not updated.
+      //    return;          // skipping this return forces update even if values match. Matching values only seem
+                             // to occur when initializing
       int opos = mapx(pos[idx]);
       int npos = mapx(val);
 
@@ -62,7 +74,7 @@ void Canvas::setPos(int idx, unsigned val, bool adjustScrollbar)
                         break;
                   case Song::JUMP:
                         if (npos >= width()) {
-                              int ppos =  val - rmapxDev(width()/8);
+                              int ppos =  val - xorg - rmapxDev(width()/8);
                               if (ppos < 0)
                                     ppos = 0;
                               emit followEvent(ppos);
@@ -70,7 +82,7 @@ void Canvas::setPos(int idx, unsigned val, bool adjustScrollbar)
                               npos = mapx(val);
                               }
                         else if (npos < 0) {
-                              int ppos =  val - rmapxDev(width()*3/4);
+                              int ppos =  val - xorg - rmapxDev(width()*3/4);
                               if (ppos < 0)
                                     ppos = 0;
                               emit followEvent(ppos);
@@ -80,7 +92,7 @@ void Canvas::setPos(int idx, unsigned val, bool adjustScrollbar)
                         break;
                   case Song::CONTINUOUS:
                         if (npos > (width()/2)) {
-                              int ppos =  pos[idx] - rmapxDev(width()/2);
+                              int ppos =  pos[idx] - xorg - rmapxDev(width()/2);
                               if (ppos < 0)
                                     ppos = 0;
                               emit followEvent(ppos);
@@ -88,7 +100,7 @@ void Canvas::setPos(int idx, unsigned val, bool adjustScrollbar)
                               npos = mapx(val);
                               }
                         else if (npos < (width()/2)) {
-                              int ppos =  pos[idx] - rmapxDev(width()/2);
+                              int ppos =  pos[idx] - xorg - rmapxDev(width()/2);
                               if (ppos < 0)
                                     ppos = 0;
                               emit followEvent(ppos);
@@ -135,21 +147,47 @@ void Canvas::draw(QPainter& p, const QRect& rect)
             //---------------------------------------------------
 
             iCItem to(items.lower_bound(x2));
-            for (iCItem i = items.begin(); i != to; ++i) {
-                  if (i->second->intersects(rect))
-                        drawItem(p, i->second, i->second->bbox().intersect(rect));
-                  }
-            to = moving.lower_bound(x2);
-            for (iCItem i = moving.begin(); i != to; ++i) {
-                  if (i->second->intersects(rect))
-                        drawItem(p, i->second, i->second->bbox().intersect(rect));
-                  }
-
+            
+            // Draw items from other parts behind all others.
+            // Only for items with events (not arranger parts).
+            for(iCItem i = items.begin(); i != to; ++i)
+            { 
+              CItem* ci = i->second;
+              if(!ci->event().empty() && ci->part() != curPart)
+              {
+                drawItem(p, ci, rect);
+              }
             }
-      else {
+            
+            for (iCItem i = items.begin(); i != to; ++i) 
+            {
+                  CItem* ci = i->second;
+                  // Draw unselected parts behind selected.
+                  if(!ci->isSelected() && !ci->isMoving() && (ci->event().empty() || ci->part() == curPart))
+                  {
+                        drawItem(p, ci, rect);
+                  }      
+            }
+            
+            // Draw selected parts in front of unselected.
+            for (iCItem i = items.begin(); i != to; ++i) 
+            {
+                  CItem* ci = i->second;
+                  if(ci->isSelected() && !ci->isMoving() && (ci->event().empty() || ci->part() == curPart))
+                  {
+                        drawItem(p, ci, rect);
+                  }      
+            }
+            to = moving.lower_bound(x2);
+            for (iCItem i = moving.begin(); i != to; ++i) 
+            {
+                        drawItem(p, i->second, rect);
+                  }
+            }
+      else {  
             p.save();
             setPainter(p);
-
+           
             if (xmag <= 0) {
                   x -= 1;
                   w += 2;
@@ -187,20 +225,40 @@ void Canvas::draw(QPainter& p, const QRect& rect)
             //---------------------------------------------------
             // draw Canvas Items
             //---------------------------------------------------
-
+           
+            // Draw items from other parts behind all others.
+            // Only for items with events (not arranger parts).
+            for(iCItem i = items.begin(); i != items.end(); ++i)
+            { 
+              CItem* ci = i->second;
+              if(!ci->event().empty() && ci->part() != curPart)
+              {
+                drawItem(p, ci, rect);
+              }
+            }  
+              
             for (iCItem i = items.begin(); i != items.end(); ++i) {
                   CItem* ci = i->second;
-                  QRect r(ci->bbox());
-                  r.moveCenter(map((ci->pos())));
-                  if (r.intersects(rect))
+                  // Draw unselected parts behind selected.
+                  if(!ci->isSelected() && !ci->isMoving() && (ci->event().empty() || ci->part() == curPart))
+                    {
                         drawItem(p, ci, rect);
+                    }      
                   }
-            for (iCItem i = moving.begin(); i != moving.end(); ++i) {
-                  QRect r(i->second->bbox());
-                  r.moveCenter(map(i->second->pos()));
-                  if (r.intersects(rect))
+            
+            // Draw selected parts in front of unselected.
+            for (iCItem i = items.begin(); i != items.end(); ++i) {
+                  CItem* ci = i->second;
+                  if(ci->isSelected() && !ci->isMoving() && (ci->event().empty() || ci->part() == curPart))
+                    {    
+                        drawItem(p, ci, rect);
+                    }    
+                  } 
+            for (iCItem i = moving.begin(); i != moving.end(); ++i) 
+                  {
                         drawItem(p, i->second, rect);
                   }
+            p.save();
             setPainter(p);
             }
 
@@ -232,7 +290,7 @@ void Canvas::draw(QPainter& p, const QRect& rect)
       if (pos[0] >= unsigned(x) && pos[0] < unsigned(x2)) {
             p.drawLine(pos[0], y, pos[0], y2);
             }
-
+      
       //---------------------------------------------------
       //    draw lasso
       //---------------------------------------------------
@@ -242,6 +300,23 @@ void Canvas::draw(QPainter& p, const QRect& rect)
             p.setBrush(NoBrush);
             p.drawRect(lasso);
             }
+      
+      //---------------------------------------------------
+      //    draw moving items
+      //---------------------------------------------------
+      
+      if(virt()) 
+      {
+        for(iCItem i = moving.begin(); i != moving.end(); ++i) 
+          drawMoving(p, i->second, rect);
+      }
+      else 
+      {  
+        p.restore();
+        for(iCItem i = moving.begin(); i != moving.end(); ++i) 
+          drawMoving(p, i->second, rect);
+        setPainter(p);
+      }
       }
 
 #define WHEEL_STEPSIZE 40
@@ -272,6 +347,11 @@ void Canvas::wheelEvent(QWheelEvent* ev)
 
       }
 
+void Canvas::redirectedWheelEvent(QWheelEvent* ev)
+      {
+      wheelEvent(ev);
+      }
+
 //---------------------------------------------------------
 //   deselectAll
 //---------------------------------------------------------
@@ -314,9 +394,13 @@ void Canvas::startMoving(const QPoint& pos, DragType)
 //          2     move only vertical
 //---------------------------------------------------------
 
-void Canvas::moveItems(const QPoint& pos, int dir = 0)
+void Canvas::moveItems(const QPoint& pos, int dir = 0, bool rasterize)
       {
-      int dp = y2pitch(pos.y()) - y2pitch(start.y());
+      int dp;
+      if(rasterize)
+        dp = y2pitch(pos.y()) - y2pitch(start.y());
+      else  
+        dp = pos.y() - start.y();
       int dx = pos.x() - start.x();
       if (dir == 1)
             dp = 0;
@@ -326,8 +410,18 @@ void Canvas::moveItems(const QPoint& pos, int dir = 0)
             int x = i->second->pos().x();
             int y = i->second->pos().y();
             int nx = x + dx;
-            int ny = pitch2y(y2pitch(y) + dp);
-            QPoint mp(raster(QPoint(nx, ny)));
+            int ny;
+            QPoint mp;
+            if(rasterize)
+            {
+              ny = pitch2y(y2pitch(y) + dp);
+              mp = raster(QPoint(nx, ny));
+            }  
+            else  
+            {  
+              ny = y + dp;
+              mp = QPoint(nx, ny);
+            }  
             if (i->second->mp() != mp) {
                   i->second->setMp(mp);
                   itemMoved(i->second, mp);
@@ -356,18 +450,18 @@ void Canvas::viewMousePressEvent(QMouseEvent* event)
       // special events if right button is clicked while operations
       // like moving or drawing lasso is performed.
       if (event->stateAfter() & RightButton) {
-	  switch (drag) {
-	        case DRAG_LASSO:
-	    		drag = DRAG_OFF;
-			redraw();
-			return;
-		case DRAG_MOVE:
-			drag = DRAG_OFF;
-			endMoveItems (start, MOVE_MOVE, 0);
-			return;
-		default:
-			break;
-	    }
+    switch (drag) {
+          case DRAG_LASSO:
+          drag = DRAG_OFF;
+      redraw();
+      return;
+    case DRAG_MOVE:
+      drag = DRAG_OFF;
+      endMoveItems (start, MOVE_MOVE, 0);
+      return;
+    default:
+      break;
+      }
       }
 
       // ignore event if (another) button is already active:
@@ -388,6 +482,8 @@ void Canvas::viewMousePressEvent(QMouseEvent* event)
             curItem = items.find(start);
       else {
             curItem = 0;
+            iCItem ius;
+            bool usfound = false;
             for (iCItem i = items.begin(); i != items.end(); ++i) {
                   QRect box = i->second->bbox();
                   int x = rmapxDev(box.x());
@@ -397,10 +493,21 @@ void Canvas::viewMousePressEvent(QMouseEvent* event)
                   QRect r(x, y, w, h);
                   r.moveBy(i->second->pos().x(), i->second->pos().y());
                   if (r.contains(start)) {
-                        curItem = i->second;
-                        break;
+                        if(i->second->isSelected())
+                        {
+                          curItem = i->second;
+                          break;
                         }
+                        else
+                        if(!usfound)
+                        {
+                          ius = i;
+                          usfound = true;
+                        }
+                     }
                   }
+                  if(!curItem && usfound)
+                    curItem = ius->second;
             }
 
       if (curItem && (event->button() == QMouseEvent::MidButton)) {
@@ -448,19 +555,31 @@ void Canvas::viewMousePressEvent(QMouseEvent* event)
             switch (_tool) {
                   case PointerTool:
                         if (curItem) {
-                              itemPressed(curItem);
                               if (curItem->part() != curPart) {
                                     curPart = curItem->part();
                                     curPartId = curPart->sn();
                                     curPartChanged();
                                     }
+                              itemPressed(curItem);
+                              // Changed by T356. Alt is default reserved for moving the whole window in KDE. Changed to Shift-Alt.
+                              // Hmm, nope, shift-alt is also reserved sometimes. Must find a way to bypass, 
+                              //  why make user turn off setting? Left alone for now...
                               if (shift)
                                     drag = DRAG_COPY_START;
                               else if (alt) {
                                     drag = DRAG_CLONE_START;
                                     }
-                             else if (ctrl) { //Select all on the same pitch (e.g. same y-value)
+                              //
+                              //if (shift)
+                              //{
+                              //  if (alt)
+                              //    drag = DRAG_CLONE_START;
+                              //  else
+                              //    drag = DRAG_COPY_START;
+                              //}       
+                              else if (ctrl) { //Select all on the same pitch (e.g. same y-value)
                                     deselectAll();
+                                    //printf("Yes, ctrl and press\n");
                                     for (iCItem i = items.begin(); i != items.end(); ++i) {
                                           if (i->second->y() == curItem->y() )
                                                 selectItem(i->second, true);
@@ -515,18 +634,260 @@ void Canvas::viewMousePressEvent(QMouseEvent* event)
       mousePress(event);
       }
 
+void Canvas::scrollTimerDone()
+      {
+      if (drag != DRAG_OFF && doScroll){
+          bool doHMove = false;
+          bool doVMove = false;
+          int hoff = rmapx(xOffset())+mapx(xorg)-1;
+          int curxpos;
+          switch(hscrollDir)
+          {  
+            case HSCROLL_RIGHT:
+              hoff += scrollSpeed;
+              switch(drag) 
+              {
+                case DRAG_NEW:
+                case DRAG_RESIZE:
+                case DRAGX_MOVE:
+                case DRAGX_COPY:
+                case DRAGX_CLONE:
+                case DRAGY_MOVE:
+                case DRAGY_COPY:
+                case DRAGY_CLONE:
+                case DRAG_MOVE:
+                case DRAG_COPY:
+                case DRAG_CLONE:
+                  emit horizontalScrollNoLimit(hoff);
+                  canScrollLeft = true;
+                  ev_pos.setX(rmapxDev(rmapx(ev_pos.x()) + scrollSpeed));
+                  doHMove = true;
+                break;
+                default:  
+                  if(canScrollRight)
+                  {
+                    curxpos = xpos;
+                    emit horizontalScroll(hoff);
+                    if(xpos <= curxpos)
+                    {  
+                      canScrollRight = false;
+                    }
+                    else
+                    {
+                      canScrollLeft = true;
+                      ev_pos.setX(rmapxDev(rmapx(ev_pos.x()) + scrollSpeed));
+                      doHMove = true;
+                    }  
+                  }  
+                  else
+                  {  
+                  }
+                break;
+              }
+            break;  
+            case HSCROLL_LEFT:
+              if(canScrollLeft)
+              {
+                curxpos = xpos;
+                hoff -= scrollSpeed;
+                emit horizontalScroll(hoff);
+                if(xpos >= curxpos)
+                {  
+                  canScrollLeft = false;
+                }
+                else
+                {
+                  canScrollRight = true;
+                  ev_pos.setX(rmapxDev(rmapx(ev_pos.x()) - scrollSpeed));
+                  doHMove = true;
+                }
+              }    
+              else
+              {  
+              }
+            break; 
+            default:
+            break;   
+          }
+          int voff = rmapy(yOffset())+mapy(yorg);
+          int curypos;
+          switch(vscrollDir)
+          {
+            case VSCROLL_DOWN:
+              if(canScrollDown)
+              {
+                curypos = ypos;
+                voff += scrollSpeed;
+                emit verticalScroll(voff);
+                if(ypos <= curypos)
+                {  
+                  canScrollDown = false;
+                }
+                else
+                {
+                  canScrollUp = true;
+                  ev_pos.setY(rmapyDev(rmapy(ev_pos.y()) + scrollSpeed));
+                  doVMove = true;
+                }
+              }    
+              else
+              {  
+              }
+            break;  
+            case VSCROLL_UP:
+              if(canScrollUp)
+              {
+                curypos = ypos;
+                voff -= scrollSpeed;
+                emit verticalScroll(voff);
+                if(ypos >= curypos)
+                {  
+                  canScrollUp = false;
+                }
+                else
+                {
+                  canScrollDown = true;
+                  ev_pos.setY(rmapyDev(rmapy(ev_pos.y()) - scrollSpeed));
+                  doVMove = true;
+                } 
+              }   
+              else
+              {  
+              }
+            break;
+            default:
+            break;
+          }
+          
+          if(!doHMove && !doVMove)
+          {
+            delete scrollTimer;
+            scrollTimer=NULL;
+            doScroll = false;
+            return;
+          }  
+          QPoint dist = ev_pos - start;
+          switch(drag) 
+          {
+            case DRAG_MOVE:
+            case DRAG_COPY:
+            case DRAG_CLONE:
+                  moveItems(ev_pos, 0, false);
+                  break;
+            case DRAGX_MOVE:
+            case DRAGX_COPY:
+            case DRAGX_CLONE:
+                  moveItems(ev_pos, 1, false);
+                  break;
+            case DRAGY_MOVE:
+            case DRAGY_COPY:
+            case DRAGY_CLONE:
+                  moveItems(ev_pos, 2, false);
+                  break;
+            case DRAG_LASSO:
+                  lasso = QRect(start.x(), start.y(), dist.x(), dist.y());
+                  redraw();
+                  break;
+            case DRAG_NEW:
+            case DRAG_RESIZE:
+                  if (dist.x()) {
+                        if (dist.x() < 1)
+                              curItem->setWidth(1);
+                        else
+                              curItem->setWidth(dist.x());
+                        redraw();
+                        }
+                  break;
+            default:  
+                  break;
+          }
+          scrollTimer->start( 40, TRUE ); // X ms single-shot timer
+        }
+      else {
+          delete scrollTimer;
+          scrollTimer=NULL;
+          }
+      }
+
+
 //---------------------------------------------------------
 //   viewMouseMoveEvent
 //---------------------------------------------------------
 
 void Canvas::viewMouseMoveEvent(QMouseEvent* event)
       {
-      QPoint pos  = event->pos();
-      QPoint dist = pos - start;
+      
+      ev_pos  = event->pos();
+      QPoint dist = ev_pos - start;
       int ax      = ABS(rmapx(dist.x()));
       int ay      = ABS(rmapy(dist.y()));
       bool moving = (ax >= 2) || (ay > 2);
 
+      // set scrolling variables: doScroll, scrollRight
+      if (drag != DRAG_OFF) {
+            
+                
+            int ex = rmapx(event->x())+mapx(0);
+            if(ex < 40 && canScrollLeft)
+              hscrollDir = HSCROLL_LEFT;
+            else  
+            if(ex > (width() - 40))
+              switch(drag) 
+              {
+                case DRAG_NEW:
+                case DRAG_RESIZE:
+                case DRAGX_MOVE:
+                case DRAGX_COPY:
+                case DRAGX_CLONE:
+                case DRAGY_MOVE:
+                case DRAGY_COPY:
+                case DRAGY_CLONE:
+                case DRAG_MOVE:
+                case DRAG_COPY:
+                case DRAG_CLONE:
+                    hscrollDir = HSCROLL_RIGHT;
+                break;
+                default:
+                  if(canScrollRight)
+                    hscrollDir = HSCROLL_RIGHT;
+                  else  
+                    hscrollDir = HSCROLL_NONE;
+                break;
+              }
+            else  
+              hscrollDir = HSCROLL_NONE;
+            int ey = rmapy(event->y())+mapy(0);
+            if(ey < 15 && canScrollUp)
+              vscrollDir = VSCROLL_UP;
+            else  
+            if(ey > (height() - 15) && canScrollDown)
+              vscrollDir = VSCROLL_DOWN;
+            else  
+              vscrollDir = VSCROLL_NONE;
+            if(hscrollDir != HSCROLL_NONE || vscrollDir != VSCROLL_NONE)
+            {
+              doScroll=true;
+              if (!scrollTimer) 
+              {
+                  scrollTimer= new QTimer(this);
+                  connect( scrollTimer, SIGNAL(timeout()), SLOT(scrollTimerDone()) );
+                  scrollTimer->start( 0, TRUE ); // single-shot timer
+              }
+            }
+            else
+                doScroll=false;
+                
+          }
+      else
+      {
+        doScroll=false;
+        
+        canScrollLeft = true;
+        canScrollRight = true;
+        canScrollUp = true;
+        canScrollDown = true;
+      }
+
       switch (drag) {
             case DRAG_LASSO_START:
                   if (!moving)
@@ -536,16 +897,13 @@ void Canvas::viewMouseMoveEvent(QMouseEvent* event)
                   // proceed with DRAG_LASSO:
 
             case DRAG_LASSO:
+                  {
                   lasso = QRect(start.x(), start.y(), dist.x(), dist.y());
-                  //redraw(QRect(10, 0, width()+10, height()));
-                 /* printf("event->x()=%d start.x()=%d mapx=%d mapxdev=%d rmapx=%d, rmapxdev=%d width()=%d\n",
-                  event->x(), start.x(), mapx(1), mapxDev(1),rmapx(1), rmapxDev(1), width());
-                  printf("xOffset=%d rmapxDev(width())=%d rmapxDev(mapxDev(1))=%d\n", xOffset(), rmapxDev(width()), rmapxDev(mapxDev(1)));
 
+                  // printf("xorg=%d xmag=%d event->x=%d, mapx(xorg)=%d rmapx0=%d xOffset=%d rmapx(xOffset()=%d\n",
+                  //         xorg, xmag, event->x(),mapx(xorg), rmapx(0), xOffset(),rmapx(xOffset()));
 
-                  if (event->x() > rmapxDev(width()))
-                      emit horizontalScroll(rmapxDev(1)+40);
-                      //  emit followEvent(40);*/
+                  }
                   redraw();
                   break;
 
@@ -595,25 +953,30 @@ void Canvas::viewMouseMoveEvent(QMouseEvent* event)
                         dt = MOVE_COPY;
                   else
                         dt = MOVE_CLONE;
-                  startMoving(pos, dt);
+                  
+                  startMoving(ev_pos, dt);
                   break;
 
             case DRAG_MOVE:
             case DRAG_COPY:
             case DRAG_CLONE:
-                  moveItems(pos, 0);
+      
+                  if(!scrollTimer)
+                    moveItems(ev_pos, 0);
                   break;
 
             case DRAGX_MOVE:
             case DRAGX_COPY:
             case DRAGX_CLONE:
-                  moveItems(pos, 1);
+                  if(!scrollTimer)
+                    moveItems(ev_pos, 1);
                   break;
 
             case DRAGY_MOVE:
             case DRAGY_COPY:
             case DRAGY_CLONE:
-                  moveItems(pos, 2);
+                  if(!scrollTimer)
+                    moveItems(ev_pos, 2);
                   break;
 
             case DRAG_NEW:
@@ -627,13 +990,14 @@ void Canvas::viewMouseMoveEvent(QMouseEvent* event)
                         }
                   break;
             case DRAG_DELETE:
-                  deleteItem(pos);
+                  deleteItem(ev_pos);
                   break;
 
             case DRAG_OFF:
                   break;
             }
-      mouseMove(pos);
+                  
+      mouseMove(ev_pos);
       }
 
 //---------------------------------------------------------
@@ -644,6 +1008,11 @@ void Canvas::viewMouseReleaseEvent(QMouseEvent* event)
       {
 // printf("release %x %x\n", event->state(), event->button());
 
+      doScroll = false;
+      canScrollLeft = true;
+      canScrollRight = true;
+      canScrollUp = true;
+      canScrollDown = true;
       if (event->state() & (LeftButton|RightButton|MidButton) & ~(event->button())) {
             printf("ignore %x  %x\n", keyState, event->button());
             return;
@@ -783,6 +1152,13 @@ void Canvas::endMoveItems(const QPoint& pos, DragType dragtype, int dir)
             dp = 0;
       else if (dir == 2)
             dx = 0;
+      
+      
+      
+      int modified = 0;
+      
+      // Removed by T356.
+      /*
       for (iCItem i = moving.begin(); i != moving.end(); ++i) {
             int x = i->second->pos().x();
             int y = i->second->pos().y();
@@ -790,7 +1166,8 @@ void Canvas::endMoveItems(const QPoint& pos, DragType dragtype, int dir)
             int ny = pitch2y(y2pitch(y) + dp);
             QPoint newpos = raster(QPoint(nx, ny));
             selectItem(i->second, true);
-            if (moveItem(i->second, newpos, dragtype))
+            
+            if (moveItem(i->second, newpos, dragtype, &modified))
                   i->second->move(newpos);
             if (moving.size() == 1) {
                   itemReleased(curItem, newpos);
@@ -798,7 +1175,11 @@ void Canvas::endMoveItems(const QPoint& pos, DragType dragtype, int dir)
             if (dragtype == MOVE_COPY || dragtype == MOVE_CLONE)
                   selectItem(i->second, false);
             }
-      endUndo(dragtype);
+      */
+      
+      moveCanvasItems(moving, dp, dx, dragtype, &modified);
+      
+      endUndo(dragtype, modified);
       moving.clear();
       updateSelection();
       redraw();
@@ -845,8 +1226,8 @@ void Canvas::deleteItem(const QPoint& p)
                   if (r.contains(p)) {
                         if (deleteItem(i->second)) {
                               selectItem(i->second, false);
-                              break;
                               }
+                        break;
                         }
                   }
             }
