@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//    $Id: checkbox.h,v 1.2 2004/02/04 19:51:27 wschweer Exp $
+//    $Id: checkbox.h,v 1.2.2.2 2006/10/29 07:54:52 terminator356 Exp $
 //  (C) Copyright 2004 Werner Schweer (ws@seh.de)
 //=========================================================
 
@@ -20,11 +20,18 @@ class CheckBox : public QCheckBox {
 
       int _id;
 
+   protected:
+      void mousePressEvent(QMouseEvent *e);
+      void mouseReleaseEvent(QMouseEvent *e);
+   
    private slots:
       void hasToggled(bool val);
 
    signals:
       void toggleChanged(bool, int);
+      void checkboxPressed(int);
+      void checkboxReleased(int);
+      void checkboxRightClicked(const QPoint &, int);
 
    public:
       CheckBox(QWidget* parent, int i, const char* name = 0);
