@@ -35,7 +35,10 @@ class Knob : public SliderBase, public ScaleIf
       double d_nTurns;
 
       QRect  kRect;
+      bool _faceColSel;
       QColor d_faceColor;
+      QColor d_curFaceColor;
+      QColor d_altFaceColor;
       QColor d_markerColor;
 
       void recalcAngle();
@@ -47,7 +50,7 @@ class Knob : public SliderBase, public ScaleIf
       void paintEvent(QPaintEvent *e);
       void resizeEvent(QResizeEvent *e);
       double getValue(const QPoint &p);
-      void getScrollMode( const QPoint &p, int &scrollMode, int &direction );
+      void getScrollMode( QPoint &p, const Qt::ButtonState &button, int &scrollMode, int &direction );
       void scaleChange()             { repaint(false); }
       void fontChange(const QFont &) { repaint(false); }
 
@@ -58,6 +61,14 @@ class Knob : public SliderBase, public ScaleIf
       void setKnobWidth(int w);
       void setTotalAngle (double angle);
       void setBorderWidth(int bw);
+      void selectFaceColor(bool alt);
+      bool selectedFaceColor() { return _faceColSel; }
+      QColor faceColor() { return d_faceColor; }
+      void setFaceColor(const QColor c);
+      QColor altFaceColor() { return d_altFaceColor; }
+      void setAltFaceColor(const QColor c);
+      QColor markerColor() { return d_markerColor; }
+      void setMarkerColor(const QColor c);
       };
 
 
