@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//  $Id: xml.cpp,v 1.17.2.1 2005/07/13 18:59:14 spamatica Exp $
+//  $Id: xml.cpp,v 1.17.2.6 2009/12/07 20:48:45 spamatica Exp $
 //
 //  (C) Copyright 2000 Werner Schweer (ws@seh.de)
 //=========================================================
@@ -137,6 +137,12 @@ void Xml::stoken()
                                     c = '"';
                               else if (strcmp(entity, "amp") == 0)
                                     c = '&';
+                              else if (strcmp(entity, "lt") == 0)
+                                    c = '<';
+                              else if (strcmp(entity, "gt") == 0)
+                                    c = '>';
+                              else if (strcmp(entity, "apos") == 0)
+                                    c = '\\';
                               else
                                     entity[k] = c;
                               break;
@@ -154,9 +160,10 @@ void Xml::stoken()
                   else
                         buffer[i++] = c;
                   }
+            else if(c != EOF)
+              buffer[i++] = c;
             if (c == EOF)
                   break;
-            buffer[i++] = c;
             next();
             }
       buffer[i] = 0;
@@ -182,7 +189,7 @@ QString Xml::strip(const QString& s)
 
 Xml::Token Xml::parse()
       {
-      char buffer[1024*32];   // TODO handle overflow
+      char buffer[1024*1024];   // increase buffer -rj
       char* p;
 
  again:
@@ -352,6 +359,12 @@ Xml::Token Xml::parse()
                         *dp = 0;
                         if (strcmp(name, "lt") == 0)
                               c = '<';
+                        else if (strcmp(name, "gt") == 0)
+                              c = '>';
+                        else if (strcmp(name, "apos") == 0)
+                              c = '\\';
+                        else if (strcmp(name, "quot") == 0)
+                              c = '"';
                         else if (strcmp(name, "amp") == 0)
                               c = '&';
                         else
@@ -427,6 +440,24 @@ int Xml::parseInt()
       return n;
       }
 
+//---------------------------------------------------------
+//   parseUInt
+//---------------------------------------------------------
+// Added by Tim. p3.3.8
+
+unsigned int Xml::parseUInt()
+      {
+      QString s(parse1().simplifyWhiteSpace());
+      bool ok;
+      int base = 10;
+      if (s.startsWith("0x") || s.startsWith("0X")) {
+            base = 16;
+            s = s.mid(2);
+            }
+      unsigned int n = s.toUInt(&ok, base);
+      return n;
+      }
+
 //---------------------------------------------------------
 //   parseFloat
 //---------------------------------------------------------
@@ -557,6 +588,12 @@ void Xml::intTag(int level, const char* name, int val)
       fprintf(f, "<%s>%d</%s>\n", name, val, name);
       }
 
+void Xml::uintTag(int level, const char* name, unsigned int val)
+      {
+      putLevel(level);
+      fprintf(f, "<%s>%u</%s>\n", name, val, name);
+      }
+
 void Xml::floatTag(int level, const char* name, float val)
       {
       putLevel(level);
@@ -580,6 +617,9 @@ void Xml::strTag(int level, const char* name, const char* val)
                   switch(*val) {
                         case '&': fprintf(f, "&amp;"); break;
                         case '<': fprintf(f, "&lt;"); break;
+                        case '>': fprintf(f, "&gt;"); break;
+                        case '\\': fprintf(f, "&apos;"); break;
+                        case '"': fprintf(f, "&quot;"); break;
                         default: fputc(*val, f); break;
                         }
                   ++val;
@@ -656,6 +696,21 @@ void Xml::skip(const QString& etag)
             }
       }
 
+//---------------------------------------------------------
+//   xmlString
+//---------------------------------------------------------
+
+QString Xml::xmlString(const QString& ss)
+      {
+      QString s(ss);
+      s.replace('&', "&amp;");
+      s.replace('<', "&lt;");
+      s.replace('>', "&gt;");
+      s.replace('\'', "&apos;");
+      s.replace('"', "&quot;");
+      return s;
+      }
+
 void Xml::dump(QString &dump)
       {
       if (f == 0)
