@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//  $Id: xml.h,v 1.8.2.1 2005/07/13 18:59:14 spamatica Exp $
+//  $Id: xml.h,v 1.8.2.3 2009/11/09 20:28:28 terminator356 Exp $
 //
 //  (C) Copyright 2000 Werner Schweer (ws@seh.de)
 //=========================================================
@@ -58,6 +58,7 @@ class Xml {
       QString parse(const QString&);
       QString parse1();
       int parseInt();
+      unsigned int parseUInt();
       float parseFloat();
       double parseDouble();
       void unknown(const char*);
@@ -75,6 +76,7 @@ class Xml {
       void tag(int level, const char* format, ...);
       void etag(int level, const char* format, ...);
       void intTag(int level, const char* const name, int val);
+      void uintTag(int level, const char* const name, unsigned int val);
       void doubleTag(int level, const char* const name, double val);
       void floatTag(int level, const char* const name, float val);
       void strTag(int level, const char* const name, const char* val);
@@ -82,6 +84,7 @@ class Xml {
       void colorTag(int level, const char* name, const QColor& color);
       void geometryTag(int level, const char* name, const QWidget* g);
       void qrectTag(int level, const char* name, const QRect& r);
+      static QString xmlString(const QString&);
 
       void skip(const QString& tag);
       };
