@@ -38,6 +38,7 @@
 
 #include "deicsonzegui.h"
 
+#include "muse/midi.h"
 #define ABS(x) (x>=0?x:-x)
 
 
@@ -643,7 +644,7 @@ void DeicsOnze::loadSutulaPresets()
     file = fopen (presetPath.latin1(), "rt");
     if (file == NULL) {
 	printf("can't open ");
-	printf(presetPath.latin1());
+	printf("%s",presetPath.latin1());
 	printf("\n");
     }
     else
@@ -1107,7 +1108,6 @@ inline double env2AmpR(int sr, float* wt, Eg eg, OpVoice* p_opVoice)
 	  p_opVoice->envLevel=1.0;
 	  p_opVoice->coefVLevel=envD1R2coef(eg.d1r, sr);
 	}
-      return p_opVoice->envLevel;
       break;
     case DECAY:
       if (p_opVoice->envLevel>((double)eg.d1l/(double)MAXD1L)+COEFERRDECSUS)
@@ -1121,7 +1121,6 @@ inline double env2AmpR(int sr, float* wt, Eg eg, OpVoice* p_opVoice)
 	  p_opVoice->envLevel=((double)eg.d1l/(double)MAXD1L);
 	  p_opVoice->coefVLevel=envD1R2coef(eg.d2r, sr);//probably the same
 	}
-      return p_opVoice->envLevel;
       break;
     case SUSTAIN:
       if (p_opVoice->envLevel>COEFERRSUSREL)
@@ -1134,7 +1133,6 @@ inline double env2AmpR(int sr, float* wt, Eg eg, OpVoice* p_opVoice)
 	  p_opVoice->envState=OFF;
 	  p_opVoice->envLevel=0.0;
 	}
-      return p_opVoice->envLevel;
       break;
     case RELEASE:
       if (p_opVoice->envLevel > COEFERRSUSREL)
@@ -1146,13 +1144,17 @@ inline double env2AmpR(int sr, float* wt, Eg eg, OpVoice* p_opVoice)
 	  p_opVoice->envState=OFF;
 	  p_opVoice->envLevel=0.0;
 	}
-      return p_opVoice->envLevel;
       break;
-    case OFF: return 0.0;
-      break;
-    default: printf("Error case envelopeState");
-      break;
-    }
+    default: 
+       printf("Error case envelopeState");
+       /* fall thru */
+    case OFF:
+       p_opVoice->envLevel = 0.0;
+       break;
+     }
+    return p_opVoice->envLevel;
+
+
 }
 
 //---------------------------------------------------------
@@ -1218,8 +1220,7 @@ bool DeicsOnze::setController(int ch, int ctrl, int val)
 	}
 	break;
 	case CTRL_PITCH:
-	    printf("CONTROLE PITCH %d\n",val);
-	    setPitchBendCoef(ch, val);
+            setPitchBendCoef(ch, val);
 	    break;
 	case CTRL_SUSTAIN:
 	    setSustain(ch, val);
@@ -1234,8 +1235,13 @@ bool DeicsOnze::setController(int ch, int ctrl, int val)
 //   getPatchName
 //---------------------------------------------------------
 
-const char* DeicsOnze::getPatchName(int /*ch*/, int val, int) const
+const char* DeicsOnze::getPatchName(int /*ch*/, int val, int, bool /*drum*/) const
 {
+    int prog =   val & 0xff;
+    if(val == CTRL_VAL_UNKNOWN || prog == 0xff)
+          return "<unknown>";
+    prog &= 0x7f;
+    
     Preset* p_preset;
     int hbank = (val & 0xff0000) >> 16;
     int lbank = (val & 0xff00) >> 8;
@@ -1245,8 +1251,7 @@ const char* DeicsOnze::getPatchName(int /*ch*/, int val, int) const
 	lbank = 0;
     if (lbank == 127)       // drum HACK
 	lbank = 128;
-    int prog =   val & 0x7f;
-    char* name="---";
+    const char* name="<unknown>";
     p_preset=_categorySet->findPreset(lbank, prog);
     if (p_preset) name=const_cast<char *>(p_preset->name.c_str());
     return name;
@@ -1581,6 +1586,8 @@ void DeicsOnze::process(float** buffer, int offset, int n)
 				       || _voices[j].op[3].envState!=OFF);
 		      break;
 		  default : printf("Error : No algorithm");
+			
+			sampleOp[3]= sampleOp[2]= sampleOp[1]= sampleOp[0]=0; //prevent unitialiazed use of variable
 		      break;
 	      }
 	
@@ -1596,6 +1603,33 @@ void DeicsOnze::process(float** buffer, int offset, int n)
     }
 }
 
+//---------------------------------------------------------
+//   processEvent
+//    All events from the sequencer go here
+//---------------------------------------------------------
+bool DeicsOnze::processEvent(const MidiPlayEvent& ev)
+{
+  switch(ev.type()) {
+    case ME_CONTROLLER:
+      setController(ev.channel(), ev.dataA(), ev.dataB());
+      return true;
+    case ME_NOTEON:
+      return playNote(ev.channel(), ev.dataA(), ev.dataB());
+    case ME_NOTEOFF:
+      return playNote(ev.channel(), ev.dataA(), 0);
+    case ME_SYSEX:
+      return sysex(ev.len(), ev.data());
+    case ME_PITCHBEND:
+      setController(ev.channel(), CTRL_PITCH, ev.dataA());
+      break;            
+    case ME_PROGRAM:
+      setController(ev.channel(), CTRL_PROGRAM, ev.dataA());
+      break;   
+    default:
+      break;
+  }
+  return false;
+}
 
 //---------------------------------------------------------
 //   inst
@@ -1603,7 +1637,7 @@ void DeicsOnze::process(float** buffer, int offset, int n)
 
 class QWidget;
 
-static Mess* instantiate(int sr, QWidget*, const char*)
+static Mess* instantiate(int sr, QWidget*, QString* projectPathPtr, const char*)
 {
     DeicsOnze* deicsonze = new DeicsOnze();
     deicsonze->setSampleRate(sr);
