@@ -212,6 +212,13 @@ DeicsOnzeGui::DeicsOnzeGui(DeicsOnze* deicsOnze)
 	(void) new QListViewItemCategory
 	    (categoryListView, _deicsOnze->_categorySet->_categoryVector[i]
 	     ->_categoryName.c_str(), _deicsOnze->_categorySet->_categoryVector[i]);
+
+      // work around for probable QT/WM interaction bug.
+      // for certain window managers, e.g xfce, this window is
+      // is displayed although not specifically set to show();
+      // bug: 2811156  	 Softsynth GUI unclosable with XFCE4 (and a few others)
+      show();
+      hide();
 };
 
 
