@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//  $Id: fluidgui.cpp,v 1.6 2004/06/01 14:25:50 wschweer Exp $
+//  $Id: fluidgui.cpp,v 1.6.2.1 2009/08/12 20:47:01 spamatica Exp $
 //
 //    This is a simple GUI implemented with QT for
 //    fluid software synthesizer.
@@ -33,6 +33,13 @@ FLUIDGui::FLUIDGui()
       {
       connect(fdialogButton, SIGNAL(clicked()), SLOT(soundFontFileDialog()));
       connect(loadButton, SIGNAL(clicked()), SLOT(loadFont()));
+
+      // work around for probable QT/WM interaction bug.
+      // for certain window managers, e.g xfce, this window is
+      // is displayed although not specifically set to show();
+      // bug: 2811156  	 Softsynth GUI unclosable with XFCE4 (and a few others)
+      show();
+      hide();
       }
 
 //---------------------------------------------------------
