@@ -3,7 +3,7 @@
  *
  * Copyright (C) 2004 Mathias Lundgren (lunar_shuttle@users.sourcforge.net)
  *
- * $Id: fluidsynti.cpp,v 1.19.2.5 2005/04/29 22:45:42 lunar_shuttle Exp $
+ * $Id: fluidsynti.cpp,v 1.19.2.18 2009/12/06 10:05:00 terminator356 Exp $
  *
  */
 
@@ -12,37 +12,53 @@
 
 #include <list>
 #include <iostream>
+#include <qfileinfo.h>
 
 
 FluidCtrl FluidSynth::fluidCtrl[] = {
-      { "Gain", FS_GAIN ,0, 127},
-      { "Master reverb on/off", FS_REVERB_ON , 0, 1},
-      { "Master reverb level", FS_REVERB_LEVEL, 0, 16384 },
-      { "Master reverb size", FS_REVERB_ROOMSIZE, 0, 16384 }, // Interval: [0,1]
-      { "Master reverb damping", FS_REVERB_DAMPING, 0, 16384 }, // Interval: [0,1]
-      { "Master reverb width", FS_REVERB_WIDTH, 0, 16384 }, // Interval: [0,100]
-      { "Master chorus on/off", FS_CHORUS_ON, 0, 1},
-      { "Master chorus num delay lines", FS_CHORUS_NUM, 0, 10 }, //Default: 3
-      { "Master chorus type",  FS_CHORUS_TYPE, 0, 1 },
-      { "Master chorus speed", FS_CHORUS_SPEED, 0, 16384 }, // (0.291,5) Hz
-      { "Master chorus depth", FS_CHORUS_DEPTH, 0, 16384 }, // [0,40]
-      { "Master chorus level", FS_CHORUS_LEVEL, 0, 16384 }, // [0,1]
-      { "Modulation", CTRL_MODULATION, 0, 127 },
-      { "Portamento time", CTRL_PORTAMENTO_TIME, 0, 127 },
-      { "Volume", CTRL_VOLUME, 0, 127 },
-      { "Pan", CTRL_PANPOT, 0, 127 },
-      { "Expression", CTRL_EXPRESSION, 0, 127 },
-      { "Sustain", CTRL_SUSTAIN, 0, 127 },
-      { "Portamento", CTRL_PORTAMENTO, 0, 127 },
-      { "Soft Pedal", CTRL_SOFT_PEDAL, 0, 127 },
-      { "Variation", CTRL_VARIATION_SEND, 0, 127 },
-      { "Channel reverb send", CTRL_REVERB_SEND, 0, 127 },
-      { "Channel chorus send", CTRL_CHORUS_SEND, 0, 127 },
-      { "Pitch", CTRL_PITCH, -8192, 8191 }
+      //{ "Expression", CTRL_EXPRESSION, 0, 127 },
+      //{ "Sustain", CTRL_SUSTAIN, 0, 127 },
+      //{ "Portamento", CTRL_PORTAMENTO, 0, 127 },
+      //{ "Soft Pedal", CTRL_SOFT_PEDAL, 0, 127 },
+      //{ "Variation", CTRL_VARIATION_SEND, 0, 127 },
+      //{ "Channel reverb send", CTRL_REVERB_SEND, 0, 127 },
+      //{ "Channel chorus send", CTRL_CHORUS_SEND, 0, 127 },
+      //{ "Pitch", CTRL_PITCH, -8192, 8191 }
+      
+      // These controllers' initial values are set by the FS_PREDEF_ values, so just set them to zero here.
+      { "Gain", FS_GAIN ,0, 127, 0},
+      { "Master reverb on/off", FS_REVERB_ON , 0, 1, 0},
+      { "Master reverb level", FS_REVERB_LEVEL, 0, 16384, 0},
+      { "Master reverb size", FS_REVERB_ROOMSIZE, 0, 16384, 0}, // Interval: [0,1]
+      { "Master reverb damping", FS_REVERB_DAMPING, 0, 16384, 0}, // Interval: [0,1]
+      { "Master reverb width", FS_REVERB_WIDTH, 0, 16384, 0}, // Interval: [0,100]
+      { "Master chorus on/off", FS_CHORUS_ON, 0, 1, 0},
+      { "Master chorus num delay lines", FS_CHORUS_NUM, 0, 10, 0}, //Default: 3
+      { "Master chorus type",  FS_CHORUS_TYPE, 0, 1, 0},
+      { "Master chorus speed", FS_CHORUS_SPEED, 0, 16384, 0}, // (0.291,5) Hz
+      { "Master chorus depth", FS_CHORUS_DEPTH, 0, 16384, 0}, // [0,40]
+      { "Master chorus level", FS_CHORUS_LEVEL, 0, 16384, 0}, // [0,1]
+      
+      { "Program", CTRL_PROGRAM, 0, 0xffffff, 0},
+      { "Modulation", CTRL_MODULATION, 0, 127, 0},
+      { "Portamento time", CTRL_PORTAMENTO_TIME, 0, 127, 0},
+      { "Volume", CTRL_VOLUME, 0, 127, 100},
+      { "Pan", CTRL_PANPOT, -64, 63, 0},
+      { "Expression", CTRL_EXPRESSION, 0, 127, 127},
+      { "Sustain", CTRL_SUSTAIN, 0, 127, 0},
+      { "Portamento", CTRL_PORTAMENTO, 0, 127, 0},
+      { "Soft Pedal", CTRL_SOFT_PEDAL, 0, 127, 0},
+      { "Variation", CTRL_VARIATION_SEND, 0, 127, 0},
+      { "Channel reverb send", CTRL_REVERB_SEND, 0, 127, 40},
+      { "Channel chorus send", CTRL_CHORUS_SEND, 0, 127, 0},
+      { "Pitch", CTRL_PITCH, -8192, 8191, 0},
+      // Added by T356
+      { "Pitch bend sensitivity", FS_PITCHWHEELSENS, 0, 24, 2}
     };
 
 static int NUM_CONTROLLER = sizeof(FluidSynth::fluidCtrl)/sizeof(*(FluidSynth::fluidCtrl));
 
+QString *projPathPtr;
 //
 // Fluidsynth
 //
@@ -50,7 +66,7 @@ FluidSynth::FluidSynth(int sr, pthread_mutex_t *_Globalsfloader_mutex) : Mess(2)
       {
       setSampleRate(sr);
       fluid_settings_t* s = new_fluid_settings();
-      fluid_settings_setnum(s, "synth.sample-rate", float(sampleRate()));
+      fluid_settings_setnum(s, (char*) "synth.sample-rate", float(sampleRate()));
       fluidsynth = new_fluid_synth(s);
       if (!fluidsynth) {
             printf("Error while creating fluidsynth!\n");
@@ -124,13 +140,44 @@ bool FluidSynth::init(const char* name)
       return false;
       }
 
+//---------------------------------------------------------
+//   processMessages
+//   Called from host always, even if output path is unconnected.
+//---------------------------------------------------------
+
+void FluidSynth::processMessages()
+{
+  //Process messages from the gui
+  while (gui->fifoSize()) 
+  {
+    MidiPlayEvent ev = gui->readEvent();
+    if (ev.type() == ME_SYSEX) 
+    {
+      sysex(ev.len(), ev.data());
+      sendEvent(ev);
+    }
+    else if (ev.type() == ME_CONTROLLER) 
+    {
+      setController(ev.channel(), ev.dataA(), ev.dataB(), true);
+      sendEvent(ev);
+    }
+    else 
+    {
+      if (FS_DEBUG)
+            printf("FluidSynth::process(): unknown event, type: %d\n", ev.type());
+    }
+  }
+
+}
+
 //---------------------------------------------------------
 //   process
-//    called from host
+//   Called from host, ONLY if output path is connected.
 //---------------------------------------------------------
 
 void FluidSynth::process(float** ports, int offset, int len)
       {
+      /*
       //Process messages from the gui
       while (gui->fifoSize()) {
             MidiPlayEvent ev = gui->readEvent();
@@ -147,6 +194,7 @@ void FluidSynth::process(float** ports, int offset, int len)
                         printf("FluidSynth::process(): unknown event, type: %d\n", ev.type());
                   }
             }
+      */
 
       if (fluid_synth_write_float(fluidsynth, len, ports[0], offset, 1, ports[1], offset, 1)) {
             M_ERROR("Error writing from synth!");
@@ -160,6 +208,10 @@ void FluidSynth::process(float** ports, int offset, int len)
 //---------------------------------------------------------
 void FluidSynth::getInitData(int* n, const unsigned char** data) const
       {
+
+ //printf("projPathPtr ");
+ //std::cout << *projPathPtr << std::endl;
+
       // Data setup:
       // FS_INIT_DATA (1 byte)
       // FluidSynth version (2 bytes, x.y)
@@ -180,7 +232,14 @@ void FluidSynth::getInitData(int* n, const unsigned char** data) const
       //Calculate length:
       int len = FS_INIT_DATA_HEADER_SIZE + strlen(lastdir.c_str()) + 1; //header size
       for (std::list<FluidSoundFont>::const_iterator it = stack.begin(); it!=stack.end(); it++) {
-            len+=strlen(it->filename.c_str()) + 2;
+
+            // if the soundfont is located under the projectPath we extract this from the filename
+            int fileLen = strlen(it->filename.c_str());
+            if (QString(it->filename).startsWith(*projPathPtr)) {
+                printf("project path found in filename, len %d shortened with %d\n",fileLen, projPathPtr->length()+1);
+                fileLen = fileLen - projPathPtr->length()-1;
+                }
+            len+=fileLen + 2;
             }
       //Add length for lastdir and channels:
       len+=strlen(lastdir.c_str())+1;
@@ -205,8 +264,16 @@ void FluidSynth::getInitData(int* n, const unsigned char** data) const
       //For each font...
       chptr+=strlen(lastdir.c_str())+1;
       for (std::list<FluidSoundFont>::const_iterator it =stack.begin(); it!=stack.end(); it++) {
-            memcpy(chptr, it->filename.c_str(), strlen(it->filename.c_str())+1);
-            chptr = chptr + 1 + strlen(it->filename.c_str());
+
+            // if the soundfont is located under the projectPath we extract this from the filename
+            int offset=0;
+            if (QString(it->filename).startsWith(*projPathPtr)) {
+                offset= projPathPtr->length()+1;
+            }
+
+            memcpy(chptr, it->filename.c_str()+offset, strlen(it->filename.c_str())-offset+1);
+            //printf("path name stored=%s\n", it->filename.c_str()+offset);
+            chptr = chptr + 1 + strlen(it->filename.c_str())-offset;
             }
 
       //For each font again...
@@ -246,6 +313,9 @@ void FluidSynth::getInitData(int* n, const unsigned char** data) const
 //-----------------------------------
 void FluidSynth::parseInitData(int n, const byte* d)
       {
+      printf("projPathPtr ");
+      std::cout << *projPathPtr << std::endl;
+
       bool load_drumchannels = true; // Introduced in initdata ver 0.3
       bool handle_bankvalue  = true; // Introduced in initdata ver 0.4
 
@@ -294,6 +364,12 @@ void FluidSynth::parseInitData(int n, const byte* d)
       for (int i=0; i<nr_of_fonts; i++) {
             fonts[i].filename = (char*)(chptr);
             chptr+=(strlen(fonts[i].filename.c_str())+1);
+
+            if (QFileInfo(fonts[i].filename).isRelative()) {
+                printf("path is relative, we append full path!\n");
+                fonts[i].filename = projPathPtr->ascii() + std::string("/")+ fonts[i].filename;
+                }
+            std::cout << "SOUNDFONT FILENAME + PATH " << fonts[i].filename << std::endl;
             }
 
       if (*chptr != FS_INIT_CHANNEL_SECTION) {
@@ -367,7 +443,16 @@ bool FluidSynth::processEvent(const MidiPlayEvent& ev)
                         printf("\n");
                         }
                   return sysex(ev.len(), ev.data());
-            }
+            case ME_PITCHBEND:
+                setController(ev.channel(), CTRL_PITCH, ev.dataA(), false);
+                break;            
+            
+            case ME_PROGRAM:
+                setController(ev.channel(), CTRL_PROGRAM, ev.dataA(), false);
+            break;   
+            default:
+            break;
+      }
       return false;
       }
 
@@ -477,7 +562,7 @@ static void* fontLoadThread(void* t)
             fptr->sendError(fluid_synth_error(fptr->fluidsynth));
             if (FS_DEBUG)
                   std::cerr << DEBUG_ARGS << "error loading soundfont: " << fluid_synth_error(fptr->fluidsynth) << std::endl;
-
+            
             //Unlock the mutex, or else we might be stuck here forever...
             pthread_mutex_unlock(sfloader_mutex);
             delete h;
@@ -536,8 +621,10 @@ static void* fontLoadThread(void* t)
 
 bool FluidSynth::playNote(int channel, int pitch, int velo)
       {
-      if (channels[channel].font_intid == FS_UNSPECIFIED_FONT)
+      if (channels[channel].font_intid == FS_UNSPECIFIED_FONT ||
+          channels[channel].font_intid == FS_UNSPECIFIED_ID)
             return false;
+      
       if (velo) {
             if (fluid_synth_noteon(fluidsynth, channel, pitch, velo)) {
                   if (FS_DEBUG)
@@ -551,7 +638,6 @@ bool FluidSynth::playNote(int channel, int pitch, int velo)
             }
       return false;
       }
-
 //---------------------------------------------------------
 //   sendSoundFontData
 //---------------------------------------------------------
@@ -786,8 +872,16 @@ void FluidSynth::setController(int channel, int id, int val, bool fromGui)
             // Controllers that depend on channels
             //
             case CTRL_PITCH:
+                  // MusE's range is from -8192 to +8191, fluidsynth seems to be [0, 16384]
+                  val +=8192;
                   err = fluid_synth_pitch_bend (fluidsynth, channel, val);
-                        break;
+                  break;
+                  
+            // Added by T356
+            case FS_PITCHWHEELSENS:
+                  err = fluid_synth_pitch_wheel_sens(fluidsynth, channel, val);
+                  break;
+                  
             case CTRL_PROGRAM: {
                   //Check if MusE is trying to set a preset on an unspecified font. If so, ignore.
                   if (FS_DEBUG)
@@ -824,7 +918,7 @@ void FluidSynth::setController(int channel, int id, int val, bool fromGui)
 //---------------------------------------------------------
 //   getControllerInfo
 //---------------------------------------------------------
-int FluidSynth::getControllerInfo(int id, const char** name, int* controller, int* min, int* max) const
+int FluidSynth::getControllerInfo(int id, const char** name, int* controller, int* min, int* max, int* initval) const
       {
       if (id >= NUM_CONTROLLER)
             return 0;
@@ -832,8 +926,52 @@ int FluidSynth::getControllerInfo(int id, const char** name, int* controller, in
       *name       = fluidCtrl[id].name;
       *min        = fluidCtrl[id].min;
       *max        = fluidCtrl[id].max;
+      switch(id)
+      {
+        case 0:
+          *initval = (int)(fluidCtrl[0].max*FS_PREDEF_VOLUME);
+        break;
+        case 1:
+          *initval = 0;
+        break;
+        case 2:
+          *initval = (int)(fluidCtrl[2].max*FS_PREDEF_REVERB_LEVEL);
+        break;
+        case 3:
+          *initval = (int)(fluidCtrl[3].max*FS_PREDEF_REVERB_ROOMSIZE);
+        break;
+        case 4:
+          *initval = (int)(fluidCtrl[4].max*FS_PREDEF_REVERB_DAMPING);
+        break;
+        case 5:
+          *initval = (int)(fluidCtrl[5].max*FS_PREDEF_REVERB_WIDTH);
+        break;
+        case 6:
+          *initval = 0;
+        break;
+        case 7:
+          *initval = (int)(fluidCtrl[7].max*FS_PREDEF_CHORUS_NUM);
+        break;
+        case 8:
+          *initval = (int)(fluidCtrl[8].max*FS_PREDEF_CHORUS_TYPE);
+        break;
+        case 9:
+          *initval = (int)(fluidCtrl[9].max*FS_PREDEF_CHORUS_SPEED);
+        break;
+        case 10:
+          *initval = (int)(fluidCtrl[10].max*FS_PREDEF_CHORUS_DEPTH);
+        break;
+        case 11:
+          *initval = (int)(fluidCtrl[11].max*FS_PREDEF_CHORUS_LEVEL);
+        break;
+        default:      
+          *initval = fluidCtrl[id].initval;
+        break;  
+      }  
+
       if (FS_DEBUG)
-            printf("FluidSynth::getControllerInfo() id: %d name: %s controller: %d min: %d max: %d\n",id,*name,*controller,*min,*max);
+            //printf("FluidSynth::getControllerInfo() id: %d name: %s controller: %d min: %d max: %d\n",id,*name,*controller,*min,*max);
+            printf("FluidSynth::getControllerInfo() id: %d name: %s controller: %d min: %d max: %d initval: %d\n",id,*name,*controller,*min,*max,*initval);
       return ++id;
       }
 
@@ -935,37 +1073,42 @@ void FluidSynth::rewriteChannelSettings()
                   banknum = 128;
 
             //printf("Channel %d, font int-id %d ext_id %d, preset %d\n",i, int_id, getFontExternalIdByChannel(i), preset);
-            if (!(preset == FS_UNSPECIFIED_PRESET || int_id == FS_UNSPECIFIED_FONT)) {
+            if (!(preset == FS_UNSPECIFIED_PRESET 
+                  || int_id == FS_UNSPECIFIED_FONT
+                  || int_id == FS_UNSPECIFIED_ID)) {
                   int rv = fluid_synth_program_select(fluidsynth, i, int_id, banknum, preset);
                   if (rv)
                         std::cerr << DEBUG_ARGS << "Error changing preset! " << fluid_synth_error(fluidsynth) << std::endl;
                   }
             }
       }
-
 //---------------------------------------------------------
 //   getPatchName
 //---------------------------------------------------------
-const char* FluidSynth::getPatchName(int i, int, int) const
+const char* FluidSynth::getPatchName(int i, int, int, bool /*drum*/) const
       {
-      if (channels[i].font_intid == FS_UNSPECIFIED_FONT)
-            return "no preset";
+      if (channels[i].font_intid == FS_UNSPECIFIED_FONT ||
+          channels[i].font_intid == FS_UNSPECIFIED_ID)
+            //return "no preset";
+            return "<unknown>";
       else if (channels[i].preset == FS_UNSPECIFIED_PRESET)
-            return "no preset";
+            //return "no preset";
+            return "<unknown>";
       else {
-            //printf("Getpatchname, channel: %d\n",channel);
             fluid_preset_t *preset = fluid_synth_get_channel_preset(fluidsynth, i);
-            if (!preset) return "no preset";
+            //if (!preset) return "no preset";
+            if (!preset) return "<unknown>";
             return preset->get_name(preset);
             }
       }
-
 //---------------------------------------------------------
 //   getPatchInfo
 //---------------------------------------------------------
 const MidiPatch* FluidSynth::getPatchInfo(int i, const MidiPatch* patch) const
       {
-      if (channels[i].font_intid == FS_UNSPECIFIED_FONT)
+      //if (channels[i].font_intid == FS_UNSPECIFIED_FONT)
+      if (channels[i].font_intid == FS_UNSPECIFIED_FONT ||
+          channels[i].font_intid == FS_UNSPECIFIED_ID)
             return 0;
       //else if (channels[i].preset == FS_UNSPECIFIED_PRESET)
       //      return 0;
@@ -992,7 +1135,8 @@ const MidiPatch* FluidSynth::getFirstPatch (int channel) const
 
       fluid_preset_t* preset;
       int font_id = channels[channel].font_intid;
-      if (font_id == FS_UNSPECIFIED_FONT)
+      //if (font_id == FS_UNSPECIFIED_FONT)
+      if (font_id == FS_UNSPECIFIED_FONT || font_id == FS_UNSPECIFIED_ID)
             return 0;
 
       fluid_sfont_t* sfont = fluid_synth_get_sfont_by_id(fluidsynth, font_id);
@@ -1034,16 +1178,13 @@ const MidiPatch* FluidSynth::getNextPatch (int channel, const MidiPatch* patch)
       static MidiPatch midiPatch;
       //First check if there actually is any soundfont associated to the channel. If not, don't bother
       int font_id = channels[channel].font_intid;
-      if (font_id == FS_UNSPECIFIED_FONT)
+      if (font_id == FS_UNSPECIFIED_FONT || font_id == FS_UNSPECIFIED_ID)
             return 0;
       if (patch == 0)
             return getFirstPatch(channel);
-
       midiPatch.typ = 0;
       midiPatch.lbank = 0;
 
-      if (font_id == FS_UNSPECIFIED_FONT)
-            return 0;
       //printf("Font has internal id: %d\n",font_id);
       fluid_preset_t* preset;
       fluid_sfont_t* sfont = fluid_synth_get_sfont_by_id(fluidsynth, font_id);
@@ -1091,7 +1232,8 @@ bool FluidSynth::popSoundfont (int ext_id)
          bool success = false;
          int int_id = getFontInternalIdByExtId(ext_id);
 
-         if (int_id == FS_UNSPECIFIED_FONT) {
+         //if (int_id == FS_UNSPECIFIED_FONT) {
+         if (int_id == FS_UNSPECIFIED_FONT || int_id == FS_UNSPECIFIED_ID) {
                std::cerr << DEBUG_ARGS << "Internal error! Request for deletion of Soundfont that is not registered!" << std::endl;
                }
          else
@@ -1139,9 +1281,11 @@ class QWidget;
 static  pthread_mutex_t globalMutex;
 static bool mutexEnabled = false;
 
-static Mess* instantiate(int sr, QWidget*, const char* name)
+
+static Mess* instantiate(int sr, QWidget*, QString* projectPathPtr, const char* name)
       {
 printf("fluidsynth sampleRate %d\n", sr);
+      projPathPtr=projectPathPtr;
 
       if (!mutexEnabled) {
           pthread_mutex_init(&globalMutex,NULL);
@@ -1160,7 +1304,7 @@ extern "C"
       {
       static MESS descriptor = {
             "FluidSynth",
-            "Mathias Lundgren (lunar_shuttle@users.sf.net)",
+            "FluidSynth soundfont loader by Mathias Lundgren", //Mathias Lundgren (lunar_shuttle@users.sf.net)
             "0.1",      //Version string
             MESS_MAJOR_VERSION, MESS_MINOR_VERSION,
             instantiate,
