@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//  $Id: organ.cpp,v 1.15.2.1 2005/12/29 23:33:50 spamatica Exp $
+//  $Id: organ.cpp,v 1.15.2.8 2009/12/06 10:05:00 terminator356 Exp $
 //
 //  Parts of this file taken from:
 //      Organ - Additive Organ Synthesizer Voice
@@ -168,11 +168,39 @@ bool Organ::init(const char* name)
       }
 
 //---------------------------------------------------------
-//   write
+//   processMessages
+//   Called from host always, even if output path is unconnected.
+//---------------------------------------------------------
+
+void Organ::processMessages()
+{
+  //Process messages from the gui
+  //
+  //  get and process all pending events from the
+  //  synthesizer GUI
+  //
+  while (gui->fifoSize()) 
+  {
+    MidiPlayEvent ev = gui->readEvent();
+    if (ev.type() == ME_CONTROLLER) 
+    {
+      // process local?
+      setController(ev.dataA(), ev.dataB());
+      sendEvent(ev);
+    }
+    else
+      printf("Organ::process(): unknown event\n");
+  }
+}
+  
+//---------------------------------------------------------
+//   process
+//   Called from host, ONLY if output path is connected.
 //---------------------------------------------------------
 
 void Organ::process(float** ports, int offset, int sampleCount)
       {
+      /*
       //
       //  get and process all pending events from the
       //  synthesizer GUI
@@ -187,7 +215,8 @@ void Organ::process(float** ports, int offset, int sampleCount)
             else
                   printf("Organ::process(): unknown event\n");
             }
-
+      */
+      
       float* buffer = *ports + offset;
       for (int i = 0; i < VOICES; ++i) {
             Voice* v = &voices[i];
@@ -219,7 +248,7 @@ void Organ::process(float** ports, int offset, int sampleCount)
                   freq_256_harm4 = freq_256_harm3 * 2;
                   freq_256_harm5 = freq_256_harm4 * 2;
                   for (int i = 0; i < sampleCount; i++) {
-                        int a1, a2;
+                        int a1=0, a2=0;	//prevent compiler warning: unitialized usage of vars a1 & a2
                         switch(v->state1) {
                               case ATTACK:
                                     if (v->envL1.step(&a1))
@@ -279,7 +308,7 @@ void Organ::process(float** ports, int offset, int sampleCount)
                   freq_256_harm4 = freq_256 * 3;
                   freq_256_harm5 = freq_256_harm3 * 2;
                   for (int i = 0; i < sampleCount; i++) {
-                        int a1, a2;
+                        int a1=0, a2=0;//prevent compiler warning: unitialized usage of vars a1 & a2
                         switch(v->state1) {
                               case ATTACK:
                                     if (v->envL1.step(&a1))
@@ -402,24 +431,35 @@ void Organ::noteoff(int channel, int pitch)
 void Organ::setController(int ctrl, int data)
       {
       int sr = sampleRate();
+      
+      // Changed By T356.
+      // Because of muse's auto-bias controllers, some of these negative-range 
+      //  controls need to apply the auto-bias correction.
+      
       switch (ctrl) {
             case HARM0:
-                  harm0 = cb2amp(-data);
+                  //harm0 = cb2amp(-data);
+                  harm0 = cb2amp(-data + 8192);
                   break;
             case HARM1:
-                  harm1 = cb2amp(-data);
+                  //harm1 = cb2amp(-data);
+                  harm1 = cb2amp(-data + 8192);
                   break;
             case HARM2:
-                  harm2 = cb2amp(-data);
+                  //harm2 = cb2amp(-data);
+                  harm2 = cb2amp(-data + 8192);
                   break;
             case HARM3:
-                  harm3 = cb2amp(-data);
+                  //harm3 = cb2amp(-data);
+                  harm3 = cb2amp(-data + 8192);
                   break;
             case HARM4:
-                  harm4 = cb2amp(-data);
+                  //harm4 = cb2amp(-data);
+                  harm4 = cb2amp(-data + 8192);
                   break;
             case HARM5:
-                  harm5 = cb2amp(-data);
+                  //harm5 = cb2amp(-data);
+                  harm5 = cb2amp(-data + 8192);
                   break;
             case ATTACK_LO:   // maxval -> 500msec
                   attack0 = (data * sr) / 1000;
@@ -428,7 +468,8 @@ void Organ::setController(int ctrl, int data)
                   decay0 = (data * sr) / 1000;
                   break;
             case SUSTAIN_LO:
-                  sustain0 = -data;
+                  //sustain0 = -data;
+                  sustain0 = -data + 8192;
                   break;
             case RELEASE_LO:
                   release0 = (data * sr) / 1000;
@@ -440,7 +481,8 @@ void Organ::setController(int ctrl, int data)
                   decay1 =  (data * sr) / 1000;
                   break;
             case SUSTAIN_HI:
-                  sustain1 = -data;
+                  //sustain1 = -data;
+                  sustain1 = -data + 8192;
                   break;
             case RELEASE_HI:
                   release1 = (data * sr) / 1000;
@@ -559,14 +601,21 @@ void Organ::getInitData(int* n, const unsigned char**p) const
 //---------------------------------------------------------
 
 int Organ::getControllerInfo(int id, const char** name, int* controller,
-   int* min, int* max) const
+   int* min, int* max, int* initval) const
       {
       if (id >= NUM_CONTROLLER)
             return 0;
       *controller = synthCtrl[id].num;
       *name       = synthCtrl[id].name;
+      *initval    = synthCtrl[id].val;
       
-      gui->getControllerMinMax(id,min,max);
+      if(synthCtrl[id].num == CTRL_VOLUME)
+      {
+        *min = 0;
+        *max = 127;
+      }
+      else
+        gui->getControllerMinMax(id,min,max);
       
       //*min        = 0;
       //*max        = 128*128-1;
@@ -620,7 +669,7 @@ void Organ::setGeometry(int x, int y, int w, int h)
 //    construct a new synthesizer instance
 //---------------------------------------------------------
 
-static Mess* instantiate(int sr, QWidget*, const char* name)
+static Mess* instantiate(int sr, QWidget*, QString* projectPathPtr, const char* name)
       {
       Organ* synth = new Organ(sr);
       if (synth->init(name)) {
@@ -638,7 +687,7 @@ static Mess* instantiate(int sr, QWidget*, const char* name)
 extern "C" {
       static MESS descriptor = {
             "Organ",
-            "Organ; based on David A. Bartold's LADSPA plugin",
+            "Organ based on David A. Bartold's LADSPA plugin",
             "0.1",      // version string
             MESS_MAJOR_VERSION, MESS_MINOR_VERSION,
             instantiate,
