@@ -1,7 +1,7 @@
 //=========================================================
 //  MusE
 //  Linux Music Editor
-//  $Id: organgui.cpp,v 1.16.2.1 2005/12/29 23:33:50 spamatica Exp $
+//  $Id: organgui.cpp,v 1.16.2.3 2009/11/16 04:30:46 terminator356 Exp $
 //
 //    This is a simple GUI implemented with QT for
 //    organ software synthesizer.
@@ -65,6 +65,13 @@ OrganGui::OrganGui()
                   connect((QCheckBox*)(dctrl[i].editor), SIGNAL(toggled(bool)), map, SLOT(map()));
             }
       connect(map, SIGNAL(mapped(int)), this, SLOT(ctrlChanged(int)));
+
+      // work around for probable QT/WM interaction bug.
+      // for certain window managers, e.g xfce, this window is
+      // is displayed although not specifically set to show();
+      // bug: 2811156  	 Softsynth GUI unclosable with XFCE4 (and a few others)
+      show();
+      hide();
       }
 
 //---------------------------------------------------------
@@ -78,6 +85,9 @@ void OrganGui::ctrlChanged(int idx)
       if (ctrl->type == SynthGuiCtrl::SLIDER) {
             QSlider* slider = (QSlider*)(ctrl->editor);
             val = slider->value();
+            // By T356. Apply auto-bias center value.
+            if(slider->minValue() < 0)
+              val += 8192;
             }
       else if (ctrl->type == SynthGuiCtrl::SWITCH) {
             val = ((QCheckBox*)(ctrl->editor))->isChecked();
@@ -131,6 +141,11 @@ void OrganGui::setParam(int param, int val)
 //             int max = slider->maxValue();
 //             if(val < 0) val = (val * max + 8191) / 16383 - 1;
 //             else val = (val * max + 8191) / 16383;
+            
+            // By T356. Apply auto-bias center value.
+            if(slider->minValue() < 0)
+              val -= 8192;
+            
             slider->setValue(val);
             if (ctrl->label)
                   ((QSpinBox*)(ctrl->label))->setValue(val);
